@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -38,8 +38,10 @@ public:
     static const float DB_FLOOR;
 
     enum FaderType {
-	ShortFader = 0, // -40 -> +6  dB
-  	 LongFader = 1  // -70 -> +10 dB
+	     ShortFader = 0, // -40 -> +6  dB
+              LongFader = 1, // -70 -> +10 dB
+            IEC268Meter = 2, // -70 ->  0  dB
+	IEC268LongMeter = 3  // -70 -> +10 dB (0dB aligns with LongFader)
     };
 
     static float multiplier_to_dB(float multiplier);
