@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -20,6 +20,7 @@
 
 #include <vector>
 #include <string>
+#include <map>
 
 #include "XmlExportable.h"
 
@@ -40,8 +41,6 @@ class PluginPort
 public:
     typedef enum
     {
-    // Currently things will go wrong if these values differ
-    // from the same constants in LADSPA.  That's bad.
         Input    = 0x01,
         Output   = 0x02,
         Control  = 0x04,
@@ -50,14 +49,13 @@ public:
 
     typedef enum
     {
-    // These are our own, though.
         NoHint      = 0x00,
 	Toggled     = 0x01,
 	Integer     = 0x02,
 	Logarithmic = 0x04
     } PortDisplayHint;
 
-    PluginPort(int id,
+    PluginPort(int number,
                std::string m_name,
                PortType type,
                PortDisplayHint displayHint,
@@ -65,7 +63,7 @@ public:
                PortData upperBound,
 	       PortData defaultValue);
 
-    int getId() const { return m_id; }
+    int getNumber() const { return m_number; }
     std::string getName() const { return m_name; }
     PortType getType() const { return m_type; }
     PortDisplayHint getDisplayHint() const { return m_displayHint; }
@@ -75,7 +73,7 @@ public:
 
 protected:
 
-    int             m_id;
+    int             m_number;
     std::string     m_name;
     PortType        m_type;
     PortDisplayHint m_displayHint;
@@ -87,11 +85,11 @@ protected:
 class PluginPortInstance
 {
 public:
-    PluginPortInstance(unsigned int i,
-                       float v):id(i), value(v) {;}
+    PluginPortInstance(unsigned int n,
+                       float v):number(n), value(v) {;}
 
-    unsigned int id;
-    PortData     value;
+    int number;
+    PortData value;
 };
 
 typedef std::vector<PluginPortInstance*>::iterator PortInstanceIterator;
@@ -100,11 +98,12 @@ class AudioPluginInstance : public XmlExportable
 {
 public:
     AudioPluginInstance(unsigned int position);
-    AudioPluginInstance(unsigned long id,
-                        unsigned int position);
 
-    void setId(unsigned long id) { m_id = id; }
-    unsigned long getId() const { return m_id; }
+    AudioPluginInstance(std::string identifier,
+			unsigned int position);
+
+    void setIdentifier(std::string identifier) { m_identifier = identifier; }
+    std::string getIdentifier() const { return m_identifier; }
 
     void setPosition(unsigned int position) { m_position = position; }
     unsigned int getPosition() const { return m_position; }
@@ -114,9 +113,9 @@ public:
 
     // Port management
     //
-    void addPort(unsigned int id, PortData value);
-    bool removePort(unsigned int id);
-    PluginPortInstance* getPort(unsigned int it);
+    void addPort(int number, PortData value);
+    bool removePort(int number);
+    PluginPortInstance* getPort(int number);
     void clearPorts();
 
     unsigned int getPortCount() const { return m_ports.size(); }
@@ -132,13 +131,24 @@ public:
     void setBypass(bool bypass) { m_bypass = bypass; }
     bool isBypassed() const { return m_bypass; }
 
+    void setProgram(std::string program) { m_program = program; }
+    std::string getProgram() const { return m_program; }
+
     int getMappedId() const { return m_mappedId; }
     void setMappedId(int value) { m_mappedId = value; }
 
+    typedef std::map<std::string, std::string> ConfigMap;
+    void clearConfiguration() { m_config.clear(); }
+    const ConfigMap &getConfiguration() { return m_config; }
+    std::string getConfigurationValue(std::string k) const;
+    void setConfigurationValue(std::string k, std::string v);
+
+    std::string getDistinctiveConfigurationText() const;
+
 protected:
 
     int                                m_mappedId;
-    unsigned long                      m_id;
+    std::string                        m_identifier;
     std::vector<PluginPortInstance*>   m_ports;
     unsigned int                       m_position;
 
@@ -148,6 +158,9 @@ protected:
     bool                               m_assigned; 
     bool                               m_bypass;
 
+    std::string                        m_program;
+
+    ConfigMap                          m_config;
 };
 
 }
