@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -33,6 +33,7 @@
 #include "Configuration.h"
 #include "XmlExportable.h"
 #include "ColourMap.h"
+#include "TriggerSegment.h"
 
 #include "Marker.h"
 
@@ -66,10 +67,15 @@ public:
 
     typedef std::map<TrackId, Track*> trackcontainer;
     typedef trackcontainer::iterator trackiterator;
+    typedef trackcontainer::const_iterator trackconstiterator;
 
     typedef std::vector<Marker*> markercontainer;
-    typedef std::vector<Marker*>::iterator markeriterator;
-    typedef std::vector<Marker*>::const_iterator markerconstiterator;
+    typedef markercontainer::iterator markeriterator;
+    typedef markercontainer::const_iterator markerconstiterator;
+
+    typedef std::set<TriggerSegmentRec *, TriggerSegmentCmp> triggersegmentcontainer;
+    typedef triggersegmentcontainer::iterator triggersegmentcontaineriterator;
+    typedef triggersegmentcontainer::const_iterator triggersegmentcontainerconstiterator;
 
     Composition();
     virtual ~Composition();
@@ -104,6 +110,7 @@ public:
     //////
     //
     //  INSTRUMENT & TRACK
+
     Track* getTrackById(TrackId track);
 
     Track* getTrackByPosition(int position);
@@ -273,6 +280,78 @@ public:
     bool weakDetachSegment(Segment*);
 
 
+    //////
+    //
+    //  TRIGGER SEGMENTS
+
+    triggersegmentcontainer &getTriggerSegments() { return m_triggerSegments; }
+    const triggersegmentcontainer &getTriggerSegments() const { return m_triggerSegments; }
+
+    /**
+     * Add a new trigger Segment with a given base pitch and base
+     * velocity, and return its record.  If pitch or velocity is -1,
+     * it will be taken from the first note event in the segment
+     */
+    TriggerSegmentRec *addTriggerSegment(Segment *, int pitch = -1, int velocity = -1);
+
+    /**
+     * Delete a trigger Segment.
+     */
+    void deleteTriggerSegment(TriggerSegmentId);
+
+    /**
+     * Detach a trigger Segment from the Composition.
+     */
+    void detachTriggerSegment(TriggerSegmentId);
+
+    /**
+     * Delete all trigger Segments.
+     */
+    void clearTriggerSegments();
+    
+    /**
+     * Return the TriggerSegmentId for the given Segment, or -1 if it is
+     * not a trigger Segment.
+     */
+    int getTriggerSegmentId(Segment *);
+    
+    /**
+     * Return the Segment for a given TriggerSegmentId
+     */
+    Segment *getTriggerSegment(TriggerSegmentId);
+
+    /**
+     * Return the TriggerSegmentRec (with Segment, base pitch, base velocity,
+     * references etc) for a given TriggerSegmentId
+     */
+    TriggerSegmentRec *getTriggerSegmentRec(TriggerSegmentId);
+
+    /**
+     * Add a new trigger Segment with a given ID and base pitch and
+     * velocity.  Fails and returns 0 if the ID is already in use.
+     * This is intended for use from file load or from undo/redo.
+     */
+    TriggerSegmentRec *addTriggerSegment(Segment *, TriggerSegmentId,
+					 int basePitch = -1, int baseVelocity = -1);
+
+    /**
+     * Get the ID of the next trigger segment that will be inserted.
+     */
+    TriggerSegmentId getNextTriggerSegmentId() const;
+
+    /**
+     * Specify the next trigger ID.  This is intended for use from file
+     * load only.  Do not use this function unless you know what you're
+     * doing.
+     */
+    void setNextTriggerSegmentId(TriggerSegmentId);
+
+    /**
+     * Update the trigger segment references for all trigger segments.
+     * To be called after file load.
+     */
+    void updateTriggerSegmentReferences();
+
 
     //////
     //
@@ -630,6 +709,7 @@ public:
     ColourMap& getGeneralColourMap() { return m_generalColourMap; }
     void setGeneralColourMap(Rosegarden::ColourMap &newmap);
 
+
     //////
     //
     // QUANTIZERS
@@ -711,6 +791,7 @@ protected:
     void checkSelectedAndRecordTracks();
     TrackId getClosestValidTrackId(TrackId id) const;
     
+
     //--------------- Data members ---------------------------------
     //
     trackcontainer                    m_tracks;
@@ -832,6 +913,11 @@ protected:
     // User defined markers in the composition
     //
     markercontainer                   m_markers;
+
+    // Trigger segments (unsorted segments fired by events elsewhere)
+    //
+    triggersegmentcontainer           m_triggerSegments;
+    TriggerSegmentId                  m_nextTriggerSegmentId;
  
     ColourMap                         m_segmentColourMap;
     ColourMap                         m_generalColourMap;
