@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -101,9 +101,13 @@ private:
 
 namespace CompositionMetadataKeys
 {
-    extern const PropertyName Copyright;
     extern const PropertyName Composer;
-    extern const PropertyName Notes;
+    extern const PropertyName Title;
+    extern const PropertyName Subtitle;
+    extern const PropertyName Arranger;
+    extern const PropertyName Copyright;
+
+    std::vector<PropertyName> getFixedKeys();
 }
 
 class DocumentConfiguration : public Configuration
@@ -123,9 +127,6 @@ public:
     // Property names
     static const PropertyName SequencerOptions;
 
-    static const PropertyName PluginTaxonomyFile;
-    static const PropertyName PluginDescriptionsFile;
-
     static const PropertyName ZoomLevel;
 };
 
