@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -19,14 +19,14 @@
     COPYING included with this distribution for more information.
 */
 
+#ifndef _DEVICE_H_
+#define _DEVICE_H_
+
 #include "XmlExportable.h"
 #include "Instrument.h"
 #include <string>
 #include <vector>
 
-#ifndef _DEVICE_H_
-#define _DEVICE_H_
-
 // A Device can query underlying hardware/sound APIs to
 // generate a list of Instruments.
 //
@@ -45,7 +45,8 @@ public:
     typedef enum 
     {
         Midi,
-        Audio
+        Audio,
+	SoftSynth
     } DeviceType;
 
     // special device ids
