@@ -5,7 +5,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -24,12 +24,11 @@
 #define _NOTATION_TYPES_H_
 
 #include <list>
+#include <map>
 
 #include "Event.h"
 #include "Instrument.h"
 
-#include "StringHash.h"
-
 /*
  * NotationTypes.h
  *
@@ -119,6 +118,7 @@ typedef std::string Mark;
 namespace Marks //!!! This would be better as a class, these days
 {
     extern const Mark NoMark;         // " "
+
     extern const Mark Accent;         // ">"
     extern const Mark Tenuto;         // "-"  ("legato" in RG2.1)
     extern const Mark Staccato;       // "."
@@ -126,12 +126,22 @@ namespace Marks //!!! This would be better as a class, these days
     extern const Mark Marcato;        // "^"
     extern const Mark Sforzando;      // "sf"
     extern const Mark Rinforzando;    // "rf"
+
     extern const Mark Trill;          // "tr"
+    extern const Mark LongTrill;      // with wiggly line
+    extern const Mark TrillLine;      // line on its own
     extern const Mark Turn;           // "~"
+
     extern const Mark Pause;          // aka "fermata"
+
     extern const Mark UpBow;          // "v"
     extern const Mark DownBow;        // a square with the bottom side missing
 
+    extern const Mark Mordent;
+    extern const Mark MordentInverted;
+    extern const Mark MordentLong;
+    extern const Mark MordentLongInverted;
+
     /**
      * Given a string, return a mark that will be recognised as a
      * text mark containing that string.  For example, the Sforzando
@@ -510,7 +520,7 @@ private:
         std::string m_rg2name;
 	int    m_tonicPitch;
 
-        KeyDetails(); // ctor needed in order to live in a hash_map
+        KeyDetails(); // ctor needed in order to live in a map
 
         KeyDetails(bool sharps, bool minor, int sharpCount,
                    std::string equivalence, std::string rg2name,
@@ -522,7 +532,7 @@ private:
     };
 
 
-    typedef hash_string<KeyDetails> KeyDetailMap;
+    typedef std::map<std::string, KeyDetails> KeyDetailMap;
     static KeyDetailMap m_keyDetailMap;
     static void checkMap();
     void checkAccidentalHeights() const;
@@ -543,10 +553,10 @@ public:
     static const std::string EventType;
     static const int EventSubOrdering;
     static const PropertyName IndicationTypePropertyName;
-//    static const PropertyName IndicationDurationPropertyName;
     typedef Exception BadIndicationName;
 
     static const std::string Slur;
+    static const std::string PhrasingSlur;
     static const std::string Crescendo;
     static const std::string Decrescendo;
     static const std::string Glissando;
@@ -615,6 +625,7 @@ public:
     static const std::string ChordName;
     static const std::string KeyName;
     static const std::string Lyric;
+    static const std::string Chord;
     static const std::string Dynamic;
     static const std::string Direction;
     static const std::string LocalDirection;
@@ -960,25 +971,29 @@ public:
     TimeSignature() :
         m_numerator(DefaultTimeSignature.m_numerator),
         m_denominator(DefaultTimeSignature.m_denominator),
-	m_common(false), m_hidden(false) { }
+	m_common(false), m_hidden(false), m_hiddenBars(false) { }
 
     /**
      * Construct a TimeSignature object describing a time signature
-     * with the given numerator and denominator.  If preferCommon
-     * is true and the time signature is a common or cut-common
-     * time, the constructed object will return true for isCommon;
-     * if hidden is true, the time signature is intended not to
-     * be displayed and isHidden will return true.
+     * with the given numerator and denominator.  If preferCommon is
+     * true and the time signature is a common or cut-common time, the
+     * constructed object will return true for isCommon; if hidden is
+     * true, the time signature is intended not to be displayed and
+     * isHidden will return true; if hiddenBars is true, the bar lines
+     * between this time signature and the next will not be shown.
      */
     TimeSignature(int numerator, int denominator,
-		  bool preferCommon = false, bool hidden = false)
+		  bool preferCommon = false,
+		  bool hidden = false,
+		  bool hiddenBars = false)
         /* throw (BadTimeSignature) */;
     
     TimeSignature(const TimeSignature &ts) :
         m_numerator(ts.m_numerator),
         m_denominator(ts.m_denominator),
 	m_common(ts.m_common),
-	m_hidden(ts.m_hidden) { }
+	m_hidden(ts.m_hidden),
+	m_hiddenBars(ts.m_hiddenBars) { }
 
     ~TimeSignature() { }
 
@@ -996,6 +1011,7 @@ public:
     
     bool isCommon()        const { return m_common; }
     bool isHidden()	   const { return m_hidden; }
+    bool hasHiddenBars()   const { return m_hiddenBars; }
 
     timeT getBarDuration() const;
 
@@ -1079,6 +1095,7 @@ private:
     static const PropertyName DenominatorPropertyName;
     static const PropertyName ShowAsCommonTimePropertyName;
     static const PropertyName IsHiddenPropertyName;
+    static const PropertyName HasHiddenBarsPropertyName;
 
     /// Returned event is on heap; caller takes responsibility for ownership
     Event *getAsEvent(timeT absoluteTime) const;
@@ -1089,6 +1106,7 @@ private:
 
     bool m_common;
     bool m_hidden;
+    bool m_hiddenBars;
 
     mutable int  m_barDuration;
     mutable int  m_beatDuration;
@@ -1130,13 +1148,13 @@ public:
     };
 
     enum BarResetType {
-	BarResetTotal,      // c# | c -> omit natural
+	BarResetNone,       // c# | c -> omit natural
 	BarResetCautionary, // c# | c -> add natural to c in brackets
-	BarResetNaturals    // c# | c -> add natural to c
+	BarResetExplicit    // c# | c -> add natural to c
     };
 
     AccidentalTable(const Key &, const Clef &,
-		    OctaveType = OctavesIndependent,
+		    OctaveType = OctavesCautionary,
 		    BarResetType = BarResetCautionary);
 
     AccidentalTable(const AccidentalTable &);
