@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -31,6 +31,7 @@
 #include "BaseProperties.h"
 #include "NotationTypes.h"
 #include "MidiTypes.h"
+#include "Quantizer.h"
 
 namespace Rosegarden
 {
@@ -108,7 +109,7 @@ public:
     static Event *getAsEvent(const Iterator &i);
 
 protected:
-    AbstractSet(const Container &c, Iterator elementInSet, const Quantizer *);
+    AbstractSet(Container &c, Iterator elementInSet, const Quantizer *);
     void initialise();
 
     /// Return true if this element is not definitely beyond bounds of set
@@ -117,12 +118,12 @@ protected:
     /// Return true if this element, known to test() true, is a set member
     virtual bool sample(const Iterator &i, bool goingForwards);
 
-    const Container &getContainer() const { return m_container; }
+    Container &getContainer() const { return m_container; }
     const Quantizer &getQuantizer() const { return *m_quantizer; }
 
     // Data members:
 
-    const Container &m_container;
+    Container &m_container;
     Iterator m_initial, m_final, m_initialNote, m_finalNote;
     Iterator m_shortest, m_longest, m_highest, m_lowest;
     Iterator m_baseIterator;
@@ -164,7 +165,7 @@ public:
        making up your chord lack HEIGHT_ON_STAFF properties, in
        which case this constructor will write those properties
        in to the chord for you */
-    GenericChord(const Container &c,
+    GenericChord(Container &c,
 		 Iterator elementInChord,
 		 const Quantizer *quantizer,
 		 PropertyName stemUpProperty = PropertyName::EmptyPropertyName);
@@ -257,8 +258,8 @@ setMaybe__String(Event *e, const PropertyName &name, const std::string &value);
 
 
 template <class Element, class Container>
-AbstractSet<Element, Container>::AbstractSet(const Container &c,
-					   Iterator i, const Quantizer *q):
+AbstractSet<Element, Container>::AbstractSet(Container &c,
+					     Iterator i, const Quantizer *q):
     m_container(c),
     m_initial(c.end()),
     m_final(c.end()),
@@ -357,7 +358,7 @@ AbstractSet<Element, Container>::sample(const Iterator &i, bool)
 //////////////////////////////////////////////////////////////////////
  
 template <class Element, class Container, bool singleStaff>
-GenericChord<Element, Container, singleStaff>::GenericChord(const Container &c,
+GenericChord<Element, Container, singleStaff>::GenericChord(Container &c,
 							    Iterator i,
 							    const Quantizer *q,
 							    PropertyName stemUpProperty) :
@@ -367,10 +368,10 @@ GenericChord<Element, Container, singleStaff>::GenericChord(const Container &c,
     m_subordering(getAsEvent(i)->getSubOrdering()),
     m_firstReject(c.end())
 {
-    initialise();
+	AbstractSet<Element, Container>::initialise();
 
-    if (size() > 1) {
-        std::stable_sort(begin(), end(), PitchGreater());
+    if (std::vector<typename Container::iterator>::size() > 1) {
+        std::stable_sort(std::vector<typename Container::iterator>::begin(), std::vector<typename Container::iterator>::end(), PitchGreater());
     }
 
 /*!!! this should all be removed ultimately
@@ -402,7 +403,7 @@ bool
 GenericChord<Element, Container, singleStaff>::test(const Iterator &i)
 {
     Event *e = getAsEvent(i);
-    if (getQuantizer().getQuantizedAbsoluteTime(e) != m_time) return false;
+    if (AbstractSet<Element, Container>::getQuantizer().getQuantizedAbsoluteTime(e) != m_time) return false;
 
     // We permit note or rest events etc here, because if a chord is a
     // little staggered (for performance reasons) then it's not at all
@@ -433,7 +434,7 @@ GenericChord<Element, Container, singleStaff>::sample(const Iterator &i,
 {
     Event *e1 = getAsEvent(i);
     if (!e1->isa(Note::EventType)) {
-	if (goingForwards && m_firstReject == getContainer().end()) m_firstReject = i;
+	if (goingForwards && m_firstReject == AbstractSet<Element, Container>::getContainer().end()) m_firstReject = i;
 	return false;
     }
 
@@ -441,16 +442,17 @@ GenericChord<Element, Container, singleStaff>::sample(const Iterator &i,
 
 	// Two notes that would otherwise be in a chord but are
 	// explicitly in different groups, or have stems pointing in
-	// different directions by design, count as separate chords.
+	// different directions by design, or have substantially
+	// different x displacements, count as separate chords.
 	
 	// Per #930473 ("Inserting notes into beamed chords is
 	// broken"), if one note is in a group and the other isn't,
 	// that's no problem.  In fact we should actually modify the
 	// one that isn't so as to suggest that it is.
 
-	if (m_baseIterator != getContainer().end()) {
+	if (AbstractSet<Element, Container>::m_baseIterator != AbstractSet<Element, Container>::getContainer().end()) {
 
-	    Event *e0 = getAsEvent(m_baseIterator);
+	    Event *e0 = getAsEvent(AbstractSet<Element, Container>::m_baseIterator);
 
 	    if (!(m_stemUpProperty == PropertyName::EmptyPropertyName)) {
 
@@ -461,17 +463,26 @@ GenericChord<Element, Container, singleStaff>::sample(const Iterator &i,
 		    get__Bool(e0, m_stemUpProperty) !=
 		    get__Bool(e1, m_stemUpProperty)) {
 
-		    if (goingForwards && m_firstReject == getContainer().end())
+		    if (goingForwards && m_firstReject == AbstractSet<Element, Container>::getContainer().end())
 			m_firstReject = i;
 		    return false;
 		}
 	    }
 
+	    long dx0 = 0, dx1 = 0;
+	    get__Int(e0, BaseProperties::DISPLACED_X, dx0);
+	    get__Int(e1, BaseProperties::DISPLACED_X, dx1);
+	    if (abs(dx0 - dx1) >= 700) {
+		if (goingForwards && m_firstReject == AbstractSet<Element, Container>::getContainer().end())
+		    m_firstReject = i;
+		return false;
+	    }
+
 	    if (e0->has(BaseProperties::BEAMED_GROUP_ID)) {
 		if (e1->has(BaseProperties::BEAMED_GROUP_ID)) {
 		    if (get__Int(e1, BaseProperties::BEAMED_GROUP_ID) !=
 			get__Int(e0, BaseProperties::BEAMED_GROUP_ID)) {
-			if (goingForwards && m_firstReject == getContainer().end())
+			if (goingForwards && m_firstReject == AbstractSet<Element, Container>::getContainer().end())
 			    m_firstReject = i;
 			return false;
 		    }
@@ -527,7 +538,7 @@ GenericChord<Element, Container, singleStaff>::getMarkCountForChord() const
 
     std::set<Mark> cmarks;
 
-    for (unsigned int i = 0; i < size(); ++i) {
+    for (unsigned int i = 0; i < std::vector<typename Container::iterator>::size(); ++i) {
 
 	Event *e = getAsEvent((*this)[i]);
 	std::vector<Mark> marks(Marks::getMarks(*e));
@@ -547,7 +558,7 @@ GenericChord<Element, Container, singleStaff>::getMarksForChord() const
 {
     std::vector<Mark> cmarks;
 
-    for (unsigned int i = 0; i < size(); ++i) {
+    for (unsigned int i = 0; i < std::vector<typename Container::iterator>::size(); ++i) {
 
 	Event *e = getAsEvent((*this)[i]);
 	std::vector<Mark> marks(Marks::getMarks(*e));
@@ -575,7 +586,7 @@ GenericChord<Element, Container, singleStaff>::getPitches() const
     std::vector<int> pitches;
 
     for (typename std::vector<typename Container::iterator>::const_iterator
-	     i = begin(); i != end(); ++i) {
+	     i = std::vector<typename Container::iterator>::begin(); i != std::vector<typename Container::iterator>::end(); ++i) {
 	if (getAsEvent(*i)->has(BaseProperties::PITCH)) {
 	    int pitch = get__Int
 		(getAsEvent(*i), Rosegarden::BaseProperties::PITCH);
@@ -594,8 +605,8 @@ bool
 GenericChord<Element, Container, singleStaff>::contains(const Iterator &itr) const
 {
     for (typename std::vector<typename Container::iterator>::const_iterator
-	     i = begin();
-	 i != end(); ++i) {
+	     i = std::vector<typename Container::iterator>::begin();
+	 i != std::vector<typename Container::iterator>::end(); ++i) {
         if (*i == itr) return true;
     }
     return false;
@@ -606,9 +617,9 @@ template <class Element, class Container, bool singleStaff>
 typename GenericChord<Element, Container, singleStaff>::Iterator
 GenericChord<Element, Container, singleStaff>::getPreviousNote()
 {
-    Iterator i(getInitialElement());
+    Iterator i(AbstractSet<Element, Container>::getInitialElement());
     while (1) {
-	if (i == getContainer().begin()) return getContainer().end();
+	if (i == AbstractSet<Element, Container>::getContainer().begin()) return AbstractSet<Element, Container>::getContainer().end();
 	--i;
 	if (getAsEvent(i)->isa(Note::EventType)) {
 	    return i;
@@ -621,14 +632,14 @@ template <class Element, class Container, bool singleStaff>
 typename GenericChord<Element, Container, singleStaff>::Iterator
 GenericChord<Element, Container, singleStaff>::getNextNote()
 {
-    Iterator i(getFinalElement());
-    while (  i != getContainer().end() &&
-	   ++i != getContainer().end()) {
+    Iterator i(AbstractSet<Element, Container>::getFinalElement());
+    while (  i != AbstractSet<Element, Container>::getContainer().end() &&
+	   ++i != AbstractSet<Element, Container>::getContainer().end()) {
 	if (getAsEvent(i)->isa(Note::EventType)) {
 	    return i;
 	}
     }
-    return getContainer().end();
+    return AbstractSet<Element, Container>::getContainer().end();
 }
 
 
