@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -68,6 +68,15 @@ public:
      */
     timeT getSnapTime(double x) const;
 
+    /**
+     * Return the snap setting -- the argument that was passed to
+     * setSnapTime.  This differs from getSnapTime, which interprets
+     * the NoSnap, SnapToBar, SnapToBeat and SnapToUnit settings to
+     * return actual timeT values; instead this function returns those
+     * actual constants if set.
+     */
+    timeT getSnapSetting() const;
+
     /**
      * Return the snap size of the grid, at the given time.  (The time
      * is required in case the built-in snap size is SnapToBar,
