@@ -5,7 +5,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -25,7 +25,12 @@
 
 #include <string>
 
-#if (__GNUC__ < 3)
+#if defined(__INTEL_COMPILER)
+
+#include <ext/hash_map>
+#define __HASH_NS std
+
+#elif (__GNUC__ < 3)
 
 #include <hash_map>
 #define __HASH_NS std
@@ -73,6 +78,5 @@ class hash_string : public __HASH_NS::hash_map<std::string, T, hashstring, eqstr
  
 }
 
-
 #endif
 
