@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -47,6 +47,7 @@
 #include "multiviewcommandhistory.h"
 #include "rosegardenguiview.h"
 #include "rosestrings.h"
+#include "rgapplication.h"
 #include "rosedebug.h"
 
 #include "WAVAudioFile.h"
@@ -183,6 +184,17 @@ AudioManagerDialog::AudioManagerDialog(QWidget *parent,
     h->setMargin(10);
     h->setSpacing(5);
 
+    m_sampleRate = 0;
+    
+    QCString replyType;
+    QByteArray replyData;
+    if (rgapp->sequencerCall("getSampleRate()", replyType, replyData)) {
+        QDataStream streamIn(replyData, IO_ReadOnly);
+        unsigned int result;
+        streamIn >> result;
+        m_sampleRate = result;
+    }
+
     m_fileList = new AudioListView(h);
 
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
@@ -229,19 +241,19 @@ AudioManagerDialog::AudioManagerDialog(QWidget *parent,
 
     // Set the column names
     //
-    m_fileList->addColumn(i18n("Name"));
-    m_fileList->addColumn(i18n("Duration"));
-    m_fileList->addColumn(i18n("Envelope"));
-    m_fileList->addColumn(i18n("File"));
-    m_fileList->addColumn(i18n("Resolution"));
-    m_fileList->addColumn(i18n("Channels"));
-    m_fileList->addColumn(i18n("Sample rate"));
+    m_fileList->addColumn(i18n("Name"));           // 0
+    m_fileList->addColumn(i18n("Duration"));       // 1
+    m_fileList->addColumn(i18n("Envelope"));       // 2
+    m_fileList->addColumn(i18n("Sample rate"));    // 3
+    m_fileList->addColumn(i18n("Channels"));       // 4
+    m_fileList->addColumn(i18n("Resolution"));     // 5
+    m_fileList->addColumn(i18n("File"));           // 6
 
     m_fileList->setColumnAlignment(1, Qt::AlignHCenter);
     m_fileList->setColumnAlignment(2, Qt::AlignHCenter);
+    m_fileList->setColumnAlignment(3, Qt::AlignHCenter);
     m_fileList->setColumnAlignment(4, Qt::AlignHCenter);
     m_fileList->setColumnAlignment(5, Qt::AlignHCenter);
-    m_fileList->setColumnAlignment(6, Qt::AlignHCenter);
 
     m_fileList->restoreLayout(kapp->config(), m_listViewLayoutName);
 
@@ -413,22 +425,26 @@ AudioManagerDialog::slotPopulateFileList()
 
         // File location
         //
-        item->setText(3, QString(
+        item->setText(6, QString(
                     m_doc->getAudioFileManager().
                         substituteHomeForTilde((*it)->getFilename()).c_str()));
                                        
         // Resolution
         //
-        item->setText(4, QString("%1 bits").arg((*it)->getBitsPerSample()));
+        item->setText(5, QString("%1 bits").arg((*it)->getBitsPerSample()));
 
         // Channels
         //
-        item->setText(5, QString("%1").arg((*it)->getChannels()));
+        item->setText(4, QString("%1").arg((*it)->getChannels()));
 
         // Sample rate
         //
-        sRate.sprintf("%.1f KHz", float((*it)->getSampleRate())/ 1000.0);
-        item->setText(6, sRate);
+	if (m_sampleRate != 0 && (*it)->getSampleRate() != m_sampleRate) {
+	    sRate.sprintf("%.1f KHz *", float((*it)->getSampleRate())/ 1000.0);
+	} else {
+	    sRate.sprintf("%.1f KHz", float((*it)->getSampleRate())/ 1000.0);
+	}
+        item->setText(3, sRate);
 
         // Test audio file element for selection criteria
         //
@@ -539,8 +555,8 @@ AudioManagerDialog::slotExportAudio()
 
     QString saveFile =
         KFileDialog::getSaveFileName(":WAVS",
-                                 QString(i18n("*.wav|WAV files (*.wav)")),
-                                 this, i18n("Choose a name to save this file as"));
+                                     i18n("*.wav|WAV files (*.wav)"),
+                                     this, i18n("Choose a name to save this file as"));
     
     if (sourceFile == 0 || item == 0 || segment == 0 || saveFile.isEmpty())
         return;
@@ -641,7 +657,7 @@ AudioManagerDialog::slotRemove()
         return;
     }
 
-    QString question = QString(i18n("This will unload audio file \"%1\" and remove all associated segments.  Are you sure?"))
+    QString question = i18n("This will unload audio file \"%1\" and remove all associated segments.  Are you sure?")
         .arg(QString(audioFile->getFilename().c_str()));
 
     // Ask the question
@@ -733,12 +749,7 @@ void
 AudioManagerDialog::slotAdd()
 {
     KURL::List kurlList =
-        KFileDialog::getOpenURLs(
-#if KDE_VERSION >= 196614
-                                 ":WAVS",
-#else
-                                 QString::null,
-#endif
+        KFileDialog::getOpenURLs(":WAVS",
                                  i18n("*.wav|WAV files (*.wav)\n*.mp3|MP3 files (*.mp3)"),
                                  this, i18n("Select one or more audio files"));
 
@@ -917,8 +928,8 @@ AudioManagerDialog::slotSelectionChanged(QListViewItem *item)
 // purpose.  Set segment to 0 for parent audio entries.
 //
 void
-AudioManagerDialog::setSelected(Rosegarden::AudioFileId id,
-                                Rosegarden::Segment *segment,
+AudioManagerDialog::setSelected(Rosegarden::AudioFileId id, 
+                                const Rosegarden::Segment *segment,
                                 bool propagate)
 {
     QListViewItem *it = m_fileList->firstChild();
@@ -1008,9 +1019,9 @@ void
 AudioManagerDialog::slotSegmentSelection(
         const Rosegarden::SegmentSelection &segments)
 {
-    Rosegarden::Segment *segment = 0;
+    const Rosegarden::Segment *segment = 0;
 
-    for (SegmentSelection::iterator it = segments.begin();
+    for (SegmentSelection::const_iterator it = segments.begin();
                                     it != segments.end(); ++it)
     {
         if ((*it)->getType() == Rosegarden::Segment::Audio) 
@@ -1082,8 +1093,7 @@ AudioManagerDialog::addFile(const KURL& kurl)
 
     } else { // download locally
         if (KIO::NetAccess::download(kurl, newFilePath) == false) {
-            KMessageBox::error(this, QString(i18n("Cannot download file %1"))
-                               .arg(kurl.prettyURL()));
+            KMessageBox::error(this, i18n("Cannot download file %1").arg(kurl.prettyURL()));
             return false;
         }
     }
