@@ -1,8 +1,9 @@
+// -*- c-basic-offset: 4 -*-
 /*
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -74,7 +75,7 @@ public:
     // Scroll and expand to show this selected item
     //
     void setSelected(Rosegarden::AudioFileId id,
-                     Rosegarden::Segment *segment,
+                     const Rosegarden::Segment *segment,
                      bool propagate); // if true then we tell the segmentcanvas
 
     MultiViewCommandHistory *getCommandHistory();
@@ -193,7 +194,8 @@ protected:
     static const int         m_previewHeight;
 
     bool                     m_audiblePreview;
-
+    int                      m_sampleRate;
+    
     static const char* const AudioManagerDialogConfigGroup;
 };
 
