@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -25,8 +25,10 @@
 #include <qhbox.h>
 #include <qdial.h>
 #include <qpushbutton.h>
+#include <qstringlist.h>
 
 #include "Instrument.h"
+#include "config.h"
 
 #ifndef _AUDIOPLUGINDIALOG_H_
 #define _AUDIOPLUGINDIALOG_H_
@@ -40,6 +42,7 @@
 //
 
 class RosegardenRotary;
+class RosegardenGUIDoc;
 class KComboBox;
 class QGroupBox;
 class QCheckBox;
@@ -47,11 +50,16 @@ class QLabel;
 class QGridLayout;
 class QAccel;
 
+#ifdef HAVE_LIBLO
+class AudioPluginOSCGUIManager;
+#endif
+
 namespace Rosegarden
 {
 
 class PluginPort;
 class AudioPluginManager;
+class Studio;
 
 class PluginControl : public QObject
 {
@@ -71,12 +79,18 @@ public:
                   PluginPort *port,
                   AudioPluginManager *pluginManager,
                   int index,
-                  float initialValue);
+                  float initialValue,
+		  bool showBounds,
+		  bool hidden);
  
-    void setValue(float value);
+    void setValue(float value, bool emitSignals = true);
+    float getValue() const;
 
     int getIndex() const { return m_index; }
 
+    void show();
+    void hide();
+
 public slots:
     void slotValueChanged(float value);
 
@@ -109,6 +123,9 @@ class AudioPluginDialog : public KDialogBase
 public:
     AudioPluginDialog(QWidget *parent,
                       AudioPluginManager *aPM,
+#ifdef HAVE_LIBLO
+		      AudioPluginOSCGUIManager *aGM,
+#endif
                       Instrument *instrument,
                       int index);
 
@@ -116,15 +133,37 @@ public:
 
     QAccel* getAccelerators() { return m_accelerators; }
 
+    bool isSynth() { return m_index == int(Instrument::SYNTH_PLUGIN_POSITION); }
+
+    void updatePlugin(int number);
+    void updatePluginPortControl(int port);
+    void updatePluginProgramControl();
+    void updatePluginProgramList();
+    void guiExited() { m_guiShown = false; }
+
 public slots:
     void slotCategorySelected(int);
     void slotPluginSelected(int index);
     void slotPluginPortChanged(float value);
+    void slotPluginProgramChanged(const QString &value);
     void slotBypassChanged(bool);
+    void slotCopy();
+    void slotPaste();
+    void slotDefault();
+    void slotShowGUI();
+
+#ifdef HAVE_LIBLO
+    virtual void slotDetails();
+#endif
 
 signals:
     void pluginSelected(Rosegarden::InstrumentId, int pluginIndex, int plugin);
     void pluginPortChanged(Rosegarden::InstrumentId, int pluginIndex, int portIndex, float value);
+    void pluginProgramChanged(Rosegarden::InstrumentId, int pluginIndex, QString program);
+    void pluginConfigurationChanged(Rosegarden::InstrumentId, int pluginIndex,
+				    bool global, QString key, QString value);
+    void showPluginGUI(Rosegarden::InstrumentId, int pluginIndex);
+    void stopPluginGUI(Rosegarden::InstrumentId, int pluginIndex);
 
     // is the plugin being bypassed
     void bypassed(Rosegarden::InstrumentId, int pluginIndex, bool bp);
@@ -136,12 +175,17 @@ protected slots:
 protected:
     virtual void closeEvent(QCloseEvent *e);
 
-    void makePluginParamsBox(QWidget*);
+    void makePluginParamsBox(QWidget*, int portCount, int tooManyPorts);
+    QStringList getProgramsForInstance(AudioPluginInstance *inst, int &current);
 
     //--------------- Data members ---------------------------------
 
     AudioPluginManager  *m_pluginManager;
+#ifdef HAVE_LIBLO
+    AudioPluginOSCGUIManager *m_pluginGUIManager;
+#endif
     Instrument          *m_instrument;
+    InstrumentId         m_instrumentId;
 
     QFrame		*m_pluginParamsBox;
     QWidget             *m_pluginCategoryBox;
@@ -152,13 +196,20 @@ protected:
     QLabel              *m_insOuts;
     QLabel              *m_pluginId;
     QCheckBox		*m_bypass;
-
+    QPushButton         *m_copyButton;
+    QPushButton         *m_pasteButton;
+    QPushButton         *m_defaultButton;
+    QPushButton         *m_guiButton;
+    
+    QLabel              *m_programLabel;
+    KComboBox           *m_programCombo;
     std::vector<PluginControl*> m_pluginWidgets;
     QGridLayout         *m_gridLayout;
 
     int                  m_index;
 
     bool                 m_generating;
+    bool                 m_guiShown;
 
     QAccel              *m_accelerators;
 
@@ -166,6 +217,8 @@ protected:
     void                 populatePluginList();
 };
 
-}
+
+} // end of namespace
+
 
 #endif // _AUDIOPLUGINDIALOG_H_
