@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -25,6 +25,8 @@
 #include "rosegardendcop.h"
 #include "rosestrings.h"
 
+#include "PluginIdentifier.h"
+
 namespace Rosegarden
 {
 
@@ -35,23 +37,34 @@ AudioPluginManager::AudioPluginManager():m_sampleRate(0)
 {
     // fetch from sequencer
     fetchSampleRate();
+
+    // Clear the plugin clipboard
+    //
+    m_pluginClipboard.m_pluginNumber = -1;
+    m_pluginClipboard.m_program = "";
+    m_pluginClipboard.m_controlValues.clear();
+
 }
 
 AudioPlugin*
-AudioPluginManager::addPlugin(MappedObjectId id,
+AudioPluginManager::addPlugin(const QString &identifier,
                               const QString &name,
                               unsigned long uniqueId,
                               const QString &label,
                               const QString &author,
                               const QString &copyright,
+			      bool isSynth,
+			      bool isGrouped,
 			      const QString &category)
 {
-    AudioPlugin *newPlugin = new AudioPlugin(id, 
+    AudioPlugin *newPlugin = new AudioPlugin(identifier,
                                              name,
                                              uniqueId,
                                              label,
                                              author,
                                              copyright,
+					     isSynth,
+					     isGrouped,
 					     category);
     m_plugins.push_back(newPlugin);
 
@@ -59,13 +72,13 @@ AudioPluginManager::addPlugin(MappedObjectId id,
 }
 
 bool
-AudioPluginManager::removePlugin(MappedObjectId id)
+AudioPluginManager::removePlugin(const QString &identifier)
 {
     std::vector<AudioPlugin*>::iterator it = m_plugins.begin();
 
     for (; it != m_plugins.end(); ++it)
     {
-        if ((*it)->getId() == id)
+        if ((*it)->getIdentifier() == identifier)
         {
             delete *it;
             m_plugins.erase(it);
@@ -99,13 +112,24 @@ AudioPluginManager::getPlugin(int number)
 }
 
 int
-AudioPluginManager::getPositionByUniqueId(unsigned long uniqueId)
+AudioPluginManager::getPositionByIdentifier(QString identifier)
 {
     int pos = 0;
     PluginIterator it = m_plugins.begin();
+
     for (; it != m_plugins.end(); ++it)
     {
-        if ((*it)->getUniqueId() == uniqueId)
+        if ((*it)->getIdentifier() == identifier)
+            return pos;
+
+        pos++;
+    }
+
+    pos = 0;
+    it = m_plugins.begin();
+    for (; it != m_plugins.end(); ++it)
+    {
+        if (PluginIdentifier::areIdentifiersSimilar((*it)->getIdentifier(), identifier))
             return pos;
 
         pos++;
@@ -114,6 +138,26 @@ AudioPluginManager::getPositionByUniqueId(unsigned long uniqueId)
     return -1;
 }
 
+AudioPlugin*
+AudioPluginManager::getPluginByIdentifier(QString identifier)
+{
+    PluginIterator it = m_plugins.begin();
+    for (; it != m_plugins.end(); ++it)
+    {
+        if ((*it)->getIdentifier() == identifier)
+            return (*it);
+    }
+
+    it = m_plugins.begin();
+    for (; it != m_plugins.end(); ++it)
+    {
+        if (PluginIdentifier::areIdentifiersSimilar((*it)->getIdentifier(), identifier))
+            return (*it);
+    }
+
+    return 0;
+}
+
 AudioPlugin*
 AudioPluginManager::getPluginByUniqueId(unsigned long uniqueId)
 {
@@ -149,34 +193,38 @@ AudioPluginManager::fetchSampleRate()
 // ----------------- AudioPlugin ---------------------
 //
 //
-AudioPlugin::AudioPlugin(MappedObjectId id,
+AudioPlugin::AudioPlugin(const QString &identifier,
                          const QString &name,
                          unsigned long uniqueId,
                          const QString &label,
                          const QString &author,
                          const QString &copyright,
+			 bool isSynth,
+			 bool isGrouped,
 			 const QString &category):
-    m_id(id),
+    m_identifier(identifier),
     m_name(name),
     m_uniqueId(uniqueId),
     m_label(label),
     m_author(author),
     m_copyright(copyright),
+    m_isSynth(isSynth),
+    m_isGrouped(isGrouped),
     m_category(category),
     m_colour(Qt::darkRed)
 {
 }
 
 void
-AudioPlugin::addPort(MappedObjectId id,
-                     const QString &name,
+AudioPlugin::addPort(int number,
+		     const QString &name,
                      PluginPort::PortType type,
                      PluginPort::PortDisplayHint hint,
                      PortData lowerBound,
                      PortData upperBound,
 		     PortData defaultValue)
 {
-    PluginPort *port = new PluginPort(id,
+    PluginPort *port = new PluginPort(number,
                                       qstrtostr(name),
                                       type,
                                       hint,
