@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2006
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -24,11 +24,8 @@
 #ifdef HAVE_LIBLO
 
 #include <lo/lo.h>
-#include <iostream>
-
-#ifdef HAVE_DSSI
 #include <dssi.h>
-#endif
+#include <iostream>
 
 #include "PluginIdentifier.h"
 #include "AudioPluginInstance.h"
@@ -558,7 +555,7 @@ AudioPluginOSCGUIManager::dispatch()
 	    RG_DEBUG << "AudioPluginOSCGUIManager: setting port " << port
 		     << " to value " << value << endl;
 
-	    m_app->slotChangePluginPort(instrument, position, port, value);
+	    m_app->slotPluginPortChanged(instrument, position, port, value);
 
 	} else if (method == "program") {
 
@@ -584,7 +581,7 @@ AudioPluginOSCGUIManager::dispatch()
 	    QString programName = Rosegarden::StudioControl::getPluginProgram
 		(pluginInstance->getMappedId(), bank, program);
 
-	    m_app->slotChangePluginProgram(instrument, position, programName);
+	    m_app->slotPluginProgramChanged(instrument, position, programName);
 
 	} else if (method == "update") {
 
@@ -679,13 +676,13 @@ AudioPluginOSCGUIManager::dispatch()
 	    RG_DEBUG << "AudioPluginOSCGUIManager: configure(" << key << "," << value
 		     << ")" << endl;
 
-	    m_app->slotChangePluginConfiguration(instrument, position,
+	    m_app->slotPluginConfigurationChanged(instrument, position,
 #ifdef DSSI_GLOBAL_CONFIGURE_PREFIX
-						 key.startsWith(DSSI_GLOBAL_CONFIGURE_PREFIX),
+						  key.startsWith(DSSI_GLOBAL_CONFIGURE_PREFIX),
 #else
-						 false,
+						  false,
 #endif
-						 key, value);
+						  key, value);
 
 	} else if (method == "midi") {
 
@@ -787,8 +784,6 @@ AudioPluginOSCGUI::getGUIFilePath(QString identifier)
     QString type, soName, label;
     Rosegarden::PluginIdentifier::parseIdentifier(identifier, type, soName, label);
 
-    RG_DEBUG << "AudioPluginOSCGUI::getGUIFilePath(" << identifier << ")" << endl;
-
     QFileInfo soInfo(soName);
     if (soInfo.isRelative()) {
 	//!!!
@@ -818,31 +813,21 @@ AudioPluginOSCGUI::getGUIFilePath(QString identifier)
 	    QFileInfo *info;
 
 	    while ((info = i.current()) != 0) {
-		
-		RG_DEBUG << "Looking at " << info->fileName() << " in path "
-			 << info->filePath() << " for suffix " << (k == nsuffixes ? "(none)" : suffixes[k]) << ", fuzzy " << fuzzy << endl;
 
 		++i;
 
 		if (!(info->isFile() || info->isSymLink())
-		    || !info->isExecutable()) {
-		    RG_DEBUG << "(not executable)" << endl;
-		    continue;
-		}
+		    || !info->isExecutable()) continue;
 
 		if (fuzzy) {
 		    if (info->fileName().left(fileBase.length()) != fileBase) continue;
-		    RG_DEBUG << "(is file base)" << endl;
 		} else {
 		    if (info->fileName().left(label.length()) != label) continue;
-		    RG_DEBUG << "(is label)" << endl;
 		}
 
 		if (k == nsuffixes || info->fileName().lower().endsWith(suffixes[k])) {
-		    RG_DEBUG << "(ends with suffix " << (k == nsuffixes ? "(none)" : suffixes[k]) << " or out of suffixes)" << endl;
 		    return info->filePath();
 		}
-		RG_DEBUG << "(doesn't end with suffix " << (k == nsuffixes ? "(none)" : suffixes[k]) << ")" << endl;
 	    }
 	}
     }
@@ -938,4 +923,3 @@ TimerCallbackAssistant::slotCallback()
 }
 
 
-#include "audiopluginoscgui.moc"
