@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -63,6 +63,7 @@
 
 #include "Studio.h"
 #include "MidiDevice.h"
+#include "SoftSynthDevice.h"
 #include "MidiProgram.h"
 
 using Rosegarden::MidiBank;
@@ -1830,12 +1831,7 @@ BankEditorDialog::slotImport()
     }
 
     KURL url = KFileDialog::getOpenURL
-        (
-#if KDE_VERSION >= 196614
-         deviceDir,
-#else
-         QString::null,
-#endif
+        (deviceDir,
          "audio/x-rosegarden-device audio/x-rosegarden audio/x-soundfont",
          this, i18n("Import Banks from Device in File"));
 
@@ -2130,11 +2126,20 @@ RemapInstrumentDialog::populateCombo(int id)
 	    Rosegarden::MidiDevice *md =
 		dynamic_cast<Rosegarden::MidiDevice *>(*it);
 
-	    if (md && md->getDirection() == Rosegarden::MidiDevice::Play)
-	    {
-		m_devices.push_back(*it);
-		m_fromCombo->insertItem(strtoqstr((*it)->getName()));
-		m_toCombo->insertItem(strtoqstr((*it)->getName()));
+	    if (md) {
+		if (md->getDirection() == Rosegarden::MidiDevice::Play) {
+		    m_devices.push_back(*it);
+		    m_fromCombo->insertItem(strtoqstr((*it)->getName()));
+		    m_toCombo->insertItem(strtoqstr((*it)->getName()));
+		}
+	    } else {
+		Rosegarden::SoftSynthDevice *sd = 
+		    dynamic_cast<Rosegarden::SoftSynthDevice *>(*it);
+		if (sd) {
+		    m_devices.push_back(*it);
+		    m_fromCombo->insertItem(strtoqstr((*it)->getName()));
+		    m_toCombo->insertItem(strtoqstr((*it)->getName()));
+		}
 	    }
         }
 
