@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -35,17 +35,17 @@ using Rosegarden::timeT;
 BasicCommand::BasicCommand(const QString &name, Segment &segment,
 			   timeT start, timeT end, bool bruteForceRedo) :
     KNamedCommand(name),
+    m_startTime(calculateStartTime(start, segment)),
+    m_endTime(calculateEndTime(end, segment)),
     m_segment(segment),
-    m_savedEvents(segment.getType(), start),
-    m_startTime(start),
-    m_endTime(end),
+    m_savedEvents(segment.getType(), m_startTime),
     m_doBruteForceRedo(false),
     m_redoEvents(0)
 {
-    if (end == start) ++m_endTime;
+    if (m_endTime == m_startTime) ++m_endTime;
 
     if (bruteForceRedo) {
-        m_redoEvents = new Segment(segment.getType(), start);
+        m_redoEvents = new Segment(segment.getType(), m_startTime);
     }
 }
 
@@ -56,6 +56,36 @@ BasicCommand::~BasicCommand()
     delete m_redoEvents;
 }
 
+timeT
+BasicCommand::calculateStartTime(timeT given, Segment &segment)
+{
+    timeT actual = given;
+    Segment::iterator i = segment.findTime(given);
+
+    while (i != segment.end() && (*i)->getAbsoluteTime() == given) {
+	timeT notation = (*i)->getNotationAbsoluteTime();
+	if (notation < given) actual = notation;
+	++i;
+    }
+
+    return actual;
+}
+
+timeT
+BasicCommand::calculateEndTime(timeT given, Segment &segment)
+{
+    timeT actual = given;
+    Segment::iterator i = segment.findTime(given);
+
+    while (i != segment.end() && (*i)->getAbsoluteTime() == given) {
+	timeT notation = (*i)->getNotationAbsoluteTime();
+	if (notation > given) actual = notation;
+	++i;
+    }
+
+    return actual;
+}
+
 Rosegarden::Segment& BasicCommand::getSegment()
 {
     return m_segment;
@@ -86,6 +116,8 @@ BasicCommand::execute()
     }
 
     m_segment.updateRefreshStatuses(getStartTime(), getRelayoutEndTime());
+    RG_DEBUG << "BasicCommand(" << name() << "): updated refresh statuses "
+	     << getStartTime() << " -> " << getRelayoutEndTime() << endl;
 }
 
 void
