@@ -5,7 +5,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -115,7 +115,7 @@ RosegardenColourTable::populate_table(Rosegarden::ColourMap &input, ColourList &
         setItem(i, 0, text);
 
         list[i] = it->first;
-        m_colours[i] = RosegardenGUIColours::convertColour(it->second.first);
+        m_colours[i] = Rosegarden::GUIPalette::convertColour(it->second.first);
 
         RosegardenColourTableItem *temp = new RosegardenColourTableItem(this, m_colours[i]);
         setItem(i, 1, temp);
