@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -33,6 +33,7 @@
 
 #include <klocale.h>
 #include <klineeditdlg.h>
+// #include <kactioncollection.h>
 
 #include "MidiTypes.h"
 #include "Selection.h"
@@ -192,10 +193,13 @@ bool ControllerEventAdapter::getValue(long& val)
     }
     else if (m_event->getType() == Rosegarden::PitchBend::EventType)
     {
-        long value = 0;
-        value = m_event->get<Rosegarden::Int>(Rosegarden::PitchBend::MSB);
+	long msb = 0, lsb = 0;
+        m_event->get<Rosegarden::Int>(Rosegarden::PitchBend::MSB, msb);
+        m_event->get<Rosegarden::Int>(Rosegarden::PitchBend::LSB, lsb);
+	
+        long value = msb;
         value <<= 7;
-        value |= m_event->get<Rosegarden::Int>(Rosegarden::PitchBend::LSB);
+        value |= lsb;
 
         //RG_DEBUG << "PitchBend Get Value = " << value << endl;
 
@@ -330,6 +334,8 @@ void ControlItem::updateValue()
 
 void ControlItem::updateFromValue()
 {
+    RG_DEBUG << "ControlItem::updateFromValue() : " << this << endl;
+
     if (m_elementAdapter->getValue(m_value)) {
 //         RG_DEBUG << "ControlItem::updateFromValue() : value = " << m_value << endl;
         setHeight(m_controlRuler->valueToHeight(m_value));
@@ -430,7 +436,7 @@ void ControlItem::handleMouseMove(QMouseEvent*, int /*deltaX*/, int deltaY)
 
 void ControlItem::handleMouseWheel(QWheelEvent *)
 {
-    RG_DEBUG << "ControlItem::handleMouseWheel - got wheel event" << endl;
+//     RG_DEBUG << "ControlItem::handleMouseWheel - got wheel event" << endl;
 }
 
 void ControlItem::setSelected(bool s)
@@ -666,6 +672,7 @@ ControlRuler::ControlRuler(Segment *segment,
                            const char* name, WFlags f) :
     RosegardenCanvasView(c, parent, name, f),
     m_parentEditView(parentView),
+    m_mainHorizontalScrollBar(0),
     m_rulerScale(rulerScale),
     m_eventSelection(new EventSelection(*segment)),
     m_segment(segment),
@@ -816,7 +823,7 @@ void ControlRuler::contentsMousePressEvent(QMouseEvent* e)
         }
     }
 
-    if (topItem) { // select the top item
+    if (topItem && !m_selectedItems.contains(topItem)) { // select the top item
         m_selectedItems << topItem;
         topItem->setSelected(true);
         topItem->handleMouseButtonPress(e);
@@ -919,7 +926,7 @@ void ControlRuler::contentsMouseMoveEvent(QMouseEvent* e)
 //            ElementAdapter* adapter = item->getElementAdapter();
 
             // set value to highest in selection
-            if (item->getValue() > value)
+            if (item->getValue() >= value)
             {
                 value = item->getValue();
                 m_numberFloat->setText(QString("%1").arg(value));
@@ -939,6 +946,11 @@ ControlRuler::contentsWheelEvent(QWheelEvent *e)
     QCanvasView::contentsWheelEvent(e);
 }
 
+QScrollBar* ControlRuler::getMainHorizontalScrollBar()
+{
+    return m_mainHorizontalScrollBar ? m_mainHorizontalScrollBar : horizontalScrollBar();
+}
+
 void ControlRuler::updateSelection()
 {
     clearSelectedItems();
@@ -1006,11 +1018,12 @@ ControlRuler::clearSelectedItems()
 
 void ControlRuler::clear()
 {
-    QCanvasItemList list = canvas()->allItems();
-    QCanvasItemList::Iterator it = list.begin();
-    for (; it != list.end(); ++it) {
-	if (*it != m_selectionRect)
-	    delete *it;
+    QCanvasItemList allItems = canvas()->allItems();
+
+    for (QCanvasItemList::Iterator it=allItems.begin(); it!=allItems.end(); ++it) {
+        if (ControlItem *item = dynamic_cast<ControlItem*>(*it)) {
+            delete item;
+        }
     }
 }
 
@@ -1033,9 +1046,6 @@ long ControlRuler::heightToValue(int h)
     val *= getMaxItemValue();
     val /= (ItemHeightRange);
     val = std::min(val, long(getMaxItemValue()));
-
-     RG_DEBUG << "ControlRuler::heightToValue : height = " << h << " - val = " << val << endl;
-
     return val;
 }
 
@@ -1134,6 +1144,7 @@ PropertyControlRuler::PropertyControlRuler(Rosegarden::PropertyName propertyName
     m_propertyLineY(0)
 {
     m_staff->addObserver(this);
+    m_segment->addObserver(this);
     m_propertyLine->setZ(1000); // bring to front
 
     setMenuName("property_ruler_menu");
@@ -1147,9 +1158,11 @@ PropertyControlRuler::setStaff(Rosegarden::Staff *staff)
     RG_DEBUG << "PropertyControlRuler::setStaff(" << staff << ")" << endl;
 
     m_staff->removeObserver(this);
+    m_segment->removeObserver(this);
     m_staff = staff;
     m_segment = &m_staff->getSegment();
     m_staff->addObserver(this);
+    m_segment->addObserver(this);
 
     //!!! need to delete the control items here
 
@@ -1200,8 +1213,13 @@ PropertyControlRuler::drawBackground()
 
 PropertyControlRuler::~PropertyControlRuler()
 {
-    if (m_staff)
+    if (m_staff) {
         m_staff->removeObserver(this);
+    }
+    if (m_segment) {
+        m_segment->removeObserver(this);
+    }
+    
 }
 
 QString PropertyControlRuler::getName()
@@ -1264,6 +1282,38 @@ void PropertyControlRuler::staffDeleted(const Rosegarden::Staff *)
     m_staff = 0;
 }
 
+
+void
+PropertyControlRuler::eventAdded(const Rosegarden::Segment *, Rosegarden::Event *e)
+{
+    // nothing to do
+}
+
+void
+PropertyControlRuler::eventRemoved(const Rosegarden::Segment *, Rosegarden::Event *e)
+{
+    // nothing to do
+}
+
+void
+PropertyControlRuler::endMarkerTimeChanged(const Rosegarden::Segment *s, bool)
+{
+    timeT endMarkerTime = s->getEndMarkerTime();
+
+    RG_DEBUG << "PropertyControlRuler::endMarkerTimeChanged() " << endMarkerTime << endl;
+
+    clearSelectedItems();
+
+    clear();
+    init();
+}
+
+void
+PropertyControlRuler::segmentDeleted(const Rosegarden::Segment *)
+{
+}
+
+
 void PropertyControlRuler::computeStaffOffset()
 {
     LinedStaff* lStaff = dynamic_cast<LinedStaff*>(m_staff);
@@ -1273,7 +1323,7 @@ void PropertyControlRuler::computeStaffOffset()
 
 void PropertyControlRuler::startPropertyLine()
 {
-    RG_DEBUG << "PropertyControlRuler::startPropertyLine" << endl;
+    RG_DEBUG << "PropertyControlRuler::startPropertyLine\n";
     m_propertyLineShowing = true;
     this->setCursor(Qt::pointingHandCursor);
 }
@@ -1281,7 +1331,7 @@ void PropertyControlRuler::startPropertyLine()
 void
 PropertyControlRuler::contentsMousePressEvent(QMouseEvent *e)
 {
-    RG_DEBUG << "PropertyControlRuler::contentsMousePressEvent" << endl;
+    RG_DEBUG << "PropertyControlRuler::contentsMousePressEvent\n";
 
     if (!m_propertyLineShowing)
     {
@@ -1318,7 +1368,7 @@ PropertyControlRuler::contentsMousePressEvent(QMouseEvent *e)
 void 
 PropertyControlRuler::contentsMouseReleaseEvent(QMouseEvent *e)
 {
-    RG_DEBUG << "PropertyControlRuler::contentsMouseReleaseEvent" << endl;
+    RG_DEBUG << "PropertyControlRuler::contentsMouseReleaseEvent\n";
 
     /*
     if (m_propertyLineShowing)
@@ -1368,7 +1418,7 @@ PropertyControlRuler::contentsMouseReleaseEvent(QMouseEvent *e)
 void 
 PropertyControlRuler::contentsMouseMoveEvent(QMouseEvent *e)
 {
-    RG_DEBUG << "PropertyControlRuler::contentsMouseMoveEvent" << endl;
+    RG_DEBUG << "PropertyControlRuler::contentsMouseMoveEvent\n";
 
     if (!m_propertyLineShowing)
     {
@@ -1390,6 +1440,30 @@ PropertyControlRuler::contentsMouseMoveEvent(QMouseEvent *e)
     canvas()->update();
 }
 
+void PropertyControlRuler::contentsContextMenuEvent(QContextMenuEvent* e)
+{
+    RG_DEBUG << "PropertyControlRuler::contentsContextMenuEvent\n";
+
+    // check if we actually have some control items
+    QCanvasItemList list = canvas()->allItems();
+    bool haveItems = false;
+    
+    QCanvasItemList::Iterator it = list.begin();
+    for (; it != list.end(); ++it) {
+        if (dynamic_cast<ControlItem*>(*it)) {
+            haveItems = true;
+            break;
+        }
+    }
+
+    RG_DEBUG << "PropertyControlRuler::contentsContextMenuEvent : haveItems = "
+             << haveItems << endl;
+
+    emit stateChange("have_note_events_in_segment", haveItems);
+
+    ControlRuler::contentsContextMenuEvent(e);
+}
+
 void 
 PropertyControlRuler::drawPropertyLine(Rosegarden::timeT startTime,
                                        Rosegarden::timeT endTime,
@@ -1402,8 +1476,8 @@ PropertyControlRuler::drawPropertyLine(Rosegarden::timeT startTime,
         std::swap(startValue, endValue);
     }
 
-    RG_DEBUG << "PropertyControlRuler::drawPropertyLine - "
-             << "set velocity from " << startTime 
+    RG_DEBUG << "PropertyControlRuler::drawPropertyLine - set velocity from "
+             << startTime 
              << " to " << endTime << endl;
 
     // Add the "true" to catch Events overlapping this line
@@ -1411,15 +1485,24 @@ PropertyControlRuler::drawPropertyLine(Rosegarden::timeT startTime,
     Rosegarden::EventSelection selection(*m_segment, startTime, endTime, true);
     Rosegarden::PropertyPattern pattern = Rosegarden::DecrescendoPattern;
 
-    SelectionPropertyCommand *command = 
-        new SelectionPropertyCommand(&selection,
-                                     Rosegarden::BaseProperties::VELOCITY,
-                                     pattern,
-                                     startValue,
-                                     endValue);
+    bool haveNotes = selection.contains(Rosegarden::Note::EventType);
 
-    m_parentEditView->addCommandToHistory(command);
+    if (haveNotes) {
+        
+        SelectionPropertyCommand *command = 
+            new SelectionPropertyCommand(&selection,
+                                         Rosegarden::BaseProperties::VELOCITY,
+                                         pattern,
+                                         startValue,
+                                         endValue);
 
+        m_parentEditView->addCommandToHistory(command);
+
+    } else {
+
+        RG_DEBUG << "PropertyControlRuler::drawPropertyLine - no notes in selection\n";
+
+    }
 }
 
 void
@@ -1462,13 +1545,14 @@ PropertyControlRuler::selectAllProperties()
     emit stateChange("have_controller_item_selected", true);
 }
 
+
 //----------------------------------------
 ControllerEventsRuler::ControllerEventsRuler(Rosegarden::Segment *segment,
                                              Rosegarden::RulerScale* rulerScale,
                                              EditViewBase* parentView,
                                              QCanvas* c,
                                              QWidget* parent,
-                                             ControlParameter *controller,
+                                             const ControlParameter *controller,
                                              const char* name, WFlags f)
     : ControlRuler(segment, rulerScale, parentView, c, parent, name, f),
       m_segmentDeleted(false),
@@ -1967,7 +2051,7 @@ PropertyViewRuler::PropertyViewRuler(RulerScale *rulerScale,
     m_boldFont.setBold(true);
     m_fontMetrics = QFontMetrics(m_boldFont);
 
-    setBackgroundColor(RosegardenGUIColours::SegmentCanvas);
+    setBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentCanvas));
 
     QString tip = i18n("%1 controller").arg(strtoqstr(property));
     QToolTip::add(this, tip);
@@ -2028,7 +2112,7 @@ PropertyViewRuler::paintEvent(QPaintEvent* e)
 
     if (getHScaleFactor() != 1.0) paint.scale(getHScaleFactor(), 1.0);
 
-    paint.setPen(RosegardenGUIColours::MatrixElementBorder);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MatrixElementBorder));
 
     QRect clipRect = e->rect().normalize();
 
@@ -2100,8 +2184,8 @@ PropertyBox::paintEvent(QPaintEvent *e)
 {
     QPainter paint(this);
 
-    paint.setPen(RosegardenGUIColours::MatrixElementBorder);
-    //paint.setBrush(RosegardenGUIColours::MatrixElementBlock);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MatrixElementBorder));
+    //paint.setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MatrixElementBlock));
 
     paint.setClipRegion(e->region());
     paint.setClipRect(e->rect().normalize());
