@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -26,12 +26,14 @@
 #include <qstring.h>
 #include <qcanvas.h>
 #include <qspinbox.h>
+#include <kcombobox.h>
 
 #include <string>
 
 #include "NotationTypes.h"
 #include "editcommands.h"
 #include "notepixmapfactory.h"
+#include "commondialogs.h" // HSpinBox
 
 class QWidget;
 class QLineEdit;
@@ -97,6 +99,7 @@ protected:
 
     QCheckBox *m_commonTimeButton;
     QCheckBox *m_hideSignatureButton;
+    QCheckBox *m_hideBarsButton;
     QCheckBox *m_normalizeRestsButton;
 
     QRadioButton *m_asGivenButton;
@@ -205,11 +208,13 @@ public:
     Rosegarden::Note::Type getUnitType() const;
     int getUntupledCount() const;
     int getTupledCount() const;
+    bool hasTimingAlready() const;
 
 public slots:
     void slotUnitChanged(const QString &);
     void slotUntupledChanged(const QString &);
     void slotTupledChanged(const QString &);
+    void slotHasTimingChanged();
 
 protected:
 
@@ -223,6 +228,9 @@ protected:
     KComboBox *m_untupledCombo;
     KComboBox *m_tupledCombo;
 
+    QCheckBox *m_hasTimingAlready;
+
+    QGroupBox *m_timingDisplayBox;
     QLabel *m_selectionDurationDisplay;
     QLabel *m_untupledDurationCalculationDisplay;
     QLabel *m_untupledDurationDisplay;
@@ -378,7 +386,7 @@ public:
 		          bool inserting = false); // inserting or editing
 
     bool isModified() const { return m_modified; }
-    Rosegarden::Event getEvent() const;
+    Rosegarden::Event getEvent();
 
     // Setup the dialog for a new event type
     void setupForEvent();
@@ -387,30 +395,38 @@ public slots:
     void slotEventTypeChanged(int value);
     void slotAbsoluteTimeChanged(int value);
     void slotDurationChanged(int value);
+    void slotNotationAbsoluteTimeChanged(int value);
+    void slotNotationDurationChanged(int value);
     void slotPitchChanged(int value);
     void slotVelocityChanged(int value);
     void slotMetaChanged(const QString &);
     void slotEditAbsoluteTime();
+    void slotEditNotationAbsoluteTime();
     void slotEditDuration();
+    void slotEditNotationDuration();
+    void slotLockNotationChanged();
     void slotEditPitch();
+    void slotSysexLoad();
+    void slotSysexSave();
 
 protected:
-
-    const Rosegarden::Event &m_originalEvent;
     Rosegarden::Event        m_event;
     RosegardenGUIDoc        *m_doc;
 
     std::string              m_type;
     Rosegarden::timeT        m_absoluteTime;
+    Rosegarden::timeT        m_notationAbsoluteTime;
     Rosegarden::timeT        m_duration;
+    Rosegarden::timeT        m_notationDuration;
 
     KComboBox               *m_typeCombo;
+    QLabel                  *m_typeLabel;
+
     QLabel                  *m_timeLabel;
     QLabel                  *m_durationLabel;
     QLabel                  *m_pitchLabel;
     QLabel                  *m_velocityLabel;
     QLabel                  *m_metaLabel;
-
     QLabel                  *m_controllerLabel;
     QLabel                  *m_controllerLabelValue;
 
@@ -422,6 +438,17 @@ protected:
     QPushButton             *m_timeEditButton;
     QPushButton             *m_durationEditButton;
     QPushButton             *m_pitchEditButton;
+    QPushButton             *m_sysexLoadButton;
+    QPushButton             *m_sysexSaveButton;
+
+    QGroupBox               *m_notationGroupBox;
+    QLabel                  *m_notationTimeLabel;
+    QLabel                  *m_notationDurationLabel;
+    QSpinBox                *m_notationTimeSpinBox;
+    QSpinBox                *m_notationDurationSpinBox;
+    QPushButton             *m_notationTimeEditButton;
+    QPushButton             *m_notationDurationEditButton;
+    QCheckBox               *m_lockNotationValues;
 
     QLineEdit               *m_metaEdit;
 
@@ -470,7 +497,7 @@ protected:
     RosegardenGUIDoc     *m_doc;
     Rosegarden::timeT     m_tempoTime;
     double                m_tempoValue;
-    RosegardenSpinBox    *m_tempoValueSpinBox;
+    HSpinBox  	  	  *m_tempoValueSpinBox;
 
     QLabel	         *m_tempoBeatLabel;
     QLabel	         *m_tempoBeat;
@@ -756,7 +783,7 @@ class InterpretDialog : public KDialogBase
 public:
     InterpretDialog(QWidget *parent);
 
-    // an OR from TransformsMenuInterpretCommand's constants
+    // an OR from AdjustMenuInterpretCommand's constants
     int getInterpretations();
 
 protected slots:
@@ -906,4 +933,93 @@ protected:
 };
 
 
+class MakeOrnamentDialog : public KDialogBase
+{
+    Q_OBJECT
+
+public:
+    MakeOrnamentDialog(QWidget *parent, QString defaultName, int defaultBasePitch);
+    
+    QString getName() const;
+    int getBasePitch() const;
+
+protected:
+    QLineEdit *m_name;
+    RosegardenPitchChooser *m_pitch;
+};
+
+
+class UseOrnamentDialog : public KDialogBase
+{
+    Q_OBJECT
+
+public:
+    UseOrnamentDialog(QWidget *parent, Rosegarden::Composition *);
+
+    Rosegarden::TriggerSegmentId getId() const;
+    Rosegarden::Mark getMark() const;
+    bool getRetune() const;
+    std::string getTimeAdjust() const;
+
+public slots:
+    void slotOk();
+    void slotMarkChanged(int);
+
+protected:
+    void setupFromConfig();
+
+    std::vector<Rosegarden::Mark> m_marks;
+
+    Rosegarden::Composition  *m_composition;
+    KComboBox                *m_ornament;
+    KComboBox                *m_mark;
+    QLabel                   *m_textLabel;
+    QLineEdit                *m_text;
+    QCheckBox                *m_retune;
+    KComboBox                *m_adjustTime;
+};
+
+class TriggerSegmentDialog : public KDialogBase
+{
+    Q_OBJECT
+
+public:
+    TriggerSegmentDialog(QWidget *parent, Rosegarden::Composition *);
+
+    Rosegarden::TriggerSegmentId getId() const;
+    bool getRetune() const;
+    std::string getTimeAdjust() const;
+
+public slots:
+    void slotOk();
+
+protected:
+    void setupFromConfig();
+
+    Rosegarden::Composition  *m_composition;
+    KComboBox                *m_segment;
+    QCheckBox                *m_retune;
+    KComboBox                *m_adjustTime;
+};
+
+/**
+ * ask the user to give us information about the selected audio segment for
+ * Tempo calculations
+ */
+class BeatsBarsDialog : public KDialogBase
+{
+    Q_OBJECT
+	
+public:
+    BeatsBarsDialog();
+    BeatsBarsDialog(QWidget *parent);
+
+    int getQuantity() { return m_spinBox->value(); }
+    int getMode()     { return m_comboBox->currentItem();   } 
+
+protected:
+    QSpinBox  *m_spinBox;
+    KComboBox *m_comboBox;
+};
+
 #endif
