@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -28,6 +28,7 @@
 #include "NotationTypes.h"
 #include "Selection.h"
 #include "SegmentNotationHelper.h"
+#include "SegmentMatrixHelper.h"
 #include "BaseProperties.h"
 #include "Clipboard.h"
 #include "Profiler.h"
@@ -124,7 +125,12 @@ CutAndCloseCommand::CloseCommand::execute()
     for (Segment::iterator i = m_segment->findTime(m_gapEnd);
 	 m_segment->isBeforeEndMarker(i); ++i) {
 	events.push_back(new Event
-			 (**i, (*i)->getAbsoluteTime() - timeDifference));
+			 (**i,
+			  (*i)->getAbsoluteTime() - timeDifference,
+			  (*i)->getDuration(),
+			  (*i)->getSubOrdering(),
+			  (*i)->getNotationAbsoluteTime() - timeDifference,
+			  (*i)->getNotationDuration()));
     }
 
     timeT oldEndTime = m_segment->getEndTime();
@@ -184,7 +190,12 @@ CutAndCloseCommand::CloseCommand::unexecute()
 	Segment::iterator j(i);
 	++j;
 	events.push_back(new Event
-			 (**i, (*i)->getAbsoluteTime() + timeDifference));
+			 (**i,
+			  (*i)->getAbsoluteTime() + timeDifference,
+			  (*i)->getDuration(),
+			  (*i)->getSubOrdering(),
+			  (*i)->getNotationAbsoluteTime() + timeDifference,
+			  (*i)->getNotationDuration()));
 	m_segment->erase(i);
 	i = j;
     }
@@ -258,23 +269,24 @@ PasteSegmentsCommand::PasteSegmentsCommand(Rosegarden::Composition *composition,
     KNamedCommand(getGlobalName()),
     m_composition(composition),
     m_clipboard(clipboard),
-    m_pasteTime(pasteTime)
+    m_pasteTime(pasteTime),
+    m_detached(false)
 {
     // nothing else
 }
 
 PasteSegmentsCommand::~PasteSegmentsCommand()
 {
-    for (unsigned int i = 0; i < m_addedSegments.size(); ++i) {
-	delete m_addedSegments[i];
+    if (m_detached) {
+	for (unsigned int i = 0; i < m_addedSegments.size(); ++i) {
+	    delete m_addedSegments[i];
+	}
     }
 }
 
 void
 PasteSegmentsCommand::execute()
 {
-    if (m_clipboard->isEmpty()) return;
-
     if (m_addedSegments.size() > 0) {
 	// been here before
 	for (unsigned int i = 0; i < m_addedSegments.size(); ++i) {
@@ -284,6 +296,8 @@ PasteSegmentsCommand::execute()
 	return;
     }
 
+    if (m_clipboard->isEmpty()) return;
+
     // We want to paste such that the earliest Segment starts at
     // m_pasteTime and the others start at the same times relative
     // to that as they did before
@@ -301,7 +315,7 @@ PasteSegmentsCommand::execute()
             trackOffset = (*i)->getTrack();
 	}
 
-        if ((*i)->getEndTime() > latestEndTime)
+        if ((*i)->getEndMarkerTime() > latestEndTime)
             latestEndTime = (*i)->getEndMarkerTime();
     }
 
@@ -322,8 +336,10 @@ PasteSegmentsCommand::execute()
 	segment->setStartTime(segment->getStartTime() + offset);
         segment->setTrack(newTrackId);
         m_composition->addSegment(segment);
-	if (m_clipboard->isPartial()) segment->normalizeRests(segment->getStartTime(),
-							      segment->getEndTime());
+	if (m_clipboard->isPartial()) {
+	    segment->normalizeRests(segment->getStartTime(),
+				    segment->getEndMarkerTime() + offset);
+	}
 	m_addedSegments.push_back(segment);
     }
 
@@ -331,6 +347,8 @@ PasteSegmentsCommand::execute()
     m_composition->setPosition(latestEndTime 
                                + m_pasteTime 
                                - earliestStartTime);
+    
+    m_detached = false;
 }
 
 void
@@ -339,6 +357,7 @@ PasteSegmentsCommand::unexecute()
     for (unsigned int i = 0; i < m_addedSegments.size(); ++i) {
 	m_composition->detachSegment(m_addedSegments[i]);
     }
+    m_detached = true;
 }
     
 
@@ -478,6 +497,19 @@ PasteEventsCommand::modifySegment()
     RG_DEBUG << "PasteEventsCommand::modifySegment() : paste type = "
              << m_pasteType << " - pasteTime = "
              << pasteTime << " - origin = " << origin << endl;
+
+    // First check for group IDs, which we want to make unique in the
+    // copies in the destination segment
+
+    std::map<long, long> groupIdMap;
+    for (Segment::iterator i = source->begin(); i != source->end(); ++i) {
+	long groupId = -1;
+	if ((*i)->get<Int>(BEAMED_GROUP_ID, groupId)) {
+	    if (groupIdMap.find(groupId) == groupIdMap.end()) {
+		groupIdMap[groupId] = destination->getNextId();
+	    }
+	}
+    }
     
     switch (m_pasteType) {
 
@@ -500,6 +532,9 @@ PasteEventsCommand::modifySegment()
 	for (Segment::iterator i = destination->findTime(pasteTime);
 	     i != destination->end(); ++i) {
 	    Event *e = new Event(**i, (*i)->getAbsoluteTime() + duration);
+	    if (e->has(BEAMED_GROUP_ID)) {
+		e->set<Int>(BEAMED_GROUP_ID, groupIdMap[e->get<Int>(BEAMED_GROUP_ID)]);
+	    }
 	    copies.push_back(e);
 	}
 
@@ -517,18 +552,23 @@ PasteEventsCommand::modifySegment()
 	for (Segment::iterator i = source->begin(); i != source->end(); ++i) {
 	    if ((*i)->isa(Note::EventRestType)) continue;
 	    if ((*i)->isa(Note::EventType)) {
-		long pitch = 0;
-		Accidental explicitAccidental = NoAccidental;
-		(*i)->get<String>(ACCIDENTAL, explicitAccidental);
-		if ((*i)->get<Int>(PITCH, pitch)) {
-		    helper.insertNote
-			((*i)->getAbsoluteTime() - origin + pasteTime,
-			 Note::getNearestNote((*i)->getDuration()),
-			 pitch, explicitAccidental);
+		Event *e = new Event(**i,
+				     (*i)->getAbsoluteTime() - origin + pasteTime,
+				     (*i)->getDuration(),
+				     (*i)->getSubOrdering(),
+				     (*i)->getNotationAbsoluteTime() - origin + pasteTime,
+				     (*i)->getNotationDuration());
+		if (e->has(BEAMED_GROUP_ID)) {
+		    e->set<Int>(BEAMED_GROUP_ID, groupIdMap[e->get<Int>(BEAMED_GROUP_ID)]);
 		}
+		helper.insertNote(e); // e is model event: we retain ownership of it
+		delete e;
 	    } else {
 		Event *e = new Event
 		    (**i, (*i)->getAbsoluteTime() - origin + pasteTime);
+		if (e->has(BEAMED_GROUP_ID)) {
+		    e->set<Int>(BEAMED_GROUP_ID, groupIdMap[e->get<Int>(BEAMED_GROUP_ID)]);
+		}
 		destination->insert(e);
 	    }
 	}
@@ -550,6 +590,10 @@ PasteEventsCommand::modifySegment()
 		e->unset(BEAMED_GROUP_TYPE);
 	    }
 
+	    if (e->has(BEAMED_GROUP_ID)) {
+		e->set<Int>(BEAMED_GROUP_ID, groupIdMap[e->get<Int>(BEAMED_GROUP_ID)]);
+	    }
+
 	    destination->insert(e);
 	}
 
@@ -564,6 +608,9 @@ PasteEventsCommand::modifySegment()
     for (Segment::iterator i = source->begin(); i != source->end(); ++i) {
 	Event *e = new Event
 	    (**i, (*i)->getAbsoluteTime() - origin + pasteTime);
+	if (e->has(BEAMED_GROUP_ID)) {
+	    e->set<Int>(BEAMED_GROUP_ID, groupIdMap[e->get<Int>(BEAMED_GROUP_ID)]);
+	}
 	destination->insert(e);
     }
 
@@ -677,16 +724,21 @@ SelectionPropertyCommand::modifySegment()
 
     Rosegarden::timeT endTime = 0;
     Rosegarden::timeT startTime = 0;
+    bool haveStart = false, haveEnd = false;
 
     // Get start and end times
     //
     for (;i != m_selection->getSegmentEvents().end(); ++i)
     {
-        if ((*i)->getAbsoluteTime() < startTime)
+        if ((*i)->getAbsoluteTime() < startTime || !haveStart) {
             startTime = (*i)->getAbsoluteTime();
+	    haveStart = true;
+	}
         
-        if ((*i)->getAbsoluteTime() > endTime)
+        if ((*i)->getAbsoluteTime() > endTime || !haveEnd) {
             endTime = (*i)->getAbsoluteTime();
+	    haveEnd = true;
+	}
     }
 
     double step = double(m_value1 - m_value2) / double(endTime - startTime);
@@ -706,24 +758,27 @@ SelectionPropertyCommand::modifySegment()
 
         } else if (m_pattern == Rosegarden::CrescendoPattern)
         {
-
             (*i)->set<Rosegarden::Int>(m_property,
                                        m_value2 +
-                                       int(step * (*i)->getAbsoluteTime()));
+                                       int(step *
+					   ((*i)->getAbsoluteTime() - startTime)));
         } else if (m_pattern == Rosegarden::DecrescendoPattern)
         {
             (*i)->set<Rosegarden::Int>(m_property,
                                        m_value1 -
-                                       int(step * (*i)->getAbsoluteTime()));
+                                       int(step *
+					   ((*i)->getAbsoluteTime() - startTime)));
         } else if (m_pattern == Rosegarden::RingingPattern)
         {
             if (count % 2 == 0)
                 (*i)->set<Rosegarden::Int>
                     (m_property,
-                     m_value1 - int(step * (*i)->getAbsoluteTime()));
+                     m_value1 - int(step * 
+				    ((*i)->getAbsoluteTime() - startTime)));
             else
             {
-                int value = m_value2 - int(lowStep * (*i)->getAbsoluteTime());
+                int value = m_value2 - int(lowStep *
+					   ((*i)->getAbsoluteTime() - startTime));
                 if (value < 0) value = 0;
 
                 (*i)->set<Rosegarden::Int>(m_property, value);
@@ -1030,6 +1085,52 @@ EventUnquantizeCommand::modifySegment()
 }
 
 
+//-----------------Collapse Notes Command-------------------
+//
+//
+void
+AdjustMenuCollapseNotesCommand::modifySegment()
+{
+    SegmentNotationHelper helper(getSegment());
+    timeT endTime = getEndTime();
+
+    // This is really nasty stuff.  We can't go in forward direction
+    // using the j-iterator trick because collapseNoteAggressively may
+    // erase the following iterator as well as the preceding one.  We
+    // can't go backward naively, because collapseNoteAggressively
+    // erases i from the EventSelection now that it's a
+    // SegmentObserver.  We need the fancy hybrid j-iterator-backward
+    // technique applied to selections instead of segments.
+    
+    EventSelection::eventcontainer::iterator i =
+	m_selection->getSegmentEvents().end();
+    EventSelection::eventcontainer::iterator j = i;
+    EventSelection::eventcontainer::iterator beg =
+	m_selection->getSegmentEvents().begin();
+    bool thisOne = false;
+
+    while (i != beg && (!thisOne || (*i != *beg))) {
+	
+	--j;
+
+	if (thisOne) {
+	    helper.collapseNoteAggressively(*i, endTime);
+	}
+	
+	// rather than "true" one could perform a test to see
+	// whether j pointed to a candidate for collapsing:
+	thisOne = true;
+	
+	i = j;
+    }
+    
+    if (thisOne) {
+	helper.collapseNoteAggressively(*i, endTime);
+    }
+}
+
+
+
 SetLyricsCommand::SetLyricsCommand(Segment *segment, QString newLyricData) :
     KNamedCommand(getGlobalName()),
     m_segment(segment),
@@ -1173,10 +1274,12 @@ TransposeCommand::modifySegment()
 	 i != m_selection->getSegmentEvents().end(); ++i) {
 
 	if ((*i)->isa(Note::EventType)) {
-	    long pitch = (*i)->get<Int>(PITCH);
-	    pitch += m_semitones;
-	    (*i)->set<Int>(PITCH, pitch); 
-	    (*i)->unset(ACCIDENTAL);
+	    try {
+		long pitch = (*i)->get<Int>(PITCH);
+		pitch += m_semitones;
+		(*i)->set<Int>(PITCH, pitch); 
+		(*i)->unset(ACCIDENTAL);
+	    } catch (...) { }
 	}
     }
 }
@@ -1187,7 +1290,7 @@ RescaleCommand::RescaleCommand(EventSelection &sel,
 			       bool closeGap) :
     BasicCommand(getGlobalName(), sel.getSegment(),
 		 sel.getStartTime(),
-		 closeGap ? sel.getSegment().getEndMarkerTime() : sel.getEndTime(),
+		 getAffectedEndTime(sel, newDuration, closeGap),
 		 true),
     m_selection(&sel),
     m_oldDuration(sel.getTotalDuration()),
@@ -1197,6 +1300,25 @@ RescaleCommand::RescaleCommand(EventSelection &sel,
     // nothing else
 }
 
+timeT
+RescaleCommand::getAffectedEndTime(EventSelection &sel,
+				   timeT newDuration,
+				   bool closeGap)
+{
+    timeT preScaleEnd = sel.getEndTime();
+    if (closeGap) preScaleEnd = sel.getSegment().getEndMarkerTime();
+
+    // dupe of rescale(), but we can't use that here as the m_
+    // variables may not have been set
+    double d = preScaleEnd;
+    d *= newDuration;
+    d /= sel.getTotalDuration();
+    d += 0.5;
+    timeT postScaleEnd = (timeT)d;
+    
+    return std::max(preScaleEnd, postScaleEnd);
+}
+
 timeT
 RescaleCommand::rescale(timeT t)
 {
@@ -1315,9 +1437,15 @@ MoveCommand::modifySegment()
 	timeT newTime =
 	    (m_useNotationTimings ?
 	     (*i)->getNotationAbsoluteTime() : (*i)->getAbsoluteTime()) + m_delta;
-	Event *e = new Event(**i, newTime);
-	if (m_useNotationTimings)
-	    e->setNotationDuration((*i)->getNotationDuration());
+
+	Event *e;
+	if (m_useNotationTimings) {
+	    e = new Event(**i, newTime, (*i)->getDuration(), (*i)->getSubOrdering(),
+			  newTime, (*i)->getNotationDuration());
+	} else {
+	    e = new Event(**i, newTime);
+	}
+
 	toInsert.push_back(e);
     }
 
@@ -1575,3 +1703,234 @@ ModifyMarkerCommand::unexecute()
 }
 
 
+void
+SetTriggerCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if (!m_notesOnly || (*i)->isa(Note::EventType)) {
+	    (*i)->set<Int>(TRIGGER_SEGMENT_ID, m_triggerSegmentId);
+	    (*i)->set<Rosegarden::Bool>(TRIGGER_SEGMENT_RETUNE, m_retune);
+	    (*i)->set<Rosegarden::String>(TRIGGER_SEGMENT_ADJUST_TIMES, m_timeAdjust);
+	    if (m_mark != Rosegarden::Marks::NoMark) {
+		Rosegarden::Marks::addMark(**i, m_mark, true);
+	    }
+	}
+    }
+
+    // Update the rec references here, without bothering to do so in unexecute
+    // or in ClearTriggersCommand -- because it doesn't matter if a trigger
+    // has references to segments that don't actually trigger it, whereas it
+    // does matter if it loses a reference to something that does
+
+    Rosegarden::TriggerSegmentRec *rec =
+	m_selection->getSegment().getComposition()->getTriggerSegmentRec
+	(m_triggerSegmentId);
+
+    if (rec) rec->updateReferences();
+}
+
+void
+ClearTriggersCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	(*i)->unset(TRIGGER_SEGMENT_ID);
+	(*i)->unset(TRIGGER_SEGMENT_RETUNE);
+	(*i)->unset(TRIGGER_SEGMENT_ADJUST_TIMES);
+    }
+}
+
+
+InsertTriggerNoteCommand::InsertTriggerNoteCommand(Rosegarden::Segment &segment,
+						   Rosegarden::timeT time,
+						   Rosegarden::Note note,
+						   int pitch,
+						   int velocity,
+						   NoteStyleName noteStyle,
+						   Rosegarden::TriggerSegmentId id,
+						   bool retune,
+						   std::string timeAdjust,
+						   Rosegarden::Mark mark) :
+    BasicCommand(i18n("Insert Trigger Note"), segment,
+		 time, time + note.getDuration()),
+    m_time(time),
+    m_note(note),
+    m_pitch(pitch),
+    m_velocity(velocity),
+    m_noteStyle(noteStyle),
+    m_id(id),
+    m_retune(retune),
+    m_timeAdjust(timeAdjust),
+    m_mark(mark)
+{
+    // nothing
+}
+
+InsertTriggerNoteCommand::~InsertTriggerNoteCommand()
+{
+    // nothing
+}
+
+void
+InsertTriggerNoteCommand::modifySegment()
+{
+    // Insert via a model event, so as to apply the note style.
+    // This is a subset of the work done by NoteInsertionCommand
+    
+    Event *e = new Event(Note::EventType, m_time, m_note.getDuration());
+
+    e->set<Int>(PITCH, m_pitch);
+    e->set<Int>(VELOCITY, m_velocity);
+
+    if (m_noteStyle != NoteStyleFactory::DefaultStyle) {
+	e->set<String>(NotationProperties::NOTE_STYLE, m_noteStyle);
+    }
+
+    e->set<Int>(TRIGGER_SEGMENT_ID, m_id);
+    e->set<Rosegarden::Bool>(TRIGGER_SEGMENT_RETUNE, m_retune);
+    e->set<Rosegarden::String>(TRIGGER_SEGMENT_ADJUST_TIMES, m_timeAdjust);
+
+    if (m_mark != Rosegarden::Marks::NoMark) {
+	Rosegarden::Marks::addMark(*e, m_mark, true);
+    }
+
+    Segment &s(getSegment());
+    Segment::iterator i = Rosegarden::SegmentMatrixHelper(s).insertNote(e);
+
+    Segment::iterator j = i;
+    while (++j != s.end()) {
+	if ((*j)->getAbsoluteTime() >
+	    (*i)->getAbsoluteTime() + (*i)->getDuration()) break;
+	if ((*j)->isa(Note::EventType)) {
+	    if ((*j)->getAbsoluteTime() ==
+		(*i)->getAbsoluteTime() + (*i)->getDuration()) {
+		if ((*j)->has(TIED_BACKWARD) && (*j)->get<Rosegarden::Bool>(TIED_BACKWARD) &&
+		    (*j)->has(PITCH) && ((*j)->get<Int>(PITCH) == m_pitch)) {
+		    (*i)->set<Rosegarden::Bool>(TIED_FORWARD, true);
+		}
+	    }
+	}
+    }
+
+    Rosegarden::TriggerSegmentRec *rec =
+	getSegment().getComposition()->getTriggerSegmentRec(m_id);
+
+    if (rec) rec->updateReferences();
+}
+
+
+void
+SetNoteTypeCommand::modifySegment()
+{
+    std::vector<Event *> toErase;
+    std::vector<Event *> toInsert;
+    
+    EventSelection::eventcontainer::iterator i;
+    timeT endTime = getEndTime();
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if ((*i)->isa(Rosegarden::Note::EventType)) {
+	    toErase.push_back(*i);
+	    
+	    Event *e;
+	    if (m_notationOnly) {
+		e = new Event(**i,
+			      (*i)->getAbsoluteTime(),
+			      (*i)->getDuration(),
+			      (*i)->getSubOrdering(),
+			      (*i)->getNotationAbsoluteTime(),
+			      Rosegarden::Note(m_type).getDuration());
+	    } else {
+		e = new Event(**i,
+			      (*i)->getNotationAbsoluteTime(),
+			      Rosegarden::Note(m_type).getDuration());
+	    }
+		
+	    if (e->getNotationAbsoluteTime() + e->getNotationDuration() > endTime) {
+		endTime = e->getNotationAbsoluteTime() + e->getNotationDuration();
+	    }
+
+	    toInsert.push_back(e);
+	}
+    }
+
+    for (std::vector<Event *>::iterator i = toErase.begin(); i != toErase.end(); ++i) {
+	m_selection->getSegment().eraseSingle(*i);
+    }
+
+    for (std::vector<Event *>::iterator i = toInsert.begin(); i != toInsert.end(); ++i) {
+	m_selection->getSegment().insert(*i);
+	m_selection->addEvent(*i);
+    }
+
+    m_selection->getSegment().normalizeRests(getStartTime(), endTime);
+}
+
+void
+AddDotCommand::modifySegment()
+{
+    std::vector<Event *> toErase;
+    std::vector<Event *> toInsert;
+    
+    EventSelection::eventcontainer::iterator i;
+    timeT endTime = getEndTime();
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if ((*i)->isa(Rosegarden::Note::EventType)) {
+
+	    Rosegarden::Note note = Rosegarden::Note::getNearestNote
+		((*i)->getNotationDuration());
+	    int dots = note.getDots();
+	    if (++dots > 2) dots = 0;
+
+	    toErase.push_back(*i);
+
+	    Event *e;
+
+	    if (m_notationOnly) {
+		e = new Event(**i,
+			      (*i)->getAbsoluteTime(),
+			      (*i)->getDuration(),
+			      (*i)->getSubOrdering(),
+			      (*i)->getNotationAbsoluteTime(),
+			      Rosegarden::Note(note.getNoteType(),
+					       dots).getDuration());
+
+	    } else {
+		e = new Event(**i,
+			      (*i)->getNotationAbsoluteTime(),
+			      Rosegarden::Note(note.getNoteType(),
+					       dots).getDuration());
+	    }
+
+	    if (e->getNotationAbsoluteTime() + e->getNotationDuration() > endTime) {
+		endTime = e->getNotationAbsoluteTime() + e->getNotationDuration();
+	    }
+
+	    toInsert.push_back(e);
+	}
+    }
+
+    for (std::vector<Event *>::iterator i = toErase.begin(); i != toErase.end(); ++i) {
+	m_selection->getSegment().eraseSingle(*i);
+    }
+
+    for (std::vector<Event *>::iterator i = toInsert.begin(); i != toInsert.end(); ++i) {
+	m_selection->getSegment().insert(*i);
+	m_selection->addEvent(*i);
+    }
+
+    m_selection->getSegment().normalizeRests(getStartTime(), endTime);
+}
+
