@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -24,12 +24,15 @@
 
 
 #include "basiccommand.h"
+#include "notestyle.h"
 
 #include "Quantizer.h"
+#include "TriggerSegment.h"
 #include <klocale.h>
 
 namespace Rosegarden {
     class Clipboard;
+    class Composition;
     class EventSelection;
     class SegmentSelection;
     class Marker;
@@ -45,7 +48,6 @@ namespace Rosegarden {
         DecrescendoPattern,   // decreasing from velocity 1 to velocity 2.
         RingingPattern        // between velocity 1 and 2, dying away.
     } PropertyPattern;
-
 }
 
 
@@ -147,6 +149,7 @@ protected:
     Rosegarden::Clipboard *m_clipboard;
     Rosegarden::timeT m_pasteTime;
     std::vector<Rosegarden::Segment *> m_addedSegments;
+    bool m_detached;
 };
 
 
@@ -299,6 +302,25 @@ private:
     Rosegarden::Quantizer *makeQuantizer(QString, bool);
 };
 
+// Collapse equal-pitch notes into one event
+//
+class AdjustMenuCollapseNotesCommand : public BasicSelectionCommand
+{
+public:
+    AdjustMenuCollapseNotesCommand(Rosegarden::EventSelection &selection) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection) { }
+
+    static QString getGlobalName() { return i18n("Collapse &Equal-Pitch Notes"); }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+};
+
+
 // Set the (numerical) property of a selection according given pattern.
 //
 class SelectionPropertyCommand : public BasicSelectionCommand
@@ -408,6 +430,9 @@ protected:
 
 private:
     Rosegarden::timeT rescale(Rosegarden::timeT);
+    Rosegarden::timeT getAffectedEndTime(Rosegarden::EventSelection &selection,
+					 Rosegarden::timeT newDuration,
+					 bool closeGap);
 
     Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
     Rosegarden::timeT m_oldDuration;
@@ -554,4 +579,141 @@ protected:
 
 };
 
+
+class SetTriggerCommand : public BasicSelectionCommand
+{
+public:
+    SetTriggerCommand(Rosegarden::EventSelection &selection,
+		      Rosegarden::TriggerSegmentId triggerSegmentId,
+		      bool notesOnly,
+		      bool retune,
+		      std::string timeAdjust,
+		      Rosegarden::Mark mark,
+		      QString name = 0) :
+	BasicSelectionCommand(name ? name : getGlobalName(), selection, true),
+	m_selection(&selection),
+	m_triggerSegmentId(triggerSegmentId),
+	m_notesOnly(notesOnly),
+	m_retune(retune),
+	m_timeAdjust(timeAdjust),
+	m_mark(mark)
+    { }
+
+    static QString getGlobalName() {
+	return i18n("Tri&gger Segment");
+    }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    Rosegarden::TriggerSegmentId m_triggerSegmentId;
+    bool m_notesOnly;
+    bool m_retune;
+    std::string m_timeAdjust;
+    Rosegarden::Mark m_mark;
+};
+
+
+class ClearTriggersCommand : public BasicSelectionCommand
+{
+public:
+    ClearTriggersCommand(Rosegarden::EventSelection &selection,
+			 QString name = 0) :
+	BasicSelectionCommand(name ? name : getGlobalName(), selection, true),
+	m_selection(&selection)
+    { }
+
+    static QString getGlobalName() {
+	return i18n("&Clear Triggers");
+    }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+};
+
+
+class InsertTriggerNoteCommand : public BasicCommand
+{
+public:
+    InsertTriggerNoteCommand(Rosegarden::Segment &,
+			     Rosegarden::timeT time,
+			     Rosegarden::Note note,
+			     int pitch,
+			     int velocity,
+			     NoteStyleName noteStyle,
+			     Rosegarden::TriggerSegmentId id,
+			     bool retune,
+			     std::string timeAdjust,
+			     Rosegarden::Mark mark);
+    virtual ~InsertTriggerNoteCommand();
+
+protected:
+    virtual void modifySegment();
+
+    Rosegarden::timeT m_time;
+    Rosegarden::Note m_note;
+    int m_pitch;
+    int m_velocity;
+    NoteStyleName m_noteStyle;
+    Rosegarden::TriggerSegmentId m_id;
+    bool m_retune;
+    std::string m_timeAdjust;
+    Rosegarden::Mark m_mark;
+};
+
+
+class SetNoteTypeCommand : public BasicSelectionCommand
+{
+public:
+    SetNoteTypeCommand(Rosegarden::EventSelection &selection,
+		       Rosegarden::Note::Type type,
+		       bool notationOnly) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection),
+	m_type(type),
+	m_notationOnly(notationOnly)
+    { }
+
+    static QString getGlobalName() {
+	return i18n("&Set Note Type");
+    }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    Rosegarden::Note::Type m_type;
+    bool m_notationOnly;
+};
+
+
+class AddDotCommand : public BasicSelectionCommand
+{
+public:
+    AddDotCommand(Rosegarden::EventSelection &selection, bool notationOnly) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection),
+	m_notationOnly(notationOnly)
+    { }
+
+    static QString getGlobalName() {
+	return i18n("&Add Dot");
+    }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    bool m_notationOnly;
+};
+
+
+
 #endif
