@@ -4,7 +4,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -26,7 +26,7 @@
 
 #include "edittool.h"
 #include "editview.h"
-
+#include "rosegardencanvasview.h"
 #include "rosestrings.h"
 #include "rosedebug.h"
 
@@ -178,7 +178,7 @@ void EditTool::handleMouseDoubleClick(Rosegarden::timeT,
 
 int EditTool::handleMouseMove(Rosegarden::timeT, int, QMouseEvent*)
 {
-    return NoFollow;
+    return RosegardenCanvasView::NoFollow;
 }
 
 void EditTool::handleMouseRelease(Rosegarden::timeT, int, QMouseEvent*)
