@@ -4,7 +4,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -65,15 +65,6 @@ class BaseTool : public QObject
 
 public:
 
-    /**
-     * handleMouseMove() will return a OR-ed combination of these
-     */
-    enum {
-        NoFollow = 0x0,
-        FollowHorizontal = 0x1,
-        FollowVertical = 0x2
-    };
-
     virtual ~BaseTool();
 
     /**
