@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -32,7 +32,7 @@
 #include <kstatusbar.h>
 #include <klistbox.h>
 
-#if KDE_VERSION >= 197120 // 320
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
 #include <ktabwidget.h>
 #else
 #include "kde32_ktabwidget.h"
@@ -46,6 +46,7 @@
 #include "AnalysisTypes.h"
 #include "CompositionTimeSliceAdapter.h"
 #include "MidiTypes.h"
+#include "SoftSynthDevice.h"
 
 #include "editview.h"
 #include "rosestrings.h"
@@ -92,7 +93,7 @@ EditView::EditView(RosegardenGUIDoc *doc,
     m_topBarButtons(0),
     m_bottomBarButtons(0),
     m_controlRuler(0),
-#if KDE_VERSION < 197120
+#if KDE_VERSION < KDE_MAKE_VERSION(3,2,0)
     m_controlRulers(new KDE32Backport::KTabWidget(getBottomWidget(), "controlrulers"))
 #else
     m_controlRulers(new KTabWidget(getBottomWidget(), "controlrulers"))
@@ -112,7 +113,6 @@ EditView::EditView(RosegardenGUIDoc *doc,
     
     m_controlRulers->hide();
     m_controlRulers->setTabPosition(QTabWidget::Bottom);
-
 }
 
 EditView::~EditView()
@@ -128,9 +128,9 @@ void EditView::updateBottomWidgetGeometry()
     getCanvasView()->updateBottomWidgetGeometry();
 }
 
-
 void EditView::paintEvent(QPaintEvent* e)
 {
+    RG_DEBUG << "EditView::paintEvent()\n";
     EditViewBase::paintEvent(e);
     
     if (m_needUpdate)  {
@@ -187,7 +187,7 @@ void EditView::setControlRulersCurrentSegment()
 
     delete m_controlRulers;
     
-#if KDE_VERSION < 197120
+#if KDE_VERSION < KDE_MAKE_VERSION(3,2,0)
     m_controlRulers = new KDE32Backport::KTabWidget(getBottomWidget(), "controlrulers");
 #else
     m_controlRulers = new KTabWidget(getBottomWidget(), "controlrulers");
@@ -289,7 +289,7 @@ void EditView::addRuler(QWidget* w)
                 w, SLOT(slotScrollHoriz(int)));
         connect(m_canvasView->horizontalScrollBar(), SIGNAL(sliderMoved(int)),
                 w, SLOT(slotScrollHoriz(int)));
-    }
+    }    
 }
 
 void EditView::addPropertyBox(QWidget *w)
@@ -310,10 +310,12 @@ PropertyControlRuler* EditView::makePropertyControlRuler(PropertyName propertyNa
 	(propertyName, getCurrentStaff(), getHLayout(), this,
 	 controlRulerCanvas, m_controlRulers);
 
+    controlRuler->setMainHorizontalScrollBar(m_canvasView->horizontalScrollBar());
+
     return controlRuler;
 }
 
-ControllerEventsRuler* EditView::makeControllerEventRuler(ControlParameter *controller)
+ControllerEventsRuler* EditView::makeControllerEventRuler(const ControlParameter *controller)
 {
     QCanvas* controlRulerCanvas = new QCanvas(this);
     QSize viewSize = getViewSize();
@@ -326,6 +328,8 @@ ControllerEventsRuler* EditView::makeControllerEventRuler(ControlParameter *cont
 	(getCurrentSegment(), getHLayout(), this,
 	 controlRulerCanvas, m_controlRulers, controller);
 
+    controlRuler->setMainHorizontalScrollBar(m_canvasView->horizontalScrollBar());
+
     return controlRuler;
 }
 
@@ -342,7 +346,9 @@ void EditView::addControlRuler(ControlRuler* ruler)
         connect(m_canvasView->horizontalScrollBar(), SIGNAL(sliderMoved(int)),
                 ruler->horizontalScrollBar(), SIGNAL(sliderMoved(int)));
     }
-    
+
+    connect(ruler, SIGNAL(stateChange(const QString&, bool)),
+            this, SLOT(slotStateChanged(const QString&, bool)));
 
     stateChanged("have_control_ruler", KXMLGUIClient::StateReverse);
 }
@@ -796,11 +802,14 @@ EditView::setupAddControlRulerMenu()
 	//!!! problem here with notation view -- current segment can
 	// change after construction, but this function isn't used again
 
-	Rosegarden::MidiDevice *md = dynamic_cast<Rosegarden::MidiDevice *>
-	    (getCurrentDevice());
-	if (!md) return;
+	Rosegarden::Controllable *c = 
+	    dynamic_cast<Rosegarden::MidiDevice *>(getCurrentDevice());
+	if (!c) {
+	    c = dynamic_cast<Rosegarden::SoftSynthDevice *>(getCurrentDevice());
+	    if (!c) return;
+	}
 
-        const Rosegarden::ControlList &list = md->getControlParameters();
+        const Rosegarden::ControlList &list = c->getControlParameters();
 
         int i = 0;
         QString itemStr;
@@ -843,9 +852,12 @@ EditView::setupControllerTabs()
 
     if (list.size())
     {
-	Rosegarden::MidiDevice *md = dynamic_cast<Rosegarden::MidiDevice *>
-	    (getCurrentDevice());
-	if (!md) return;
+	Rosegarden::Controllable *c = 
+	    dynamic_cast<Rosegarden::MidiDevice *>(getCurrentDevice());
+	if (!c) {
+	    c = dynamic_cast<Rosegarden::SoftSynthDevice *>(getCurrentDevice());
+	    if (!c) return;
+	}
 
         Rosegarden::Segment::EventRulerListIterator it;
 
@@ -853,8 +865,9 @@ EditView::setupControllerTabs()
         {
             // Get ControlParameter object from controller value
             //
-            ControlParameter *controlParameter = 
-                md->getControlParameter((*it)->m_type, Rosegarden::MidiByte((*it)->m_controllerValue));
+            const ControlParameter *controlParameter = 
+		c->getControlParameter((*it)->m_type,
+				       Rosegarden::MidiByte((*it)->m_controllerValue));
 
             RG_DEBUG << "EditView::setupControllerTabs - " 
                      << "Control Parameter type = " << (*it)->m_type << endl;
@@ -882,11 +895,14 @@ EditView::slotAddControlRuler(int controller)
     RG_DEBUG << "EditView::slotAddControlRuler - item = " 
              << controller << endl;
 
-    Rosegarden::MidiDevice *md = dynamic_cast<Rosegarden::MidiDevice *>
-	(getCurrentDevice());
-    if (!md) return;
-
-    const Rosegarden::ControlList &list = md->getControlParameters();
+    Rosegarden::Controllable *c = 
+	dynamic_cast<Rosegarden::MidiDevice *>(getCurrentDevice());
+    if (!c) {
+	c = dynamic_cast<Rosegarden::SoftSynthDevice *>(getCurrentDevice());
+	if (!c) return;
+    }
+    
+    const Rosegarden::ControlList &list = c->getControlParameters();
     ControlParameter control = list[controller];
 
     int index = 0;
@@ -1061,18 +1077,14 @@ EditView::createInsertPitchActionMenu()
 
 int
 EditView::getPitchFromNoteInsertAction(QString name,
-				       Rosegarden::Accidental &accidental)
+				       Rosegarden::Accidental &accidental,
+				       const Rosegarden::Clef &clef,
+				       const Rosegarden::Key &key)
 {
     using namespace Rosegarden::Accidentals;
 
     accidental = NoAccidental;
 
-    Rosegarden::Key key;
-    Rosegarden::Clef clef;
-
-    //!!! need to be able to get clef & key -- how?
-    //!!! modify to use Pitch class
-
     if (name.left(7) == "insert_") {
 
 	name = name.right(name.length()-7);
@@ -1112,31 +1124,10 @@ EditView::getPitchFromNoteInsertAction(QString name,
 	    scalePitch = 0;
 	}
 
-	//!!! needs testing before we remove old stuff
 	Rosegarden::Pitch pitch
-	    (scalePitch, 5 + octave + clef.getOctave(), key, accidental);
+	    (scalePitch, 4 + octave + clef.getOctave(), key, accidental);
 	return pitch.getPerformancePitch();
 
-/*!!!
-	int pitch =
-	    key.getTonicPitch() + 60 + 12*(octave + clef.getOctave()) + modify;
-
-	static int scale[] = { 0, 2, 4, 5, 7, 9, 11 };
-	pitch += scale[scalePitch];
-*/
-/*!!!
-	if (accidental != NoAccidental) {
-	    Rosegarden::NotationDisplayPitch ndp(pitch, clef, key);
-	    int height = ndp.getHeightOnStaff();
-	    Rosegarden::Accidental defaultAcc = key.getAccidentalAtHeight(height, clef);
-	    if ((defaultAcc == Sharp || defaultAcc == Flat) &&
-		defaultAcc != accidental) accidental = Natural;
-	    else if (defaultAcc == Sharp) accidental = DoubleSharp;
-	    else                          accidental = DoubleFlat;
-	}
-*/
-//!!!	return pitch;
-
     } else {
 
 	throw Rosegarden::Exception("Not an insert action",
