@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -46,6 +46,7 @@
 #include "barbuttons.h"
 #include "Clipboard.h"
 #include "sequencemanager.h"
+#include "segmentcommands.h"
 
 #include "rosedebug.h"
 
@@ -168,9 +169,11 @@ void EditViewBase::setupActions(QString rcFileName, bool haveClipboard)
 
     // Actions all edit views will have
 
-    KStdAction::showToolbar(this, SLOT(slotToggleToolBar()), actionCollection());
-    KStdAction::showStatusbar(this, SLOT(slotToggleStatusBar()), actionCollection());
-
+    KStdAction::showToolbar(this, SLOT(slotToggleToolBar()),
+                                       actionCollection(), "options_show_toolbar");
+    
+    KStdAction::showStatusbar(this, SLOT(slotToggleStatusBar()),
+                                         actionCollection(), "options_show_statusbar");
 
     KStdAction::preferences(this,
                             SLOT(slotConfigure()),
@@ -209,21 +212,32 @@ void EditViewBase::setupActions(QString rcFileName, bool haveClipboard)
 
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
 
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/matrix.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/matrix.xpm");
+    QIconSet icon = QIconSet(pixmap);
     new KAction(i18n("Open in Matri&x Editor"), icon, 0, this,
                 SLOT(slotOpenInMatrix()), actionCollection(),
                 "open_in_matrix");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/notation.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/notation.xpm");
+    icon = QIconSet(pixmap);
     new KAction(i18n("Open in &Notation Editor"), icon, 0, this,
                 SLOT(slotOpenInNotation()), actionCollection(),
                 "open_in_notation");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/eventlist.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/eventlist.xpm");
+    icon = QIconSet(pixmap);
     new KAction(i18n("Open in &Event List Editor"), icon, 0, this,
                 SLOT(slotOpenInEventList()), actionCollection(),
                 "open_in_event_list");
 
+    new KAction(i18n("Set Segment Start Time..."), 0, this,
+		SLOT(slotSetSegmentStartTime()), actionCollection(),
+		"set_segment_start");
+
+    new KAction(i18n("Set Segment Duration..."), 0, this,
+		SLOT(slotSetSegmentDuration()), actionCollection(),
+		"set_segment_duration");
+
     // add undo and redo to edit menu and toolbar
     getCommandHistory()->attachView(actionCollection());
     
@@ -568,13 +582,7 @@ void EditViewBase::toggleWidget(QWidget* widget,
         return;
     }
 
-    // Qt 3.1 only :
-//     widget->setShown(toggleAction->isChecked());
-
-    if (toggleAction->isChecked())
-        widget->show();
-    else
-        widget->hide();
+    widget->setShown(toggleAction->isChecked());
 }
 
     
@@ -602,29 +610,100 @@ EditViewBase::slotTestClipboard()
 void
 EditViewBase::slotToggleSolo()
 {
-    RG_DEBUG << "EditViewBase::slotToggleSolo()\n";
+    KToggleAction* toggleSoloAction = getToggleAction("toggle_solo");
+    if (!toggleSoloAction) return;
 
-    if(getDocument()->getComposition().isSolo()) {
+    bool newSoloState = toggleSoloAction->isChecked();
 
-        getDocument()->getComposition().setSolo(false);
-
-    } else {
-
-        emit selectTrack(m_segments[0]->getTrack());
-        getDocument()->getComposition().setSolo(true);
+    RG_DEBUG << "EditViewBase::slotToggleSolo() : solo  = " << newSoloState << endl;
+    emit toggleSolo(newSoloState);
 
+    if (newSoloState) {
+        emit selectTrack(getCurrentSegment()->getTrack());
     }
-    
+
 }
 
 void
 EditViewBase::slotStateChanged(const QString& s,
                                bool noReverse)
 {
+    RG_DEBUG << "EditViewBase::slotStateChanged " << s << ", " << noReverse << endl;
     stateChanged(s, noReverse ? KXMLGUIClient::StateNoReverse : KXMLGUIClient::StateReverse);
 }
 
 
+void
+EditViewBase::slotSetSegmentStartTime()
+{
+    Rosegarden::Segment *s = getCurrentSegment();
+    if (!s) return;
+
+    TimeDialog dialog(this, i18n("Segment Start Time"),
+		      &getDocument()->getComposition(),
+		      s->getStartTime());
+
+    if (dialog.exec() == QDialog::Accepted) {
+
+	SegmentReconfigureCommand *command =
+	    new SegmentReconfigureCommand(i18n("Set Segment Start Time"));
+	
+	command->addSegment
+	    (s, dialog.getTime(),
+	     s->getEndMarkerTime() - s->getStartTime() + dialog.getTime(),
+	     s->getTrack());
+
+        addCommandToHistory(command);
+    }
+}
+
+
+void
+EditViewBase::slotSetSegmentDuration()
+{
+    Rosegarden::Segment *s = getCurrentSegment();
+    if (!s) return;
+
+    TimeDialog dialog(this, i18n("Segment Duration"),
+		      &getDocument()->getComposition(),
+		      s->getStartTime(),
+		      s->getEndMarkerTime() - s->getStartTime());
+
+    if (dialog.exec() == QDialog::Accepted) {
+	
+	SegmentReconfigureCommand *command =
+	    new SegmentReconfigureCommand(i18n("Set Segment Duration"));
+	
+	command->addSegment
+	    (s, s->getStartTime(),
+	     s->getStartTime() + dialog.getTime(),
+	     s->getTrack());
+
+        addCommandToHistory(command);
+    }
+}
+
+void EditViewBase::slotCompositionStateUpdate()
+{
+    // update state of 'solo' toggle
+    //
+    KToggleAction* toggleSolo = getToggleAction("toggle_solo");
+    if (!toggleSolo) return;
+
+    if(getDocument()->getComposition().isSolo()) {
+        bool s = m_segments[0]->getTrack() == getDocument()->getComposition().getSelectedTrack();
+        RG_DEBUG << "EditViewBase::slotCompositionStateUpdate() : set solo to " << s << endl;
+        toggleSolo->setChecked(s);
+    } else {
+        toggleSolo->setChecked(false);
+        RG_DEBUG << "EditViewBase::slotCompositionStateUpdate() : set solo to false\n";
+    }
+
+    // update the window caption
+    //
+    updateViewCaption();
+}
+
 /*
  * Let tools know if their current element has gone
  */
