@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -31,6 +31,7 @@
 #include <kstdaction.h> 
 #include <kaction.h>
 #include <kglobal.h>
+#include <klineeditdlg.h>
 #include <kstddirs.h>
 
 #include <qvbox.h>
@@ -41,6 +42,7 @@
 #include <qbuttongroup.h>
 #include <qpopupmenu.h>
 #include <qiconset.h>
+#include <qspinbox.h>
 
 #include <klistview.h>
 
@@ -54,6 +56,8 @@
 #include "matrixtool.h"
 #include "rosedebug.h"
 #include "eventcommands.h"
+#include "segmentcommands.h"
+#include "widgets.h"
 
 #include "Segment.h"
 #include "SegmentPerformanceHelper.h"
@@ -65,7 +69,6 @@
 
 using Rosegarden::Int;
 using Rosegarden::String;
-using Rosegarden::BaseProperties;
 
 // EventView specialisation of a QListViewItem with the
 // addition of a segment pointer
@@ -177,6 +180,17 @@ EventView::EventView(RosegardenGUIDoc *doc,
 		  ProgramChange | PitchBend | Indication | Other),
     m_menu(0)
 {
+    m_isTriggerSegment = false;
+    m_triggerName = m_triggerPitch = m_triggerVelocity = 0;
+
+    if (!segments.empty()) {
+	Rosegarden::Segment *s = *segments.begin();
+	if (s->getComposition()) {
+	    int id = s->getComposition()->getTriggerSegmentId(s);
+	    if (id >= 0) m_isTriggerSegment = true;
+	}
+    }
+
     if (m_lastSetEventFilter < 0) m_lastSetEventFilter = m_eventFilter;
     else m_eventFilter = m_lastSetEventFilter;
 
@@ -211,19 +225,80 @@ EventView::EventView(RosegardenGUIDoc *doc,
 
     m_grid->addWidget(m_eventList, 2, 1);
 
-    if (segments.size() == 1) {
+    if (m_isTriggerSegment) {
 
-        setCaption(QString("%1 - Segment Track #%2 - Event List")
-                   .arg(doc->getTitle())
-                   .arg(segments[0]->getTrack()));
+	int id = segments[0]->getComposition()->getTriggerSegmentId(segments[0]);
+	Rosegarden::TriggerSegmentRec *rec =
+	    segments[0]->getComposition()->getTriggerSegmentRec(id);
 
-    } else {
+	QGroupBox *groupBox = new QGroupBox
+	    (1, Horizontal, i18n("Triggered Segment Properties"), getCentralWidget());
+	
+	QFrame *frame = new QFrame(groupBox);
+	QGridLayout *layout = new QGridLayout(frame, 5, 3, 5, 5);
 
-        setCaption(QString("%1 - %2 Segments - Event List")
-                   .arg(doc->getTitle())
-                   .arg(segments.size()));
+	layout->addWidget(new QLabel(i18n("Label:  "), frame), 0, 0);
+	QString label = strtoqstr(segments[0]->getLabel());
+	if (label == "") label = i18n("<no label>");
+	m_triggerName = new QLabel(label, frame);
+	layout->addWidget(m_triggerName, 0, 1);
+	QPushButton *editButton = new QPushButton(i18n("..."), frame);
+	layout->addWidget(editButton, 0, 2);
+	connect(editButton, SIGNAL(clicked()), this, SLOT(slotEditTriggerName()));
+
+	layout->addWidget(new QLabel(i18n("Base pitch:  "), frame), 1, 0);
+	m_triggerPitch = new QLabel(QString("%1").arg(rec->getBasePitch()), frame);
+	layout->addWidget(m_triggerPitch, 1, 1);
+	editButton = new QPushButton(i18n("..."), frame);
+	layout->addWidget(editButton, 1, 2);
+	connect(editButton, SIGNAL(clicked()), this, SLOT(slotEditTriggerPitch()));
+
+	layout->addWidget(new QLabel(i18n("Base velocity:  "), frame), 2, 0);
+	m_triggerVelocity = new QLabel(QString("%1").arg(rec->getBaseVelocity()), frame);
+	layout->addWidget(m_triggerVelocity, 2, 1);
+	editButton = new QPushButton(i18n("..."), frame);
+	layout->addWidget(editButton, 2, 2);
+	connect(editButton, SIGNAL(clicked()), this, SLOT(slotEditTriggerVelocity()));
+
+	/*!!! Comment out these two options, which are not yet used
+	  anywhere else -- intended for use with library ornaments, not
+	  yet implemented
+	
+	layout->addWidget(new QLabel(i18n("Default timing:  "), frame), 3, 0);
+
+	KComboBox *adjust = new KComboBox(frame);
+	layout->addMultiCellWidget(adjust, 3, 3, 1, 2);
+	adjust->insertItem(i18n("As stored"));
+	adjust->insertItem(i18n("Truncate if longer than note"));
+	adjust->insertItem(i18n("End at same time as note")); 
+	adjust->insertItem(i18n("Stretch or squash segment to note duration"));
+
+	std::string timing = rec->getDefaultTimeAdjust();
+	if (timing == Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_NONE) {
+	    adjust->setCurrentItem(0);
+	} else if (timing == Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SQUISH) {
+	    adjust->setCurrentItem(3);
+	} else if (timing == Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SYNC_START) {
+	    adjust->setCurrentItem(1);
+	} else if (timing == Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SYNC_END) {
+	    adjust->setCurrentItem(2);
+	}
+
+	connect(adjust, SIGNAL(activated(int)), this, SLOT(slotTriggerTimeAdjustChanged(int)));
+	    
+	QCheckBox *retune = new QCheckBox(i18n("Adjust pitch to trigger note by default"), frame);
+	retune->setChecked(rec->getDefaultRetune());
+	connect(retune, SIGNAL(clicked()), this, SLOT(slotTriggerRetuneChanged()));
+	layout->addMultiCellWidget(retune, 4, 4, 1, 2);
+
+	*/
+
+	m_grid->addWidget(groupBox, 2, 2);
+	
     }
 
+    updateViewCaption();
+
     for (unsigned int i = 0; i < m_segments.size(); ++i) {
 	m_segments[i]->addObserver(this);
     }
@@ -257,6 +332,8 @@ EventView::EventView(RosegardenGUIDoc *doc,
 
     makeInitialSelection(doc->getComposition().getPosition());
 
+    slotCompositionStateUpdate();
+
     setOutOfCtor();
 }
 
@@ -360,16 +437,16 @@ EventView::applyLayout(int /*staffNo*/)
 
 	    // avoid debug stuff going to stderr if no properties found
 
-	    if ((*it)->has(BaseProperties::PITCH)) {
+	    if ((*it)->has(Rosegarden::BaseProperties::PITCH)) {
 		pitchStr = QString("%1  ").
-                    arg((*it)->get<Int>(BaseProperties::PITCH));
+                    arg((*it)->get<Int>(Rosegarden::BaseProperties::PITCH));
 	    } else if ((*it)->isa(Rosegarden::Note::EventType)) {
 		pitchStr = "<not set>";
 	    }
 
-	    if ((*it)->has(BaseProperties::VELOCITY)) {
+	    if ((*it)->has(Rosegarden::BaseProperties::VELOCITY)) {
 		velyStr = QString("%1  ").
-                    arg((*it)->get<Int>(BaseProperties::VELOCITY));
+                    arg((*it)->get<Int>(Rosegarden::BaseProperties::VELOCITY));
 	    } else if ((*it)->isa(Rosegarden::Note::EventType)) {
 		velyStr = "<not set>";
 	    }
@@ -398,10 +475,10 @@ EventView::applyLayout(int /*staffNo*/)
 	    } else if ((*it)->has(Rosegarden::PitchBend::MSB)) {
                 data1Str = QString("%1  ").
                     arg((*it)->get<Int>(Rosegarden::PitchBend::MSB));
-            } else if ((*it)->has(BaseProperties::BEAMED_GROUP_TYPE)) {
+            } else if ((*it)->has(Rosegarden::BaseProperties::BEAMED_GROUP_TYPE)) {
 		data1Str = QString("%1  ").
 		    arg(strtoqstr((*it)->get<String>
-				  (BaseProperties::BEAMED_GROUP_TYPE)));
+				  (Rosegarden::BaseProperties::BEAMED_GROUP_TYPE)));
 	    }
 
             if ((*it)->has(Rosegarden::Controller::VALUE)) {
@@ -421,14 +498,14 @@ EventView::applyLayout(int /*staffNo*/)
 	    } else if ((*it)->has(Rosegarden::PitchBend::LSB)) {
                 data2Str = QString("%1  ").
                     arg((*it)->get<Int>(Rosegarden::PitchBend::LSB));
-            } else if ((*it)->has(BaseProperties::BEAMED_GROUP_ID)) {
+            } else if ((*it)->has(Rosegarden::BaseProperties::BEAMED_GROUP_ID)) {
 		data2Str = i18n("(group %1)  ").
-		    arg((*it)->get<Int>(BaseProperties::BEAMED_GROUP_ID));
+		    arg((*it)->get<Int>(Rosegarden::BaseProperties::BEAMED_GROUP_ID));
 	    }
 
             if ((*it)->has(Rosegarden::ProgramChange::PROGRAM)) {
                 data1Str = QString("%1  ").
-                    arg((*it)->get<Int>(Rosegarden::ProgramChange::PROGRAM));
+                    arg((*it)->get<Int>(Rosegarden::ProgramChange::PROGRAM) + 1);
             }
 
             if ((*it)->has(Rosegarden::ChannelPressure::PRESSURE)) {
@@ -527,6 +604,23 @@ EventView::makeInitialSelection(Rosegarden::timeT time)
     EventViewItem *goodItem = 0;
     int goodItemNo = 0;
 
+    int i = 0;
+    
+    for (QListViewItem *child = m_eventList->firstChild(); 
+	 child;
+	 child = child->nextSibling()) {
+
+	EventViewItem *item = dynamic_cast<EventViewItem *>(child);
+
+	if (item) {
+	    if (item->getEvent()->getAbsoluteTime() > time) break;
+	    goodItem = item;
+	    goodItemNo = i;
+	}
+
+	++i;
+    }
+/*!!!
     for (int i = 0; m_eventList->itemAtIndex(i); ++i) {
 
 	EventViewItem *item = dynamic_cast<EventViewItem *>
@@ -538,7 +632,7 @@ EventView::makeInitialSelection(Rosegarden::timeT time)
 	    goodItemNo = i;
 	}
     }
-
+*/
     if (goodItem) {
 	m_listSelection.push_back(goodItemNo);
 	m_eventList->setSelected(goodItem, true);
@@ -632,6 +726,120 @@ EventView::updateView()
     m_eventList->update();
 }
 
+void
+EventView::slotEditTriggerName()
+{
+    bool ok = false;
+    QString newLabel = KLineEditDlg::getText(i18n("Segment label"), i18n("Label:"),
+					     strtoqstr(m_segments[0]->getLabel()),
+					     &ok, this);
+
+    if (ok) {
+	Rosegarden::SegmentSelection selection;
+	selection.insert(m_segments[0]);
+	SegmentLabelCommand *cmd = new SegmentLabelCommand(selection, newLabel);
+	addCommandToHistory(cmd);
+	m_triggerName->setText(newLabel);
+    }
+}
+
+void
+EventView::slotEditTriggerPitch()
+{
+    int id = m_segments[0]->getComposition()->getTriggerSegmentId(m_segments[0]);
+
+    Rosegarden::TriggerSegmentRec *rec =
+	m_segments[0]->getComposition()->getTriggerSegmentRec(id);
+
+    PitchDialog *dlg = new PitchDialog(this, i18n("Base pitch"), rec->getBasePitch());
+
+    if (dlg->exec() == QDialog::Accepted) {
+	addCommandToHistory(new SetTriggerSegmentBasePitchCommand
+			    (&getDocument()->getComposition(), id, dlg->getPitch()));
+	m_triggerPitch->setText(QString("%1").arg(dlg->getPitch()));
+    }
+}
+
+class TrivialVelocityDialog : public KDialogBase
+{
+public:
+    TrivialVelocityDialog(QWidget *parent, QString label, int deft) :
+	KDialogBase(parent, 0, true, label, Ok | Cancel)
+    {
+	QHBox *hbox = makeHBoxMainWidget();
+	new QLabel(label, hbox);
+	m_spin = new QSpinBox(0, 127, 1, hbox);
+	m_spin->setValue(deft);
+    }
+
+    int getVelocity() { return m_spin->value(); }
+
+protected:
+    QSpinBox *m_spin;
+};
+
+void
+EventView::slotEditTriggerVelocity()
+{
+    int id = m_segments[0]->getComposition()->getTriggerSegmentId(m_segments[0]);
+
+    Rosegarden::TriggerSegmentRec *rec =
+	m_segments[0]->getComposition()->getTriggerSegmentRec(id);
+
+    TrivialVelocityDialog *dlg = new TrivialVelocityDialog
+	(this, i18n("Base velocity"), rec->getBaseVelocity());
+
+    if (dlg->exec() == QDialog::Accepted) {
+	addCommandToHistory(new SetTriggerSegmentBaseVelocityCommand
+			    (&getDocument()->getComposition(), id, dlg->getVelocity()));
+	m_triggerVelocity->setText(QString("%1").arg(dlg->getVelocity()));
+    }
+}
+
+void
+EventView::slotTriggerTimeAdjustChanged(int option)
+{
+    std::string adjust = Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SQUISH;
+
+    switch(option) {
+
+    case  0:
+	adjust = Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_NONE;
+	break;
+    case  1:
+	adjust = Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SYNC_START;
+	break;
+    case  2:
+	adjust = Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SYNC_END;
+	break;
+    case  3:
+	adjust = Rosegarden::BaseProperties::TRIGGER_SEGMENT_ADJUST_SQUISH;
+	break;
+
+    default: break;
+    }
+    
+    int id = m_segments[0]->getComposition()->getTriggerSegmentId(m_segments[0]);
+
+    Rosegarden::TriggerSegmentRec *rec =
+	m_segments[0]->getComposition()->getTriggerSegmentRec(id);
+
+    addCommandToHistory(new SetTriggerSegmentDefaultTimeAdjustCommand
+			(&getDocument()->getComposition(), id, adjust));
+}
+
+void
+EventView::slotTriggerRetuneChanged()
+{
+    int id = m_segments[0]->getComposition()->getTriggerSegmentId(m_segments[0]);
+
+    Rosegarden::TriggerSegmentRec *rec =
+	m_segments[0]->getComposition()->getTriggerSegmentRec(id);
+
+    addCommandToHistory(new SetTriggerSegmentDefaultRetuneCommand
+			(&getDocument()->getComposition(), id, !rec->getDefaultRetune()));
+}
+
 void
 EventView::slotEditCut()
 {
@@ -987,19 +1195,22 @@ EventView::setupActions()
                 SLOT(slotEditInsert()), actionCollection(),
                 "insert");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-delete.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/event-delete.xpm");
+    icon = QIconSet(pixmap);
     
     new KAction(i18n("&Delete Event"), icon, Key_Delete, this,
                 SLOT(slotEditDelete()), actionCollection(),
                 "delete");
     
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-edit.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/event-edit.xpm");
+    icon = QIconSet(pixmap);
 
     new KAction(i18n("&Edit Event"), icon, Key_E, this,
                 SLOT(slotEditEvent()), actionCollection(),
                 "edit_simple");
     
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-edit-advanced.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/event-edit-advanced.xpm");
+    icon = QIconSet(pixmap);
 
     new KAction(i18n("&Advanced Event Editor"), icon, Key_A, this,
                 SLOT(slotEditEventAdvanced()), actionCollection(),
@@ -1023,7 +1234,8 @@ EventView::setupActions()
 
     KRadioAction *action;
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-musical.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-musical.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Musical Times"), icon, 0, this,
 			      SLOT(slotMusicalTime()),
@@ -1031,7 +1243,8 @@ EventView::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 0) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-real.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-real.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Real Times"), icon, 0, this,
 			      SLOT(slotRealTime()),
@@ -1039,7 +1252,8 @@ EventView::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 1) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-raw.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-raw.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("Ra&w Times"), icon, 0, this,
 			      SLOT(slotRawTime()),
@@ -1047,6 +1261,13 @@ EventView::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 2) action->setChecked(true);
 
+    if (m_isTriggerSegment) {
+	KAction *action = actionCollection()->action("open_in_matrix");
+	if (action) delete action;
+	action = actionCollection()->action("open_in_notation");
+	if (action) delete action;
+    }
+
     createGUI(getRCFileName());
 }
 
@@ -1106,6 +1327,13 @@ EventView::slotSaveOptions()
     m_eventList->saveLayout(m_config, LayoutConfigGroupName);
 }
 
+Rosegarden::Segment *
+EventView::getCurrentSegment()
+{
+    if (m_segments.empty()) return 0;
+    else return *m_segments.begin();
+}
+
 void 
 EventView::slotModifyFilter(int button)
 {
@@ -1315,6 +1543,8 @@ EventView::slotPopupEventEditor(QListViewItem *item)
 {
     EventViewItem *eItem = dynamic_cast<EventViewItem*>(item);
 
+    //!!! trigger events
+
     if (eItem)
     {
 	Rosegarden::Event *event = eItem->getEvent();
@@ -1416,6 +1646,38 @@ EventView::slotMenuActivated(int value)
     return;
 }
 
+void
+EventView::updateViewCaption()
+{
+    if (m_isTriggerSegment) {
+
+	setCaption(i18n("%1 - Triggered Segment: %2")
+		   .arg(getDocument()->getTitle())
+		   .arg(strtoqstr(m_segments[0]->getLabel())));
+
+        
+    }  else if (m_segments.size() == 1) {
+
+	Rosegarden::TrackId trackId = m_segments[0]->getTrack();
+	Rosegarden::Track *track =
+	    m_segments[0]->getComposition()->getTrackById(trackId);
+
+	int trackPosition = -1;
+	if (track) trackPosition = track->getPosition();
+
+        setCaption(i18n("%1 - Segment Track #%2 - Event List")
+                   .arg(getDocument()->getTitle())
+                   .arg(trackPosition + 1));
+
+    } else {
+
+        setCaption(i18n("%1 - %2 Segments - Event List")
+                   .arg(getDocument()->getTitle())
+                   .arg(m_segments.size()));
+    }
+    
+}
+
 
 
 const char* const EventView::ConfigGroup = "EventList Options";
