@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -39,8 +39,10 @@ class RosegardenGUIDoc;
 class KListView;
 class QListViewItem;
 class QPushButton;
+class QLabel;
 class QCheckBox;
 class QButtonGroup;
+class RosegardenPitchChooser;
 
 class EventView : public EditViewBase, public Rosegarden::SegmentObserver
 {
@@ -133,10 +135,18 @@ public slots:
     virtual void endMarkerTimeChanged(const Rosegarden::Segment *, bool) { }
     virtual void segmentDeleted(const Rosegarden::Segment *) { }
 
-protected slots:
+signals:
+    void editTriggerSegment(int);
 
+protected slots:
     virtual void slotSaveOptions();
 
+    void slotEditTriggerName();
+    void slotEditTriggerPitch();
+    void slotEditTriggerVelocity();
+    void slotTriggerTimeAdjustChanged(int);
+    void slotTriggerRetuneChanged();
+
 protected:
 
     virtual void readOptions();
@@ -144,8 +154,17 @@ protected:
     QString makeTimeString(Rosegarden::timeT time, int timeMode);
     QString makeDurationString(Rosegarden::timeT time,
 			       Rosegarden::timeT duration, int timeMode);
+    virtual Rosegarden::Segment *getCurrentSegment();
+
+    virtual void updateViewCaption();
 
     //--------------- Data members ---------------------------------
+
+    bool         m_isTriggerSegment;
+    QLabel      *m_triggerName;
+    QLabel      *m_triggerPitch;
+    QLabel      *m_triggerVelocity;
+
     KListView   *m_eventList;
     int          m_eventFilter;
 
