@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -52,21 +52,35 @@ ImportDeviceDialog::ImportDeviceDialog(QWidget *parent, KURL url) :
     KDialogBase(parent, "importdevicedialog", true,
                 i18n("Import from Device..."),
                 Ok | Cancel, Ok),
+    m_url(url),
     m_fileDoc(0),
     m_device(0)
+{
+}
+
+ImportDeviceDialog::~ImportDeviceDialog()
+{
+    if (m_fileDoc) {
+	delete m_fileDoc;
+    } else {
+	delete m_device;
+    }
+}
+
+bool
+ImportDeviceDialog::doImport()
 {
     QVBox *mainFrame = makeVBoxMainWidget();
 
-    if (url.isEmpty()) {
+    if (m_url.isEmpty()) {
 	reject();
-	return;
+	return false;
     }
 
     QString target;
-    if (KIO::NetAccess::download(url, target) == false) {
-        KMessageBox::error(this, QString(i18n("Cannot download file %1"))
-                           .arg(url.prettyURL()));
-        return;
+    if (KIO::NetAccess::download(m_url, target) == false) {
+        KMessageBox::error(this, i18n("Cannot download file %1").arg(m_url.prettyURL()));
+        return false;
     }
 
     bool fileRead = false;
@@ -77,17 +91,17 @@ ImportDeviceDialog::ImportDeviceDialog(QWidget *parent, KURL url) :
     }
     if (!fileRead) {
 	KMessageBox::error
-	    (this, i18n("Cannot open file %1").arg(url.prettyURL()));
+	    (this, i18n("Cannot open file %1").arg(m_url.prettyURL()));
 	reject();
 	close();
-	return;
+	return false;
     }
     if (m_devices.size() == 0) {
 	KMessageBox::sorry
-	    (this, i18n("No devices found in file %1").arg(url.prettyURL()));
+	    (this, i18n("No devices found in file %1").arg(m_url.prettyURL()));
 	reject();
 	close();
-	return;
+	return false;
     }
 
     QGroupBox *groupBox = new QGroupBox(2, Qt::Horizontal,
@@ -160,16 +174,10 @@ ImportDeviceDialog::ImportDeviceDialog(QWidget *parent, KURL url) :
     bool overwrite = config->readBoolEntry("importbanksoverwrite", true);
     if (overwrite) m_buttonGroup->setButton(1);
     else m_buttonGroup->setButton(0);
+
+    return true;
 }
 
-ImportDeviceDialog::~ImportDeviceDialog()
-{
-    if (m_fileDoc) {
-	delete m_fileDoc;
-    } else {
-	delete m_device;
-    }
-}
 
 void
 ImportDeviceDialog::slotOk()
@@ -328,7 +336,8 @@ ImportDeviceDialog::importFromSF2(QString filename)
 	int lsb = bankNumber % 128;
 
         Rosegarden::MidiBank bank
-	    (false, msb, lsb, qstrtostr(i18n("Bank %1:%2").arg(msb).arg(lsb)));
+	    (msb == 1, msb, lsb,
+	     qstrtostr(i18n("Bank %1:%2").arg(msb).arg(lsb)));
 
         banks.push_back(bank);
 
