@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -43,6 +43,8 @@ public:
     ImportDeviceDialog(QWidget *parent, KURL url);
     virtual ~ImportDeviceDialog();
 
+    bool doImport();
+
     bool shouldImportBanks() const;
     bool shouldImportControllers() const;
     bool shouldOverwriteBanks() const; 
@@ -64,6 +66,8 @@ protected:
     bool importFromRG(QString fileName);
     bool importFromSF2(QString fileName);
 
+    KURL               m_url;
+
     KComboBox          *m_deviceCombo;
     QLabel             *m_deviceLabel;
 
