@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -39,6 +39,8 @@
 #include <klocale.h>
 #include <kapp.h>
 
+#include <algorithm>
+
 #include "Midi.h"
 #include "Instrument.h"
 #include "MidiDevice.h"
@@ -67,15 +69,14 @@ using Rosegarden::MidiDevice;
 InstrumentParameterBox::InstrumentParameterBox(RosegardenGUIDoc *doc,
                                                QWidget *parent)
     : RosegardenParameterBox(1, Qt::Horizontal, i18n("Instrument Parameters"), parent),
-//      m_widgetStack(new QWidgetStack(this)),
-      m_widgetStack(0), //!!!
+      m_widgetStack(new QWidgetStack(this)),
       m_noInstrumentParameters(new QVBox(this)),
       m_midiInstrumentParameters(new MIDIInstrumentParameterPanel(doc, this)),
       m_audioInstrumentParameters(new AudioInstrumentParameterPanel(doc, this)),
       m_selectedInstrument(0),
       m_doc(doc)
 {
-//     m_widgetStack->setFont(m_font);
+    m_widgetStack->setFont(m_font);
     m_noInstrumentParameters->setFont(m_font);
     m_midiInstrumentParameters->setFont(m_font);
     m_audioInstrumentParameters->setFont(m_font);
@@ -95,9 +96,9 @@ InstrumentParameterBox::InstrumentParameterBox(RosegardenGUIDoc *doc,
     QLabel *label = new QLabel(i18n("<no instrument>"), m_noInstrumentParameters);
     label->setAlignment(label->alignment() | Qt::AlignHCenter);
 
-//     m_widgetStack->addWidget(m_midiInstrumentParameters);
-//     m_widgetStack->addWidget(m_audioInstrumentParameters);
-//     m_widgetStack->addWidget(m_noInstrumentParameters);
+    m_widgetStack->addWidget(m_midiInstrumentParameters);
+    m_widgetStack->addWidget(m_audioInstrumentParameters);
+    m_widgetStack->addWidget(m_noInstrumentParameters);
 
     m_midiInstrumentParameters->adjustSize();
     m_audioInstrumentParameters->adjustSize();
@@ -111,25 +112,15 @@ InstrumentParameterBox::InstrumentParameterBox(RosegardenGUIDoc *doc,
 	    this,
 	    SIGNAL(instrumentParametersChanged(Rosegarden::InstrumentId)));
 
-    connect(m_audioInstrumentParameters,
-            SIGNAL(muteButton(Rosegarden::InstrumentId, bool)),
-            this, 
-            SIGNAL(setMute(Rosegarden::InstrumentId, bool)));
-    
-    connect(m_audioInstrumentParameters,
-            SIGNAL(soloButton(Rosegarden::InstrumentId, bool)),
-            this,
-            SIGNAL(setSolo(Rosegarden::InstrumentId, bool)));
-    
-    connect(m_audioInstrumentParameters,
-            SIGNAL(recordButton(Rosegarden::InstrumentId, bool)),
-            this,
-            SIGNAL(setRecord(Rosegarden::InstrumentId, bool)));
-
     connect(m_audioInstrumentParameters,
 	    SIGNAL(selectPlugin(QWidget *, Rosegarden::InstrumentId, int)),
 	    this,
 	    SIGNAL(selectPlugin(QWidget *, Rosegarden::InstrumentId, int)));
+
+    connect(m_audioInstrumentParameters,
+	    SIGNAL(showPluginGUI(Rosegarden::InstrumentId, int)),
+	    this,
+	    SIGNAL(showPluginGUI(Rosegarden::InstrumentId, int)));
     
     connect(m_midiInstrumentParameters, SIGNAL(updateAllBoxes()),
             this, SLOT(slotUpdateAllBoxes()));
@@ -207,11 +198,7 @@ InstrumentParameterBox::useInstrument(Instrument *instrument)
 
     if (instrument == 0)
     {
-//         m_widgetStack->raiseWidget(m_noInstrumentParameters);
-	m_audioInstrumentParameters->hide();
-	m_midiInstrumentParameters->hide();
-	if (m_noInstrumentParameters->isHidden()) 
-	    m_noInstrumentParameters->show();
+        m_widgetStack->raiseWidget(m_noInstrumentParameters);
         return;
     } 
 
@@ -220,69 +207,24 @@ InstrumentParameterBox::useInstrument(Instrument *instrument)
 
     // Hide or Show according to Instrument type
     //
-    if (instrument->getType() == Instrument::Audio)
+    if (instrument->getType() == Instrument::Audio ||
+	instrument->getType() == Instrument::SoftSynth)
     {
-	m_noInstrumentParameters->hide();
-	m_midiInstrumentParameters->hide();
         m_audioInstrumentParameters->setupForInstrument(m_selectedInstrument);
-	if (m_audioInstrumentParameters->isHidden())
-	    m_audioInstrumentParameters->show();
-
-//         m_widgetStack->raiseWidget(m_audioInstrumentParameters);
+        m_widgetStack->raiseWidget(m_audioInstrumentParameters);
 
     } else { // Midi
 
-	m_noInstrumentParameters->hide();
-	m_audioInstrumentParameters->hide();
         m_midiInstrumentParameters->setupForInstrument(m_selectedInstrument);
-	if (m_midiInstrumentParameters->isHidden())
-	    m_midiInstrumentParameters->show();
-
-//         m_widgetStack->raiseWidget(m_midiInstrumentParameters);
+	m_widgetStack->raiseWidget(m_midiInstrumentParameters);
     }
     
 }
 
-void
-InstrumentParameterBox::setMute(bool value)
-{
-    if (m_selectedInstrument && 
-            m_selectedInstrument->getType() == Instrument::Audio)
-    {
-        m_audioInstrumentParameters->slotSetMute(value);
-    }
-}
-
-/*
- * Set the record state of the audio instrument parameter panel
- */
-void
-InstrumentParameterBox::setRecord(bool value)
-{
-    if (m_selectedInstrument &&
-	m_selectedInstrument->getType() == Instrument::Audio)
-    {
-        m_audioInstrumentParameters->slotSetRecord(value);
-    }
-}
-
-void
-InstrumentParameterBox::setSolo(bool value)
-{
-    if (m_selectedInstrument &&
-	m_selectedInstrument->getType() == Instrument::Audio)
-    {
-        m_audioInstrumentParameters->slotSetSolo(value);
-    }
-}
-
-    
 
 void
 InstrumentParameterBox::slotUpdateAllBoxes()
 {
-    RG_DEBUG << "InstrumentParameterBox::slotUpdateAllBoxes" << endl;
-
     std::vector<InstrumentParameterBox*>::iterator it =
         instrumentParamBoxes.begin();
 
@@ -324,10 +266,8 @@ AudioInstrumentParameterPanel::slotSelectAudioLevel(float dB)
     if (m_selectedInstrument == 0)
         return;
 
-    std::cerr << "AudioInstrumentParameterPanel::slotSelectAudioLevel("
-	      << dB << ")" << std::endl;
-
-    if (m_selectedInstrument->getType() == Instrument::Audio)
+    if (m_selectedInstrument->getType() == Instrument::Audio ||
+	m_selectedInstrument->getType() == Instrument::SoftSynth)
     {
 	m_selectedInstrument->setLevel(dB);
 
@@ -364,50 +304,6 @@ AudioInstrumentParameterPanel::slotSelectAudioRecordLevel(float dB)
     }
 }
 
-void 
-AudioInstrumentParameterPanel::slotSetMute(bool value)
-{
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotSetMute - "
-             << "value = " << value << endl;
-    m_audioFader->m_muteButton->setOn(value);
-    emit instrumentParametersChanged(m_selectedInstrument->getId());
-}
-
-void
-AudioInstrumentParameterPanel::slotSetSolo(bool value)
-{
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotSetSolo - "
-             << "value = " << value << endl;
-    m_audioFader->m_soloButton->setOn(value);
-    emit instrumentParametersChanged(m_selectedInstrument->getId());
-}
-
-void 
-AudioInstrumentParameterPanel::slotSetRecord(bool value)
-{
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotSetRecord - "
-             << "value = " << value << endl;
-
-    //if (m_selectedInstrument)
-        //cout << "INSTRUMENT NAME = " 
-               //<< m_selectedInstrument->getName() << endl;
-
-    // Set the background colour for the button
-    //
-    if (value)
-    {
-        m_audioFader->m_recordButton->
-            setPalette(QPalette(RosegardenGUIColours::ActiveRecordTrack));
-    }
-    else
-    {
-        m_audioFader->m_recordButton->unsetPalette();
-    }
-
-    m_audioFader->m_recordButton->setOn(value);
-    emit instrumentParametersChanged(m_selectedInstrument->getId());
-}
-
 
 void
 AudioInstrumentParameterPanel::slotPluginSelected(Rosegarden::InstrumentId instrumentId,
@@ -424,10 +320,26 @@ AudioInstrumentParameterPanel::slotPluginSelected(Rosegarden::InstrumentId instr
     QColor pluginBackgroundColour = Qt::black;
     bool bypassed = false;
 
+    QPushButton *button = 0;
+    QString noneText;
+
+    // updates synth gui button &c:
+    m_audioFader->slotSetInstrument(&m_doc->getStudio(), m_selectedInstrument);
+
+    if (index == Rosegarden::Instrument::SYNTH_PLUGIN_POSITION) {
+	button = m_audioFader->m_synthButton;
+	noneText = i18n("<no synth>");
+    } else {
+	button = m_audioFader->m_plugins[index];
+	noneText = i18n("<no plugin>");
+    }
+
+    if (!button) return;
+
     if (plugin == -1) {
 
-	m_audioFader->m_plugins[index]->setText(i18n("<no plugin>"));
-        QToolTip::add(m_audioFader->m_plugins[index], i18n("<no plugin>"));
+	button->setText(noneText);
+        QToolTip::add(button, noneText);
 
     } else {
 
@@ -436,11 +348,9 @@ AudioInstrumentParameterPanel::slotPluginSelected(Rosegarden::InstrumentId instr
 
 	if (pluginClass)
         {
-	    m_audioFader->m_plugins[index]->
-		setText(pluginClass->getLabel());
+	    button->setText(pluginClass->getLabel());
 
-            QToolTip::add(m_audioFader->m_plugins[index], 
-                    pluginClass->getLabel());
+            QToolTip::add(button, pluginClass->getLabel());
 
             pluginBackgroundColour = pluginClass->getColour();
         }
@@ -472,7 +382,7 @@ AudioInstrumentParameterPanel::slotPluginBypassed(Rosegarden::InstrumentId instr
         Rosegarden::AudioPlugin *pluginClass 
             = m_doc->getPluginManager()->getPlugin(
                     m_doc->getPluginManager()->
-                        getPositionByUniqueId(inst->getId()));
+                        getPositionByIdentifier(inst->getIdentifier().c_str()));
 
         /// Set the colour on the button
         //
@@ -494,47 +404,61 @@ AudioInstrumentParameterPanel::setButtonColour(
         << ", bypassState = " << bypassState
         << ", rgb = " << colour.name() << endl;
 
+    QPushButton *button = 0;
+
+    if (pluginIndex == Rosegarden::Instrument::SYNTH_PLUGIN_POSITION) {
+	button = m_audioFader->m_synthButton;
+    } else {
+	button = m_audioFader->m_plugins[pluginIndex];
+    }
+
+    if (!button) return;
+
     // Set the bypass colour on the plugin button
     if (bypassState)
     {
-        m_audioFader->m_plugins[pluginIndex]->
+	button->
             setPaletteForegroundColor(kapp->palette().
                     color(QPalette::Active, QColorGroup::Button));
 
-        m_audioFader->m_plugins[pluginIndex]->
+	button->
             setPaletteBackgroundColor(kapp->palette().
                     color(QPalette::Active, QColorGroup::ButtonText));
     }
     else if (colour == Qt::black)
     {
-        m_audioFader->m_plugins[pluginIndex]->
+	button->
             setPaletteForegroundColor(kapp->palette().
                     color(QPalette::Active, QColorGroup::ButtonText));
 
-        m_audioFader->m_plugins[pluginIndex]->
+	button->
             setPaletteBackgroundColor(kapp->palette().
                     color(QPalette::Active, QColorGroup::Button));
     }
     else
     {
-        m_audioFader->m_plugins[pluginIndex]->
+	button->
             setPaletteForegroundColor(Qt::white);
-        m_audioFader->m_plugins[pluginIndex]->setPaletteBackgroundColor(colour);
+
+	button->
+	    setPaletteBackgroundColor(colour);
     }
 }
 
 
 AudioInstrumentParameterPanel::AudioInstrumentParameterPanel(RosegardenGUIDoc* doc, QWidget* parent)
     : InstrumentParameterPanel(doc, parent),
-      m_audioFader(new AudioFaderWidget(this, AudioFaderWidget::FaderBox))
+      m_audioFader(new AudioFaderBox(this))
 {
-    QGridLayout *gridLayout = new QGridLayout(this, 2, 1, 5, 5);
+    QGridLayout *gridLayout = new QGridLayout(this, 3, 2, 5, 5);
 
     // Instrument label : first row, all cols
-    gridLayout->addMultiCellWidget(m_instrumentLabel, 0, 0, 0, 0, AlignCenter);
+    gridLayout->addMultiCellWidget(m_instrumentLabel, 0, 0, 0, 1, AlignCenter);
 
     // fader and connect it
-    gridLayout->addMultiCellWidget(m_audioFader, 1, 1, 0, 0);
+    gridLayout->addMultiCellWidget(m_audioFader, 1, 1, 0, 1);
+
+    gridLayout->setRowStretch(2, 1);
 
     connect(m_audioFader, SIGNAL(audioChannelsChanged(int)),
             this, SLOT(slotAudioChannels(int)));
@@ -548,15 +472,6 @@ AudioInstrumentParameterPanel::AudioInstrumentParameterPanel(RosegardenGUIDoc* d
     connect(m_audioFader->m_recordFader, SIGNAL(faderChanged(float)),
             this, SLOT(slotSelectAudioRecordLevel(float)));
 
-    connect(m_audioFader->m_muteButton, SIGNAL(clicked()),
-            this, SLOT(slotMute()));
-
-    connect(m_audioFader->m_soloButton, SIGNAL(clicked()),
-            this, SLOT(slotSolo()));
-
-    connect(m_audioFader->m_recordButton, SIGNAL(clicked()),
-            this, SLOT(slotRecord()));
-
     connect(m_audioFader->m_pan, SIGNAL(valueChanged(float)),
             this, SLOT(slotSetPan(float)));
 
@@ -565,42 +480,25 @@ AudioInstrumentParameterPanel::AudioInstrumentParameterPanel(RosegardenGUIDoc* d
 
     connect(m_audioFader->m_audioInput, SIGNAL(changed()),
             this, SLOT(slotAudioRoutingChanged()));
+
+    connect(m_audioFader->m_synthButton, SIGNAL(clicked()),
+	    this, SLOT(slotSynthButtonClicked()));
+
+    connect(m_audioFader->m_synthGUIButton, SIGNAL(clicked()),
+	    this, SLOT(slotSynthGUIButtonClicked()));
 }
 
 void
-AudioInstrumentParameterPanel::slotMute()
+AudioInstrumentParameterPanel::slotSynthButtonClicked()
 {
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotMute" << endl;
-    emit muteButton(m_selectedInstrument->getId(),
-                    m_audioFader->m_muteButton->isOn());
+    slotSelectPlugin(Rosegarden::Instrument::SYNTH_PLUGIN_POSITION);
 }
 
 void
-AudioInstrumentParameterPanel::slotSolo()
+AudioInstrumentParameterPanel::slotSynthGUIButtonClicked()
 {
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotSolo" << endl;
-    emit soloButton(m_selectedInstrument->getId(),
-                    m_audioFader->m_soloButton->isOn());
-}
-
-void
-AudioInstrumentParameterPanel::slotRecord()
-{
-    RG_DEBUG << "AudioInstrumentParameterPanel::slotRecord - " 
-             << " isOn = " <<  m_audioFader->m_recordButton->isOn() << endl;
-
-    // At the moment we can't turn a recording button off
-    //
-    if (m_audioFader->m_recordButton->isOn())
-    {
-        m_audioFader->m_recordButton->setOn(true);
-        emit recordButton(m_selectedInstrument->getId(),
-                          m_audioFader->m_recordButton->isOn());
-    }
-    else
-    {
-        m_audioFader->m_recordButton->setOn(true);
-    }
+    emit showPluginGUI(m_selectedInstrument->getId(),
+			Rosegarden::Instrument::SYNTH_PLUGIN_POSITION);
 }
 
 void
@@ -626,11 +524,9 @@ AudioInstrumentParameterPanel::setAudioMeter(float dBleft, float dBright)
     
     if (m_selectedInstrument)
     {
-        if (m_selectedInstrument->getAudioChannels() == 1) {
-	    m_audioFader->m_vuMeter->setLevel(dBleft);
-	} else {
-	    m_audioFader->m_vuMeter->setLevel(dBleft, dBright);
-	}
+	// Always set stereo, because we have to reflect what's happening
+	// with the pan setting even on mono tracks
+	m_audioFader->m_vuMeter->setLevel(dBleft, dBright);
     }
 }
 
@@ -638,6 +534,8 @@ AudioInstrumentParameterPanel::setAudioMeter(float dBleft, float dBright)
 void
 AudioInstrumentParameterPanel::setupForInstrument(Instrument* instrument)
 {
+    blockSignals(true);
+
     m_selectedInstrument = instrument;
 
     m_instrumentLabel->setText(strtoqstr(instrument->getName()));
@@ -647,37 +545,50 @@ AudioInstrumentParameterPanel::setupForInstrument(Instrument* instrument)
 
     m_audioFader->slotSetInstrument(&m_doc->getStudio(), instrument);
 
-    for (unsigned int i = 0; i < m_audioFader->m_plugins.size(); i++)
-    {
-        m_audioFader->m_plugins[i]->show();
+    int start = 0;
 
-        Rosegarden::AudioPluginInstance *inst = instrument->getPlugin(i);
+    if (instrument->getType() == Rosegarden::Instrument::SoftSynth) start = -1;
+
+    for (int i = start; i < int(m_audioFader->m_plugins.size()); i++)
+    {
+	int index;
+	QPushButton *button;
+	QString noneText;
+
+	if (i == -1) {
+	    index = Rosegarden::Instrument::SYNTH_PLUGIN_POSITION;
+	    button = m_audioFader->m_synthButton;
+	    noneText = i18n("<no synth>");
+	} else {
+	    index = i;
+	    button = m_audioFader->m_plugins[i];
+	    noneText = i18n("<no plugin>");
+	}
+
+	button->show();
+
+        Rosegarden::AudioPluginInstance *inst = instrument->getPlugin(index);
 
         if (inst && inst->isAssigned())
         {
             Rosegarden::AudioPlugin *pluginClass 
                 = m_doc->getPluginManager()->getPlugin(
                         m_doc->getPluginManager()->
-                            getPositionByUniqueId(inst->getId()));
+                            getPositionByIdentifier(inst->getIdentifier().c_str()));
 
             if (pluginClass)
             {
-                m_audioFader->m_plugins[i]->
-                    setText(pluginClass->getLabel());
-
-                QToolTip::add(m_audioFader->m_plugins[i], 
-                    pluginClass->getLabel());
-
-                setButtonColour(i, inst->isBypassed(), 
+                button->setText(pluginClass->getLabel());
+                QToolTip::add(button, pluginClass->getLabel());
+                setButtonColour(index, inst->isBypassed(), 
                         pluginClass->getColour());
             }
         }
         else
         {
-            m_audioFader->m_plugins[i]->setText(i18n("<no plugin>"));
-            QToolTip::add(m_audioFader->m_plugins[i], i18n("<no plugin>"));
-
-            setButtonColour(i, inst ? inst->isBypassed() : false, Qt::black);
+            button->setText(noneText);
+            QToolTip::add(button, noneText);
+            setButtonColour(index, inst ? inst->isBypassed() : false, Qt::black);
         }
     }
 
@@ -688,6 +599,12 @@ AudioInstrumentParameterPanel::setupForInstrument(Instrument* instrument)
     // Pan - adjusted backwards
     //
     m_audioFader->m_pan->setPosition(instrument->getPan() - 100);
+
+    // Tell fader box whether to include e.g. audio input selection
+    // 
+    m_audioFader->setIsSynth(instrument->getType() == Rosegarden::Instrument::SoftSynth);
+
+    blockSignals(false);
 }
 
 void
@@ -740,6 +657,10 @@ MIDIInstrumentParameterPanel::MIDIInstrumentParameterPanel(RosegardenGUIDoc *doc
     m_variationCheckBox = new QCheckBox(this);
     m_percussionCheckBox = new QCheckBox(this);
 
+    m_bankValue->setSizeLimit(20);
+    m_programValue->setSizeLimit(20);
+    m_variationValue->setSizeLimit(20);
+
     m_bankLabel = new QLabel(i18n("Bank"), this);
     m_variationLabel = new QLabel(i18n("Variation"), this);
     QLabel* programLabel = new QLabel(i18n("Program"), this);
@@ -1027,7 +948,10 @@ MIDIInstrumentParameterPanel::setupControllers(MidiDevice *md)
 		 smallStep,
 		 bigStep,
 		 it->getDefault(),
-		 20);
+		 20,
+		 RosegardenRotary::NoTicks,
+		 false,
+		 it->getDefault() == 64); //!!! hacky
 
 	    rotary->setKnobColour(knobColour);
 
@@ -1140,13 +1064,20 @@ MIDIInstrumentParameterPanel::populateBankList()
 
     if (md->getVariationType() == MidiDevice::NoVariations) {
 
-	if (m_bankLabel->isHidden()) {
-	    m_bankLabel->show();
-	    m_bankCheckBox->show();
-	    m_bankValue->show();
-	}
 	banks = md->getBanks(m_selectedInstrument->isPercussion());
 
+	if (!banks.empty()) {
+	    if (m_bankLabel->isHidden()) {
+		m_bankLabel->show();
+		m_bankCheckBox->show();
+		m_bankValue->show();
+	    }
+	} else {
+	    m_bankLabel->hide();
+	    m_bankCheckBox->hide();
+	    m_bankValue->hide();
+	}
+
 	for (unsigned int i = 0; i < banks.size(); ++i) {
 	    if (m_selectedInstrument->getProgram().getBank() == banks[i]) {
 		currentBank = i;
@@ -1209,9 +1140,15 @@ MIDIInstrumentParameterPanel::populateBankList()
 	m_banks.push_back(*i);
 	m_bankValue->insertItem(strtoqstr(i->getName()));
     }
-	    
-    m_bankValue->setCurrentItem(currentBank);
+
     m_bankValue->setEnabled(m_selectedInstrument->sendsBankSelect());
+    
+    if (currentBank < 0 && !banks.empty()) {
+	m_bankValue->setCurrentItem(0);
+	slotSelectBank(0);
+    } else {
+	m_bankValue->setCurrentItem(currentBank);
+    }
 }    
 	
 // Populate program list by bank context
@@ -1244,6 +1181,8 @@ MIDIInstrumentParameterPanel::populateProgramList()
 	bank = m_selectedInstrument->getProgram().getBank();
     }
 
+    int currentProgram = -1;
+
     Rosegarden::ProgramList programs = md->getPrograms(bank);
     for (unsigned int i = 0; i < programs.size(); ++i) {
 	std::string programName = programs[i].getName();
@@ -1252,7 +1191,7 @@ MIDIInstrumentParameterPanel::populateProgramList()
 				       .arg(programs[i].getProgram() + 1)
 				       .arg(strtoqstr(programName)));
 	    if (m_selectedInstrument->getProgram() == programs[i]) {
-		m_programValue->setCurrentItem(i);
+		currentProgram = i;
 	    }
 	    m_programs.push_back(programs[i]);
 	}
@@ -1260,13 +1199,20 @@ MIDIInstrumentParameterPanel::populateProgramList()
 
     m_programValue->setEnabled(m_selectedInstrument->sendsProgramChange());
 
-    // Ensure that stored program change value is same as the one
-    // we're now showing (BUG 937371)
-    //
-    if (m_programs.size())
-    {
-        m_selectedInstrument->setProgramChange
-            ((m_programs[m_programValue->currentItem()]).getProgram());
+    if (currentProgram == 0 && !m_programs.empty()) {
+	m_programValue->setCurrentItem(0);
+	slotSelectProgram(0);
+    } else {
+	m_programValue->setCurrentItem(currentProgram);
+
+	// Ensure that stored program change value is same as the one
+	// we're now showing (BUG 937371)
+	//
+	if (!m_programs.empty())
+	{
+	    m_selectedInstrument->setProgramChange
+		((m_programs[m_programValue->currentItem()]).getProgram());
+	}
     }
 }
 
@@ -1336,6 +1282,8 @@ MIDIInstrumentParameterPanel::populateVariationList()
     }
     std::string defaultProgramName = md->getProgramName(defaultProgram);
 
+    int currentVariation = -1;
+
     for (unsigned int i = 0; i < variations.size(); ++i) {
 
 	Rosegarden::MidiProgram program;
@@ -1366,12 +1314,19 @@ MIDIInstrumentParameterPanel::populateVariationList()
 					 .arg(variations[i] + 1)
 					 .arg(strtoqstr(programName)));
 	    if (m_selectedInstrument->getProgram() == program) {
-		m_variationValue->setCurrentItem(i);
+		currentVariation = i;
 	    }
 	    m_variations.push_back(variations[i]);
 	}
     }
 
+    if (currentVariation < 0 && !m_variations.empty()) {
+	m_variationValue->setCurrentItem(0);
+	slotSelectVariation(0);
+    } else {
+	m_variationValue->setCurrentItem(currentVariation);
+    }
+
     if (m_variations.size() < 2) {
 	if (!m_variationLabel->isHidden()) {
 	    m_variationLabel->hide();
@@ -1518,18 +1473,27 @@ MIDIInstrumentParameterPanel::slotSelectBank(int index)
 
     const Rosegarden::MidiBank *bank = &m_banks[index];
 
+    bool change = false;
+
     if (md->getVariationType() != MidiDevice::VariationFromLSB) {
-	m_selectedInstrument->setLSB(bank->getLSB());
+	if (m_selectedInstrument->getLSB() != bank->getLSB()) {
+	    m_selectedInstrument->setLSB(bank->getLSB());
+	    change = true;
+	}
     }
     if (md->getVariationType() != MidiDevice::VariationFromMSB) {
-	m_selectedInstrument->setMSB(bank->getMSB());
+	if (m_selectedInstrument->getMSB() != bank->getMSB()) {
+	    m_selectedInstrument->setMSB(bank->getMSB());
+	    change = true;
+	}
     }
 
     populateProgramList();
 
-    sendBankAndProgram();
-
-    emit updateAllBoxes();
+    if (change) {
+	sendBankAndProgram();
+	emit updateAllBoxes();
+    }
 }
 
 void
@@ -1540,16 +1504,22 @@ MIDIInstrumentParameterPanel::slotSelectProgram(int index)
         RG_DEBUG << "program change not found in bank" << endl;
         return;
     }
-    m_selectedInstrument->setProgramChange(prg->getProgram());
 
-    sendBankAndProgram();
+    bool change = false;
+    if (m_selectedInstrument->getProgramChange() != prg->getProgram()) {
+	m_selectedInstrument->setProgramChange(prg->getProgram());
+	change = true;
+    }
 
     populateVariationList();
 
-    emit changeInstrumentLabel(m_selectedInstrument->getId(),
-			       strtoqstr(m_selectedInstrument->
-					 getProgramName()));
-    emit updateAllBoxes();
+    if (change) {
+	sendBankAndProgram();
+	emit changeInstrumentLabel(m_selectedInstrument->getId(),
+				   strtoqstr(m_selectedInstrument->
+					     getProgramName()));
+	emit updateAllBoxes();
+    }
 }
 
 void
@@ -1570,13 +1540,23 @@ MIDIInstrumentParameterPanel::slotSelectVariation(int index)
 
     Rosegarden::MidiByte v = m_variations[index];
     
-    if (md->getVariationType() == MidiDevice::VariationFromLSB) {
-	m_selectedInstrument->setLSB(v);
-    } else if (md->getVariationType() == MidiDevice::VariationFromMSB) {
-	m_selectedInstrument->setMSB(v);
-    }
+    bool change = false;
 
-    sendBankAndProgram();
+    if (md->getVariationType() == MidiDevice::VariationFromLSB) {
+	if (m_selectedInstrument->getLSB() != v) {
+	    m_selectedInstrument->setLSB(v);
+	    change = true;
+	}
+    } else if (md->getVariationType() == MidiDevice::VariationFromMSB) {
+	if (m_selectedInstrument->getMSB() != v) {
+	    m_selectedInstrument->setMSB(v);
+	    change = true;
+	}
+    }
+    
+    if (change) {
+	sendBankAndProgram();
+    }
 }
 
 void
@@ -1588,7 +1568,7 @@ MIDIInstrumentParameterPanel::sendBankAndProgram()
     MidiDevice *md = dynamic_cast<MidiDevice*>
 	(m_selectedInstrument->getDevice());
     if (!md) {
-	RG_DEBUG << "WARNING: MIDIInstrumentParameterPanel::slotSelectBank: No MidiDevice for Instrument "
+	RG_DEBUG << "WARNING: MIDIInstrumentParameterPanel::sendBankAndProgram: No MidiDevice for Instrument "
 		  << m_selectedInstrument->getId() << endl;
 	return;
     }
@@ -1601,12 +1581,24 @@ MIDIInstrumentParameterPanel::sendBankAndProgram()
                                       Rosegarden::MappedEvent::MidiController,
                                       Rosegarden::MIDI_CONTROLLER_BANK_MSB,
                                       m_selectedInstrument->getMSB());
+
+	RG_DEBUG << "MIDIInstrumentParameterPanel::sendBankAndProgram - "
+		 << "sending MSB = " 
+		 << int(m_selectedInstrument->getMSB()) 
+		 << endl;
+
         Rosegarden::StudioControl::sendMappedEvent(mEMSB);
 
         Rosegarden::MappedEvent mELSB(m_selectedInstrument->getId(), 
                                       Rosegarden::MappedEvent::MidiController,
                                       Rosegarden::MIDI_CONTROLLER_BANK_LSB,
                                       m_selectedInstrument->getLSB());
+
+	RG_DEBUG << "MIDIInstrumentParameterPanel::sendBankAndProgram - "
+		 << "sending LSB = " 
+		 << int(m_selectedInstrument->getLSB()) 
+		 << endl;
+
         Rosegarden::StudioControl::sendMappedEvent(mELSB);
     }
 
@@ -1662,7 +1654,8 @@ MIDIInstrumentParameterPanel::slotControllerChanged(int controllerNumber)
     if (controllerNumber == int(Rosegarden::MIDI_CONTROLLER_PAN))
     {
         float adjValue = value;
-        if (m_selectedInstrument->getType() == Instrument::Audio)
+        if (m_selectedInstrument->getType() == Instrument::Audio ||
+	    m_selectedInstrument->getType() == Instrument::SoftSynth)
             value += 100;
 
         m_selectedInstrument->setPan(Rosegarden::MidiByte(adjValue));
