@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -59,6 +59,8 @@
 
 #include "rosedebug.h"
 
+//#define DEBUG_PITCH
+
 using namespace Rosegarden::BaseProperties;
 using Rosegarden::Bool;
 using Rosegarden::Clef;
@@ -72,11 +74,9 @@ using Rosegarden::String;
 using Rosegarden::timeT;
 using Rosegarden::TimeSignature;
 using Rosegarden::Accidental;
-using Rosegarden::Accidentals;
 using Rosegarden::Text;
 using Rosegarden::PropertyName;
 using Rosegarden::Mark;
-using Rosegarden::Marks;
 using Rosegarden::Configuration;
 
 const Rosegarden::PropertyName LilypondExporter::SKIP_PROPERTY
@@ -89,7 +89,7 @@ LilypondExporter::LilypondExporter(QObject *parent,
                                    m_composition(composition),
                                    m_fileName(fileName)
 {
-    // nothing
+    m_pitchBorked = false;
 }
 
 LilypondExporter::~LilypondExporter()
@@ -109,6 +109,8 @@ LilypondExporter::handleStartingEvents(eventstartlist &eventsToStart,
 	    Indication i(**m);
 
 	    if (i.getIndicationType() == Indication::Slur) {
+		str << "( ";
+	    } else if (i.getIndicationType() == Indication::PhrasingSlur) {
 		str << "\\( ";
 	    } else if (i.getIndicationType() == Indication::Crescendo) {
 		str << "\\< ";
@@ -153,10 +155,13 @@ LilypondExporter::handleEndingEvents(eventendlist &eventsInProgress,
 		(*j)->getNotationAbsoluteTime() + (*j)->getNotationDuration();
 	    
 	    if (indicationEnd < eventEnd ||
-		(i.getIndicationType() == Indication::Slur &&
+		((i.getIndicationType() == Indication::Slur ||
+		  i.getIndicationType() == Indication::PhrasingSlur) &&
 		 indicationEnd == eventEnd)) {
 
                 if (i.getIndicationType() == Indication::Slur) {
+                    str << ") ";
+		} else if (i.getIndicationType() == Indication::PhrasingSlur) {
                     str << "\\) ";
                 } else if (i.getIndicationType() == Indication::Crescendo ||
                            i.getIndicationType() == Indication::Decrescendo) {
@@ -179,6 +184,9 @@ LilypondExporter::handleEndingEvents(eventendlist &eventsInProgress,
 
 // processes input to produce a Lilypond-format note written correctly for all
 // keys and out-of-key accidental combinations.
+//
+// this code is deprecated, as it has been supplanted by the Pitch class,
+// but it remains, for the time being, as a necessary failsafe
 std::string
 LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
 					 const Rosegarden::Key &key) 
@@ -224,7 +232,7 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
         
     // assign out-of-key accidentals first, by BaseProperty::ACCIDENTAL
     if (accidental != "") {
-        if (accidental == Accidentals::Sharp) {
+        if (accidental == Rosegarden::Accidentals::Sharp) {
             switch (pitchNote) {
                 case  5: lilyNote = "eis"; // 5 + Sharp = E#
                          break;
@@ -240,7 +248,7 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
                          break;
                 case 10: lilyNote = "ais";
             }
-        } else if (accidental == Accidentals::Flat) {
+        } else if (accidental == Rosegarden::Accidentals::Flat) {
             switch (pitchNote) {
                 case 11: lilyNote = "ces"; // 11 + Flat = Cb
                          break;
@@ -256,7 +264,7 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
                          break;
                 case 10: lilyNote = "bes";
             }
-        } else if (accidental == Accidentals::DoubleSharp) {
+        } else if (accidental == Rosegarden::Accidentals::DoubleSharp) {
             switch (pitchNote) {
                 case  1: lilyNote = "bisis"; // 1 + ## = B##
                          break;
@@ -273,7 +281,7 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
                 case 11: lilyNote = "aisis"; //11 + ## = A##
                          break;
             }
-        } else if (accidental == Accidentals::DoubleFlat) {
+        } else if (accidental == Rosegarden::Accidentals::DoubleFlat) {
             switch (pitchNote) {
                 case 10: lilyNote = "ceses"; //10 + bb = Cbb
                          break;
@@ -290,7 +298,7 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
                 case  9: lilyNote = "beses"; // 9 + bb = Bbb
                          break;
             }
-        } else if (accidental == Accidentals::Natural) {
+        } else if (accidental == Rosegarden::Accidentals::Natural) {
             // do we have anything explicit left to do in this
             // case?  probably not, but I'll leave this placeholder for now
             //
@@ -327,32 +335,42 @@ LilypondExporter::convertPitchToLilyNote(int pitch, Accidental accidental,
        } 
     }
 
-    // leave this in to see if there are any _other_ problems that are going
-    // to break this method...
-    if (lilyNote == "") {
+    // the failsafe's failsafe
+#ifndef DEBUG_PITCH
+    if (lilyNote == "") { 
         std::cerr << "LilypondExporter::convertPitchToLilyNote() -  WARNING: cannot resolve note"
+#else
+        std::cerr << "LilypondExporter::cPTLN() chewing on: "
+#endif	
                   << std::endl << "pitch = " << pitchNote << "\tkey sig. = "
                   << ((isFlatKeySignature) ? "flat" : "sharp") << "\tno. of accidentals = "
                   << accidentalCount << "\textra accidental = \"" << accidental << "\""
                   << std::endl;
-        
+#ifndef DEBUG_PITCH
+	m_pitchBorked = true;
     }
+#endif        
 
     //!!! Alternative implementation in test:
 
-    Rosegarden::Pitch p(pitch, accidental);
+    Rosegarden::Pitch p(pitch, accidental);  // pitch + accidental, but no key? (DMM)
     std::string origLilyNote = lilyNote;
     lilyNote = "";
 
     lilyNote += (char)tolower(p.getNoteName(key));
     Accidental acc = p.getAccidental(key.isSharp());
-    if      (acc == Accidentals::DoubleFlat)  lilyNote += "eses";
-    else if (acc == Accidentals::Flat)        lilyNote += "es";
-    else if (acc == Accidentals::Sharp)       lilyNote += "is";
-    else if (acc == Accidentals::DoubleSharp) lilyNote += "isis";
+    if      (acc == Rosegarden::Accidentals::DoubleFlat)  lilyNote += "eses";
+    else if (acc == Rosegarden::Accidentals::Flat)        lilyNote += "es";
+    else if (acc == Rosegarden::Accidentals::Sharp)       lilyNote += "is";
+    else if (acc == Rosegarden::Accidentals::DoubleSharp) lilyNote += "isis";
     
     if (lilyNote != origLilyNote) {
 	std::cerr << "WARNING: LilypondExporter::convertPitchToLilyNote: " << lilyNote << " != " << origLilyNote << std::endl;
+	lilyNote = origLilyNote;  // makes it return the correct value with my old, trusted code
+	                          // until I can fix this properly
+				  // (deferred until after 1.0 because users
+				  // don't care which version as long as it
+				  // works) - DMM
     }
 
     return lilyNote;
@@ -372,8 +390,8 @@ LilypondExporter::composeLilyMark(std::string eventMark, bool stemUp) {
     std::string prefix = (stemUp) ? "_" : "^";
     
     // shoot text mark straight through unless it's sf or rf
-    if (Marks::isTextMark(eventMark)) {
-        inStr = protectIllegalChars(Marks::getTextFromMark(eventMark));
+    if (Rosegarden::Marks::isTextMark(eventMark)) {
+        inStr = protectIllegalChars(Rosegarden::Marks::getTextFromMark(eventMark));
         
         if (inStr == "sf") {
             inStr = "\\sf";
@@ -386,12 +404,12 @@ LilypondExporter::composeLilyMark(std::string eventMark, bool stemUp) {
 
         outStr = prefix + inStr;
 
-    } else if (Marks::isFingeringMark(eventMark)) {
+    } else if (Rosegarden::Marks::isFingeringMark(eventMark)) {
 	
 	// treat fingering marks like text marks for the moment,
 	// though they probably shouldn't be
 
-        inStr = protectIllegalChars(Marks::getFingeringFromMark(eventMark));
+        inStr = protectIllegalChars(Rosegarden::Marks::getFingeringFromMark(eventMark));
 	
 	// FIX 1.X / 2.X VERSION DIFFERENCE HERE
 	inStr = "#'(italic \"" + inStr + "\")";
@@ -403,25 +421,25 @@ LilypondExporter::composeLilyMark(std::string eventMark, bool stemUp) {
 
         // use full \accent format for everything, even though some shortcuts
         // exist, for the sake of consistency
-        if (eventMark == Marks::Accent) {
+        if (eventMark == Rosegarden::Marks::Accent) {
             outStr += "\\accent";
-        } else if (eventMark == Marks::Tenuto) {
+        } else if (eventMark == Rosegarden::Marks::Tenuto) {
             outStr += "\\tenuto";
-        } else if (eventMark == Marks::Staccato) {
+        } else if (eventMark == Rosegarden::Marks::Staccato) {
             outStr += "\\staccato";
-        } else if (eventMark == Marks::Staccatissimo) {
+        } else if (eventMark == Rosegarden::Marks::Staccatissimo) {
             outStr += "\\staccatissimo";
-        } else if (eventMark == Marks::Marcato) {
+        } else if (eventMark == Rosegarden::Marks::Marcato) {
             outStr += "\\marcato";
-        } else if (eventMark == Marks::Trill) {
+        } else if (eventMark == Rosegarden::Marks::Trill) {
             outStr += "\\trill";
-        } else if (eventMark == Marks::Turn) {
+        } else if (eventMark == Rosegarden::Marks::Turn) {
             outStr += "\\turn";
-        } else if (eventMark == Marks::Pause) {
+        } else if (eventMark == Rosegarden::Marks::Pause) {
             outStr += "\\fermata";
-        } else if (eventMark == Marks::UpBow) {
+        } else if (eventMark == Rosegarden::Marks::UpBow) {
             outStr += "\\upbow";
-        } else if (eventMark == Marks::DownBow) {
+        } else if (eventMark == Rosegarden::Marks::DownBow) {
             outStr += "\\downbow";
         } else {
             outStr = "";
@@ -872,6 +890,18 @@ LilypondExporter::write()
 	str << indent(col) << "\\tempo 4 = " << tempo << std::endl;
         str << indent(--col) << "} " << std::endl;
     }    
+   
+    // DMM - a friendly userland warning if they happen to tease out any lingering
+    // trouble with this stuff;  since users don't usually see the debug messages
+    if (m_pitchBorked) {
+	CurrentProgressDialog::freeze();
+	KMessageBox::sorry(0,
+		i18n("You encountered an enharmonic resolution bug somewhere during this "
+		     "operation.  You may wish to save a copy of your current composition "
+		     "and email it to dmmcintyr@users.sourceforge.net for analysis.\n\n"
+		     "In the meantime, you will probably need to make manual adjustments "
+		     "to the file you've just exported."));
+    }	
 
     // close \score section and close out the file
     str << "} % score" << std::endl;
@@ -1368,6 +1398,9 @@ LilypondExporter::handleText(const Rosegarden::Event *textEvent,
 			     int languageLevel)
 {
     try {
+	
+	//!! DMM -  this needs to deal with the new Text::Chord type
+	 
 	Rosegarden::Text text(*textEvent);
 	std::string s = protectIllegalChars(text.getText());
 	
@@ -1410,13 +1443,16 @@ LilypondExporter::handleText(const Rosegarden::Event *textEvent,
 
 	} else if (text.getTextType() == Text::Direction) {
 
-	    lilyText += " \\mark \"" + s + "\"";
+	    // \mark is syntactically a different thing from the
+	    // others here, they don't mix
+//	    lilyText += " \\mark \"" + s + "\" ";
+	    lilyText += "^\\markup { " + s + " } ";
 
 	} else if (text.getTextType() == Text::LocalDirection) {
 
 	    // print below staff, bold italics, small
 	    if (languageLevel < 1) {
-		lilyText += "_#'((bold italic) \"" + s + "\")";
+		lilyText += "_#'((bold italic) \"" + s + "\") ";
 	    } else {
 		lilyText += "_\\markup { \\bold \\italic " + s + " } ";
 	    }
