@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -24,8 +24,11 @@
 #include "rosestrings.h"
 #include "rosedebug.h"
 #include "SnapGrid.h"
-#include "qcanvas.h"
-#include "qcolor.h"
+#include <qcanvas.h>
+#include <qcolor.h>
+#include <qpainter.h>
+
+#include <algorithm>
 
 #include "LayoutEngine.h"
 #include "Profiler.h"
@@ -34,6 +37,168 @@
 //
 const int pointerWidth = 3;
 
+class BarLine : public QCanvasPolygonalItem
+{
+public:
+    BarLine(QCanvas *canvas, double layoutX,
+	    int barLineHeight, int baseBarThickness, int lineSpacing,
+	    int inset, LinedStaff::BarStyle style) :
+	QCanvasPolygonalItem(canvas),
+	m_layoutX(layoutX),
+	m_barLineHeight(barLineHeight),
+	m_baseBarThickness(baseBarThickness),
+	m_lineSpacing(lineSpacing),
+	m_inset(inset),
+	m_style(style) { }
+
+    double getLayoutX() const { return m_layoutX; }
+    
+    virtual QPointArray areaPoints() const;
+    virtual void drawShape(QPainter &);
+
+protected:
+    double m_layoutX;
+    int m_barLineHeight;
+    int m_baseBarThickness;
+    int m_lineSpacing;
+    int m_inset;
+    LinedStaff::BarStyle m_style;
+};
+
+
+void
+BarLine::drawShape(QPainter &painter)
+{
+    int bx = int(x());
+    int by = int(y());
+
+    switch (m_style) {
+
+    case LinedStaff::PlainBar:
+	painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	break;
+
+    case LinedStaff::DoubleBar:
+	painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 3, by,
+			 m_baseBarThickness, m_barLineHeight);
+	break;
+
+    case LinedStaff::HeavyDoubleBar:
+	bx -= m_baseBarThickness * 5;
+	painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 3, by,
+			 m_baseBarThickness * 3, m_barLineHeight);
+	break;
+
+    case LinedStaff::RepeatEndBar:
+	bx -= m_baseBarThickness * 5 + m_lineSpacing * 2 / 3;
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 - (m_lineSpacing * 2 / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 + (m_lineSpacing / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	bx += m_lineSpacing * 2 / 3;
+	painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 3, by,
+			 m_baseBarThickness * 3, m_barLineHeight);
+	break;
+
+    case LinedStaff::RepeatStartBar:
+
+	if (m_inset > 0) {
+	    painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	    bx += m_inset;
+	}
+
+	painter.drawRect(bx, by, m_baseBarThickness * 3, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 5, by,
+			 m_baseBarThickness, m_barLineHeight);
+	bx += m_baseBarThickness * 6 + (m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 - (m_lineSpacing * 2 / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 + (m_lineSpacing / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	break;
+
+    case LinedStaff::RepeatBothBar:
+
+	if (m_inset > 0) {
+	    painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	    bx += m_inset;
+	}
+
+	bx -= m_baseBarThickness * 4 + m_lineSpacing * 2 / 3;
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 - (m_lineSpacing * 2 / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 + (m_lineSpacing / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	bx += m_lineSpacing * 2 / 3;
+	painter.drawRect(bx, by, m_baseBarThickness, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 3, by,
+			 m_baseBarThickness * 3, m_barLineHeight);
+	painter.drawRect(bx + m_baseBarThickness * 8, by,
+			 m_baseBarThickness, m_barLineHeight);
+	bx += m_baseBarThickness * 9 + (m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 - (m_lineSpacing * 2 / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	painter.drawEllipse(bx, by + m_barLineHeight / 2 + (m_lineSpacing / 3),
+			    m_lineSpacing / 3, m_lineSpacing / 3);
+	
+	break;
+
+    case LinedStaff::NoVisibleBar:
+	break;
+    }
+}
+
+QPointArray
+BarLine::areaPoints() const
+{
+    int bx = int(x());
+    int by = int(y());
+    int x0 = bx, y0 = by, x1, y1 = by + m_barLineHeight;
+
+    switch (m_style) {
+
+    case LinedStaff::PlainBar:
+	x1 = x0 + m_baseBarThickness;
+	break;
+
+    case LinedStaff::DoubleBar:
+	x1 = x0 + m_baseBarThickness * 4;
+	break;
+
+    case LinedStaff::HeavyDoubleBar:
+	x0 -= m_baseBarThickness * 6;
+	x1 = bx;
+	break;
+
+    case LinedStaff::RepeatEndBar:
+	x0 -= m_baseBarThickness * 6 + m_lineSpacing * 2 / 3;
+	x1 = bx;
+	break;
+
+    case LinedStaff::RepeatStartBar:
+	x1 = x0 + m_baseBarThickness * 6 + m_lineSpacing * 2 / 3 + m_inset;
+	break;
+
+    case LinedStaff::RepeatBothBar:
+	x0 -= m_baseBarThickness * 4 + m_lineSpacing * 2 / 3;
+	x1 = x0 + m_baseBarThickness * 9 + m_lineSpacing * 2 / 3 + m_inset;
+	break;
+
+    case LinedStaff::NoVisibleBar:
+	x1 = x0 + 1;
+	break;
+    }
+
+    QPointArray p(4);
+    p[0] = QPoint(x0, y0);
+    p[1] = QPoint(x1, y0);
+    p[2] = QPoint(x1, y1);
+    p[3] = QPoint(x0, y1);
+    return p;
+}
 
 LinedStaff::LinedStaff(QCanvas *canvas, Rosegarden::Segment *segment,
                           Rosegarden::SnapGrid *snapGrid, int id,
@@ -121,23 +286,26 @@ LinedStaff::LinedStaff(QCanvas *canvas, Rosegarden::Segment *segment,
 
 LinedStaff::~LinedStaff()
 {
+/*!!! No, the canvas items are all deleted by the canvas on destruction.
+
     deleteBars();
     for (int i = 0; i < (int)m_staffLines.size(); ++i) clearStaffLineRow(i);
+*/
 }
 
 void
 LinedStaff::initCursors()
 {
-    QPen pen(RosegardenGUIColours::Pointer);
+    QPen pen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::Pointer));
     pen.setWidth(pointerWidth);
 
     m_pointer->setPen(pen);
-    m_pointer->setBrush(RosegardenGUIColours::Pointer);
+    m_pointer->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::Pointer));
 
-    pen.setColor(RosegardenGUIColours::InsertCursor);
+    pen.setColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::InsertCursor));
 
     m_insertCursor->setPen(pen);
-    m_insertCursor->setBrush(RosegardenGUIColours::InsertCursor);
+    m_insertCursor->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::InsertCursor));
 }
 
 void
@@ -401,20 +569,20 @@ LinedStaff::getBarExtents(double x, int y) const
 
     for (int i = 1; i < m_barLines.size(); ++i) {
 
-	double layoutX = m_barLines[i].first;
+	double layoutX = m_barLines[i]->getLayoutX();
         int barRow = getRowForLayoutX(layoutX);
 
         if (m_pageMode != LinearMode && (barRow < row)) continue;
 
-        QCanvasRectangle *line = (QCanvasRectangle *)m_barLines[i].second;
+        BarLine *line = m_barLines[i];
 
         if (line)
         {
             if (line->x() <= x) continue;
 
-            return QRect(int(m_barLines[i-1].second->x()),
+            return QRect(int(m_barLines[i-1]->x()),
                      getCanvasYForTopOfStaff(barRow),
-                     int(line->x() - m_barLines[i-1].second->x()),
+                     int(line->x() - m_barLines[i-1]->x()),
                      getHeightOfRow());
         }
     }
@@ -527,6 +695,7 @@ LinedStaff::sizeStaff(Rosegarden::HorizontalLayoutEngine &layout)
     Rosegarden::Profiler profiler("LinedStaff::sizeStaff", true);
 
     deleteBars();
+    deleteRepeatedClefsAndKeys();
     deleteTimeSignatures();
 
 //    RG_DEBUG << "LinedStaff::sizeStaff" << endl;
@@ -559,18 +728,19 @@ LinedStaff::sizeStaff(Rosegarden::HorizontalLayoutEngine &layout)
 	    insertTimeSignature(timeSigX, currentTimeSignature);
 	}
 
-//	RG_DEBUG << "LinedStaff::sizeStaff: inserting bar at " << x << " on staff " << this << endl;
+	RG_DEBUG << "LinedStaff::sizeStaff: inserting bar at " << x << " on staff " << this << " (isNew " << isNew << ", timeSigX " << timeSigX << ")" << endl;
 	
-	int displayBarNo = 0;
-	if (showBarNumbersEvery() > 0 &&
-	    ((barNo + 1) % showBarNumbersEvery()) == 0) displayBarNo = barNo + 1;
+	bool showBarNo = 
+	    (showBarNumbersEvery() > 0 &&
+	     ((barNo + 1) % showBarNumbersEvery()) == 0);
 
 	insertBar(x,
 		  ((barNo == lastBar) ? 0 :
 		   (layout.getBarPosition(barNo + 1) - x)),
 		  layout.isBarCorrectOnStaff(*this, barNo - 1),
 		  currentTimeSignature,
-		  displayBarNo);
+		  barNo,
+		  showBarNo);
     }
 
     m_startLayoutX = xleft;
@@ -585,21 +755,25 @@ LinedStaff::deleteBars()
 {
     for (BarLineList::iterator i = m_barLines.begin();
          i != m_barLines.end(); ++i) {
-        delete i->second;
+	(*i)->hide();
+        delete *i;
     }
 
-    for (BarLineList::iterator i = m_beatLines.begin();
+    for (LineRecList::iterator i = m_beatLines.begin();
 	 i != m_beatLines.end(); ++i) {
+	i->second->hide();
 	delete i->second;
     }
 
-    for (BarLineList::iterator i = m_barConnectingLines.begin();
+    for (LineRecList::iterator i = m_barConnectingLines.begin();
          i != m_barConnectingLines.end(); ++i) {
+	i->second->hide();
         delete i->second;
     }
 
     for (ItemList::iterator i = m_barNumbers.begin();
          i != m_barNumbers.end(); ++i) {
+	(*i)->hide();
         delete *i;
     }
 
@@ -611,7 +785,8 @@ LinedStaff::deleteBars()
 
 void
 LinedStaff::insertBar(double layoutX, double width, bool isCorrect,
-		      const Rosegarden::TimeSignature &timeSig, int barNo)
+		      const Rosegarden::TimeSignature &timeSig,
+		      int barNo, bool showBarNo)
 {
 //    RG_DEBUG << "insertBar: " << layoutX << ", " << width
 //			 << ", " << isCorrect << endl;
@@ -633,69 +808,101 @@ LinedStaff::insertBar(double layoutX, double width, bool isCorrect,
     double x = getCanvasXForLayoutX(layoutX);
     int y = getCanvasYForTopLine(row);
 
-    QCanvasRectangle *line = new QCanvasRectangle
-	(0, 0, barThickness, getBarLineHeight(), m_canvas);
-    line->moveBy(x, y);
+    bool firstBarInRow = false, lastBarInRow = false;
 
-    if (isCorrect) {
-	line->setPen(RosegardenGUIColours::BarLine);
-	line->setBrush(RosegardenGUIColours::BarLine);
-    } else {
-	line->setPen(RosegardenGUIColours::BarLineIncorrect);
-	line->setBrush(RosegardenGUIColours::BarLineIncorrect);
-    }
-
-    line->setZ(-1);
-    line->show();
-
-    // The bar lines have to be in order of layout-x (there's no
-    // such interesting stipulation for beat or connecting lines)
-    BarLine barLine(layoutX, line);
-    BarLineList::iterator insertPoint = lower_bound
-	(m_barLines.begin(), m_barLines.end(), barLine, compareBars);
-    m_barLines.insert(insertPoint, barLine);
+    if (m_pageMode != LinearMode &&
+	(getRowForLayoutX(layoutX) >
+	 getRowForLayoutX(layoutX - getMargin() - 2))) firstBarInRow = true;
 
     if (m_pageMode != LinearMode &&
 	width > 0.01 && // width == 0 for final bar in staff
 	(getRowForLayoutX(layoutX) <
-	 getRowForLayoutX(layoutX + width + getMargin() + 2))) {
+	 getRowForLayoutX(layoutX + width + getMargin() + 2))) lastBarInRow = true;
 
-	// end of this row
+    BarStyle style = getBarStyle(barNo);
 
-	double xe = x + width - barThickness;
+    if (style == RepeatBothBar && firstBarInRow) style = RepeatStartBar;
 
-	QCanvasRectangle *eline = new QCanvasRectangle
-	    (0, 0, barThickness, getBarLineHeight(), m_canvas);
-	eline->moveBy(xe, y);
+    if (firstBarInRow) insertRepeatedClefAndKey(layoutX, barNo);
 
-	eline->setPen(RosegardenGUIColours::BarLine);
-	eline->setBrush(RosegardenGUIColours::BarLine);
+    // If we're supposed to be hiding bar lines, we do just that --
+    // create them as normal, then hide them.  We can't simply not
+    // create them because we rely on this to find bar extents for
+    // things like double-click selection in notation.
+    bool hidden = false;
+    if (style == PlainBar && timeSig.hasHiddenBars()) hidden = true;
 
-	eline->setZ(-1);
-	eline->show();
-
-	BarLine barLine(layoutX, eline);
-	BarLineList::iterator insertPoint = lower_bound
-	    (m_barLines.begin(), m_barLines.end(), barLine, compareBars);
-	m_barLines.insert(insertPoint, barLine);
+    double inset = 0.0;
+    if (style == RepeatStartBar || style == RepeatBothBar) {
+	inset = getBarInset(barNo, firstBarInRow);
     }
 
-    if (barNo > 0) {
+    BarLine *line = new BarLine(m_canvas, layoutX,
+				getBarLineHeight(), barThickness, getLineSpacing(),
+				(int)inset, style);
+
+    line->moveBy(x, y);
+
+    if (isCorrect) {
+	line->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLine));
+	line->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLine));
+    } else {
+	line->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLineIncorrect));
+	line->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLineIncorrect));
+    }
+
+    line->setZ(-1);
+    if (hidden) line->hide();
+    else line->show();
+
+    // The bar lines have to be in order of layout-x (there's no
+    // such interesting stipulation for beat or connecting lines)
+    BarLineList::iterator insertPoint = lower_bound
+	(m_barLines.begin(), m_barLines.end(), line, compareBars);
+    m_barLines.insert(insertPoint, line);
+
+    if (lastBarInRow) {
+
+	double xe = x + width - barThickness;
+	style = getBarStyle(barNo + 1);
+	if (style == RepeatBothBar) style = RepeatEndBar;
+
+	BarLine *eline = new BarLine(m_canvas, layoutX,
+				     getBarLineHeight(), barThickness, getLineSpacing(),
+				     0, style);
+	eline->moveBy(xe, y);
+
+	eline->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLine));
+	eline->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BarLine));
+
+	eline->setZ(-1);
+	if (hidden) eline->hide();
+	else eline->show();
+
+	BarLineList::iterator insertPoint = lower_bound
+	    (m_barLines.begin(), m_barLines.end(), eline, compareBars);
+	m_barLines.insert(insertPoint, eline);
+    }
+
+    if (showBarNo) {
 
 	QFont font;
 	font.setPixelSize(m_resolution * 3 / 2);
 	QFontMetrics metrics(font);
-	QString text = QString("%1").arg(barNo);
+	QString text = QString("%1").arg(barNo + 1);
 
 	QCanvasItem *barNoText = new QCanvasText(text, font, m_canvas);
 	barNoText->setX(x);
 	barNoText->setY(y - metrics.height() - m_resolution * 2);
 	barNoText->setZ(-1);
-	barNoText->show();
+	if (hidden) barNoText->hide();
+	else barNoText->show();
 
 	m_barNumbers.push_back(barNoText);
     }
 
+    QCanvasRectangle *rect = 0;
+
     if (showBeatLines()) {
 
 	double gridLines; // number of grid lines per bar may be fractional
@@ -710,61 +917,62 @@ LinedStaff::insertBar(double layoutX, double width, bool isCorrect,
 
 	double dx = width / gridLines;
 
-	for (int gridLine = 1; gridLine < gridLines; ++gridLine) {
+	for (int gridLine = hidden ? 0 : 1; gridLine < gridLines; ++gridLine) {
 
-	    line = new QCanvasRectangle
+	    rect = new QCanvasRectangle
 		(0, 0, barThickness, getBarLineHeight(), m_canvas);
 
-	    line->moveBy(x + gridLine * dx, y);
+	    rect->moveBy(x + gridLine * dx, y);
 
             double currentGrid = gridLines/double(timeSig.getBeatsPerBar());
 
-	    line->setPen(RosegardenGUIColours::BeatLine);
-	    line->setBrush(RosegardenGUIColours::BeatLine);
+	    rect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BeatLine));
+	    rect->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::BeatLine));
 
             // Reset to SubBeatLine colour if we're not a beat line - avoid div by zero!
             //
             if (currentGrid > 1.0 && double(gridLine)/currentGrid != gridLine/int(currentGrid))
             {
-	        line->setPen(RosegardenGUIColours::SubBeatLine);
-	        line->setBrush(RosegardenGUIColours::SubBeatLine);
+	        rect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SubBeatLine));
+	        rect->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SubBeatLine));
             }
 
-	    line->setZ(-1);
-	    line->show();
+	    rect->setZ(-1);
+	    rect->show(); // show beat lines even if the bar lines are hidden
 
-	    BarLine beatLine(layoutX + gridLine * dx, line);
+	    LineRec beatLine(layoutX + gridLine * dx, rect);
 	    m_beatLines.push_back(beatLine);
 	}
     }
     
     if (m_connectingLineLength > 0) {
 	
-	line = new QCanvasRectangle
+	rect = new QCanvasRectangle
 	    (0, 0, barThickness, m_connectingLineLength, m_canvas);
 
-	line->moveBy(x, y);
+	rect->moveBy(x, y);
 
-	line->setPen(RosegardenGUIColours::StaffConnectingLine);
-	line->setBrush(RosegardenGUIColours::StaffConnectingLine);
-	line->setZ(-3);
-	line->show();
+	rect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::StaffConnectingLine));
+	rect->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::StaffConnectingLine));
+	rect->setZ(-3);
+	if (hidden) rect->hide();
+	else rect->show();
 	
-	BarLine connectingLine(layoutX, line);
+	LineRec connectingLine(layoutX, rect);
 	m_barConnectingLines.push_back(connectingLine);
     }
 }
 
 bool
-LinedStaff::compareBars(const BarLine &barLine1, const BarLine &barLine2)
+LinedStaff::compareBars(const BarLine *barLine1, const BarLine *barLine2)
 {
-    return (barLine1.first < barLine2.first);
+    return (barLine1->getLayoutX() < barLine2->getLayoutX());
 }
 
 bool
-LinedStaff::compareBarToLayoutX(const BarLine &barLine1, int x)
+LinedStaff::compareBarToLayoutX(const BarLine *barLine1, int x)
 {
-    return (barLine1.first < x);
+    return (barLine1->getLayoutX() < x);
 }
 
 void
@@ -779,6 +987,18 @@ LinedStaff::insertTimeSignature(double, const Rosegarden::TimeSignature &)
     // default implementation is empty
 }
 
+void
+LinedStaff::deleteRepeatedClefsAndKeys()
+{
+    // default implementation is empty
+}
+
+void
+LinedStaff::insertRepeatedClefAndKey(double, int)
+{
+    // default implementation is empty
+}
+
 void
 LinedStaff::drawStaffName()
 {
@@ -867,7 +1087,7 @@ LinedStaff::resizeStaffLineRow(int row, double x, double length)
     // of the staff lines somewhat to make them less intrusive
 
     int level = 0;
-    int z = 1;
+    int z = 2;
     if (m_resolution < 6) {
         z = -1;
         level = (9 - m_resolution) * 32;
@@ -891,17 +1111,19 @@ LinedStaff::resizeStaffLineRow(int row, double x, double length)
 
     delete m_staffConnectingLines[row];
 
-    if (m_pageMode != LinearMode && m_connectingLineLength > 0.1) { //!!! handle multi page
+    if (m_pageMode != LinearMode && m_connectingLineLength > 0.1) {
+
 	// rather arbitrary (dup in insertBar)
 	int barThickness = m_resolution / 12 + 1;
         y = getCanvasYForTopLine(row);
 	QCanvasRectangle *line = new QCanvasRectangle
 	    (int(x + length), y, barThickness, m_connectingLineLength, m_canvas);
-	line->setPen(RosegardenGUIColours::StaffConnectingTerminatingLine);
-	line->setBrush(RosegardenGUIColours::StaffConnectingTerminatingLine);
+	line->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::StaffConnectingTerminatingLine));
+	line->setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::StaffConnectingTerminatingLine));
         line->setZ(-2);
         line->show();
 	m_staffConnectingLines[row] = line;
+
     } else {
 	m_staffConnectingLines[row] = 0;
     }
