@@ -5,7 +5,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -35,6 +35,7 @@ namespace Rosegarden { class SnapGrid; class HorizontalLayoutEngine; }
 class QCanvas;
 class QCanvasLine;
 class QCanvasItem;
+class BarLine;
 
 /**
  * LinedStaffManager is a trivial abstract base for classes that own
@@ -76,6 +77,16 @@ public:
 	MultiPageMode
     };
 
+    enum BarStyle {
+	PlainBar = 0,
+	DoubleBar,
+	HeavyDoubleBar,
+	RepeatEndBar,
+	RepeatStartBar,
+	RepeatBothBar,
+	NoVisibleBar
+    };
+
 protected:
     /**
      * Create a new LinedStaff for the given Segment, with a
@@ -196,6 +207,22 @@ protected:
 	return 0;
     }
 
+    /**
+     * Returns the bar line / repeat style for the start of the given bar.
+     */
+    virtual BarStyle getBarStyle(int /* barNo */) const {
+	return PlainBar;
+    }
+
+    /**
+     * Returns the distance the opening (repeat) bar is inset from the
+     * nominal barline position.  This is to accommodate the situation
+     * where a repeat bar has to appear after the clef and key.
+     */
+    virtual double getBarInset(int /* barNo */, bool /* isFirstBarInRow */) const {
+	return 0;
+    }
+
 protected:
     /// Subclass may wish to expose this
     virtual void setResolution(int resolution);
@@ -559,7 +586,10 @@ protected:
     // shows staff lines, connecting lines (where appropriate) and bar
     // lines, but does not show time signatures.  To see time
     // signatures, override the deleteTimeSignatures and
-    // insertTimeSignature methods.
+    // insertTimeSignature methods.  For repeated clefs and keys at
+    // the start of each row, override deleteRepeatedClefsAndKeys
+    // and insertRepeatedClefAndKey, but note that your layout class
+    // will need to allot the space for them separately.
 
     virtual void resizeStaffLines();
     virtual void clearStaffLineRow(int row);
@@ -568,13 +598,17 @@ protected:
     virtual void deleteBars();
     virtual void insertBar(double layoutX, double width, bool isCorrect,
 			   const Rosegarden::TimeSignature &,
-			   int barNo);
+			   int barNo, bool showBarNo);
 
     // The default implementations of the following two are empty.
     virtual void deleteTimeSignatures();
     virtual void insertTimeSignature(double layoutX,
 				     const Rosegarden::TimeSignature &);
 
+    // The default implementations of the following two are empty.
+    virtual void deleteRepeatedClefsAndKeys();
+    virtual void insertRepeatedClefAndKey(double layoutX, int barNo);
+
     // The default implementation of the following is empty.  The
     // subclass is presumed to know what the staff's name is and
     // where to put it; this is simply called at some point during
@@ -615,13 +649,14 @@ protected:
     ItemMatrix m_staffLines;
     ItemList m_staffConnectingLines;
 
-    typedef std::pair<double, QCanvasItem *> BarLine; // layout-x, line
-    typedef FastVector<BarLine> BarLineList;
-    static bool compareBars(const BarLine &, const BarLine &);
-    static bool compareBarToLayoutX(const BarLine &, int);
+    typedef std::pair<double, QCanvasItem *> LineRec; // layout-x, line
+    typedef FastVector<LineRec> LineRecList;
+    typedef FastVector<BarLine *> BarLineList;//!!! should be multiset I reckon
+    static bool compareBars(const BarLine *, const BarLine *);
+    static bool compareBarToLayoutX(const BarLine *, int);
     BarLineList m_barLines;
-    BarLineList m_beatLines;
-    BarLineList m_barConnectingLines;
+    LineRecList m_beatLines;
+    LineRecList m_barConnectingLines;
     ItemList m_barNumbers;
 
     QCanvasLine *m_pointer;
