@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -399,11 +399,13 @@ MarkerEditorDialog::setupActions()
                             KStdAction::stdName(KStdAction::Redo));
 
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
+    kapp->config()->setGroup(MarkerEditorConfigGroup);
     int timeMode = kapp->config()->readNumEntry("timemode", 0);
 
     KRadioAction *action;
 
-    QIconSet icon(QCanvasPixmap(pixmapDir + "/toolbar/time-musical.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/time-musical.xpm");
+    QIconSet icon(pixmap);
 
     action = new KRadioAction(i18n("&Musical Times"), icon, 0, this,
                               SLOT(slotMusicalTime()),
@@ -411,7 +413,8 @@ MarkerEditorDialog::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 0) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-real.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-real.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Real Times"), icon, 0, this,
                               SLOT(slotRealTime()),
@@ -419,7 +422,8 @@ MarkerEditorDialog::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 1) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-raw.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-raw.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("Ra&w Times"), icon, 0, this,
                               SLOT(slotRawTime()),
@@ -472,6 +476,12 @@ MarkerEditorDialog::slotEdit(QListViewItem *i)
 {
     RG_DEBUG << "MarkerEditorDialog::slotEdit" << endl;
 
+    if (m_listView->selectionMode() == QListView::NoSelection) {
+	// The marker list is empty, so we shouldn't allow editing the
+	// <none> placeholder
+	return;
+    }
+
     // Need to get the raw time from the ListViewItem
     //
     MarkerEditorViewItem *item = 
