@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -32,10 +32,8 @@
 #include <qlineedit.h>
 #include <qspinbox.h>
 
-#include "widgets.h" // or other RosegardenTimeWidget header
-
 #include "Composition.h"
-#include "Event.h" // for timeT definition
+#include "timewidget.h"
 
 class RosegardenGUIDoc;
 class KCommand;
@@ -61,7 +59,6 @@ public:
 protected:
     RosegardenGUIDoc             *m_doc;
 
-//    QSpinBox                     *m_timeEdit;
     RosegardenTimeWidget         *m_timeEdit;
     QLineEdit                    *m_nameEdit;
     QLineEdit                    *m_desEdit;
