@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -53,6 +53,10 @@ void MatrixInsertionCommand::modifySegment()
 {
     MATRIX_DEBUG << "MatrixInsertionCommand::modifySegment()\n";
 
+    if (!m_event->has(Rosegarden::BaseProperties::VELOCITY)) {
+	m_event->set<Rosegarden::Int>(Rosegarden::BaseProperties::VELOCITY, 100);
+    }
+
     Rosegarden::SegmentMatrixHelper helper(getSegment());
     m_lastInsertedEvent = new Event(*m_event);
     helper.insertNote(m_lastInsertedEvent);
@@ -92,7 +96,8 @@ void MatrixEraseCommand::modifySegment()
 MatrixModifyCommand::MatrixModifyCommand(Rosegarden::Segment &segment,
                                          Rosegarden::Event *oldEvent,
                                          Rosegarden::Event *newEvent,
-                                         bool isMove):
+                                         bool isMove,
+					 bool normalize):
       BasicCommand((isMove ? i18n("Move Note") : i18n("Modify Note")),
                    segment,
                    std::min(newEvent->getAbsoluteTime(),
@@ -102,8 +107,9 @@ MatrixModifyCommand::MatrixModifyCommand(Rosegarden::Segment &segment,
                             newEvent->getAbsoluteTime() +
                             newEvent->getDuration()),
                    true),
-    m_oldEvent(oldEvent),
-    m_newEvent(newEvent)
+      m_normalize(normalize),
+      m_oldEvent(oldEvent),
+      m_newEvent(newEvent)
 {
 }
 
@@ -124,6 +130,9 @@ void MatrixModifyCommand::modifySegment()
         Rosegarden::Segment &segment(getSegment());
         segment.insert(m_newEvent);
         segment.eraseSingle(m_oldEvent);
-        segment.normalizeRests(normalizeStart, normalizeEnd);
+
+	if (m_normalize) {
+	    segment.normalizeRests(normalizeStart, normalizeEnd);
+	}
     }
 }
