@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -69,12 +69,13 @@ public:
     MatrixModifyCommand(Rosegarden::Segment &segment,
                         Rosegarden::Event *oldEvent,
                         Rosegarden::Event *newEvent,
-                        bool isMove);
+                        bool isMove,
+			bool normalize);
 
 protected:
     virtual void modifySegment();
 
-    int m_newPitch;
+    bool m_normalize;
 
     Rosegarden::Event *m_oldEvent;
     Rosegarden::Event *m_newEvent;
