@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -51,7 +51,7 @@ void MatrixElement::setCanvas(QCanvas* c)
         //
         //m_canvasRect->setBrush(RosegardenGUIColours::MatrixElementBlock);
 
-        m_canvasRect->setPen(RosegardenGUIColours::MatrixElementBorder);
+        m_canvasRect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MatrixElementBorder));
         m_canvasRect->show();
     }
 }
