@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -108,8 +108,10 @@ void MatrixStaff::positionElement(Rosegarden::ViewElement* vel)
     Rosegarden::EventSelection *selection = m_view->getCurrentSelection();
 
     if (selection && selection->contains(el->event()))
-        el->setColour(RosegardenGUIColours::SelectedElement);
-    else
+        el->setColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SelectedElement));
+    else if (el->event()->has(Rosegarden::BaseProperties::TRIGGER_SEGMENT_ID))
+	el->setColour(Qt::gray);
+    else 
         el->setColour(DefaultVelocityColour::getInstance()->getColour(velocity));
 
     el->setCanvasX(coords.first);
