@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -23,6 +23,8 @@
 #include <klocale.h>
 #include <kaction.h>
 #include <kstddirs.h>
+#include <qapplication.h>
+#include <qtimer.h>
 
 #include "BaseProperties.h"
 #include "SegmentMatrixHelper.h"
@@ -32,7 +34,9 @@
 #include "matrixview.h"
 #include "matrixstaff.h"
 #include "matrixcommands.h"
+#include "notationcommands.h"
 #include "velocitycolour.h"
+#include "zoomslider.h"
 
 #include "rosestrings.h"
 #include "rosedebug.h"
@@ -163,7 +167,8 @@ MatrixPainter::MatrixPainter(MatrixView* parent)
       m_currentStaff(0)
 {
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/select.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/select.xpm");
+    QIconSet icon = QIconSet(pixmap);
 
     new KAction(i18n("Switch to Select Tool"), icon, 0, this,
                 SLOT(slotSelectSelected()), actionCollection(),
@@ -177,7 +182,8 @@ MatrixPainter::MatrixPainter(MatrixView* parent)
                 SLOT(slotMoveSelected()), actionCollection(),
                 "move");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/resize.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/resize.xpm");
+    icon = QIconSet(pixmap);
     new KAction(i18n("Switch to Resize Tool"), icon, 0, this,
                 SLOT(slotResizeSelected()), actionCollection(),
                 "resize");
@@ -259,7 +265,7 @@ int MatrixPainter::handleMouseMove(Rosegarden::timeT time,
                                    QMouseEvent *)
 {
     // sanity check
-    if (!m_currentElement) return NoFollow;
+    if (!m_currentElement) return RosegardenCanvasView::NoFollow;
 
     MATRIX_DEBUG << "MatrixPainter::handleMouseMove : pitch = "
                          << pitch << ", time : " << time << endl;
@@ -277,7 +283,8 @@ int MatrixPainter::handleMouseMove(Rosegarden::timeT time,
 
     m_currentElement->setWidth(int(width));
     
-    if (pitch != m_currentElement->event()->get<Rosegarden::Int>(PITCH)) {
+    if (m_currentElement->event()->has(PITCH) &&
+	pitch != m_currentElement->event()->get<Rosegarden::Int>(PITCH)) {
 	m_currentElement->event()->set<Rosegarden::Int>(PITCH, pitch);
 	int y = m_currentStaff->getLayoutYForHeight(pitch) -
 	    m_currentStaff->getElementHeight() / 2;
@@ -289,7 +296,7 @@ int MatrixPainter::handleMouseMove(Rosegarden::timeT time,
     }
     m_mParentView->update();
 
-    return FollowHorizontal;
+    return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
 }
 
 void MatrixPainter::handleMouseRelease(Rosegarden::timeT endTime,
@@ -333,6 +340,38 @@ void MatrixPainter::handleMouseRelease(Rosegarden::timeT endTime,
     m_currentElement = 0;
 }
 
+void MatrixPainter::ready()
+{
+    connect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixPainter::stow()
+{
+    disconnect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixPainter::slotMatrixScrolled(int newX, int newY) 
+{
+    if (!m_currentElement) return;
+
+    QPoint newP1(newX, newY), oldP1(m_parentView->getCanvasView()->contentsX(),
+                                    m_parentView->getCanvasView()->contentsY());
+
+    QPoint offset = newP1 - oldP1;
+
+    offset = m_mParentView->inverseMapPoint(offset);
+
+    QPoint p(m_currentElement->getCanvasX() + m_currentElement->getWidth(), m_currentElement->getCanvasY());
+    p += offset;
+    
+    timeT newTime = m_mParentView->getSnapGrid().snapX(p.x());
+    int newPitch = m_currentStaff->getHeightAtCanvasCoords(p.x(), p.y());
+
+    handleMouseMove(newTime, newPitch, 0);
+}
+
 //------------------------------
 
 MatrixEraser::MatrixEraser(MatrixView* parent)
@@ -340,7 +379,8 @@ MatrixEraser::MatrixEraser(MatrixView* parent)
       m_currentStaff(0)
 {
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/select.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/select.xpm");
+    QIconSet icon = QIconSet(pixmap);
 
     new KAction(i18n("Switch to Select Tool"), icon, 0, this,
                 SLOT(slotSelectSelected()), actionCollection(),
@@ -354,7 +394,8 @@ MatrixEraser::MatrixEraser(MatrixView* parent)
                 SLOT(slotMoveSelected()), actionCollection(),
                 "move");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/resize.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/resize.xpm");
+    icon = QIconSet(pixmap);
     new KAction(i18n("Switch to Resize Tool"), icon, 0, this,
                 SLOT(slotResizeSelected()), actionCollection(),
                 "resize");
@@ -400,6 +441,8 @@ MatrixSelector::MatrixSelector(MatrixView* view)
       m_currentStaff(0),
       m_clickedElement(0),
       m_dispatchTool(0),
+      m_justSelectedBar(false),
+      m_matrixView(view),
       m_selectionToMerge(0)
 {
     connect(m_parentView, SIGNAL(usedSelection()),
@@ -418,7 +461,8 @@ MatrixSelector::MatrixSelector(MatrixView* view)
                 "move");
 
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/resize.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/resize.xpm");
+    QIconSet icon = QIconSet(pixmap);
 
     new KAction(i18n("Switch to Resize Tool"), icon, 0, this,
                 SLOT(slotResizeSelected()), actionCollection(),
@@ -443,6 +487,11 @@ void MatrixSelector::handleEventRemoved(Rosegarden::Event *event)
     }
 }
 
+void MatrixSelector::slotClickTimeout()
+{
+    m_justSelectedBar = false;
+}
+
 void MatrixSelector::handleLeftButtonPress(Rosegarden::timeT time,
                                            int height,
                                            int staffNo,
@@ -451,6 +500,12 @@ void MatrixSelector::handleLeftButtonPress(Rosegarden::timeT time,
 {
     MATRIX_DEBUG << "MatrixSelector::handleMousePress" << endl;
 
+    if (m_justSelectedBar) {
+	handleMouseTripleClick(time, height, staffNo, e, element);
+	m_justSelectedBar = false;
+	return;
+    }
+
     QPoint p = m_mParentView->inverseMapPoint(e->pos());
 
     m_currentStaff = m_mParentView->getStaff(staffNo);
@@ -499,8 +554,31 @@ void MatrixSelector::handleLeftButtonPress(Rosegarden::timeT time,
     }
     else
     {
-        m_selectionRect->setX(p.x());
-        m_selectionRect->setY(p.y());
+	// Workaround for #930420 Positional error in sweep-selection box
+	// boundary
+	int zoomValue = (int)m_matrixView->m_hZoomSlider->getCurrentSize();
+	MatrixStaff *staff = m_mParentView->getStaff(staffNo);
+	int pitch = m_currentStaff->getHeightAtCanvasCoords(p.x(), p.y());
+	int pitchCentreHeight = staff->getTotalHeight() -
+				pitch * staff->getLineSpacing() - 2; // 2 or ?
+	int pitchLineHeight = pitchCentreHeight + staff->getLineSpacing()/2;
+	int drawHeight = p.y();
+	if (drawHeight <= pitchLineHeight + 1 &&
+	    drawHeight >= pitchLineHeight - 1) {
+	    if (drawHeight == pitchLineHeight)
+		drawHeight += 2;
+	    else
+		drawHeight += 2 * (drawHeight - pitchLineHeight);
+	}
+	MATRIX_DEBUG << "#### MatrixSelector::handleLeftButtonPress() : zoom "
+		<< zoomValue
+		<< " pitch " << pitch
+		<< " pitchCentreHeight " << pitchCentreHeight
+		<< " pitchLineHeight " << pitchLineHeight
+		<< " lineSpacing " << staff->getLineSpacing()
+		<< " drawHeight " << drawHeight << endl;
+        m_selectionRect->setX(int(p.x()/4)*4); // more workaround for #930420
+        m_selectionRect->setY(drawHeight);
         m_selectionRect->setSize(0,0);
 
         m_selectionRect->show();
@@ -540,7 +618,7 @@ void MatrixSelector::handleMidButtonPress(Rosegarden::timeT time,
 void MatrixSelector::handleMouseDoubleClick(Rosegarden::timeT ,
 					    int ,
 					    int staffNo,
-					    QMouseEvent* ,
+					    QMouseEvent *ev,
 					    Rosegarden::ViewElement *element)
 {
 /*
@@ -557,20 +635,97 @@ void MatrixSelector::handleMouseDoubleClick(Rosegarden::timeT ,
 
     if (m_clickedElement) {
 
-	EventEditDialog dialog(m_mParentView, *m_clickedElement->event(), true);
-
-	if (dialog.exec() == QDialog::Accepted &&
-	    dialog.isModified()) {
-
-	    EventEditCommand *command = new EventEditCommand
-		(staff->getSegment(),
-		 m_clickedElement->event(),
-		 dialog.getEvent());
-
-	    m_mParentView->addCommandToHistory(command);
+	if (m_clickedElement->event()->isa(Rosegarden::Note::EventType) &&
+	    m_clickedElement->event()->has(Rosegarden::BaseProperties::TRIGGER_SEGMENT_ID)) {
+	    
+	    int id = m_clickedElement->event()->get<Rosegarden::Int>
+		(Rosegarden::BaseProperties::TRIGGER_SEGMENT_ID);
+	    emit editTriggerSegment(id);
+	    return;
 	}
-    }
 
+	if (ev->state() & ShiftButton) { // advanced edit
+
+	    EventEditDialog dialog(m_mParentView, *m_clickedElement->event(), true);
+
+	    if (dialog.exec() == QDialog::Accepted &&
+		dialog.isModified()) {
+		
+		EventEditCommand *command = new EventEditCommand
+		    (staff->getSegment(),
+		     m_clickedElement->event(),
+		     dialog.getEvent());
+		
+		m_mParentView->addCommandToHistory(command);
+	    }
+	} else {
+
+	    SimpleEventEditDialog dialog(m_mParentView, m_mParentView->getDocument(),
+					 *m_clickedElement->event(), false);
+
+	    if (dialog.exec() == QDialog::Accepted &&
+		dialog.isModified()) {
+		
+		EventEditCommand *command = new EventEditCommand
+		    (staff->getSegment(),
+		     m_clickedElement->event(),
+		     dialog.getEvent());
+		
+		m_mParentView->addCommandToHistory(command);
+	    }
+	}	    
+
+    } /*  
+	  
+      #988167: Matrix:Multiclick select methods don't work in matrix editor
+      Postponing this, as it falls foul of world-matrix transformation
+      etiquette and other such niceties
+
+	  else {
+
+	QRect rect = staff->getBarExtents(ev->x(), ev->y());
+
+	m_selectionRect->setX(rect.x() + 2);
+	m_selectionRect->setY(rect.y());
+	m_selectionRect->setSize(rect.width() - 4, rect.height());
+
+	m_selectionRect->show();
+	m_updateRect = false;
+	
+	m_justSelectedBar = true;
+	QTimer::singleShot(QApplication::doubleClickInterval(), this,
+			   SLOT(slotClickTimeout()));
+    } */
+}
+
+void MatrixSelector::handleMouseTripleClick(Rosegarden::timeT t,
+					    int height,
+					    int staffNo,
+					    QMouseEvent *ev,
+					    Rosegarden::ViewElement *element)
+{
+    if (!m_justSelectedBar) return;
+    m_justSelectedBar = false;
+
+    MatrixStaff *staff = m_mParentView->getStaff(staffNo);
+    if (!staff) return;
+
+    if (m_clickedElement) {
+
+	// should be safe, as we've already set m_justSelectedBar false
+	handleLeftButtonPress(t, height, staffNo, ev, element);
+	return;
+
+    } else {
+
+	m_selectionRect->setX(staff->getX());
+	m_selectionRect->setY(staff->getY());
+	m_selectionRect->setSize(int(staff->getTotalWidth()) - 1,
+				 staff->getTotalHeight() - 1);
+
+	m_selectionRect->show();
+	m_updateRect = false;
+    }
 }
 
 int MatrixSelector::handleMouseMove(timeT time, int height,
@@ -583,7 +738,7 @@ int MatrixSelector::handleMouseMove(timeT time, int height,
         return m_dispatchTool->handleMouseMove(time, height, e);
     }
 
-    if (!m_updateRect) return NoFollow;
+    if (!m_updateRect) return RosegardenCanvasView::NoFollow;
 
     int w = int(p.x() - m_selectionRect->x());
     int h = int(p.y() - m_selectionRect->y());
@@ -592,11 +747,18 @@ int MatrixSelector::handleMouseMove(timeT time, int height,
     if (w > 0) ++w; else --w;
     if (h > 0) ++h; else --h;
 
-    m_selectionRect->setSize(w,h);
+    // Workaround for #930420 Positional error in sweep-selection box boundary
+    int wFix = (w > 0) ? 3 : 0;
+    int hFix = (h > 0) ? 3 : 0;
+    int xFix = (w < 0) ? 3 : 0;
+    m_selectionRect->setSize(w-wFix,h-hFix);
+    m_selectionRect->setX(m_selectionRect->x()+xFix);
     setViewCurrentSelection();
+    m_selectionRect->setSize(w,h);
+    m_selectionRect->setX(m_selectionRect->x()-xFix);
     m_mParentView->canvas()->update();
 
-    return FollowHorizontal | FollowVertical;
+    return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
 }
 
 void MatrixSelector::handleMouseRelease(timeT time, int height, QMouseEvent *e)
@@ -641,11 +803,15 @@ void MatrixSelector::ready()
     {
         m_selectionRect = new QCanvasRectangle(m_mParentView->canvas());
         m_selectionRect->hide();
-        m_selectionRect->setPen(RosegardenGUIColours::SelectionRectangle);
+        m_selectionRect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SelectionRectangle));
 
         m_mParentView->setCanvasCursor(Qt::arrowCursor);
         //m_mParentView->setPositionTracking(false);
     }
+
+    connect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotMatrixScrolled(int, int)));
+
 }
 
 void MatrixSelector::stow()
@@ -656,6 +822,10 @@ void MatrixSelector::stow()
         m_selectionRect = 0;
         m_mParentView->canvas()->update();
     }
+
+    disconnect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotMatrixScrolled(int, int)));
+
 }
 
 
@@ -667,6 +837,26 @@ void MatrixSelector::slotHideSelection()
     m_mParentView->canvas()->update();
 }
 
+void MatrixSelector::slotMatrixScrolled(int newX, int newY)
+{
+    if (m_updateRect) {
+        int offsetX = newX - m_parentView->getCanvasView()->contentsX();
+        int offsetY = newY - m_parentView->getCanvasView()->contentsY();
+
+        int w = int(m_selectionRect->width() + offsetX);
+        int h = int(m_selectionRect->height() + offsetY);
+
+        // Qt rectangle dimensions appear to be 1-based
+        if (w > 0) ++w; else --w;
+        if (h > 0) ++h; else --h;
+
+        m_selectionRect->setSize(w,h);
+        setViewCurrentSelection();
+        m_mParentView->canvas()->update();
+    }
+}
+
+
 void MatrixSelector::setViewCurrentSelection()
 {
     EventSelection* selection = getSelection();
@@ -725,7 +915,8 @@ MatrixMover::MatrixMover(MatrixView* parent)
       m_oldY(0)
 {
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/select.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/select.xpm");
+    QIconSet icon = QIconSet(pixmap);
 
     new KAction(i18n("Switch to Select Tool"), icon, 0, this,
                 SLOT(slotSelectSelected()), actionCollection(),
@@ -739,7 +930,8 @@ MatrixMover::MatrixMover(MatrixView* parent)
                 SLOT(slotEraseSelected()), actionCollection(),
                 "erase");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/resize.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/resize.xpm");
+    icon = QIconSet(pixmap);
     new KAction(i18n("Switch to Resize Tool"), icon, 0, this,
                 SLOT(slotResizeSelected()), actionCollection(),
                 "resize");
@@ -820,7 +1012,7 @@ int MatrixMover::handleMouseMove(Rosegarden::timeT newTime,
     MATRIX_DEBUG << "MatrixMover::handleMouseMove() time = "
                          << newTime << endl;
 
-    if (!m_currentElement || !m_currentStaff) return NoFollow;
+    if (!m_currentElement || !m_currentStaff) return RosegardenCanvasView::NoFollow;
 
     /*
     using Rosegarden::BaseProperties::PITCH;
@@ -921,7 +1113,7 @@ int MatrixMover::handleMouseMove(Rosegarden::timeT newTime,
     }
 
     m_mParentView->canvas()->update();
-    return FollowHorizontal | FollowVertical;
+    return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
 }
 
 void MatrixMover::handleMouseRelease(Rosegarden::timeT newTime,
@@ -963,8 +1155,11 @@ void MatrixMover::handleMouseRelease(Rosegarden::timeT newTime,
 
     using Rosegarden::BaseProperties::PITCH;
     timeT diffTime = newTime - m_currentElement->getViewAbsoluteTime();
-    int diffPitch = newPitch -
-        m_currentElement->event()->get<Rosegarden::Int>(PITCH);
+    int diffPitch = 0;
+    if (m_currentElement->event()->has(PITCH)) {
+	diffPitch = newPitch -
+	    m_currentElement->event()->get<Rosegarden::Int>(PITCH);
+    }
 
     if (diffTime == 0 && diffPitch == 0) { // don't generate command or refresh
 	m_mParentView->canvas()->update();
@@ -994,7 +1189,10 @@ void MatrixMover::handleMouseRelease(Rosegarden::timeT newTime,
         for (; it != selection->getSegmentEvents().end(); it++)
         {
             timeT newTime = (*it)->getAbsoluteTime() + diffTime;
-            int newPitch = (*it)->get<Rosegarden::Int>(PITCH) + diffPitch;
+            int newPitch = 60;
+	    if ((*it)->has(PITCH)) {
+		newPitch = (*it)->get<Rosegarden::Int>(PITCH) + diffPitch;
+	    }
             
             Rosegarden::Event *newEvent = new Rosegarden::Event(**it, newTime);
             newEvent->set<Rosegarden::Int>
@@ -1004,10 +1202,13 @@ void MatrixMover::handleMouseRelease(Rosegarden::timeT newTime,
                     new MatrixModifyCommand(m_currentStaff->getSegment(),
                                             (*it),
                                             newEvent,
-                                            true));
+                                            true,
+					    false));
             newSelection->addEvent(newEvent);
         }
 
+	macro->addCommand(new AdjustMenuNormalizeRestsCommand(*newSelection));
+
         m_mParentView->setCurrentSelection(0, false, false);
         m_mParentView->addCommandToHistory(macro);
         m_mParentView->setCurrentSelection(newSelection, false, false);
@@ -1018,6 +1219,39 @@ void MatrixMover::handleMouseRelease(Rosegarden::timeT newTime,
     m_currentElement = 0;
 }
 
+void MatrixMover::ready()
+{
+    connect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixMover::stow()
+{
+    disconnect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixMover::slotMatrixScrolled(int newX, int newY) 
+{
+    if (!m_currentElement) return;
+
+    QPoint newP1(newX, newY), oldP1(m_parentView->getCanvasView()->contentsX(),
+                                    m_parentView->getCanvasView()->contentsY());
+
+    QPoint offset = newP1 - oldP1;
+
+    offset = m_mParentView->inverseMapPoint(offset);
+
+    QPoint p(m_currentElement->getCanvasX(), m_currentElement->getCanvasY());
+    p += offset;
+    
+    timeT newTime = m_mParentView->getSnapGrid().snapX(p.x());
+    int newPitch = m_currentStaff->getHeightAtCanvasCoords(p.x(), p.y());
+
+    handleMouseMove(newTime, newPitch, 0);
+}
+
+
 //------------------------------
 MatrixResizer::MatrixResizer(MatrixView* parent)
     : MatrixTool("MatrixResizer", parent),
@@ -1025,7 +1259,8 @@ MatrixResizer::MatrixResizer(MatrixView* parent)
       m_currentStaff(0)
 {
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-    QIconSet icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/select.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/select.xpm");
+    QIconSet icon = QIconSet(pixmap);
 
     new KAction(i18n("Switch to Select Tool"), icon, 0, this,
                 SLOT(slotSelectSelected()), actionCollection(),
@@ -1109,7 +1344,7 @@ int MatrixResizer::handleMouseMove(Rosegarden::timeT newTime,
                                    int,
                                    QMouseEvent *)
 {
-    if (!m_currentElement || !m_currentStaff) return NoFollow;
+    if (!m_currentElement || !m_currentStaff) return RosegardenCanvasView::NoFollow;
     timeT newDuration = newTime - m_currentElement->getViewAbsoluteTime();
 
     int initialWidth = m_currentElement->getWidth();
@@ -1145,7 +1380,7 @@ int MatrixResizer::handleMouseMove(Rosegarden::timeT newTime,
     }
 
     m_mParentView->canvas()->update();
-    return FollowHorizontal;
+    return RosegardenCanvasView::FollowHorizontal;
 }
 
 void MatrixResizer::handleMouseRelease(Rosegarden::timeT newTime,
@@ -1208,11 +1443,14 @@ void MatrixResizer::handleMouseRelease(Rosegarden::timeT newTime,
                     new MatrixModifyCommand(m_currentStaff->getSegment(),
                                             *it,
                                             newEvent,
-                                            false));
+                                            false,
+					    false));
 
             newSelection->addEvent(newEvent);
         }
 
+	macro->addCommand(new AdjustMenuNormalizeRestsCommand(*newSelection));
+
         m_mParentView->setCurrentSelection(0, false, false);
         m_mParentView->addCommandToHistory(macro);
         m_mParentView->setCurrentSelection(newSelection, false, false);
@@ -1222,6 +1460,34 @@ void MatrixResizer::handleMouseRelease(Rosegarden::timeT newTime,
     m_currentElement = 0;
 }
 
+void MatrixResizer::ready()
+{
+    connect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixResizer::stow()
+{
+    disconnect(m_parentView->getCanvasView(), SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotMatrixScrolled(int, int)));
+}
+
+void MatrixResizer::slotMatrixScrolled(int newX, int newY) 
+{
+    QPoint newP1(newX, newY), oldP1(m_parentView->getCanvasView()->contentsX(),
+                                    m_parentView->getCanvasView()->contentsY());
+
+    QPoint p(newX, newY);
+    
+    if (newP1.x() > oldP1.x()) {
+        p.setX(newX + m_parentView->getCanvasView()->visibleWidth());
+    }
+
+    p = m_mParentView->inverseMapPoint(p);
+    int newTime = m_mParentView->getSnapGrid().snapX(p.x());
+    handleMouseMove(newTime, 0, 0);
+}
+
 //------------------------------
 
 const QString MatrixPainter::ToolName   = "painter";
