@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -118,6 +118,13 @@ public:
      */
     virtual void handleEventRemoved(Rosegarden::Event *event);
 
+    virtual void ready();
+    virtual void stow();
+
+protected slots:
+
+    void slotMatrixScrolled(int x, int y);
+
 protected:
     MatrixPainter(MatrixView*);
     MatrixPainter(QString name, MatrixView*);
@@ -183,7 +190,7 @@ public:
                                     QMouseEvent *event);
 
     /**
-     *
+     * Double-click: edit an event or make a whole-bar selection
      */
     virtual void handleMouseDoubleClick(Rosegarden::timeT time,
 					int height,
@@ -191,6 +198,15 @@ public:
 					QMouseEvent* event,
 					Rosegarden::ViewElement *element);
 
+    /**
+     * Triple-click: maybe make a whole-staff selection
+     */
+    virtual void handleMouseTripleClick(Rosegarden::timeT time,
+					int height,
+					int staffNo,
+					QMouseEvent* event,
+					Rosegarden::ViewElement *element);
+
 
     /**
      * Create the selection rect
@@ -231,8 +247,15 @@ public slots:
      */
     void slotHideSelection();
 
+    void slotClickTimeout();
+
+protected slots:
+
+    void slotMatrixScrolled(int x, int y);
+
 signals:
     void gotSelection(); // inform that we've got a new selection
+    void editTriggerSegment(int);
     
 protected:
     MatrixSelector(MatrixView*);
@@ -252,12 +275,18 @@ protected:
     // tool to delegate to
     EditTool*    m_dispatchTool;
 
+    bool m_justSelectedBar;
+
+    MatrixView * m_matrixView;
+
     Rosegarden::EventSelection *m_selectionToMerge;
 };
 
 
 class MatrixMover : public MatrixTool
 {
+    Q_OBJECT
+
     friend class MatrixToolBox;
 
 public:
@@ -289,6 +318,13 @@ public:
      */
     virtual void handleEventRemoved(Rosegarden::Event *event);
 
+    virtual void ready();
+    virtual void stow();
+
+protected slots:
+
+    void slotMatrixScrolled(int x, int y);
+
 protected:
     MatrixMover(MatrixView*);
 
@@ -304,6 +340,8 @@ protected:
 
 class MatrixResizer : public MatrixTool
 {
+    Q_OBJECT
+
     friend class MatrixToolBox;
 
 public:
@@ -335,6 +373,13 @@ public:
      */
     virtual void handleEventRemoved(Rosegarden::Event *event);
 
+    virtual void ready();
+    virtual void stow();
+
+protected slots:
+
+    void slotMatrixScrolled(int x, int y);
+
 protected:
     MatrixResizer(MatrixView*);
 
