@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -49,6 +49,7 @@
 #include "BaseProperties.h"
 #include "Profiler.h"
 #include "Property.h"
+#include "AudioLevel.h"
 
 #include "matrixview.h"
 #include "matrixstaff.h"
@@ -69,17 +70,20 @@
 #include "chordnameruler.h"
 #include "pianokeyboard.h"
 #include "editcommands.h"
+#include "notationcommands.h"
 #include "qdeferscrollview.h"
-#include "matrixparameterbox.h"
+#include "instrumentparameterbox.h"
 #include "velocitycolour.h"
 #include "widgets.h"
 #include "zoomslider.h"
+#include "rosegardengui.h"
 #include "notepixmapfactory.h"
 #include "controlruler.h"
 #include "studiocontrol.h"
 #include "Clipboard.h"
 #include "eventfilter.h"
 #include "MidiTypes.h"
+#include "tempoview.h"
 
 #include "rosedebug.h"
 
@@ -111,9 +115,10 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
       m_quantizations(Rosegarden::BasicQuantizer::getStandardQuantizations()),
       m_chordNameRuler(0),
       m_tempoRuler(0),
-      m_playTracking(true)
+      m_playTracking(true),
+      m_dockVisible(true)
 {
-    MATRIX_DEBUG << "MatrixView ctor\n";
+    RG_DEBUG << "MatrixView ctor\n";
 
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/toolbar");
     QPixmap matrixPixmap(pixmapDir + "/matrix.xpm");
@@ -151,6 +156,7 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
 	    tCanvas->setBackgroundPixmap(background);
 	}
     }
+
     m_config->setGroup(ConfigGroup);
 
     MATRIX_DEBUG << "MatrixView : creating staff\n";
@@ -177,9 +183,39 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
 
     m_grid->addWidget(m_pianoView, CANVASVIEW_ROW, 1);
 
-    m_parameterBox = new MatrixParameterBox(getDocument(), m_dockLeft, "matrixparameterbox");
+    m_parameterBox = new InstrumentParameterBox(getDocument(), m_dockLeft);
     m_dockLeft->setWidget(m_parameterBox);
 
+    RosegardenGUIApp *app = RosegardenGUIApp::self();
+    connect(app,
+	    SIGNAL(pluginSelected(Rosegarden::InstrumentId, int, int)),
+	    m_parameterBox,
+	    SLOT(slotPluginSelected(Rosegarden::InstrumentId, int, int)));
+    connect(app,
+	    SIGNAL(pluginBypassed(Rosegarden::InstrumentId, int, bool)),
+	    m_parameterBox,
+	    SLOT(slotPluginBypassed(Rosegarden::InstrumentId, int, bool)));
+    connect(app,
+	    SIGNAL(instrumentParametersChanged(Rosegarden::InstrumentId)),
+	    m_parameterBox,
+	    SLOT(slotInstrumentParametersChanged(Rosegarden::InstrumentId)));
+    connect(m_parameterBox,
+	    SIGNAL(instrumentParametersChanged(Rosegarden::InstrumentId)),
+	    app,
+	    SIGNAL(instrumentParametersChanged(Rosegarden::InstrumentId)));
+    connect(m_parameterBox,
+	    SIGNAL(selectPlugin(QWidget *, Rosegarden::InstrumentId, int)),
+	    app,
+	    SLOT(slotShowPluginDialog(QWidget *, Rosegarden::InstrumentId, int)));
+    connect(m_parameterBox,
+	    SIGNAL(showPluginGUI(Rosegarden::InstrumentId, int)),
+	    app,
+	    SLOT(slotShowPluginGUI(Rosegarden::InstrumentId, int)));
+    connect(parent, // RosegardenGUIView
+	    SIGNAL(checkTrackAssignments()),
+	    this,
+	    SLOT(slotCheckTrackAssignments()));
+    
     // Set the instrument we're using on this segment
     //
     Rosegarden::Track *track =
@@ -311,7 +347,12 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
 	 this, SLOT(slotSetInsertCursorPosition(Rosegarden::timeT)));
 
     topBarButtons->getLoopRuler()->setBackgroundColor
-	(RosegardenGUIColours::InsertCursorRuler);
+	(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::InsertCursorRuler));
+
+    connect(topBarButtons->getLoopRuler(), SIGNAL(startMouseMove(int)),
+            m_canvasView, SLOT(startAutoScroll(int)));
+    connect(topBarButtons->getLoopRuler(), SIGNAL(stopMouseMove()),
+            m_canvasView, SLOT(stopAutoScroll()));
 
     BarButtons *bottomBarButtons = new BarButtons(getDocument(),
                                                   &m_hlayout, 0, 25,
@@ -319,30 +360,17 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
     bottomBarButtons->connectRulerToDocPointer(doc);
     setBottomBarButtons(bottomBarButtons);
 
+    connect(bottomBarButtons->getLoopRuler(), SIGNAL(startMouseMove(int)),
+            m_canvasView, SLOT(startAutoScroll(int)));
+    connect(bottomBarButtons->getLoopRuler(), SIGNAL(stopMouseMove()),
+            m_canvasView, SLOT(stopAutoScroll()));
+
     // Force height for the moment
     //
     m_pianoKeyboard->setFixedHeight(canvas()->height());
 
 
-    // Set client label
-    //
-    if (segments.size() == 1) {
-
-        setCaption(QString("%1 - Segment Track #%2 - Matrix")
-                   .arg(doc->getTitle())
-                   .arg(segments[0]->getTrack()));
-
-    } else if (segments.size() == comp.getNbSegments()) {
-
-        setCaption(QString("%1 - All Segments - Matrix")
-                   .arg(doc->getTitle()));
-
-    } else {
-
-        setCaption(QString("%1 - %2 Segments - Matrix ")
-                   .arg(doc->getTitle())
-                   .arg(segments.size()));
-    }
+    updateViewCaption();
 
     // Add a velocity ruler
     //
@@ -357,6 +385,11 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
 	(&m_hlayout, doc, 0, 20, false, getCentralWidget());
     addRuler(m_tempoRuler);
 
+    // make tempo ruler double click editable
+    connect(m_tempoRuler,
+	    SIGNAL(doubleClicked(Rosegarden::timeT)),
+	    SLOT(slotEditTempos(Rosegarden::timeT)));
+
     // Scroll view to centre middle-C and warp to pointer position
     //
     m_canvasView->scrollBy(0, m_staffs[0]->getCanvasYForHeight(60) / 2);
@@ -387,6 +420,7 @@ MatrixView::MatrixView(RosegardenGUIDoc *doc,
     setupControllerTabs();
 
     setRewFFwdToAutoRepeat();
+    slotCompositionStateUpdate();
 }
 
 MatrixView::~MatrixView()
@@ -424,6 +458,8 @@ void MatrixView::slotSaveOptions()
 
     m_config->writeEntry("Show Chord Name Ruler", getToggleAction("show_chords_ruler")->isChecked());
     m_config->writeEntry("Show Tempo Ruler",      getToggleAction("show_tempo_ruler")->isChecked());
+    m_config->writeEntry("Show Parameters",       m_dockVisible);
+    //getToggleAction("m_dockLeft->isVisible());
 
     m_config->sync();
 }
@@ -442,6 +478,16 @@ void MatrixView::readOptions()
     opt = m_config->readBoolEntry("Show Tempo Ruler", false);
     getToggleAction("show_tempo_ruler")->setChecked(opt);
     slotToggleTempoRuler();
+
+    opt = m_config->readBoolEntry("Show Parameters", true);
+    if (!opt)
+    {
+        m_dockLeft->undock();
+        m_dockLeft->hide();
+        stateChanged("parametersbox_closed", KXMLGUIClient::StateNoReverse);
+        m_dockVisible = false;
+    }
+
 }
 
 void MatrixView::setupActions()
@@ -457,28 +503,29 @@ void MatrixView::setupActions()
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
     QIconSet icon(QPixmap(pixmapDir + "/toolbar/select.xpm"));
 
-    toolAction = new KRadioAction(i18n("&Select"), icon, Key_F1,
+    toolAction = new KRadioAction(i18n("&Select"), icon, Key_F2,
                                   this, SLOT(slotSelectSelected()),
                                   actionCollection(), "select");
     toolAction->setExclusiveGroup("tools");
 
-    toolAction = new KRadioAction(i18n("&Draw"), "pencil", Key_F2,
+    toolAction = new KRadioAction(i18n("&Draw"), "pencil", Key_F3,
                                   this, SLOT(slotPaintSelected()),
                                   actionCollection(), "draw");
     toolAction->setExclusiveGroup("tools");
 
-    toolAction = new KRadioAction(i18n("&Erase"), "eraser", Key_F3,
+    toolAction = new KRadioAction(i18n("&Erase"), "eraser", Key_F4,
                                   this, SLOT(slotEraseSelected()),
                                   actionCollection(), "erase");
     toolAction->setExclusiveGroup("tools");
 
-    toolAction = new KRadioAction(i18n("&Move"), "move", Key_F4,
+    toolAction = new KRadioAction(i18n("&Move"), "move", Key_F5,
                                   this, SLOT(slotMoveSelected()),
                                   actionCollection(), "move");
     toolAction->setExclusiveGroup("tools");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/resize.xpm"));
-    toolAction = new KRadioAction(i18n("Resi&ze"), icon, Key_F5,
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/resize.xpm");
+    icon = QIconSet(pixmap);
+    toolAction = new KRadioAction(i18n("Resi&ze"), icon, Key_F6,
                                   this, SLOT(slotResizeSelected()),
                                   actionCollection(), "resize");
     toolAction->setExclusiveGroup("tools");
@@ -489,12 +536,14 @@ void MatrixView::setupActions()
 		       actionCollection(), "chord_mode"))->
 	setChecked(false);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/step_by_step.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/step_by_step.xpm");
+    icon = QIconSet(pixmap);
     new KToggleAction(i18n("Ste&p Recording"), icon, 0, this,
                 SLOT(slotToggleStepByStep()), actionCollection(),
                 "toggle_step_by_step");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/quantize.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/quantize.xpm");
+    icon = QIconSet(pixmap);
     new KAction(EventQuantizeCommand::getGlobalName(), icon, Key_Equal, this,
                 SLOT(slotTransformsQuantize()), actionCollection(),
                 "quantize");
@@ -502,6 +551,10 @@ void MatrixView::setupActions()
     new KAction(i18n("Repeat Last Quantize"), Key_Plus, this,
                 SLOT(slotTransformsRepeatQuantize()), actionCollection(),
                 "repeat_quantize");
+    
+    new KAction(AdjustMenuCollapseNotesCommand::getGlobalName(), Key_Equal + CTRL, this,
+                SLOT(slotTransformsCollapseNotes()), actionCollection(),
+               "collapse_notes");
 
     new KAction(i18n("&Legato"), Key_Minus, this,
                 SLOT(slotTransformsLegato()), actionCollection(),
@@ -533,6 +586,14 @@ void MatrixView::setupActions()
                 SLOT(slotSetVelocities()), actionCollection(),
                 "set_velocities");
 
+    new KAction(i18n("Trigger Se&gment..."), 0, this,
+                SLOT(slotTriggerSegment()), actionCollection(),
+                "trigger_segment");
+
+    new KAction(i18n("Remove Triggers..."), 0, this,
+                SLOT(slotRemoveTriggers()), actionCollection(),
+                "remove_trigger");
+
     new KAction(i18n("Select &All"), Key_A + CTRL, this,
                 SLOT(slotSelectAll()), actionCollection(),
                 "select_all");
@@ -573,7 +634,9 @@ void MatrixView::setupActions()
 		SLOT(slotExtendSelectionForwardBar()), actionCollection(),
 		"extend_selection_forward_bar");
 
-    new KAction(i18n("Cursor to St&art"), 0, Key_A + CTRL, this,
+    new KAction(i18n("Cursor to St&art"), 0,
+		/* #1025717: conflicting meanings for ctrl+a - dupe with Select All
+		  Key_A + CTRL, */ this,
 		SLOT(slotJumpToStart()), actionCollection(),
 		"cursor_start");
 
@@ -725,14 +788,29 @@ void MatrixView::slotDockParametersBack()
 void MatrixView::slotParametersClosed()
 {
     stateChanged("parametersbox_closed");
+    m_dockVisible = false;
 }
 
 void MatrixView::slotParametersDockedBack(KDockWidget* dw, KDockWidget::DockPosition)
 {
     if (dw == m_dockLeft)
+    {
         stateChanged("parametersbox_closed", KXMLGUIClient::StateReverse);
+        m_dockVisible = true;
+    }
 }
 
+void MatrixView::slotCheckTrackAssignments()
+{
+    Rosegarden::Track *track =
+        m_staffs[0]->getSegment().getComposition()->
+	getTrackById(m_staffs[0]->getSegment().getTrack());
+
+    Rosegarden::Instrument *instr = getDocument()->getStudio().
+        getInstrumentById(track->getInstrument());
+
+    m_parameterBox->useInstrument(instr);
+}
 
 void MatrixView::initStatusBar()
 {
@@ -819,8 +897,14 @@ QSize MatrixView::getViewSize()
 
 void MatrixView::setViewSize(QSize s)
 {
+    MATRIX_DEBUG  << "MatrixView::setViewSize() w = " << s.width()
+                  << endl;
+
     canvas()->resize(s.width(), s.height());
     getCanvasView()->resizeContents(s.width(), s.height());
+
+    MATRIX_DEBUG  << "MatrixView::setViewSize() contentsWidth = " << getCanvasView()->contentsWidth()
+                  << endl;
 }
 
 void MatrixView::repaintRulers()
@@ -899,7 +983,10 @@ void MatrixView::setCurrentSelection(EventSelection* s, bool preview,
 		    long velocity = -1;
 		    (void)((*i)->get<Rosegarden::Int>
 			   (Rosegarden::BaseProperties::VELOCITY, velocity));
-		    playNote(s->getSegment(), pitch, velocity);
+		    if (!((*i)->has(Rosegarden::BaseProperties::TIED_BACKWARD)&&
+		          (*i)->get<Rosegarden::Bool>
+			           (Rosegarden::BaseProperties::TIED_BACKWARD)))
+		        playNote(s->getSegment(), pitch, velocity);
 		}
 	    }
 	}
@@ -1037,9 +1124,13 @@ void MatrixView::slotEraseSelected()
 void MatrixView::slotSelectSelected()
 {
     EditTool* selector = m_toolBox->getTool(MatrixSelector::ToolName);
+
     connect(selector, SIGNAL(gotSelection()),
             this, SLOT(slotNewSelection()));
 
+    connect(selector, SIGNAL(editTriggerSegment(int)),
+            this, SIGNAL(editTriggerSegment(int)));
+
     setTool(selector);
 }
 
@@ -1085,6 +1176,16 @@ void MatrixView::slotTransformsRepeatQuantize()
 			 "Quantize Dialog Grid", false)); // no i18n (config group name)
 }
 
+void MatrixView::slotTransformsCollapseNotes()
+{
+    if (!m_currentEventSelection) return;
+	KTmpStatusMsg msg(i18n("Collapsing notes..."), this);
+
+    addCommandToHistory(new AdjustMenuCollapseNotesCommand
+		        (*m_currentEventSelection));
+}
+
+
 void MatrixView::slotTransformsLegato()
 {
     using Rosegarden::Quantizer;
@@ -1159,6 +1260,10 @@ void MatrixView::slotMousePressed(Rosegarden::timeT time, int pitch,
 
     m_tool->handleMousePress(time, pitch, 0, e, el);
 
+    if (e->button() != RightButton) {
+	getCanvasView()->startAutoScroll();
+    }
+
     // play a preview
     //playPreview(pitch);
 }
@@ -1177,13 +1282,12 @@ void MatrixView::slotMouseMoved(Rosegarden::timeT time, int pitch, QMouseEvent*
     else 
     {
         int follow = m_tool->handleMouseMove(time, pitch, e);
+        getCanvasView()->setScrollDirectionConstraint(follow);
+        
+//        if (follow != RosegardenCanvasView::NoFollow) {
+//            getCanvasView()->doAutoScroll();
+//        }
         
-        if (follow & EditTool::FollowHorizontal)
-	    getCanvasView()->slotScrollHorizSmallSteps(e->pos().x());
-
-        if (follow & EditTool::FollowVertical)
-	    getCanvasView()->slotScrollVertSmallSteps(e->pos().y());
-	    
         // play a preview
         if (pitch != m_previousEvPitch)
         {
@@ -1210,6 +1314,7 @@ void MatrixView::slotMouseReleased(Rosegarden::timeT time, int pitch, QMouseEven
     // send the real event time now (not adjusted for beginning of bar)
     m_tool->handleMouseRelease(time, pitch, e);
     m_previousEvPitch = 0;
+    getCanvasView()->stopAutoScroll();
 }
 
 void
@@ -1265,14 +1370,28 @@ MatrixView::slotSetPointerPosition(timeT time, bool scroll)
     Rosegarden::Composition &comp = getDocument()->getComposition();
     int barNo = comp.getBarNumber(time);
 
-    if (barNo < m_hlayout.getFirstVisibleBarOnStaff(*m_staffs[0]) ||
-        barNo > m_hlayout. getLastVisibleBarOnStaff(*m_staffs[0])) {
-        m_staffs[0]->hidePointer();
+    if (barNo >= m_hlayout.getLastVisibleBarOnStaff(*m_staffs[0])) {
+
+	Segment &seg = m_staffs[0]->getSegment();
+
+	if (seg.isRepeating() && time < seg.getRepeatEndTime()) {
+	    time =
+		seg.getStartTime() +
+		((time - seg.getStartTime()) %
+		 (seg.getEndMarkerTime() - seg.getStartTime()));
+	    m_staffs[0]->setPointerPosition(m_hlayout, time);
+	} else {
+	    m_staffs[0]->hidePointer();
+	    scroll = false;
+	}
+    } else if (barNo < m_hlayout.getFirstVisibleBarOnStaff(*m_staffs[0])) {
+	m_staffs[0]->hidePointer();
+	scroll = false;
     } else {
         m_staffs[0]->setPointerPosition(m_hlayout, time);
     }
 
-    if (scroll)
+    if (scroll && !getCanvasView()->isAutoScrolling())
         getCanvasView()->slotScrollHoriz(static_cast<int>(getXbyWorldMatrix(m_hlayout.getXForTime(time))));
 
     updateView();
@@ -1286,7 +1405,7 @@ MatrixView::slotSetInsertCursorPosition(timeT time, bool scroll)
 
     m_staffs[0]->setInsertCursorPosition(m_hlayout, time);
 
-    if (scroll) {
+    if (scroll && !getCanvasView()->isAutoScrolling()) {
         getCanvasView()->slotScrollHoriz(static_cast<int>(getXbyWorldMatrix(m_hlayout.getXForTime(time))));
     }
 
@@ -1379,7 +1498,7 @@ void MatrixView::slotKeyPressed(unsigned int y, bool repeating)
     if (!repeating) m_firstNote = evPitch;
 
     Rosegarden::Track *track = comp.getTrackById(
-            m_staffs[0]->getSegment().getTrack());
+            staff.getSegment().getTrack());
 
     Rosegarden::Instrument *ins =
         studio.getInstrumentById(track->getInstrument());
@@ -1389,14 +1508,12 @@ void MatrixView::slotKeyPressed(unsigned int y, bool repeating)
     if (ins == 0)
         return;
 
-    // Send out note of half second duration
-    //
     Rosegarden::MappedEvent mE(ins->getId(),
                                Rosegarden::MappedEvent::MidiNoteOneShot,
-                               evPitch,
+                               evPitch + staff.getSegment().getTranspose(),
                                Rosegarden::MidiMaxValue,
                                Rosegarden::RealTime::zeroTime,
-                               Rosegarden::RealTime(0, 500000000),
+                               Rosegarden::RealTime(0, 250000000),
                                Rosegarden::RealTime::zeroTime);
     Rosegarden::StudioControl::sendMappedEvent(mE);
 
@@ -1460,14 +1577,12 @@ void MatrixView::slotKeySelected(unsigned int y, bool repeating)
     if (ins == 0)
         return;
 
-    // Send out note of half second duration
-    //
     Rosegarden::MappedEvent mE(ins->getId(),
                                Rosegarden::MappedEvent::MidiNoteOneShot,
-                               evPitch,
+                               evPitch + segment.getTranspose(),
                                Rosegarden::MidiMaxValue,
                                Rosegarden::RealTime::zeroTime,
-                               Rosegarden::RealTime(0, 500000000),
+                               Rosegarden::RealTime(0, 250000000),
                                Rosegarden::RealTime::zeroTime);
     Rosegarden::StudioControl::sendMappedEvent(mE);
 }
@@ -1486,17 +1601,22 @@ void MatrixView::slotInsertNoteFromAction()
 
     Segment &segment = *getCurrentSegment();
     int pitch = 0;
+
     Rosegarden::Accidental accidental = 
 	Rosegarden::Accidentals::NoAccidental;
 
+    Rosegarden::timeT time(getInsertionTime());
+    Rosegarden::Key key = segment.getKeyAtTime(time);
+    Rosegarden::Clef clef = segment.getClefAtTime(time);
+
     try {
 
-	pitch = getPitchFromNoteInsertAction(name, accidental);
+	pitch = getPitchFromNoteInsertAction(name, accidental, clef, key);
 
     } catch (...) {
 	
 	KMessageBox::sorry
-	    (this, QString(i18n("Unknown note insert action %1").arg(name)));
+	    (this, i18n("Unknown note insert action %1").arg(name));
 	return;
     }
 
@@ -1507,7 +1627,6 @@ void MatrixView::slotInsertNoteFromAction()
     Rosegarden::Event modelEvent(Rosegarden::Note::EventType, 0, 1);
     modelEvent.set<Rosegarden::Int>(Rosegarden::BaseProperties::PITCH, pitch);
     modelEvent.set<Rosegarden::String>(Rosegarden::BaseProperties::ACCIDENTAL, accidental);
-    Rosegarden::timeT time(getInsertionTime());
     Rosegarden::timeT endTime(time + m_snapGrid->getSnapTime(time));
 
     MatrixInsertionCommand* command = 
@@ -1586,7 +1705,8 @@ void MatrixView::playNote(Rosegarden::Event *event)
                                Rosegarden::MappedEvent::MidiNoteOneShot,
                                (Rosegarden::MidiByte)
                                event->get<Rosegarden::Int>
-                               (Rosegarden::BaseProperties::PITCH),
+                               (Rosegarden::BaseProperties::PITCH) +
+			       m_staffs[0]->getSegment().getTranspose(),
                                velocity,
                                Rosegarden::RealTime::zeroTime,
                                duration,
@@ -1614,14 +1734,12 @@ void MatrixView::playNote(const Rosegarden::Segment &segment, int pitch,
 
     if (velocity < 0) velocity = Rosegarden::MidiMaxValue;
 
-    // Send out note of half second duration
-    //
     Rosegarden::MappedEvent mE(ins->getId(),
                                Rosegarden::MappedEvent::MidiNoteOneShot,
-                               pitch,
+                               pitch + segment.getTranspose(),
                                velocity,
                                Rosegarden::RealTime::zeroTime,
-                               Rosegarden::RealTime(0, 500000000),
+                               Rosegarden::RealTime(0, 250000000),
                                Rosegarden::RealTime::zeroTime);
 
     Rosegarden::StudioControl::sendMappedEvent(mE);
@@ -1675,7 +1793,7 @@ MatrixView::slotNewSelection()
 {
     MATRIX_DEBUG << "MatrixView::slotNewSelection\n";
 
-    m_parameterBox->setSelection(m_currentEventSelection);
+//    m_parameterBox->setSelection(m_currentEventSelection);
 }
 
 
@@ -1835,6 +1953,10 @@ MatrixView::initActionsToolbar()
 	    QPixmap pixmap = NotePixmapFactory::toQPixmap(NotePixmapFactory::makeNoteMenuPixmap(m_snapValues[i], err));
 	    m_snapGridCombo->insertItem((err ? noMap : pixmap), label);
 	}
+
+	if (m_snapValues[i] == m_snapGrid->getSnapSetting()) {
+	    m_snapGridCombo->setCurrentItem(m_snapGridCombo->count() - 1);
+	}
     }
 
     connect(m_snapGridCombo, SIGNAL(activated(int)),
@@ -1888,6 +2010,8 @@ MatrixView::initZoomToolbar()
     for (unsigned int i = 0; i < sizeof(factors)/sizeof(factors[0]); ++i)
     {
 //         zoomSizes.push_back(duration44 / (defaultBarWidth44 * factors[i]));
+
+//         zoomSizes.push_back(factors[i] / 2); // GROSS HACK - see in matrixstaff.h - BREAKS MATRIX VIEW, see bug 1000595
         zoomSizes.push_back(factors[i]);
     }
 
@@ -1912,6 +2036,7 @@ MatrixView::slotChangeHorizontalZoom(int)
 {
     double zoomValue = m_hZoomSlider->getCurrentSize();
 
+//     m_zoomLabel->setText(i18n("%1%").arg(zoomValue*100.0 * 2)); // GROSS HACK - see in matrixstaff.h - BREAKS MATRIX VIEW, see bug 1000595
     m_zoomLabel->setText(i18n("%1%").arg(zoomValue*100.0));
 
     MATRIX_DEBUG << "MatrixView::slotChangeHorizontalZoom() : zoom factor = "
@@ -1944,10 +2069,9 @@ MatrixView::slotChangeHorizontalZoom(int)
     // and restore.
     //
 
-    int endX = int(m_hlayout.getXForTime(m_segments[0]->getEndMarkerTime()));
-    int startX = int(m_hlayout.getXForTime(m_segments[0]->getStartTime()));
+    int newWidth = computePostLayoutWidth();
 
-    int newWidth = int(getXbyWorldMatrix(endX - startX));
+//     int newWidth = int(getXbyWorldMatrix(getCanvasView()->canvas()->width()));
 
     // We DO NOT resize the canvas(), only the area it's displaying on
     //
@@ -2179,16 +2303,7 @@ MatrixView::readjustCanvasSize()
 
     }
 
-    Segment *segment = m_segments[0];
-    Rosegarden::Composition *composition = segment->getComposition();
-    int   endX = int(m_hlayout.getXForTime
-		     (composition->getBarEndForTime
-		      (segment->getEndMarkerTime())));
-    int startX = int(m_hlayout.getXForTime
-		     (composition->getBarStartForTime
-		      (segment->getStartTime())));
-
-    int newWidth = int(getXbyWorldMatrix(endX - startX));
+    int newWidth = computePostLayoutWidth();
 
     // now get the EditView to do the biz
     readjustViewSize(QSize(newWidth, maxHeight), true);
@@ -2259,6 +2374,33 @@ MatrixView::slotSetVelocities()
     }
 }
 
+void
+MatrixView::slotTriggerSegment()
+{
+    if (!m_currentEventSelection) return;
+    
+    TriggerSegmentDialog dialog(this, &getDocument()->getComposition());
+    if (dialog.exec() != QDialog::Accepted) return;
+
+    addCommandToHistory(new SetTriggerCommand(*m_currentEventSelection,
+					      dialog.getId(),
+					      true,
+					      dialog.getRetune(),
+					      dialog.getTimeAdjust(),
+					      Rosegarden::Marks::NoMark,
+					      i18n("Trigger Segment")));
+}
+
+void
+MatrixView::slotRemoveTriggers()
+{
+    if (!m_currentEventSelection) return;
+
+    addCommandToHistory(new ClearTriggersCommand(*m_currentEventSelection,
+						 i18n("Remove Triggers")));
+}
+
+
 void
 MatrixView::slotToggleChordsRuler()
 {
@@ -2271,6 +2413,24 @@ MatrixView::slotToggleTempoRuler()
     toggleWidget(m_tempoRuler, "show_tempo_ruler");
 }
 
+void
+MatrixView::slotEditTempos(Rosegarden::timeT t)
+{
+    TempoView *tempoView = new TempoView(getDocument(), this, t);
+
+    connect(tempoView,
+            SIGNAL(changeTempo(Rosegarden::timeT,
+                               double, TempoDialog::TempoDialogAction)),
+	    RosegardenGUIApp::self(),
+            SLOT(slotChangeTempo(Rosegarden::timeT,
+                                 double, TempoDialog::TempoDialogAction)));
+
+    connect(tempoView, SIGNAL(saveFile()),
+	    RosegardenGUIApp::self(), SLOT(slotFileSave()));
+
+    tempoView->show();
+}
+
 void
 MatrixView::paintEvent(QPaintEvent* e)
 {
@@ -2295,6 +2455,61 @@ MatrixView::paintEvent(QPaintEvent* e)
     }
 }
 
+void
+MatrixView::updateViewCaption()
+{
+    // Set client label
+    //
+    if (m_segments.size() == 1) {
+
+	Rosegarden::TrackId trackId = m_segments[0]->getTrack();
+	Rosegarden::Track *track =
+	    m_segments[0]->getComposition()->getTrackById(trackId);
+
+	int trackPosition = -1;
+	if (track) trackPosition = track->getPosition();
+
+        setCaption(i18n("%1 - Segment Track #%2 - Matrix")
+                   .arg(getDocument()->getTitle())
+                   .arg(trackPosition + 1));
+
+    } else if (m_segments.size() == getDocument()->getComposition().getNbSegments()) {
+
+        setCaption(i18n("%1 - All Segments - Matrix")
+                   .arg(getDocument()->getTitle()));
+
+    } else {
+
+        setCaption(i18n("%1 - %2 Segments - Matrix ")
+                   .arg(getDocument()->getTitle())
+                   .arg(m_segments.size()));
+    }
+}
+
+int MatrixView::computePostLayoutWidth()
+{
+    Segment *segment = m_segments[0];
+    Rosegarden::Composition *composition = segment->getComposition();
+    int   endX = int(m_hlayout.getXForTime
+		     (composition->getBarEndForTime
+		      (segment->getEndMarkerTime())));
+    int startX = int(m_hlayout.getXForTime
+		     (composition->getBarStartForTime
+		      (segment->getStartTime())));
+
+    int newWidth = int(getXbyWorldMatrix(endX - startX));
+
+    MATRIX_DEBUG << "MatrixView::readjustCanvasSize() : startX = " 
+                 << startX
+                 << " endX = " << endX
+                 << " newWidth = " << newWidth
+                 << " endmarkertime : " << segment->getEndMarkerTime()
+                 << " barEnd for time : " << composition->getBarEndForTime(segment->getEndMarkerTime())
+                 << endl;
+
+    return newWidth;
+}
+
 
 // Ignore velocity for the moment -- we need the option to use or ignore it
 void
@@ -2317,6 +2532,16 @@ MatrixView::slotInsertableNoteEventReceived(int pitch, int velocity, bool noteOn
 
     Segment &segment = *getCurrentSegment();
 
+    // If the segment is transposed, we want to take that into
+    // account.  But the note has already been played back to the user
+    // at its untransposed pitch, because that's done by the MIDI THRU
+    // code in the sequencer which has no way to know whether a note
+    // was intended for step recording.  So rather than adjust the
+    // pitch for playback according to the transpose setting, we have
+    // to adjust the stored pitch in the opposite direction.
+
+    pitch -= segment.getTranspose();
+
     KTmpStatusMsg msg(i18n("Inserting note"), this);
 
     MATRIX_DEBUG << "Inserting note at pitch " << pitch << endl;
@@ -2330,6 +2555,15 @@ MatrixView::slotInsertableNoteEventReceived(int pitch, int velocity, bool noteOn
     }
     Rosegarden::timeT endTime(time + m_snapGrid->getSnapTime(time));
 
+    if (endTime <= time) {
+	static bool showingError = false;
+	if (showingError) return;
+	showingError = true;
+	KMessageBox::sorry(this, i18n("Can't insert note: No grid duration selected"));
+	showingError = false;
+        return;
+    }
+
     MatrixInsertionCommand* command = 
 	new MatrixInsertionCommand(segment, time, endTime, &modelEvent);
 
@@ -2395,6 +2629,30 @@ MatrixView::slotStepByStepTargetRequested(QObject *obj)
     action->setChecked(obj == this);
 }
 
+void
+MatrixView::slotInstrumentLevelsChanged(Rosegarden::InstrumentId id,
+					const Rosegarden::LevelInfo &info)
+{
+    if (!m_parameterBox) return;
+
+    Rosegarden::Composition &comp = getDocument()->getComposition();
+
+    Rosegarden::Track *track =
+        comp.getTrackById(m_staffs[0]->getSegment().getTrack());
+    if (!track || track->getInstrument() != id) return;
+
+    Rosegarden::Instrument *instr = getDocument()->getStudio().
+        getInstrumentById(track->getInstrument());
+    if (!instr || instr->getType() != Rosegarden::Instrument::SoftSynth) return;
+
+    float dBleft = Rosegarden::AudioLevel::fader_to_dB
+	(info.level, 127, Rosegarden::AudioLevel::LongFader);
+    float dBright = Rosegarden::AudioLevel::fader_to_dB
+	(info.levelRight, 127, Rosegarden::AudioLevel::LongFader);
+    
+    m_parameterBox->setAudioMeter(dBleft, dBright);
+}
+
 void
 MatrixView::slotCanvasBottomWidgetHeightChanged(int newHeight)
 {
