@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -75,7 +75,8 @@ void MatrixVLayout::scanStaff(Rosegarden::Staff& staffBase,
 
         if (!el->isNote()) continue; // notes only
         
-        int pitch = el->event()->get<Rosegarden::Int>(PITCH);
+	long pitch = 60;
+	el->event()->get<Rosegarden::Int>(PITCH, pitch);
 
 	int y = staff.getLayoutYForHeight(pitch) - staff.getElementHeight()/2;
 
