@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -27,6 +27,8 @@
 
 #include <qdir.h>
 #include <kstddirs.h>
+#include <kconfig.h>
+#include <algorithm>
 
 #include "mmapper.h"
 
@@ -36,11 +38,14 @@
 #include "ControlBlock.h"
 #include "MappedEvent.h"
 #include "SegmentPerformanceHelper.h"
+#include "BaseProperties.h"
+#include "Midi.h"
 
 #include "rosestrings.h"
 #include "rosegardenguidoc.h"
 #include "rosedebug.h"
 #include "rgapplication.h"
+#include "constants.h"
 
 // Seems not to be properly defined under some gcc 2.95 setups
 #ifndef MREMAP_MAYMOVE
@@ -53,6 +58,9 @@ using Rosegarden::Segment;
 using Rosegarden::Composition;
 using Rosegarden::MappedEvent;
 using Rosegarden::timeT;
+using Rosegarden::Int;
+using Rosegarden::Bool;
+using namespace Rosegarden::BaseProperties;
 
 ControlBlockMmapper::ControlBlockMmapper(RosegardenGUIDoc* doc)
     : m_doc(doc),
@@ -257,6 +265,13 @@ size_t SegmentMmapper::computeMmappedSize()
 }
 
 
+size_t SegmentMmapper::addMmappedSize(Segment *s)
+{
+    int repeatCount = getSegmentRepeatCount();
+    return m_mmappedSize + (repeatCount + 1) * s->size() * sizeof(MappedEvent);
+}
+
+
 SegmentMmapper::~SegmentMmapper()
 {
     SEQMAN_DEBUG << "~SegmentMmapper : " << this
@@ -432,8 +447,6 @@ void SegmentMmapper::dump()
     Rosegarden::RealTime duration;
     Rosegarden::Track* track = comp.getTrackById(m_segment->getTrack());
     
-    Rosegarden::SegmentPerformanceHelper helper(*m_segment);
-
     timeT segmentStartTime = m_segment->getStartTime();
     timeT segmentEndTime = m_segment->getEndMarkerTime();
     timeT segmentDuration = segmentEndTime - segmentStartTime;
@@ -447,57 +460,130 @@ void SegmentMmapper::dump()
 
     for (int repeatNo = 0; repeatNo <= repeatCount; ++repeatNo) {
 
+	Segment *triggered = 0;
+	Segment::iterator *i = 0;
+
         for (Segment::iterator j = m_segment->begin();
-             j != m_segment->end(); ++j) {
+	     m_segment->isBeforeEndMarker(j) || (i && *i != triggered->end()); ) {
 
-            // Skip rests
-            //
-            if ((*j)->isa(Rosegarden::Note::EventRestType)) continue;
+	    bool usingi = false;
+	    Segment::iterator *k = &j;
 
-            timeT playTime =
-                helper.getSoundingAbsoluteTime(j) + repeatNo * segmentDuration;
-            if (playTime >= repeatEndTime) break;
+	    if (i && *i != triggered->end() &&
+		(!m_segment->isBeforeEndMarker(j) ||
+		 (**i)->getAbsoluteTime() < (*j)->getAbsoluteTime())) {
+		k = i;
+		usingi = true;
+	    }
 
-	    timeT playDuration = helper.getSoundingDuration(j);
+	    if (!usingi) { // don't permit nested triggered segments
 
-            // No duration and we're a note?  Probably in a tied
-            // series, but not as first note
-            //
-            if (playDuration == 0 && (*j)->isa(Rosegarden::Note::EventType))
-                continue;
-                
-	    if (playTime + playDuration > repeatEndTime)
-		playDuration = repeatEndTime - playTime;
+		long triggerId = -1;
+		(**k)->get<Int>(TRIGGER_SEGMENT_ID, triggerId);
 
-	    playTime = playTime + m_segment->getDelay();
-            eventTime = comp.getElapsedRealTime(playTime);
+		if (triggerId >= 0) {
 
-	    // slightly quicker than calling helper.getRealSoundingDuration()
-	    duration =
-		comp.getElapsedRealTime(playTime + playDuration) - eventTime;
+		    Rosegarden::TriggerSegmentRec *rec =
+			comp.getTriggerSegmentRec(triggerId);
 
-	    eventTime = eventTime + m_segment->getRealTimeDelay();
+		    if (rec && rec->getSegment()) {
+			Rosegarden::timeT performanceDuration =
+			    Rosegarden::SegmentPerformanceHelper(*m_segment).
+			    getSoundingDuration(j);
+			if (performanceDuration > 0) {
+			    mergeTriggerSegment(&triggered, *j,
+						performanceDuration, rec);
+			    size_t sz = addMmappedSize(rec->getSegment());
+			    size_t offset = bufPos - m_mmappedEventBuffer;
+			    setFileSize(sz);
+			    remap(sz);
+			    bufPos = m_mmappedEventBuffer + offset;
+			}
+		    }
 
-            try {
-                // Create mapped event in mmapped buffer
-                MappedEvent *mE = new (bufPos) MappedEvent(0, // the instrument will be extracted from the ControlBlock by the sequencer
-                                                           **j,
-                                                           eventTime,
-                                                           duration);
-                mE->setTrackId(track->getId());
+		    if (triggered) {
+			if (i) delete i;
+			i = new Segment::iterator
+			    (triggered->findTime((*j)->getAbsoluteTime()));
+		    }
 
-		if (m_segment->getTranspose() != 0 &&
-		    (*j)->isa(Rosegarden::Note::EventType)) {
-		    mE->setPitch(mE->getPitch() + m_segment->getTranspose());
+		    // Use the next triggered event (presumably the
+		    // first of the current triggered segment) instead
+		    // of the one that triggered it
+
+		    ++j; // whatever happens, we don't want to write this one
+
+		    if (i && *i != triggered->end() &&
+			(!m_segment->isBeforeEndMarker(j) ||
+			 ((**i)->getAbsoluteTime() < (*j)->getAbsoluteTime()))) {
+			k = i;
+			usingi = true;
+		    } else {
+			// no joy at all
+			continue; 
+		    }
 		}
+	    }
 
-                ++bufPos;
+	    // Ignore rests
+	    //
+            if (!(**k)->isa(Rosegarden::Note::EventRestType)) {
 
-            } catch(...) {
-                SEQMAN_DEBUG << "SegmentMmapper::dump - caught exception while trying to create MappedEvent\n";
-            }
+		Rosegarden::SegmentPerformanceHelper helper
+		    (usingi ? *triggered : *m_segment);
+
+		timeT playTime =
+		    helper.getSoundingAbsoluteTime(*k) + repeatNo * segmentDuration;
+		if (playTime >= repeatEndTime) break;
+
+		timeT playDuration = helper.getSoundingDuration(*k);
+
+		// Ignore notes without duration -- they're probably in a tied
+		// series but not as first note
+		//
+		if (playDuration > 0 || !(**k)->isa(Rosegarden::Note::EventType)) {
+                
+		    if (playTime + playDuration > repeatEndTime)
+			playDuration = repeatEndTime - playTime;
+
+		    playTime = playTime + m_segment->getDelay();
+		    eventTime = comp.getElapsedRealTime(playTime);
+
+		    // slightly quicker than calling helper.getRealSoundingDuration()
+		    duration =
+			comp.getElapsedRealTime(playTime + playDuration) - eventTime;
+
+		    eventTime = eventTime + m_segment->getRealTimeDelay();
+
+		    try {
+			// Create mapped event in mmapped buffer.  The
+			// instrument will be extracted from the ControlBlock
+			// by the sequencer, so we set it to zero here.
+			MappedEvent *mE = new (bufPos)
+			    MappedEvent(0,
+					***k, // three stars! what an accolade
+					eventTime,
+					duration);
+			mE->setTrackId(track->getId());
+
+			if (m_segment->getTranspose() != 0 &&
+			    (**k)->isa(Rosegarden::Note::EventType)) {
+			    mE->setPitch(mE->getPitch() + m_segment->getTranspose());
+			}
+
+			++bufPos;
+
+		    } catch(...) {
+			SEQMAN_DEBUG << "SegmentMmapper::dump - caught exception while trying to create MappedEvent\n";
+		    }
+		}
+	    }
+	
+	    ++*k; // increment either i or j, whichever one we just used
         }
-        
+
+	delete i;
+	delete triggered;
     }
 
     // Store the number of events at the start of the shared memory region
@@ -526,6 +612,95 @@ unsigned int SegmentMmapper::getSegmentRepeatCount()
     return repeatCount;
 }
 
+void
+SegmentMmapper::mergeTriggerSegment(Rosegarden::Segment **target,
+				    Rosegarden::Event *trigger,
+				    Rosegarden::timeT evDuration,
+				    Rosegarden::TriggerSegmentRec *rec)
+{
+    if (!rec || !rec->getSegment() || rec->getSegment()->empty()) return;
+    if (!*target) *target = new Rosegarden::Segment;
+
+    Rosegarden::timeT evTime = trigger->getAbsoluteTime();
+    Rosegarden::timeT trStart = rec->getSegment()->getStartTime();
+    Rosegarden::timeT trEnd = rec->getSegment()->getEndMarkerTime();
+    Rosegarden::timeT trDuration = trEnd - trStart;
+    if (trDuration == 0) return;
+
+    bool retune = false;
+    std::string timeAdjust = TRIGGER_SEGMENT_ADJUST_NONE;
+
+    trigger->get<Bool>
+	(TRIGGER_SEGMENT_RETUNE, retune);
+    
+    trigger->get<Rosegarden::String>
+	(TRIGGER_SEGMENT_ADJUST_TIMES, timeAdjust);
+
+    long evPitch = rec->getBasePitch();
+    (void)trigger->get<Int>(PITCH, evPitch);
+    int pitchDiff = evPitch - rec->getBasePitch();
+
+    long evVelocity = rec->getBaseVelocity();
+    (void)trigger->get<Int>(VELOCITY, evVelocity);
+    int velocityDiff = evVelocity - rec->getBaseVelocity();
+
+    Rosegarden::timeT offset = 0;
+    if (timeAdjust == TRIGGER_SEGMENT_ADJUST_SYNC_END) {
+	offset = evDuration - trDuration;
+    }
+
+    for (Segment::iterator i = rec->getSegment()->begin();
+	 rec->getSegment()->isBeforeEndMarker(i); ++i) {
+	
+	Rosegarden::timeT t = (*i)->getAbsoluteTime() - trStart;
+	Rosegarden::timeT d = (*i)->getDuration();
+
+	if (evDuration != trDuration &&
+	    timeAdjust == TRIGGER_SEGMENT_ADJUST_SQUISH) {
+	    t = Rosegarden::timeT(double(t * evDuration) / double(trDuration));
+	    d = Rosegarden::timeT(double(d * evDuration) / double(trDuration));
+	}
+
+	t += evTime + offset;
+
+	if (t < evTime) {
+	    if (t + d <= evTime) continue;
+	    else {
+		d -= (evTime - t);
+		t = evTime;
+	    }
+	}
+	
+	if (timeAdjust == TRIGGER_SEGMENT_ADJUST_SYNC_START) {
+	    if (t + d > evTime + evDuration) {
+		if (t >= evTime + evDuration) continue;
+		else {
+		    d = evTime + evDuration - t;
+		}
+	    }
+	}
+
+	Rosegarden::Event *newEvent = new Rosegarden::Event(**i, t, d);
+
+	if (retune && newEvent->has(PITCH)) {
+	    int pitch = newEvent->get<Int>(PITCH) + pitchDiff;
+	    if (pitch > 127) pitch = 127;
+	    if (pitch < 0) pitch = 0;
+	    newEvent->set<Int>(PITCH, pitch);
+	}
+
+	if (newEvent->has(VELOCITY)) {
+	    int velocity = newEvent->get<Int>(VELOCITY) + velocityDiff;
+	    if (velocity > 127) velocity = 127;
+	    if (velocity < 0) velocity = 0;
+	    newEvent->set<Int>(VELOCITY, velocity);
+	}
+
+	(*target)->insert(newEvent);
+    }
+}
+    
+
 //----------------------------------------
 
 AudioSegmentMmapper::AudioSegmentMmapper(RosegardenGUIDoc* doc, Rosegarden::Segment* s,
@@ -588,6 +763,26 @@ void AudioSegmentMmapper::dump()
                                                  audioStart);
         mE->setTrackId(track->getId());
         mE->setRuntimeSegmentId(m_segment->getRuntimeId());
+
+        // Send the autofade if required
+        //
+        if (m_segment->isAutoFading())
+        {
+            mE->setAutoFade(true);
+            mE->setFadeInTime(m_segment->getFadeInTime());
+            mE->setFadeOutTime(m_segment->getFadeOutTime());
+            std::cout << "AudioSegmentMmapper::dump - "
+                      << "SETTING AUTOFADE "
+                      << "in = " << m_segment->getFadeInTime()
+                      << ", out = " << m_segment->getFadeOutTime()
+                      << std::endl;
+        }
+        else
+        {
+//            std::cout << "AudioSegmentMmapper::dump - "
+//                      << "NO AUTOFADE SET ON SEGMENT" << std::endl;
+        }
+
         ++bufPos;
     }
 
@@ -656,6 +851,10 @@ bool CompositionMmapper::segmentModified(Segment* segment)
 {
     SegmentMmapper* mmapper = m_segmentMmappers[segment];
 
+    if (!mmapper)
+        return false; // this can happen with the SegmentSplitCommand, where the new segment's transpose is set
+    // even though it's not mapped yet
+
     SEQMAN_DEBUG << "CompositionMmapper::segmentModified(" << segment << ") - mmapper = "
                  << mmapper << endl;
 
@@ -774,6 +973,32 @@ MetronomeMmapper::MetronomeMmapper(RosegardenGUIDoc* doc)
         t = c.getBarEndForTime(t);
     }
     
+    KConfig *config = kapp->config();
+    config->setGroup(Rosegarden::SequencerOptionsConfigGroup);
+    bool midiClock = config->readBoolEntry("midiclock", false);
+    bool mtcMaster = config->readBoolEntry("mtcmaster", false);
+
+    if (midiClock)
+    {
+        using Rosegarden::Note;
+        Rosegarden::timeT quarterNote = Note(Note::Crotchet).getDuration();
+
+        // Insert 24 clocks per quarter note
+        //
+        for (Rosegarden::timeT insertTime = c.getStartMarker();
+             insertTime < c.getEndMarker(); 
+             insertTime += quarterNote / 24)
+        {
+            m_ticks.push_back(Tick(insertTime, 3));
+        }
+    }
+
+
+    if (mtcMaster)
+    {
+        // do something
+    }
+
     sortTicks();
 
     if (m_ticks.size() == 0) {
@@ -813,7 +1038,38 @@ unsigned int MetronomeMmapper::getSegmentRepeatCount()
 
 size_t MetronomeMmapper::computeMmappedSize()
 {
-    return m_ticks.size() * sizeof(MappedEvent);
+    KConfig *config = kapp->config();
+    config->setGroup(Rosegarden::SequencerOptionsConfigGroup);
+    bool midiClock = config->readBoolEntry("midiclock", false);
+    bool mtcMaster = config->readBoolEntry("mtcmaster", false);
+
+    // base size for Metronome ticks
+    size_t size = m_ticks.size() * sizeof(MappedEvent);
+    Composition& comp = m_doc->getComposition();
+
+    if (midiClock)
+    {
+        using Rosegarden::Note;
+
+        // Allow room for MIDI clocks
+        int clocks = ( 24 * ( comp.getEndMarker() - comp.getStartMarker() ) ) / 
+            Note(Note::Crotchet).getDuration();
+
+        /*
+        SEQMAN_DEBUG << "MetronomeMmapper::computeMmappedSize - " 
+                     << "Number of clock events catered for = " << clocks
+                     << endl;
+        */
+
+        size += clocks * sizeof(MappedEvent);
+    }
+
+    if (mtcMaster)
+    {
+        // Allow room for MTC timing messages (how?)
+    }
+
+    return size;
 }
 
 void MetronomeMmapper::dump()
@@ -823,17 +1079,10 @@ void MetronomeMmapper::dump()
 
     SEQMAN_DEBUG << "MetronomeMmapper::dump: instrument is " << m_metronome->getInstrument() << endl;
 
-    MappedEvent* bufPos = m_mmappedEventBuffer;
+    MappedEvent* bufPos = m_mmappedEventBuffer, *mE;
 
     for (TickContainer::iterator i = m_ticks.begin(); i != m_ticks.end(); ++i) {
 
-        Rosegarden::MidiByte velocity;
-	switch (i->second) {
-	case 0:  velocity = m_metronome->getBarVelocity(); break;
-	case 1:  velocity = m_metronome->getBeatVelocity(); break;
-	default: velocity = m_metronome->getSubBeatVelocity(); break;
-	}
-
         /*
         SEQMAN_DEBUG << "MetronomeMmapper::dump: velocity = "
                      << int(velocity) << endl;
@@ -841,16 +1090,39 @@ void MetronomeMmapper::dump()
 
         eventTime = comp.getElapsedRealTime(i->first);
 
-        new (bufPos) MappedEvent(m_metronome->getInstrument(),
-                                 m_metronome->getPitch(),
-                                 velocity,
-                                 eventTime,
-                                 m_tickDuration);
+        if (i->second == 3) // MIDI Clock
+        {
+            mE = new (bufPos) MappedEvent(0, MappedEvent::MidiSystemMessage);
+            mE->setData1(Rosegarden::MIDI_TIMING_CLOCK);
+            mE->setEventTime(eventTime);
+        }
+        else
+        {
+            Rosegarden::MidiByte velocity;
+	    switch (i->second) {
+	    case 0:  velocity = m_metronome->getBarVelocity(); break;
+	    case 1:  velocity = m_metronome->getBeatVelocity(); break;
+	    default: velocity = m_metronome->getSubBeatVelocity(); break;
+            }
+
+            new (bufPos) MappedEvent(m_metronome->getInstrument(),
+				     MappedEvent::MidiNoteOneShot,
+                                     m_metronome->getPitch(),
+                                     velocity,
+                                     eventTime,
+                                     m_tickDuration,
+				     Rosegarden::RealTime::zeroTime);
+        }
+
         ++bufPos;
     }
 
     // Store the number of events at the start of the shared memory region
     *(size_t *)m_mmappedRegion = (bufPos - m_mmappedEventBuffer);
+
+    SEQMAN_DEBUG << "MetronomeMmapper::dump: - "
+                 << "Total events written = " << *(size_t *)m_mmappedRegion
+                 << endl;
 }
 
 
