@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -191,6 +191,22 @@ void NotationCanvasView::contentsMousePressEvent(QMouseEvent *e)
 
 	NotationElement &el = sprite->getNotationElement();
 
+	// #957364 (Notation: Hard to select upper note in chords of
+	// seconds) -- adjust x-coord for shifted note head
+
+	double cx = el.getCanvasX();
+	int nbw = 10;
+
+	if (staff) {
+	    nbw = staff->getNotePixmapFactory(false).getNoteBodyWidth();
+	    bool shifted = false;
+
+	    if (el.event()->get<Rosegarden::Bool>
+		(staff->getProperties().NOTE_HEAD_SHIFTED, shifted) && shifted) {
+		cx += nbw;
+	    }
+	}
+
 	if (el.isNote() && haveClickHeight) {
 	    long eventHeight = 0;
 	    if (el.event()->get<Rosegarden::Int>
@@ -199,14 +215,12 @@ void NotationCanvasView::contentsMousePressEvent(QMouseEvent *e)
 		if (eventHeight == clickHeight) {
 
 		    if (!clickedNote &&
-			e->x() >= el.getCanvasX() &&
-			e->x() <= el.getCanvasX() +
-			staff->getNotePixmapFactory(false).getNoteBodyWidth()) {
+			e->x() >= cx &&
+			e->x() <= cx + nbw) {
 			clickedNote = &el;
 		    } else if (!clickedVagueNote &&
-			       e->x() >= el.getCanvasX() - 2 &&
-			       e->x() <= el.getCanvasX() +
-			       staff->getNotePixmapFactory(false).getNoteBodyWidth() + 2) {
+			       e->x() >= cx - 2 &&
+			       e->x() <= cx + nbw + 2) {
 			clickedVagueNote = &el;
 		    }
 
