@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -52,7 +52,6 @@ using Rosegarden::Text;
 using Rosegarden::Accidental;
 using namespace Rosegarden::Accidentals;
 using Rosegarden::Mark;
-using Rosegarden::Marks;
 using Rosegarden::Indication;
 using Rosegarden::EventSelection;
 
@@ -436,6 +435,34 @@ MultiKeyInsertionCommand::~MultiKeyInsertionCommand()
     // nothing
 }
 
+
+SustainInsertionCommand::SustainInsertionCommand(Segment &segment, timeT time,
+						 bool down,
+						 int controllerNumber) :
+    BasicCommand(getGlobalName(down), segment, time, time),
+    m_down(down),
+    m_controllerNumber(controllerNumber),
+    m_lastInsertedEvent(0)
+{
+    // nothing
+}
+
+SustainInsertionCommand::~SustainInsertionCommand()
+{
+    // nothing
+}
+
+void
+SustainInsertionCommand::modifySegment()
+{
+    Event *e = new Event(Rosegarden::Controller::EventType, getStartTime(), 0,
+			 Rosegarden::Controller::EventSubOrdering);
+    e->set<Int>(Rosegarden::Controller::NUMBER, m_controllerNumber);
+    e->set<Int>(Rosegarden::Controller::VALUE, m_down ? 127 : 0);
+    m_lastInsertedEvent = *getSegment().insert(e);
+}
+
+
 EraseEventCommand::EraseEventCommand(Segment &segment,
 				     Event *event,
 				     bool collapseRest) :
@@ -505,7 +532,7 @@ EraseEventCommand::modifySegment()
 
 
 void
-GroupMenuBeamCommand::modifySegment()
+NotesMenuBeamCommand::modifySegment()
 {
     int id = getSegment().getNextId();
 
@@ -521,14 +548,14 @@ GroupMenuBeamCommand::modifySegment()
 }
 
 void
-GroupMenuAutoBeamCommand::modifySegment()
+NotesMenuAutoBeamCommand::modifySegment()
 {
     SegmentNotationHelper helper(getSegment());
     helper.autoBeam(getStartTime(), getEndTime(), GROUP_TYPE_BEAMED);
 }
 
 void
-GroupMenuBreakCommand::modifySegment()
+NotesMenuBreakCommand::modifySegment()
 {
     for (EventSelection::eventcontainer::iterator i =
 	     m_selection->getSegmentEvents().begin();
@@ -541,7 +568,7 @@ GroupMenuBreakCommand::modifySegment()
     }
 }
 
-GroupMenuGraceCommand::GroupMenuGraceCommand(Rosegarden::EventSelection &selection) :
+AdjustMenuGraceCommand::AdjustMenuGraceCommand(Rosegarden::EventSelection &selection) :
     BasicCommand(getGlobalName(),
 		 selection.getSegment(),
 		 selection.getStartTime(),
@@ -552,7 +579,7 @@ GroupMenuGraceCommand::GroupMenuGraceCommand(Rosegarden::EventSelection &selecti
 }
 
 timeT
-GroupMenuGraceCommand::getEffectiveEndTime(Rosegarden::EventSelection &
+AdjustMenuGraceCommand::getEffectiveEndTime(Rosegarden::EventSelection &
 					   selection)
 {
     EventSelection::eventcontainer::iterator i =
@@ -568,7 +595,7 @@ GroupMenuGraceCommand::getEffectiveEndTime(Rosegarden::EventSelection &
 }
 
 void
-GroupMenuGraceCommand::modifySegment()
+AdjustMenuGraceCommand::modifySegment()
 {
     Segment &s(getSegment());
     timeT startTime = getStartTime();
@@ -610,35 +637,59 @@ GroupMenuGraceCommand::modifySegment()
 }
 
 void
-GroupMenuUnGraceCommand::modifySegment()
+AdjustMenuUnGraceCommand::modifySegment()
 {
     //!!!
 }
 
 
-GroupMenuTupletCommand::GroupMenuTupletCommand(Rosegarden::Segment &segment,
+AdjustMenuTupletCommand::AdjustMenuTupletCommand(Rosegarden::Segment &segment,
 					       timeT startTime,
 					       timeT unit,
-					       int untupled, int tupled) :
+					       int untupled, int tupled,
+					       bool hasTimingAlready) :
     BasicCommand(getGlobalName((untupled == 3) && (tupled == 2)),
 		 segment, startTime, startTime + (unit * untupled)),
     m_unit(unit),
     m_untupled(untupled),
-    m_tupled(tupled)
+    m_tupled(tupled),
+    m_hasTimingAlready(hasTimingAlready)
 {
     // nothing else
 }
 
 void
-GroupMenuTupletCommand::modifySegment()
+AdjustMenuTupletCommand::modifySegment()
 {
-    SegmentNotationHelper helper(getSegment());
-    helper.makeTupletGroup(getStartTime(), m_untupled, m_tupled, m_unit);
+    if (m_hasTimingAlready) {
+
+	int groupId = getSegment().getNextId();
+
+	for (Segment::iterator i = getSegment().findTime(getStartTime());
+	     getSegment().isBeforeEndMarker(i); ++i) {
+
+	    if ((*i)->getNotationAbsoluteTime() >=
+	        getStartTime() + (m_unit * m_tupled)) break;
+
+	    Event *e = *i;
+
+	    e->set<Int>(BEAMED_GROUP_ID, groupId);
+	    e->set<String>(BEAMED_GROUP_TYPE, GROUP_TYPE_TUPLED);
+	    
+	    e->set<Int>(BEAMED_GROUP_TUPLET_BASE, m_unit);
+	    e->set<Int>(BEAMED_GROUP_TUPLED_COUNT, m_tupled);
+	    e->set<Int>(BEAMED_GROUP_UNTUPLED_COUNT, m_untupled);
+	}
+
+    } else {
+	SegmentNotationHelper helper(getSegment());
+	helper.makeTupletGroup(getStartTime(), m_untupled, m_tupled, m_unit);
+    }
 }
 
 
 void
-GroupMenuUnTupletCommand::modifySegment()
+AdjustMenuUnTupletCommand::modifySegment()
 {
     for (EventSelection::eventcontainer::iterator i =
 	     m_selection->getSegmentEvents().begin();
@@ -649,13 +700,12 @@ GroupMenuUnTupletCommand::modifySegment()
 	(*i)->unset(BEAMED_GROUP_TUPLET_BASE);
 	(*i)->unset(BEAMED_GROUP_TUPLED_COUNT);
 	(*i)->unset(BEAMED_GROUP_UNTUPLED_COUNT);
-//!!!	(*i)->unset(TUPLET_NOMINAL_DURATION);
     }
 }
 
 
 
-GroupMenuAddIndicationCommand::GroupMenuAddIndicationCommand(std::string indicationType, 
+NotesMenuAddIndicationCommand::NotesMenuAddIndicationCommand(std::string indicationType, 
 							     EventSelection &selection) :
     BasicCommand(getGlobalName(indicationType),
 		 selection.getSegment(),
@@ -668,13 +718,13 @@ GroupMenuAddIndicationCommand::GroupMenuAddIndicationCommand(std::string indicat
     // nothing else
 }
 
-GroupMenuAddIndicationCommand::~GroupMenuAddIndicationCommand()
+NotesMenuAddIndicationCommand::~NotesMenuAddIndicationCommand()
 {
     // empty
 }
 
 bool
-GroupMenuAddIndicationCommand::canExecute()
+NotesMenuAddIndicationCommand::canExecute()
 {
     Segment &s(getSegment());
     
@@ -730,7 +780,7 @@ GroupMenuAddIndicationCommand::canExecute()
 }
 
 void
-GroupMenuAddIndicationCommand::modifySegment()
+NotesMenuAddIndicationCommand::modifySegment()
 {
     SegmentNotationHelper helper(getSegment());
 
@@ -752,10 +802,12 @@ GroupMenuAddIndicationCommand::modifySegment()
 }
 
 QString
-GroupMenuAddIndicationCommand::getGlobalName(std::string indicationType)
+NotesMenuAddIndicationCommand::getGlobalName(std::string indicationType)
 {
     if (indicationType == Rosegarden::Indication::Slur) {
 	return i18n("Add S&lur");
+    } else if (indicationType == Rosegarden::Indication::PhrasingSlur) {
+	return i18n("Add &Phrasing Slur");
     } else if (indicationType == Rosegarden::Indication::QuindicesimaUp) {
 	return i18n("Add Double-Octave Up");
     } else if (indicationType == Rosegarden::Indication::OttavaUp) {
@@ -764,6 +816,15 @@ GroupMenuAddIndicationCommand::getGlobalName(std::string indicationType)
 	return i18n("Add Octave &Down");
     } else if (indicationType == Rosegarden::Indication::QuindicesimaDown) {
 	return i18n("Add Double Octave Down");
+
+    // We used to generate these ones from the internal names plus
+    // caps, but that makes them untranslateable:
+    } else if (indicationType == Rosegarden::Indication::Crescendo) {
+	return i18n("Add &Crescendo");
+    } else if (indicationType == Rosegarden::Indication::Decrescendo) {
+	return i18n("Add &Decrescendo");
+    } else if (indicationType == Rosegarden::Indication::Glissando) {
+	return i18n("Add &Glissando");
     }
 
     QString n = i18n("Add &%1%2").arg((char)toupper(indicationType[0])).arg(strtoqstr(indicationType.substr(1)));
@@ -772,7 +833,7 @@ GroupMenuAddIndicationCommand::getGlobalName(std::string indicationType)
 
 
 void
-GroupMenuMakeChordCommand::modifySegment()
+AdjustMenuMakeChordCommand::modifySegment()
 {
     // find all the notes in the selection, and bring them back to align
     // with the start of the selection, giving them the same duration as
@@ -807,7 +868,7 @@ GroupMenuMakeChordCommand::modifySegment()
 }	 
 
 
-TransformsMenuNormalizeRestsCommand::TransformsMenuNormalizeRestsCommand
+AdjustMenuNormalizeRestsCommand::AdjustMenuNormalizeRestsCommand
 (Rosegarden::EventSelection &selection) :
     BasicCommand(getGlobalName(),
 		 selection.getSegment(),
@@ -817,12 +878,12 @@ TransformsMenuNormalizeRestsCommand::TransformsMenuNormalizeRestsCommand
     // nothing else
 }
 
-void TransformsMenuNormalizeRestsCommand::modifySegment()
+void AdjustMenuNormalizeRestsCommand::modifySegment()
 {
     getSegment().normalizeRests(getStartTime(), getEndTime());
 }
 
-TransformsMenuCollapseRestsCommand::TransformsMenuCollapseRestsCommand
+AdjustMenuCollapseRestsCommand::AdjustMenuCollapseRestsCommand
 (Rosegarden::EventSelection &selection) :
     BasicCommand(getGlobalName(),
 		 selection.getSegment(),
@@ -832,54 +893,14 @@ TransformsMenuCollapseRestsCommand::TransformsMenuCollapseRestsCommand
     // nothing else
 }
 
-void TransformsMenuCollapseRestsCommand::modifySegment()
+void AdjustMenuCollapseRestsCommand::modifySegment()
 {
     SegmentNotationHelper helper(getSegment());
     helper.collapseRestsAggressively(getStartTime(), getEndTime());
 }
 
 void
-TransformsMenuCollapseNotesCommand::modifySegment()
-{
-    SegmentNotationHelper helper(getSegment());
-    timeT endTime = getEndTime();
-
-    // This is really nasty stuff.  We can't go in forward direction
-    // using the j-iterator trick because collapseNoteAggressively may
-    // erase the following iterator as well as the preceding one.  We
-    // can't go backward naively, because collapseNoteAggressively
-    // erases i from the EventSelection now that it's a
-    // SegmentObserver.  We need the fancy hybrid j-iterator-backward
-    // technique applied to selections instead of segments.
-    
-    EventSelection::eventcontainer::iterator i =
-	m_selection->getSegmentEvents().end();
-    EventSelection::eventcontainer::iterator j = i;
-    bool thisOne = false;
-
-    while (i != m_selection->getSegmentEvents().begin()) {
-	
-	--j;
-	
-	if (thisOne) {
-	    helper.collapseNoteAggressively(*i, endTime);
-	}
-	
-	// rather than "true" one could perform a test to see
-	// whether j pointed to a candidate for collapsing:
-	thisOne = true;
-	
-	i = j;
-    }
-    
-    if (thisOne) {
-	helper.collapseNoteAggressively(*i, endTime);
-    }
-}
-
-
-void
-TransformsMenuTieNotesCommand::modifySegment()
+NotesMenuTieNotesCommand::modifySegment()
 {
     Segment &segment(getSegment());
     SegmentNotationHelper helper(segment);
@@ -910,7 +931,7 @@ TransformsMenuTieNotesCommand::modifySegment()
 
 
 void
-TransformsMenuUntieNotesCommand::modifySegment()
+AdjustMenuUntieNotesCommand::modifySegment()
 {
     for (EventSelection::eventcontainer::iterator i =
 	     m_selection->getSegmentEvents().begin();
@@ -923,7 +944,7 @@ TransformsMenuUntieNotesCommand::modifySegment()
 
 
 void
-TransformsMenuMakeNotesViableCommand::modifySegment()
+AdjustMenuMakeNotesViableCommand::modifySegment()
 {
     Segment &segment(getSegment());
     SegmentNotationHelper helper(segment);
@@ -942,7 +963,7 @@ TransformsMenuMakeNotesViableCommand::modifySegment()
 }
 
 void
-TransformsMenuMakeRegionViableCommand::modifySegment()
+AdjustMenuMakeRegionViableCommand::modifySegment()
 {
     Segment &segment(getSegment());
     SegmentNotationHelper helper(segment);
@@ -952,7 +973,7 @@ TransformsMenuMakeRegionViableCommand::modifySegment()
 }
 
 void
-TransformsMenuDeCounterpointCommand::modifySegment()
+AdjustMenuDeCounterpointCommand::modifySegment()
 {
     Segment &segment(getSegment());
     SegmentNotationHelper helper(segment);
@@ -972,7 +993,7 @@ TransformsMenuDeCounterpointCommand::modifySegment()
 
 
 void
-TransformsMenuChangeStemsCommand::modifySegment()
+AdjustMenuChangeStemsCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -987,7 +1008,7 @@ TransformsMenuChangeStemsCommand::modifySegment()
 
 
 void
-TransformsMenuRestoreStemsCommand::modifySegment()
+AdjustMenuRestoreStemsCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1000,15 +1021,56 @@ TransformsMenuRestoreStemsCommand::modifySegment()
     }
 }
 
+
+void
+AdjustMenuChangeSlurPositionCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if ((*i)->isa(Indication::EventType)) {
+	    std::string indicationType;
+	    if ((*i)->get<String>(Indication::IndicationTypePropertyName, indicationType)
+		&& (indicationType == Indication::Slur ||
+		    indicationType == Indication::PhrasingSlur)) {
+		(*i)->set<Rosegarden::Bool>(NotationProperties::SLUR_ABOVE, m_above);
+	    }
+	}
+    }
+}
+
+
+void
+AdjustMenuRestoreSlursCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if ((*i)->isa(Indication::EventType)) {
+	    std::string indicationType;
+	    if ((*i)->get<String>(Indication::IndicationTypePropertyName, indicationType)
+		&& (indicationType == Indication::Slur ||
+		    indicationType == Indication::PhrasingSlur)) {
+		(*i)->unset(NotationProperties::SLUR_ABOVE);
+	    }
+	}
+    }
+}
+
+
 QString
-TransformsMenuChangeStyleCommand::getGlobalName(NoteStyleName style)
+AdjustMenuChangeStyleCommand::getGlobalName(NoteStyleName style)
 {
     return strtoqstr(style);
 }
 
 
 void
-TransformsMenuChangeStyleCommand::modifySegment()
+AdjustMenuChangeStyleCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1045,7 +1107,7 @@ NotesMenuAddSlashesCommand::modifySegment()
 
 
 QString
-MarksMenuAddMarkCommand::getGlobalName(Rosegarden::Mark markType)
+NotesMenuAddMarkCommand::getGlobalName(Rosegarden::Mark markType)
 {
     QString m = strtoqstr(markType);
 
@@ -1055,6 +1117,8 @@ MarksMenuAddMarkCommand::getGlobalName(Rosegarden::Mark markType)
     else if (markType == Rosegarden::Marks::Rinforzando) m = i18n("R&inforzando");
     else if (markType == Rosegarden::Marks::Tenuto) m = i18n("T&enuto");
     else if (markType == Rosegarden::Marks::Trill) m = i18n("Tri&ll");
+    else if (markType == Rosegarden::Marks::LongTrill) m = i18n("Trill &with Line");
+    else if (markType == Rosegarden::Marks::TrillLine) m = i18n("Trill Line");
     else if (markType == Rosegarden::Marks::Turn) m = i18n("&Turn");
     else if (markType == Rosegarden::Marks::Accent) m = i18n("&Accent");
     else if (markType == Rosegarden::Marks::Staccatissimo) m = i18n("&Staccatissimo");
@@ -1062,6 +1126,14 @@ MarksMenuAddMarkCommand::getGlobalName(Rosegarden::Mark markType)
     else if (markType == Rosegarden::Marks::Pause) m = i18n("&Pause");
     else if (markType == Rosegarden::Marks::UpBow) m = i18n("&Up-Bow");
     else if (markType == Rosegarden::Marks::DownBow) m = i18n("&Down-Bow");
+    else if (markType == Rosegarden::Marks::Mordent)
+	m = i18n("Mo&rdent");
+    else if (markType == Rosegarden::Marks::MordentInverted)
+	m = i18n("Inverted Mordent");
+    else if (markType == Rosegarden::Marks::MordentLong)
+	m = i18n("Long Mordent");
+    else if (markType == Rosegarden::Marks::MordentLongInverted)
+	m = i18n("Lon&g Inverted Mordent");
     else m = i18n("&%1%2").arg(m[0].upper()).arg(m.right(m.length()-1));
     // FIXME: That last i18n has very little chance of working, unless
     // by some miracle the exact same string was translated elsewhere already
@@ -1072,7 +1144,7 @@ MarksMenuAddMarkCommand::getGlobalName(Rosegarden::Mark markType)
 }
 
 void
-MarksMenuAddMarkCommand::modifySegment()
+NotesMenuAddMarkCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1088,7 +1160,7 @@ MarksMenuAddMarkCommand::modifySegment()
 }
 
 void
-MarksMenuAddTextMarkCommand::modifySegment()
+NotesMenuAddTextMarkCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1104,7 +1176,7 @@ MarksMenuAddTextMarkCommand::modifySegment()
 }
 
 QString
-MarksMenuAddFingeringMarkCommand::getGlobalName(QString fingering)
+NotesMenuAddFingeringMarkCommand::getGlobalName(QString fingering)
 {
     if (fingering == "") return i18n("Add Other &Fingering...");
     else if (fingering == "0") return i18n("Add Fingering &0 (Thumb)");
@@ -1112,7 +1184,7 @@ MarksMenuAddFingeringMarkCommand::getGlobalName(QString fingering)
 }
 
 void
-MarksMenuAddFingeringMarkCommand::modifySegment()
+NotesMenuAddFingeringMarkCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
     Segment &segment(m_selection->getSegment());
@@ -1174,7 +1246,7 @@ MarksMenuAddFingeringMarkCommand::modifySegment()
 }
 
 void
-MarksMenuRemoveMarksCommand::modifySegment()
+NotesMenuRemoveMarksCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1192,7 +1264,7 @@ MarksMenuRemoveMarksCommand::modifySegment()
 }
 
 void
-MarksMenuRemoveFingeringMarksCommand::modifySegment()
+NotesMenuRemoveFingeringMarksCommand::modifySegment()
 {
     EventSelection::eventcontainer::iterator i;
 
@@ -1210,7 +1282,7 @@ MarksMenuRemoveFingeringMarksCommand::modifySegment()
 }
 
 void
-TransformsMenuFixNotationQuantizeCommand::modifySegment()
+AdjustMenuFixNotationQuantizeCommand::modifySegment()
 {
     std::vector<Event *> toErase;
     std::vector<Event *> toInsert;
@@ -1256,16 +1328,48 @@ TransformsMenuFixNotationQuantizeCommand::modifySegment()
     //!!! normalizeRests?
 }
 
+void
+AdjustMenuRemoveNotationQuantizeCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
 
-const int TransformsMenuInterpretCommand::NoInterpretation      = 0;
-const int TransformsMenuInterpretCommand::GuessDirections	= (1<<0);
-const int TransformsMenuInterpretCommand::ApplyTextDynamics	= (1<<1);
-const int TransformsMenuInterpretCommand::ApplyHairpins		= (1<<2);
-const int TransformsMenuInterpretCommand::StressBeats		= (1<<3);
-const int TransformsMenuInterpretCommand::Articulate		= (1<<4);
-const int TransformsMenuInterpretCommand::AllInterpretations	= (1<<5) - 1;
+    std::vector<Event *> toInsert;
+    std::vector<Event *> toErase;
 
-TransformsMenuInterpretCommand::~TransformsMenuInterpretCommand()
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	toInsert.push_back(new Event(**i,
+				     (*i)->getAbsoluteTime(),
+				     (*i)->getDuration(),
+				     (*i)->getSubOrdering(),
+				     (*i)->getAbsoluteTime(),
+				     (*i)->getDuration()));
+
+	toErase.push_back(*i);
+    }
+
+    for (std::vector<Event *>::iterator i = toErase.begin(); i != toErase.end();
+	 ++i) {
+	m_selection->getSegment().eraseSingle(*i);
+    }
+
+    for (std::vector<Event *>::iterator i = toInsert.begin(); i != toInsert.end();
+	 ++i) {
+	m_selection->getSegment().insert(*i);
+    }
+}
+
+
+const int AdjustMenuInterpretCommand::NoInterpretation      = 0;
+const int AdjustMenuInterpretCommand::GuessDirections	= (1<<0);
+const int AdjustMenuInterpretCommand::ApplyTextDynamics	= (1<<1);
+const int AdjustMenuInterpretCommand::ApplyHairpins		= (1<<2);
+const int AdjustMenuInterpretCommand::StressBeats		= (1<<3);
+const int AdjustMenuInterpretCommand::Articulate		= (1<<4);
+const int AdjustMenuInterpretCommand::AllInterpretations	= (1<<5) - 1;
+
+AdjustMenuInterpretCommand::~AdjustMenuInterpretCommand()
 {
     for (IndicationMap::iterator i = m_indications.begin();
 	 i != m_indications.end(); ++i) {
@@ -1274,7 +1378,7 @@ TransformsMenuInterpretCommand::~TransformsMenuInterpretCommand()
 }
 
 void
-TransformsMenuInterpretCommand::modifySegment()
+AdjustMenuInterpretCommand::modifySegment()
 {
     // Of all the interpretations, Articulate is the only one that
     // changes event times or durations.  This means we must apply it
@@ -1327,13 +1431,13 @@ TransformsMenuInterpretCommand::modifySegment()
 }
 
 void
-TransformsMenuInterpretCommand::guessDirections()
+AdjustMenuInterpretCommand::guessDirections()
 {
     //...
 }
 
 void
-TransformsMenuInterpretCommand::applyTextDynamics()
+AdjustMenuInterpretCommand::applyTextDynamics()
 {
     // laborious
 
@@ -1366,7 +1470,7 @@ TransformsMenuInterpretCommand::applyTextDynamics()
 }
 
 int
-TransformsMenuInterpretCommand::getVelocityForDynamic(std::string text)
+AdjustMenuInterpretCommand::getVelocityForDynamic(std::string text)
 {
     int velocity = 100;
 		    
@@ -1386,13 +1490,13 @@ TransformsMenuInterpretCommand::getVelocityForDynamic(std::string text)
     else if (text == "ffff")  velocity = 120;
     else if (text == "fffff") velocity = 125;
 
-    NOTATION_DEBUG << "TransformsMenuInterpretCommand::getVelocityForDynamic: unrecognised dynamic " << text << endl;
+    NOTATION_DEBUG << "AdjustMenuInterpretCommand::getVelocityForDynamic: unrecognised dynamic " << text << endl;
 
     return velocity;
 }
 
 void
-TransformsMenuInterpretCommand::applyHairpins()
+AdjustMenuInterpretCommand::applyHairpins()
 {
     Segment &segment(getSegment());
     int velocityToApply = -1;
@@ -1443,9 +1547,9 @@ TransformsMenuInterpretCommand::applyHairpins()
 	    --itr;
 	}
 
-	int startingVelocity = 100;
+	long startingVelocity = 100;
 	if (itr != segment.end()) {
-	    startingVelocity = (*itr)->get<Int>(VELOCITY);
+	    (*itr)->get<Int>(VELOCITY, startingVelocity);
 	}
 	
 	// The ending velocity is harder.  If there's a dynamic change
@@ -1455,7 +1559,7 @@ TransformsMenuInterpretCommand::applyHairpins()
 	// we should probably make the degree of change caused by the
 	// hairpin depend on its total duration.
 
-	int endingVelocity = startingVelocity;
+	long endingVelocity = startingVelocity;
 	timeT hairpinEndTime = inditr->first +
                                inditr->second->getIndicationDuration();
 	itr = segment.findTime(hairpinEndTime);
@@ -1494,7 +1598,7 @@ TransformsMenuInterpretCommand::applyHairpins()
 	    int((endingVelocity - startingVelocity) * proportion +
 		startingVelocity);
 
-	NOTATION_DEBUG << "TransformsMenuInterpretCommand::applyHairpins: velocity of note at " << e->getAbsoluteTime() << " is " << velocity << " (" << proportion << " through hairpin from " << startingVelocity << " to " << endingVelocity <<")" << endl;
+	NOTATION_DEBUG << "AdjustMenuInterpretCommand::applyHairpins: velocity of note at " << e->getAbsoluteTime() << " is " << velocity << " (" << proportion << " through hairpin from " << startingVelocity << " to " << endingVelocity <<")" << endl;
 	if (velocity < 10) velocity = 10;
 	if (velocity > 127) velocity = 127;
 	e->set<Int>(VELOCITY, velocity);
@@ -1503,7 +1607,7 @@ TransformsMenuInterpretCommand::applyHairpins()
 }
 
 void
-TransformsMenuInterpretCommand::stressBeats()
+AdjustMenuInterpretCommand::stressBeats()
 {
     Rosegarden::Composition *c = getSegment().getComposition();
 
@@ -1535,7 +1639,7 @@ TransformsMenuInterpretCommand::stressBeats()
 }
 
 void
-TransformsMenuInterpretCommand::articulate()
+AdjustMenuInterpretCommand::articulate()
 {
     // Basic articulations:
     //
@@ -1595,26 +1699,26 @@ TransformsMenuInterpretCommand::articulate()
 	for (std::vector<Mark>::iterator i = marks.begin();
 	     i != marks.end(); ++i) {
 
-	    if (*i == Marks::Accent) {
+	    if (*i == Rosegarden::Marks::Accent) {
 		velocityChange += 30;
-	    } else if (*i == Marks::Tenuto) {
+	    } else if (*i == Rosegarden::Marks::Tenuto) {
 		tenuto = true;
-	    } else if (*i == Marks::Staccato) {
+	    } else if (*i == Rosegarden::Marks::Staccato) {
 		staccato = true;
-	    } else if (*i == Marks::Staccatissimo) {
+	    } else if (*i == Rosegarden::Marks::Staccatissimo) {
 		staccatissimo = true;
 		velocityChange -= 5;
-	    } else if (*i == Marks::Marcato) {
+	    } else if (*i == Rosegarden::Marks::Marcato) {
 		marcato = true;
 		velocityChange += 15;
-	    } else if (*i == Marks::Sforzando) {
+	    } else if (*i == Rosegarden::Marks::Sforzando) {
 		velocityChange += 35;
-	    } else if (*i == Marks::Rinforzando) {
+	    } else if (*i == Rosegarden::Marks::Rinforzando) {
 		rinforzando = true;
 		velocityChange += 15;
-	    } else if (*i == Marks::DownBow) {
+	    } else if (*i == Rosegarden::Marks::DownBow) {
 		velocityChange += 5;
-	    } else if (*i == Marks::UpBow) {
+	    } else if (*i == Rosegarden::Marks::UpBow) {
 		velocityChange -= 5;
 	    }
 	}
@@ -1658,14 +1762,14 @@ TransformsMenuInterpretCommand::articulate()
 	    else durationChange = -10;
 	}
 
-	NOTATION_DEBUG << "TransformsMenuInterpretCommand::modifySegment: chord has " << chord.size() << " notes in it" << endl;
+	NOTATION_DEBUG << "AdjustMenuInterpretCommand::modifySegment: chord has " << chord.size() << " notes in it" << endl;
 
 	for (Rosegarden::Chord::iterator ci = chord.begin();
 	     ci != chord.end(); ++ci) {
 
 	    e = **ci;
 
-	NOTATION_DEBUG << "TransformsMenuInterpretCommand::modifySegment: For note at " << e->getAbsoluteTime() << ", velocityChange is " << velocityChange << " and durationChange is " << durationChange << endl;
+	NOTATION_DEBUG << "AdjustMenuInterpretCommand::modifySegment: For note at " << e->getAbsoluteTime() << ", velocityChange is " << velocityChange << " and durationChange is " << durationChange << endl;
 
 	    // do this even if velocityChange == 0, in case the event
 	    // has no velocity yet
@@ -1697,8 +1801,12 @@ TransformsMenuInterpretCommand::articulate()
 		
 		    //!!! deal with tuplets
 		    
-		    Event *newEvent = new Event(*e, e->getAbsoluteTime(), newDuration);
-		    newEvent->setNotationDuration(duration);
+		    Event *newEvent = new Event(*e,
+						e->getAbsoluteTime(),
+						newDuration,
+						e->getSubOrdering(),
+						e->getNotationAbsoluteTime(),
+						duration);
 		    toInsert.insert(newEvent);
 		    toErase.insert(e);
 		}
@@ -1724,8 +1832,8 @@ TransformsMenuInterpretCommand::articulate()
     }
 }
 
-TransformsMenuInterpretCommand::IndicationMap::iterator
-TransformsMenuInterpretCommand::findEnclosingIndication(Event *e,
+AdjustMenuInterpretCommand::IndicationMap::iterator
+AdjustMenuInterpretCommand::findEnclosingIndication(Event *e,
 							std::string type)
 {
     // a bit slow, but let's wait and see whether it's a bottleneck
@@ -1863,4 +1971,52 @@ MakeAccidentalsCautionaryCommand::modifySegment()
 }
 
 
+void
+IncrementDisplacementsCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	long prevX = 0, prevY = 0;
+	(*i)->get<Int>(DISPLACED_X, prevX);
+	(*i)->get<Int>(DISPLACED_Y, prevY);
+	(*i)->setMaybe<Int>(DISPLACED_X, prevX + long(m_dx));
+	(*i)->setMaybe<Int>(DISPLACED_Y, prevY + long(m_dy));
+    }
+}
+
+
+void
+ResetDisplacementsCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	(*i)->unset(DISPLACED_X);
+	(*i)->unset(DISPLACED_Y);
+    }
+}
+
+
+void
+SetVisibilityCommand::modifySegment()
+{
+    EventSelection::eventcontainer::iterator i;
+
+    for (i  = m_selection->getSegmentEvents().begin();
+	 i != m_selection->getSegmentEvents().end(); ++i) {
+
+	if (m_visible) {
+	    (*i)->unset(INVISIBLE);
+	} else {
+	    (*i)->set<Bool>(INVISIBLE, true);
+	}
+    }
+}
+
+
 		
