@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -181,6 +181,33 @@ public:
     }
 };
 
+class SustainInsertionCommand : public BasicCommand
+{
+public:
+    SustainInsertionCommand(Rosegarden::Segment &segment,
+			    Rosegarden::timeT time,
+			    bool down,
+			    int controllerNumber);
+    virtual ~SustainInsertionCommand();
+
+    static QString getGlobalName(bool down) {
+	if (down) {
+	    return i18n("Add Pedal &Press");
+	} else {
+	    return i18n("Add Pedal &Release");
+	}
+    }
+
+    Rosegarden::Event *getLastInsertedEvent() { return m_lastInsertedEvent; }
+
+protected:
+    virtual void modifySegment();
+
+    bool m_down;
+    int m_controllerNumber;
+    Rosegarden::Event *m_lastInsertedEvent;
+};
+
 
 class EraseEventCommand : public BasicCommand
 {
@@ -207,10 +234,10 @@ protected:
 // Group menu commands
 
 
-class GroupMenuBeamCommand : public BasicSelectionCommand
+class NotesMenuBeamCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuBeamCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuBeamCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -222,13 +249,13 @@ protected:
 };
 
 
-class GroupMenuAutoBeamCommand : public BasicSelectionCommand
+class NotesMenuAutoBeamCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuAutoBeamCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuAutoBeamCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection) { }
 
-    GroupMenuAutoBeamCommand(Rosegarden::Segment &segment) :
+    NotesMenuAutoBeamCommand(Rosegarden::Segment &segment) :
 	BasicSelectionCommand(getGlobalName(), segment) { }
 
     static QString getGlobalName() { return i18n("&Auto-Beam"); }
@@ -238,10 +265,10 @@ protected:
 };
 
 
-class GroupMenuBreakCommand : public BasicSelectionCommand
+class NotesMenuBreakCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuBreakCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuBreakCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -253,17 +280,18 @@ protected:
 };
 
 
-class GroupMenuTupletCommand : public BasicCommand
+class AdjustMenuTupletCommand : public BasicCommand
 {
 public:
-    GroupMenuTupletCommand(Rosegarden::Segment &segment,
+    AdjustMenuTupletCommand(Rosegarden::Segment &segment,
 			   Rosegarden::timeT startTime,
 			   Rosegarden::timeT unit,
-			   int untupled = 3, int tupled = 2);
+			   int untupled = 3, int tupled = 2,
+			   bool groupHasTimingAlready = false);
 
     static QString getGlobalName(bool simple = true) {
-	if (simple) return i18n("&Simple Tuplet");
-	else return i18n("&Tuplet...");
+	if (simple) return i18n("&Triplet");
+	else return i18n("Tu&plet...");
     }
 
 protected:
@@ -273,13 +301,14 @@ private:
     Rosegarden::timeT m_unit;
     int m_untupled;
     int m_tupled;
+    bool m_hasTimingAlready;
 };
 
 
-class GroupMenuUnTupletCommand : public BasicSelectionCommand
+class AdjustMenuUnTupletCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuUnTupletCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuUnTupletCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -293,10 +322,10 @@ protected:
 };
 
 
-class GroupMenuGraceCommand : public BasicCommand
+class AdjustMenuGraceCommand : public BasicCommand
 {
 public:
-    GroupMenuGraceCommand(Rosegarden::EventSelection &selection);
+    AdjustMenuGraceCommand(Rosegarden::EventSelection &selection);
 
     static QString getGlobalName() { return i18n("Make &Grace Notes"); }
 
@@ -306,10 +335,10 @@ protected:
     Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
 };
 
-class GroupMenuUnGraceCommand : public BasicSelectionCommand
+class AdjustMenuUnGraceCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuUnGraceCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuUnGraceCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection) { }
 
     static QString getGlobalName() { return i18n("Ung&race"); }
@@ -319,12 +348,12 @@ protected:
 };
 
 
-class GroupMenuAddIndicationCommand : public BasicCommand
+class NotesMenuAddIndicationCommand : public BasicCommand
 {
 public:
-    GroupMenuAddIndicationCommand(std::string indicationType,
+    NotesMenuAddIndicationCommand(std::string indicationType,
 				  Rosegarden::EventSelection &selection);
-    virtual ~GroupMenuAddIndicationCommand();
+    virtual ~NotesMenuAddIndicationCommand();
 
     // tests whether the indication can be added without overlapping
     // another one of the same type
@@ -348,10 +377,10 @@ protected:
 };
     
 
-class GroupMenuMakeChordCommand : public BasicSelectionCommand
+class AdjustMenuMakeChordCommand : public BasicSelectionCommand
 {
 public:
-    GroupMenuMakeChordCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuMakeChordCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -368,15 +397,15 @@ private:
 // Transforms menu commands
 
 
-class TransformsMenuNormalizeRestsCommand : public BasicCommand
+class AdjustMenuNormalizeRestsCommand : public BasicCommand
 {
 public:
-    TransformsMenuNormalizeRestsCommand(Rosegarden::Segment &s,
+    AdjustMenuNormalizeRestsCommand(Rosegarden::Segment &s,
 					Rosegarden::timeT startTime,
 					Rosegarden::timeT endTime) :
 	BasicCommand(getGlobalName(), s, startTime, endTime) { }
 
-    TransformsMenuNormalizeRestsCommand(Rosegarden::EventSelection &selection);
+    AdjustMenuNormalizeRestsCommand(Rosegarden::EventSelection &selection);
 
     static QString getGlobalName() { return i18n("&Normalize Rests"); }
 
@@ -385,15 +414,15 @@ protected:
 };
 
 
-class TransformsMenuCollapseRestsCommand : public BasicCommand
+class AdjustMenuCollapseRestsCommand : public BasicCommand
 {
 public:
-    TransformsMenuCollapseRestsCommand(Rosegarden::Segment &s,
+    AdjustMenuCollapseRestsCommand(Rosegarden::Segment &s,
 				       Rosegarden::timeT startTime,
 				       Rosegarden::timeT endTime) :
 	BasicCommand(getGlobalName(), s, startTime, endTime) { }
 
-    TransformsMenuCollapseRestsCommand(Rosegarden::EventSelection &selection);
+    AdjustMenuCollapseRestsCommand(Rosegarden::EventSelection &selection);
 
     static QString getGlobalName() { return i18n("&Collapse Rests"); }
 
@@ -401,32 +430,14 @@ protected:
     virtual void modifySegment();
 };
 
-
-class TransformsMenuCollapseNotesCommand : public BasicSelectionCommand
+class NotesMenuTieNotesCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuCollapseNotesCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuTieNotesCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
-    static QString getGlobalName() { return i18n("Collapse &Equal-Pitch Notes"); }
-
-protected:
-    virtual void modifySegment();
-
-private:
-    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
-};
-    
-
-class TransformsMenuTieNotesCommand : public BasicSelectionCommand
-{
-public:
-    TransformsMenuTieNotesCommand(Rosegarden::EventSelection &selection) :
-	BasicSelectionCommand(getGlobalName(), selection, true),
-	m_selection(&selection) { }
-
-    static QString getGlobalName() { return i18n("&Tie Equal-Pitch Notes"); }
+    static QString getGlobalName() { return i18n("&Tie"); }
 
 protected:
     virtual void modifySegment();
@@ -436,14 +447,14 @@ private:
 };
      
 
-class TransformsMenuUntieNotesCommand : public BasicSelectionCommand
+class AdjustMenuUntieNotesCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuUntieNotesCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuUntieNotesCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
-    static QString getGlobalName() { return i18n("&Untie Notes"); }
+    static QString getGlobalName() { return i18n("&Untie"); }
 
 protected:
     virtual void modifySegment();
@@ -454,14 +465,14 @@ private:
     
 
 /// MakeNotesViable works on a selection or entire segment
-class TransformsMenuMakeNotesViableCommand : public BasicSelectionCommand
+class AdjustMenuMakeNotesViableCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuMakeNotesViableCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuMakeNotesViableCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
-    TransformsMenuMakeNotesViableCommand(Rosegarden::Segment &segment) :
+    AdjustMenuMakeNotesViableCommand(Rosegarden::Segment &segment) :
 	BasicSelectionCommand(getGlobalName(), segment, true),
 	m_selection(0) { }
 
@@ -475,10 +486,10 @@ private:
 };
  
 /// MakeRegionViable works on part of a segment
-class TransformsMenuMakeRegionViableCommand : public BasicCommand
+class AdjustMenuMakeRegionViableCommand : public BasicCommand
 {
 public:
-    TransformsMenuMakeRegionViableCommand(Rosegarden::Segment &segment,
+    AdjustMenuMakeRegionViableCommand(Rosegarden::Segment &segment,
 					  Rosegarden::timeT startTime,
 					  Rosegarden::timeT endTime) :
 	BasicCommand(getGlobalName(), segment, startTime, endTime) { }
@@ -490,14 +501,14 @@ protected:
 };
     
 
-class TransformsMenuDeCounterpointCommand : public BasicSelectionCommand
+class AdjustMenuDeCounterpointCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuDeCounterpointCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuDeCounterpointCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
-    TransformsMenuDeCounterpointCommand(Rosegarden::Segment &segment) :
+    AdjustMenuDeCounterpointCommand(Rosegarden::Segment &segment) :
 	BasicSelectionCommand(getGlobalName(), segment, true),
 	m_selection(0) { }
 
@@ -511,10 +522,10 @@ private:
 };
   
 
-class TransformsMenuChangeStemsCommand : public BasicSelectionCommand
+class AdjustMenuChangeStemsCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuChangeStemsCommand(bool up, Rosegarden::EventSelection &selection) :
+    AdjustMenuChangeStemsCommand(bool up, Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(up), selection, true),
 	m_selection(&selection), m_up(up) { }
 
@@ -531,10 +542,10 @@ private:
 };
 
 
-class TransformsMenuRestoreStemsCommand : public BasicSelectionCommand
+class AdjustMenuRestoreStemsCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuRestoreStemsCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuRestoreStemsCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -548,10 +559,47 @@ private:
 };
 
 
-class TransformsMenuChangeStyleCommand : public BasicSelectionCommand
+class AdjustMenuChangeSlurPositionCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuChangeStyleCommand(NoteStyleName style,
+    AdjustMenuChangeSlurPositionCommand(bool above, Rosegarden::EventSelection &selection) :
+	BasicSelectionCommand(getGlobalName(above), selection, true),
+	m_selection(&selection), m_above(above) { }
+
+    static QString getGlobalName(bool above) {
+	return above ? i18n("Slur &Above") : i18n("Slur &Below");
+    }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    bool m_above;
+};
+
+
+class AdjustMenuRestoreSlursCommand : public BasicSelectionCommand
+{
+public:
+    AdjustMenuRestoreSlursCommand(Rosegarden::EventSelection &selection) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection) { }
+
+    static QString getGlobalName() { return i18n("&Restore Computed Slur Position"); }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+};
+
+
+class AdjustMenuChangeStyleCommand : public BasicSelectionCommand
+{
+public:
+    AdjustMenuChangeStyleCommand(NoteStyleName style,
 				     Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(style), selection, true),
 	m_selection(&selection), m_style(style) { }
@@ -587,10 +635,10 @@ private:
     int m_number;
 };    
 
-class MarksMenuAddMarkCommand : public BasicSelectionCommand
+class NotesMenuAddMarkCommand : public BasicSelectionCommand
 {
 public:
-    MarksMenuAddMarkCommand(Rosegarden::Mark mark,
+    NotesMenuAddMarkCommand(Rosegarden::Mark mark,
 			    Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(mark), selection, true),
 	m_selection(&selection), m_mark(mark) { }
@@ -606,10 +654,10 @@ private:
 };
 
 
-class MarksMenuAddTextMarkCommand : public BasicSelectionCommand
+class NotesMenuAddTextMarkCommand : public BasicSelectionCommand
 {
 public:
-    MarksMenuAddTextMarkCommand(std::string text,
+    NotesMenuAddTextMarkCommand(std::string text,
 				Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection), m_text(text) { }
@@ -625,10 +673,10 @@ private:
 };
 
 
-class MarksMenuAddFingeringMarkCommand : public BasicSelectionCommand
+class NotesMenuAddFingeringMarkCommand : public BasicSelectionCommand
 {
 public:
-    MarksMenuAddFingeringMarkCommand(std::string text,
+    NotesMenuAddFingeringMarkCommand(std::string text,
 				     Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection), m_text(text) { }
@@ -644,10 +692,10 @@ private:
 };
 
 
-class MarksMenuRemoveMarksCommand : public BasicSelectionCommand
+class NotesMenuRemoveMarksCommand : public BasicSelectionCommand
 {
 public:
-    MarksMenuRemoveMarksCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuRemoveMarksCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -660,10 +708,10 @@ private:
     Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
 };
 
-class MarksMenuRemoveFingeringMarksCommand : public BasicSelectionCommand
+class NotesMenuRemoveFingeringMarksCommand : public BasicSelectionCommand
 {
 public:
-    MarksMenuRemoveFingeringMarksCommand(Rosegarden::EventSelection &selection) :
+    NotesMenuRemoveFingeringMarksCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
 
@@ -676,10 +724,10 @@ private:
     Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
 };
 
-class TransformsMenuFixNotationQuantizeCommand : public BasicSelectionCommand
+class AdjustMenuFixNotationQuantizeCommand : public BasicSelectionCommand
 {
 public:
-    TransformsMenuFixNotationQuantizeCommand(Rosegarden::EventSelection &selection) :
+    AdjustMenuFixNotationQuantizeCommand(Rosegarden::EventSelection &selection) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
 	m_selection(&selection) { }
     
@@ -692,7 +740,23 @@ private:
     Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
 };
 
-class TransformsMenuInterpretCommand : public BasicSelectionCommand
+class AdjustMenuRemoveNotationQuantizeCommand : public BasicSelectionCommand
+{
+public:
+    AdjustMenuRemoveNotationQuantizeCommand(Rosegarden::EventSelection &selection) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection) { }
+    
+    static QString getGlobalName() { return i18n("Remo&ve Notation Quantization"); }
+    
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+};
+
+class AdjustMenuInterpretCommand : public BasicSelectionCommand
 {
 public:
     // bit masks: pass an OR of these to the constructor
@@ -704,7 +768,7 @@ public:
     static const int Articulate;         // slurs, marks, legato etc
     static const int AllInterpretations; // all of the above
 
-    TransformsMenuInterpretCommand(Rosegarden::EventSelection &selection,
+    AdjustMenuInterpretCommand(Rosegarden::EventSelection &selection,
 				   const Rosegarden::Quantizer *quantizer,
 				   int interpretations) :
 	BasicSelectionCommand(getGlobalName(), selection, true),
@@ -712,7 +776,7 @@ public:
 	m_quantizer(quantizer),
 	m_interpretations(interpretations) { }
 
-    virtual ~TransformsMenuInterpretCommand();
+    virtual ~AdjustMenuInterpretCommand();
 
     static QString getGlobalName() { return i18n("&Interpret..."); }
     
@@ -788,4 +852,59 @@ private:
     bool m_cautionary;
 };
 
+class IncrementDisplacementsCommand : public BasicSelectionCommand
+{
+public:
+    IncrementDisplacementsCommand(Rosegarden::EventSelection &selection,
+				  long dx, long dy) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection),
+	m_dx(dx),
+	m_dy(dy) { }
+
+    static QString getGlobalName() { return i18n("Fine Reposition"); }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    long m_dx;
+    long m_dy;
+};
+
+class ResetDisplacementsCommand : public BasicSelectionCommand
+{
+public:
+    ResetDisplacementsCommand(Rosegarden::EventSelection &selection) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection) { }
+
+    static QString getGlobalName() { return i18n("Restore Computed Positions"); }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+};
+
+class SetVisibilityCommand : public BasicSelectionCommand
+{
+public:
+    SetVisibilityCommand(Rosegarden::EventSelection &selection, bool visible) :
+	BasicSelectionCommand(getGlobalName(), selection, true),
+	m_selection(&selection),
+	m_visible(visible) { }
+
+    static QString getGlobalName() { return i18n("Set Visibility"); }
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::EventSelection *m_selection;// only used on 1st execute (cf bruteForceRedo)
+    bool m_visible;
+};
+
 #endif
