@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -84,6 +84,10 @@ NotationHLayout::NotationHLayout(Composition *c, NotePixmapFactory *npf,
     m_staffCount(0)
 {
 //    NOTATION_DEBUG << "NotationHLayout::NotationHLayout()" << endl;
+
+    KConfig *config = kapp->config();
+    config->setGroup("Notation Options");
+    m_keySigCancelMode = config->readNumEntry("keysigcancelmode", 1);
 }
 
 NotationHLayout::~NotationHLayout()
@@ -140,7 +144,7 @@ NotationHLayout::getBarData(Staff &staff) const
 
 
 NotationElementList::iterator
-NotationHLayout::getStartOfQuantizedSlice(const NotationElementList *notes,
+NotationHLayout::getStartOfQuantizedSlice(NotationElementList *notes,
 					  timeT t)
     const
 {
@@ -241,16 +245,30 @@ NotationHLayout::scanStaff(Staff &staff, timeT startTime, timeT endTime)
 
     NOTATION_DEBUG << "ottava shift at start:" << ottavaShift << ", ottavaEnd " << ottavaEnd << endl;
 
-    //!!! problematic: if we're scanning from the middle, we won't
-    //have recorded any accidentals in the previous bar for the
-    //desirable BarResetCautionary/BarResetNaturals accidental table
-    //modes
-    
-    Rosegarden::AccidentalTable::OctaveType octaveType = 
-	Rosegarden::AccidentalTable::OctavesCautionary;
+    KConfig *config = kapp->config();
+    config->setGroup("Notation Options");
 
+    int accOctaveMode = config->readNumEntry("accidentaloctavemode", 1);
+    Rosegarden::AccidentalTable::OctaveType octaveType = 
+	(accOctaveMode == 0 ? Rosegarden::AccidentalTable::OctavesIndependent :
+	 accOctaveMode == 1 ? Rosegarden::AccidentalTable::OctavesCautionary :
+	 Rosegarden::AccidentalTable::OctavesEquivalent);
+
+    int accBarMode = config->readNumEntry("accidentalbarmode", 0);
     Rosegarden::AccidentalTable::BarResetType barResetType =
-	Rosegarden::AccidentalTable::BarResetCautionary;
+	(accBarMode == 0 ? Rosegarden::AccidentalTable::BarResetNone :
+	 accBarMode == 1 ? Rosegarden::AccidentalTable::BarResetCautionary :
+	 Rosegarden::AccidentalTable::BarResetExplicit);
+
+    bool showInvisibles = config->readBoolEntry("showinvisibles", true);
+
+    if (barResetType != Rosegarden::AccidentalTable::BarResetNone) {
+	//!!! very crude and expensive way of making sure we see the
+	// accidentals from previous bar:
+	if (startBarNo > segment.getComposition()->getBarNumber(segment.getStartTime())) {
+	    --startBarNo;
+	}
+    }
 
     Rosegarden::AccidentalTable accTable(key, clef, octaveType, barResetType);
 
@@ -324,6 +342,11 @@ NotationHLayout::scanStaff(Staff &staff, timeT startTime, timeT endTime)
 		}
 	    }
 
+	    bool invisible = false;
+	    if (el->event()->get<Bool>(INVISIBLE, invisible) && invisible) {
+		if (!showInvisibles) continue;
+	    }
+
 	    if (el->event()->has(BEAMED_GROUP_ID)) {
 		NOTATION_DEBUG << "element is beamed" << endl;
 		long groupId = el->event()->get<Int>(BEAMED_GROUP_ID);
@@ -491,6 +514,7 @@ NotationHLayout::setBarSizeData(Staff &staff,
     i->second.sizeData.actualDuration = actualDuration;
     i->second.sizeData.idealWidth = 0.0;
     i->second.sizeData.reconciledWidth = 0.0;
+    i->second.sizeData.clefKeyWidth = 0;
     i->second.sizeData.fixedWidth = fixedWidth;
 }
 
@@ -769,7 +793,13 @@ NotationHLayout::preSquishBar(int barNo)
 	if (bdli != bdl.end()) {
 
 	    bdli->second.sizeData.idealWidth =
-		bdli->second.sizeData.fixedWidth + x + getBarMargin();
+		bdli->second.sizeData.fixedWidth + x;
+
+	    if (!bdli->second.basicData.timeSignature.hasHiddenBars()) {
+		bdli->second.sizeData.idealWidth += getBarMargin();
+	    } else if (bdli->second.basicData.newTimeSig) {
+		bdli->second.sizeData.idealWidth += getPostBarMargin();
+	    }
 
 	    bdli->second.sizeData.reconciledWidth =
 		bdli->second.sizeData.idealWidth;
@@ -809,6 +839,40 @@ NotationHLayout::getStaffWithWidestBar(int barNo)
     return widest;
 }
 
+int
+NotationHLayout::getMaxRepeatedClefAndKeyWidth(int barNo)
+{
+    int max = 0;
+
+    timeT barStart = 0;
+
+    for (BarDataMap::iterator mi = m_barData.begin();
+	 mi != m_barData.end(); ++mi) {
+
+	Staff *staff = mi->first;
+	if (mi == m_barData.begin()) {
+	    barStart = staff->getSegment().getComposition()->getBarStart(barNo);
+	}
+
+	timeT t;
+	int w = 0;
+
+	Clef clef = staff->getSegment().getClefAtTime(barStart, t);
+	if (t < barStart) w += m_npf->getClefWidth(clef);
+
+	Rosegarden::Key key = staff->getSegment().getKeyAtTime(barStart, t);
+	if (t < barStart) w += m_npf->getKeyWidth(key);
+
+	if (w > max) max = w;
+    }
+
+    NOTATION_DEBUG << "getMaxRepeatedClefAndKeyWidth(" << barNo << "): " << max
+		   << endl;
+
+    if (max > 0) return max + getFixedItemSpacing() * 2;
+    else return 0;
+}
+
 void
 NotationHLayout::reconcileBarsLinear()
 {
@@ -982,8 +1046,24 @@ NotationHLayout::reconcileBarsPage()
 	    rowData.push_back(std::pair<int, double>(barNoThisRow,
 						     pageWidthSoFar));
 	    barNoThisRow = 1;
-	    pageWidthSoFar = maxWidth;
-	    stretchFactor = m_pageWidth / maxWidth;
+
+	    // When we start a new row, we always need to allow for the
+	    // repeated clef and key at the start of it.
+	    int maxClefKeyWidth = getMaxRepeatedClefAndKeyWidth(barNo);
+
+	    for (BarDataMap::iterator i = m_barData.begin();
+		 i != m_barData.end(); ++i) {
+		
+		BarDataList &list = i->second;
+		BarDataList::iterator bdli = list.find(barNo);
+		
+		if (bdli != list.end()) {
+		    bdli->second.sizeData.clefKeyWidth = maxClefKeyWidth;
+		}
+	    }
+
+	    pageWidthSoFar = maxWidth + maxClefKeyWidth;
+	    stretchFactor = m_pageWidth / pageWidthSoFar;
 	} else {
 	    ++barNoThisRow;
 	    pageWidthSoFar = nextPageWidth;
@@ -1023,16 +1103,17 @@ NotationHLayout::reconcileBarsPage()
 		if (barNo >= getLastVisibleBar()) break; // yes
 		else maxWidth = stretchFactor * (m_spacing / 3);
 	    } else {
-		maxWidth =
-		    (stretchFactor *
-		     m_barData[widest].find(barNo)->
-		     second.sizeData.idealWidth);
+		BarData &bd = m_barData[widest].find(barNo)->second;
+		maxWidth = (stretchFactor * bd.sizeData.idealWidth) +
+		    bd.sizeData.clefKeyWidth;
+		NOTATION_DEBUG << "setting maxWidth to " << (stretchFactor * bd.sizeData.idealWidth) << " + " << bd.sizeData.clefKeyWidth << " = " << maxWidth << endl;
 	    }
 
 	    if (barNoThisRow == finalBarThisRow) {
 		if (!finalRow ||
 		    (maxWidth > (m_pageWidth - pageWidthSoFar))) {
 		    maxWidth = m_pageWidth - pageWidthSoFar;
+		    NOTATION_DEBUG << "reset maxWidth to " << m_pageWidth << " - " << pageWidthSoFar << " = " << maxWidth << endl;
 		}
 	    }
 
@@ -1128,12 +1209,16 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 	(isFullLayout && (notes->begin() != notes->end())) ?
 	(*notes->begin())->getViewAbsoluteTime() : startTime;
 
-    Rosegarden::Key key = notationStaff.getSegment().getKeyAtTime(lastIncrement);;
+    Rosegarden::Key key = notationStaff.getSegment().getKeyAtTime(lastIncrement);
     Clef clef = notationStaff.getSegment().getClefAtTime(lastIncrement);
     TimeSignature timeSignature;
 
     int startBar = getComposition()->getBarNumber(startTime);
 
+    KConfig *config = kapp->config();
+    config->setGroup("Notation Options");
+    bool showInvisibles = config->readBoolEntry("showinvisibles", true);
+
     for (BarPositionList::iterator bpi = m_barPositions.begin();
 	 bpi != m_barPositions.end(); ++bpi) {
 
@@ -1203,26 +1288,45 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 	    timeSignature = bdi->second.basicData.timeSignature;
 	    timeSigToPlace = !bdi->second.basicData.timeSignature.isHidden();
 	}
-
         if (timeSigToPlace) {
 	    NOTATION_DEBUG << "NotationHLayout::layout(): there's a time sig in this bar" << endl;
 	}
 
+	bool repeatClefAndKey = false;
+	if (bdi->second.sizeData.clefKeyWidth > 0) {
+	    repeatClefAndKey = true;
+	}
+	if (repeatClefAndKey) {
+	    NOTATION_DEBUG << "NotationHLayout::layout(): need to repeat clef & key in this bar" << endl;
+	}
+
+	double barInset = notationStaff.getBarInset(barNo, repeatClefAndKey);
+
 	NotationElement *lastDynamicText = 0;
 	int count = 0;
 
+	double offset = 0.0;
+	double reconciledWidth = bdi->second.sizeData.reconciledWidth;
+
+	if (repeatClefAndKey) {
+	    offset = bdi->second.sizeData.clefKeyWidth;
+	    reconciledWidth -= offset;
+	}
+
+	if (bdi->second.basicData.newTimeSig ||
+	    !bdi->second.basicData.timeSignature.hasHiddenBars()) {
+	    offset += getPostBarMargin();
+	}
+
 	ChunkList &chunks = bdi->second.chunks;
 	ChunkList::iterator chunkitr = chunks.begin();
 	double reconcileRatio = 1.0;
 	if (bdi->second.sizeData.idealWidth > 0.0) {
-	    reconcileRatio =
-		bdi->second.sizeData.reconciledWidth /
-		bdi->second.sizeData.idealWidth;
+	    reconcileRatio = reconciledWidth / bdi->second.sizeData.idealWidth;
 	}
 
 	NOTATION_DEBUG << "have " << chunks.size() << " chunks, reconciledWidth " << bdi->second.sizeData.reconciledWidth << ", idealWidth " << bdi->second.sizeData.idealWidth << ", ratio " << reconcileRatio << endl;
 
-	double offset = getPostBarMargin();
 	double delta = 0;
 
         for (NotationElementList::iterator it = from; it != to; ++it) {
@@ -1231,7 +1335,18 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 	    delta = 0;
 	    float fixed = 0;
 
-	    NOTATION_DEBUG << "element is a " << el->event()->getType() << endl;
+	    if (el->event()->isa(Note::EventType)) {
+		long pitch = 0;
+		el->event()->get<Int>(PITCH, pitch);
+		NOTATION_DEBUG << "element is a " << el->event()->getType() << " (pitch " << pitch << ")" << endl;
+	    } else {
+		NOTATION_DEBUG << "element is a " << el->event()->getType() << endl;
+	    }
+
+	    bool invisible = false;
+	    if (el->event()->get<Bool>(INVISIBLE, invisible) && invisible) {
+		if (!showInvisibles) continue;
+	    }
 
 	    if (chunkitr != chunks.end()) {
 		NOTATION_DEBUG << "new chunk: addr " << &(*chunkitr) << " duration=" << (*chunkitr).duration << " subordering=" << (*chunkitr).subordering << " fixed=" << (*chunkitr).fixed << " stretchy=" << (*chunkitr).stretchy << " x=" << (*chunkitr).x << endl;
@@ -1242,13 +1357,13 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 		if (++chunkscooter != chunks.end()) {
 		    delta = (*chunkscooter).x - (*chunkitr).x;
 		} else {
-		    delta = bdi->second.sizeData.reconciledWidth -
+		    delta = reconciledWidth -
 			bdi->second.sizeData.fixedWidth - (*chunkitr).x;
 		}
 		delta *= reconcileRatio;
 		++chunkitr;
 	    } else {
-		x = barX + bdi->second.sizeData.reconciledWidth - getPreBarMargin();
+		x = barX + reconciledWidth - getPreBarMargin();
 		delta = 0;
 	    }
 
@@ -1265,15 +1380,37 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 		timeSigToPlace = false;
 	    }
 
+	    if (barInset >= 1.0) {
+		if (el->event()->isa(Rosegarden::Clef::EventType) ||
+		    el->event()->isa(Rosegarden::Key::EventType)) {
+		    NOTATION_DEBUG << "Pulling clef/key back by " << getPreBarMargin() << endl;
+		    x -= getPostBarMargin() * 2 / 3;
+		} else {
+		    barInset = 0.0;
+		}
+	    }
+
             NOTATION_DEBUG << "NotationHLayout::layout(): setting element's x to " << x << " (was " << el->getLayoutX() << ")" << endl;
 
-	    el->setLayoutX(x);
+	    double displacedX = 0.0;
+	    long dxRaw = 0;
+	    el->event()->get<Int>(DISPLACED_X, dxRaw);
+	    displacedX = double(dxRaw * m_npf->getNoteBodyWidth()) / 1000.0;
+
+	    el->setLayoutX(x + displacedX);
 	    el->setLayoutAirspace(x, int(delta));
 
+	    // #704958 (multiple tuplet spanners created when entering
+	    // triplet chord) -- only do this here for non-notes,
+	    // notes get it from positionChord
+	    if (!el->isNote()) {
+		sampleGroupElement(staff, clef, key, it);
+	    }
+
 	    if (el->isNote()) {
 
 		// This modifies "it" and "tieMap"
-		positionChord(staff, it, bdi, timeSignature, clef, key, tieMap, to);
+		positionChord(staff, it, clef, key, tieMap, to);
 
 	    } else if (el->isRest()) {
 
@@ -1322,7 +1459,7 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 			m_npf->getNoteBodyWidth() / 4;
 		}
 
-		el->setLayoutX(ix);
+		el->setLayoutX(ix + displacedX);
 		el->setLayoutAirspace(ix, delta - (ix - x));
 
 	    } else {    
@@ -1330,31 +1467,6 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 		// nothing else
 	    }
 
-	    if (it != to && el->event()->has(BEAMED_GROUP_ID)) {
-
-		//!!! Gosh.  We need some clever logic to establish
-		// whether one group is happening while another has
-		// not yet ended -- perhaps we decide one has ended
-		// if we see another, and then re-open the case of
-		// the first if we meet another note that claims to
-		// be in it.  Then we need to hint to both of the
-		// groups that they should choose appropriate stem
-		// directions -- we could just use HEIGHT_ON_STAFF
-		// of their first notes to determine this, as if that
-		// doesn't work, nothing will
-
-		long groupId = el->event()->get<Int>(BEAMED_GROUP_ID);
-//		NOTATION_DEBUG << "group id: " << groupId << endl;
-		if (m_groupsExtant.find(groupId) == m_groupsExtant.end()) {
-//		    NOTATION_DEBUG << "(new group)" << endl;
-		    m_groupsExtant[groupId] =
-			new NotationGroup(*staff.getViewElementList(),
-					  m_notationQuantizer,
-					  m_properties, clef, key);
-		}
-		m_groupsExtant[groupId]->sample(it, true);
-	    }
-
 	    if (m_timePerProgressIncrement > 0 && (++count == 100)) {
 		count = 0;
 		timeT sinceIncrement = el->getViewAbsoluteTime() - lastIncrement;
@@ -1371,7 +1483,7 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
 
 	if (timeSigToPlace) {
 	    // no other events in this bar, so we never managed to place it
-	    x = x + delta - m_npf->getTimeSigWidth(bdi->second.basicData.timeSignature);
+	    x = barX + offset;
 	    NOTATION_DEBUG << "Placing timesig reluctantly at " << x << endl;
 	    bdi->second.layoutData.timeSigX = (int)(x);
 	    timeSigToPlace = false;
@@ -1389,6 +1501,37 @@ NotationHLayout::layout(BarDataMap::iterator i, timeT startTime, timeT endTime)
     }
 }
 
+void
+NotationHLayout::sampleGroupElement(Staff &staff,
+				    const Clef &clef,
+				    const Rosegarden::Key &key,
+				    const NotationElementList::iterator &itr)
+{
+    NotationElement *el = static_cast<NotationElement *>(*itr);
+
+    if (el->event()->has(BEAMED_GROUP_ID)) {
+
+	//!!! Gosh.  We need some clever logic to establish whether
+	// one group is happening while another has not yet ended --
+	// perhaps we decide one has ended if we see another, and then
+	// re-open the case of the first if we meet another note that
+	// claims to be in it.  Then we need to hint to both of the
+	// groups that they should choose appropriate stem directions
+	// -- we could just use HEIGHT_ON_STAFF of their first notes
+	// to determine this, as if that doesn't work, nothing will
+
+	long groupId = el->event()->get<Int>(BEAMED_GROUP_ID);
+	NOTATION_DEBUG << "group id: " << groupId << endl;
+	if (m_groupsExtant.find(groupId) == m_groupsExtant.end()) {
+	    NOTATION_DEBUG << "(new group)" << endl;
+	    m_groupsExtant[groupId] =
+		new NotationGroup(*staff.getViewElementList(),
+				  m_notationQuantizer,
+				  m_properties, clef, key);
+	}
+	m_groupsExtant[groupId]->sample(itr, true);
+    }
+}
 
 timeT
 NotationHLayout::getSpacingDuration(Staff &staff,
@@ -1430,8 +1573,6 @@ NotationHLayout::getSpacingDuration(Staff &staff,
 void
 NotationHLayout::positionChord(Staff &staff,
                                NotationElementList::iterator &itr,
-				const BarDataList::iterator &, //!!!unreqd?
-				const TimeSignature &, //!!! unreqd?
 			       const Clef &clef, const Rosegarden::Key &key,
 			       TieMap &tieMap, 
 			       NotationElementList::iterator &to)
@@ -1456,14 +1597,25 @@ NotationHLayout::positionChord(Staff &staff,
 	
 	if (citr == to) barEndsInChord = true;
 
-	NotationElement *elt = static_cast<NotationElement*>(*citr);//!!!
+	// #704958 (multiple tuplet spanners created when entering
+	// triplet chord) -- layout() updates the beamed group data
+	// for non-notes, but we have to do it for notes so as to
+	// ensure every note in the chord is accounted for
+	sampleGroupElement(staff, clef, key, citr);
 
-	elt->setLayoutX(baseX);
+	NotationElement *elt = static_cast<NotationElement*>(*citr);
+
+	double displacedX = 0.0;
+	long dxRaw = 0;
+	elt->event()->get<Int>(DISPLACED_X, dxRaw);
+	displacedX = double(dxRaw * m_npf->getNoteBodyWidth()) / 1000.0;
+
+	elt->setLayoutX(baseX + displacedX);
 	elt->setLayoutAirspace(baseX, delta);
 
 	NOTATION_DEBUG << "NotationHLayout::positionChord: assigned x to elt at " << elt->getViewAbsoluteTime() << endl;
 
-	if (citr == chord.getFinalElement()) break;//!!!
+	if (citr == chord.getFinalElement()) break;
     }
 
     // Check for any ties going back, and if so work out how long they
@@ -1472,9 +1624,9 @@ NotationHLayout::positionChord(Staff &staff,
     for (NotationElementList::iterator citr = chord.getInitialElement();
 	 citr != staff.getViewElementList()->end(); ++citr) {
 	
-	NotationElement *note = static_cast<NotationElement*>(*citr);//!!!
+	NotationElement *note = static_cast<NotationElement*>(*citr);
 	if (!note->isNote()) {
-	    if (citr == chord.getFinalElement()) break; //!!!
+	    if (citr == chord.getFinalElement()) break;
 	    continue;
 	}
 
@@ -1484,6 +1636,7 @@ NotationHLayout::positionChord(Staff &staff,
 	note->event()->get<Bool>(TIED_FORWARD,  tiedForwards);
 	note->event()->get<Bool>(TIED_BACKWARD, tiedBack);
 
+	if (!note->event()->has(PITCH)) continue;
 	int pitch = note->event()->get<Int>(PITCH);
 
 	if (tiedBack) {
@@ -1531,7 +1684,7 @@ NotationHLayout::getLayoutWidth(Rosegarden::ViewElement &ve,
 {
     NotationElement& e = static_cast<NotationElement&>(ve);
 
-    if (e.isNote() || e.isRest()) {
+    if ((e.isNote() || e.isRest()) && e.event()->has(NOTE_TYPE)) {
 
         long noteType = e.event()->get<Int>(NOTE_TYPE);
 	long dots = 0;
@@ -1571,20 +1724,31 @@ NotationHLayout::getLayoutWidth(Rosegarden::ViewElement &ve,
 
 	    Rosegarden::Key key(*e.event());
 
-	    if (key.getAccidentalCount() == 0) {
-		w += m_npf->getKeyWidth(previousKey, true);
-	    } else {
-		w += m_npf->getKeyWidth(key);
+	    Rosegarden::Key cancelKey = previousKey;
+
+	    if (m_keySigCancelMode == 0) { // only when entering C maj / A min
+
+		if (key.getAccidentalCount() != 0) cancelKey = Rosegarden::Key();
+
+	    } else if (m_keySigCancelMode == 1) { // only when reducing acc count
+
+		if (!(key.isSharp() == cancelKey.isSharp() &&
+		      key.getAccidentalCount() < cancelKey.getAccidentalCount())) {
+		    cancelKey = Rosegarden::Key();
+		}
 	    }
 
+	    w += m_npf->getKeyWidth(key, cancelKey);
+
 	} else if (e.event()->isa(Indication::EventType) ||
 		   e.event()->isa(Text::EventType)) {
 
 	    w = 0;
 
 	} else {
-	    NOTATION_DEBUG << "NotationHLayout::getLayoutWidth(): no case for event type " << e.event()->getType() << endl;
-	    w += 24;
+//	    NOTATION_DEBUG << "NotationHLayout::getLayoutWidth(): no case for event type " << e.event()->getType() << endl;
+//	    w += 24;
+	    w = 0;
 	}
 
 	return w;
@@ -1792,7 +1956,6 @@ NotationHLayout::getXForTimeByEvent(Rosegarden::timeT time)
 		if (vli == staff->getViewElementList()->end()) break;
 		NotationElement *element = static_cast<NotationElement *>(*vli);
 		if (element->getCanvasItem()) {
-//!!!		    && (element->isNote() || element->isRest())) {
 		    x = element->getLayoutX();
 		    double temp;
 		    element->getLayoutAirspace(temp, dx);
