@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -262,6 +262,7 @@ protected:
 	    float idealWidth;    // theoretical width of bar following barline
 	    float reconciledWidth;
 	    float fixedWidth;       // width of non-chunk items in bar
+	    int clefKeyWidth;
 	    Rosegarden::timeT actualDuration; // may exceed nominal duration
 
 	} sizeData;
@@ -283,7 +284,7 @@ protected:
 	    sizeData.idealWidth = 0;
 	    sizeData.reconciledWidth = 0;
 	    sizeData.fixedWidth = 0;
-//	    sizeData.baseWidth = 0;
+	    sizeData.clefKeyWidth = 0;
 	    sizeData.actualDuration = 0;
 	    layoutData.needsLayout = true;
 	    layoutData.x = -1;
@@ -329,6 +330,9 @@ protected:
     /// Find the staff in which bar "barNo" is widest
     Rosegarden::Staff *getStaffWithWidestBar(int barNo);
 
+    /// Find width of clef+key in the staff in which they're widest in this bar
+    int getMaxRepeatedClefAndKeyWidth(int barNo);
+
     /// For a single bar, makes sure synchronisation points align in all staves
     void preSquishBar(int barNo);
 
@@ -344,7 +348,7 @@ protected:
     
     /// Find earliest element with quantized time of t or greater
     NotationElementList::iterator getStartOfQuantizedSlice 
-    (const NotationElementList *, Rosegarden::timeT t) const;
+    (NotationElementList *, Rosegarden::timeT t) const;
 
     void scanChord
     (NotationElementList *notes, NotationElementList::iterator &i,
@@ -361,10 +365,13 @@ protected:
     // note within the chord
     void positionChord
     (Rosegarden::Staff &staff, 
-     NotationElementList::iterator &, const BarDataList::iterator &,
-     const Rosegarden::TimeSignature &, const Rosegarden::Clef &clef,
+     NotationElementList::iterator &, const Rosegarden::Clef &clef,
      const Rosegarden::Key &key, TieMap &, NotationElementList::iterator &to);
 
+    void sampleGroupElement
+    (Rosegarden::Staff &staff, const Rosegarden::Clef &clef,
+     const Rosegarden::Key &key, const NotationElementList::iterator &);
+
     /// Difference between absolute time of next event and of this
     Rosegarden::timeT getSpacingDuration
     (Rosegarden::Staff &staff, const NotationElementList::iterator &);
@@ -393,6 +400,7 @@ protected:
     double m_pageWidth;
     int m_spacing;
     int m_proportion;
+    int m_keySigCancelMode;
     NotePixmapFactory *m_npf;
 
     static std::vector<int> m_availableSpacings;
