@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -29,6 +29,7 @@ const Rosegarden::PropertyName NotationProperties::SLASHES	      = "Slashes";
 const Rosegarden::PropertyName NotationProperties::STEM_UP	      = "NoteStemUp";
 const Rosegarden::PropertyName NotationProperties::USE_CAUTIONARY_ACCIDENTAL = "UseCautionaryAccidental";
 const Rosegarden::PropertyName NotationProperties::OTTAVA_SHIFT       = "OttavaShift";
+const Rosegarden::PropertyName NotationProperties::SLUR_ABOVE	      = "SlurAbove";
 
 
 NotationProperties::NotationProperties(const std::string &prefix) :
@@ -50,11 +51,11 @@ NotationProperties::NotationProperties(const std::string &prefix) :
     CHORD_PRIMARY_NOTE		(prefix + "ChordPrimaryNote"),
     CHORD_MARK_COUNT		(prefix + "ChordMarkCount"),
     TIE_LENGTH			(prefix + "TieLength"),
-    SLUR_ABOVE			(prefix + "SlurAbove"),
     SLUR_Y_DELTA		(prefix + "SlurYDelta"),
     SLUR_LENGTH			(prefix + "SlurLength"),
     LYRIC_EXTRA_WIDTH		(prefix + "LyricExtraWidth"),
     REST_TOO_SHORT              (prefix + "RestTooShort"),
+    REST_OUTSIDE_STAVE          (prefix + "RestOutsideStave"),
 
     BEAM_GRADIENT		(prefix + "BeamGradient"),
     BEAM_SECTION_WIDTH		(prefix + "BeamSectionWidth"),
