@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -40,7 +40,6 @@ using Rosegarden::Clef;
 using Rosegarden::Key;
 using Rosegarden::Note;
 using Rosegarden::Mark;
-using Rosegarden::Marks;
 using Rosegarden::Segment;
 using Rosegarden::Equation;
 using Rosegarden::Quantizer;
@@ -61,7 +60,7 @@ AbstractSet<NotationElement, NotationElementList>::getAsEvent(const NotationElem
 
 }
 
-NotationChord::NotationChord(const NotationElementList &c,
+NotationChord::NotationChord(NotationElementList &c,
 			     NotationElementList::iterator i,
 			     const Rosegarden::Quantizer *quantizer,
 			     const NotationProperties &properties,
@@ -108,7 +107,8 @@ NotationChord::hasStem() const
     
     Iterator i(getInitialNote());
     for (;;) {
-	Note::Type note = getAsEvent(i)->get<Int>(NOTE_TYPE);
+	long note;
+	if (!getAsEvent(i)->get<Int>(NOTE_TYPE, note)) return true;
 	if (NoteStyleFactory::getStyleForEvent(getAsEvent(i))->hasStem(note))
 	    return true;
 	if (i == getFinalNote()) return false;
@@ -232,7 +232,8 @@ NotationChord::applyAccidentalShiftProperties()
     }
 
     int lastShift = minShift;
-    int lastHeight = 0, maxHeight = 0;
+    int lastHeight = 0, maxHeight = 999;
+    int lastWidth = 1;
 
     for (iterator i = end(); i != begin(); ) {
 
@@ -249,6 +250,10 @@ NotationChord::applyAccidentalShiftProperties()
 	}
     }
 
+    if (maxHeight == 999) {
+	return;
+    }
+
     for (iterator i = begin(); i != end(); ++i) {
 
 	Event *e = getAsEvent(*i);
@@ -260,32 +265,39 @@ NotationChord::applyAccidentalShiftProperties()
 	}
 
 	Rosegarden::Accidental acc;
+
 	if (e->get<String>(m_properties.DISPLAY_ACCIDENTAL, acc) &&
 	    acc != Rosegarden::Accidentals::NoAccidental) {
 
+	    int shift = lastShift;
+
 	    if (height < lastHeight) { // lastHeight was the first, up top
-		if (lastHeight - height >= 6) {
-		    e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, lastShift);
-		} else {
-		    e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, lastShift+1);
-		    lastShift = lastShift+1;
+		if (lastHeight - height < 6) {
+		    shift = lastShift + lastWidth;
 		}
 	    } else {
 		if (height - lastHeight >= 6) {
 		    if (maxHeight - height >= 6) {
-			e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, minShift);
-			lastShift = minShift;
+			shift = minShift;
 		    } else {
-			e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, minShift+1);
-			lastShift = minShift+1;
+			shift = minShift + 1;
 		    }
 		} else {
-		    e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, lastShift+1);
-		    lastShift = lastShift+1;
+		    shift = lastShift + lastWidth;
 		}
 	    }
 
+	    e->setMaybe<Int>(m_properties.ACCIDENTAL_SHIFT, shift);
+
 	    lastHeight = height;
+	    lastShift = shift;
+
+	    lastWidth = 1;
+	    bool c = false;
+	    if (e->get<Bool>(m_properties.DISPLAY_ACCIDENTAL_IS_CAUTIONARY, c)
+		&& c) {
+		lastWidth = 2;
+	    }
 	}
     }
 }
@@ -324,7 +336,7 @@ NotationChord::getAccidentalShift(const Iterator &i, bool &extra) const
 
 //////////////////////////////////////////////////////////////////////
 
-NotationGroup::NotationGroup(const NotationElementList &nel,
+NotationGroup::NotationGroup(NotationElementList &nel,
                              NELIterator i, const Quantizer *q,
 			     std::pair<timeT, timeT> barRange,
 			     const NotationProperties &p,
@@ -359,7 +371,7 @@ NotationGroup::NotationGroup(const NotationElementList &nel,
     */
 }
 
-NotationGroup::NotationGroup(const NotationElementList &nel,
+NotationGroup::NotationGroup(NotationElementList &nel,
 			     const Quantizer *q,
 			     const NotationProperties &p,
 			     const Clef &clef, const Key &key) :
@@ -450,8 +462,8 @@ NotationGroup::sample(const NELIterator &i, bool goingForwards)
     // e.g. BEAM_GRADIENT to indicate that a beam should be drawn;
     // it's possible the gradient might be left over from a previous
     // calculation and the group might have changed since.  Instead it
-    // should test BEAM_NECESSARY, which may be false even if there
-    // is a gradient present.
+    // should test BEAMED, which may be false even if there is a
+    // gradient present.
     (*i)->event()->setMaybe<Bool>(NotationProperties::BEAMED, false);
 
     int h = height(i);
@@ -556,6 +568,7 @@ NotationGroup::applyStemProperties()
 	el->event()->setMaybe<Bool>(NotationProperties::BEAM_ABOVE, aboveNotes);
         
         if (el->isNote() &&
+	    el->event()->has(NOTE_TYPE) &&
 	    el->event()->get<Int>(NOTE_TYPE) < Note::Crotchet &&
 	    el->event()->has(BEAMED_GROUP_ID) &&
 	    el->event()->get<Int>(BEAMED_GROUP_ID) == m_groupNo) {
@@ -588,6 +601,7 @@ NotationGroup::haveInternalRest() const
         NotationElement* el = static_cast<NotationElement*>(*i);
 
 	if (el->isNote() &&
+	    el->event()->has(NOTE_TYPE) &&
 	    el->event()->get<Int>(NOTE_TYPE) < Note::Crotchet &&
 	    el->event()->has(BEAMED_GROUP_ID) &&
 	    el->event()->get<Int>(BEAMED_GROUP_ID) == m_groupNo) {
@@ -820,13 +834,13 @@ void
 NotationGroup::applyBeam(NotationStaff &staff)
 {
 //    NOTATION_DEBUG << "NotationGroup::applyBeam, group no is " << m_groupNo << endl;
-/*!!!
+/*
     NOTATION_DEBUG << "\nNotationGroup::applyBeam" << endl;
     NOTATION_DEBUG << "Group id: " << m_groupNo << ", type " << m_type << endl;
     NOTATION_DEBUG << "Coverage:" << endl;
     int i = 0;
     for (NELIterator i = getInitialElement(); i != getContainer().end(); ++i) {
-	(*i)->event()->dump(NOTATION_DEBUG);
+	(*i)->event()->dump(cerr);
 	if (i == getFinalElement()) break;
     }
     {
@@ -846,11 +860,11 @@ NotationGroup::applyBeam(NotationStaff &staff)
 	NOTATION_DEBUG << "Lowest note: " << (i == getContainer().end() ? -1 : (*i)->event()->getAbsoluteTime()) << endl;
     }
 */
-
     Beam beam(calculateBeam(staff));
     if (!beam.necessary) {
 	for (NELIterator i = getInitialNote(); i != getContainer().end(); ++i) {
 	    (*i)->event()->unset(NotationProperties::BEAMED);
+	    (*i)->event()->unset(m_properties.TUPLING_LINE_MY_Y);
 	    if (i == getFinalNote()) break;
 	}
 	return;
@@ -891,8 +905,16 @@ NotationGroup::applyBeam(NotationStaff &staff)
 
     for (NELIterator i = getInitialNote(); i != getContainer().end(); ++i) {
         NotationElement* el = static_cast<NotationElement*>(*i);
+
+	// Clear tuplingness for all events in the group, to be
+	// reinstated by any subsequent call to applyTuplingLine.  We
+	// do this because applyTuplingLine doesn't clear these
+	// properties from notes that don't need them; it only applies
+	// them to notes that do.
+	el->event()->unset(m_properties.TUPLING_LINE_MY_Y);
         
         if (el->isNote() &&
+	    el->event()->has(NOTE_TYPE) &&
 	    el->event()->get<Int>(NOTE_TYPE) < Note::Crotchet &&
 	    el->event()->has(BEAMED_GROUP_ID) &&
 	    el->event()->get<Int>(BEAMED_GROUP_ID) == m_groupNo) {
@@ -1139,7 +1161,7 @@ NotationGroup::applyTuplingLine(NotationStaff &staff)
 		if (bc > maxEndBeamCount) maxEndBeamCount = bc;
 	    }
 
-	    int extraBeamSpace = maxEndBeamCount * nh;
+	    int extraBeamSpace = maxEndBeamCount * nh + nh/2;
 
 	    if (beam.aboveNotes) {
 		startY -= extraBeamSpace; endY -= extraBeamSpace;
