@@ -4,7 +4,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -53,7 +53,6 @@
 #include "colours.h"
 
 using Rosegarden::Accidental;
-using Rosegarden::Accidentals;
 using Rosegarden::Event;
 using Rosegarden::EventSelection;
 using Rosegarden::Clef;
@@ -164,7 +163,9 @@ NoteInserter::NoteInserter(NotationView* view)
       m_noteType(Rosegarden::Note::Quaver),
       m_noteDots(0),
       m_autoBeam(true),
-      m_accidental(Accidentals::NoAccidental)
+      m_accidental(Rosegarden::Accidentals::NoAccidental),
+      m_lastAccidental(Rosegarden::Accidentals::NoAccidental),
+      m_followAccidental(false)
 {
     QIconSet icon;
 
@@ -220,8 +221,8 @@ NoteInserter::NoteInserter(NotationView* view)
 
     createMenu("noteinserter.rc");
 
-    connect(m_parentView, SIGNAL(changeAccidental(Rosegarden::Accidental)),
-            this,         SLOT(slotSetAccidental(Rosegarden::Accidental)));
+    connect(m_parentView, SIGNAL(changeAccidental(Rosegarden::Accidental, bool)),
+            this,         SLOT(slotSetAccidental(Rosegarden::Accidental, bool)));
 }
 
 NoteInserter::NoteInserter(const QString& menuName, NotationView* view)
@@ -230,10 +231,12 @@ NoteInserter::NoteInserter(const QString& menuName, NotationView* view)
       m_noteDots(0),
       m_autoBeam(false),
       m_clickHappened(false),
-      m_accidental(Accidentals::NoAccidental)
+      m_accidental(Rosegarden::Accidentals::NoAccidental),
+      m_lastAccidental(Rosegarden::Accidentals::NoAccidental),
+      m_followAccidental(false)
 {
-    connect(m_parentView, SIGNAL(changeAccidental(Rosegarden::Accidental)),
-            this,         SLOT(slotSetAccidental(Rosegarden::Accidental)));
+    connect(m_parentView, SIGNAL(changeAccidental(Rosegarden::Accidental, bool)),
+            this,         SLOT(slotSetAccidental(Rosegarden::Accidental, bool)));
 }
 
 NoteInserter::~NoteInserter()
@@ -266,9 +269,11 @@ NoteInserter::handleMouseMove(Rosegarden::timeT,
 			      int,
 			      QMouseEvent *e)
 {
-    if (!m_clickHappened) return NoFollow;
-    computeLocationAndPreview(e);
-    return NoFollow;
+    if (m_clickHappened) {
+        computeLocationAndPreview(e);
+    }
+    
+    return RosegardenCanvasView::NoFollow;
 }
 
 
@@ -296,7 +301,10 @@ NoteInserter::handleMouseRelease(Rosegarden::timeT,
     }
 
     Event *lastInsertedEvent = doAddCommand
-	(segment, m_clickTime, endTime, note, m_clickPitch, m_accidental);
+	(segment, m_clickTime, endTime, note, m_clickPitch,
+	 (m_accidental == Rosegarden::Accidentals::NoAccidental &&
+	  m_followAccidental) ?
+	    m_lastAccidental : m_accidental);
 
     if (lastInsertedEvent) {
 
@@ -401,6 +409,35 @@ NoteInserter::computeLocationAndPreview(QMouseEvent *e)
     Rosegarden::Pitch p(height, clef, key, m_accidental);
     int pitch = p.getPerformancePitch();
 
+    // [RFE 987960] When inserting via mouse, if no accidental is
+    // selected, we use the same accidental (and thus the same pitch)
+    // as of the previous note found at this height -- iff such a note
+    // is found more recently than the last key signature.
+
+    if (m_accidental == Rosegarden::Accidentals::NoAccidental &&
+	m_followAccidental) {
+	Rosegarden::Segment &segment = staff->getSegment();
+	m_lastAccidental = m_accidental;
+	Segment::iterator i = segment.findNearestTime(time);
+	while (i != segment.end()) {
+	    if ((*i)->isa(Rosegarden::Key::EventType)) break;
+	    if ((*i)->isa(Rosegarden::Note::EventType)) {
+		if ((*i)->has(NotationProperties::HEIGHT_ON_STAFF) &&
+		    (*i)->has(Rosegarden::BaseProperties::PITCH)) {
+		    int h = (*i)->get<Int>(NotationProperties::HEIGHT_ON_STAFF);
+		    if (h == height) {
+			pitch = (*i)->get<Int>(Rosegarden::BaseProperties::PITCH);
+			(*i)->get<String>(Rosegarden::BaseProperties::ACCIDENTAL,
+					  m_lastAccidental);
+			break;
+		    }
+		}
+	    }
+	    if (i == segment.begin()) break;
+	    --i;
+	}
+    }
+
     bool changed = false;
 
     if (m_clickHappened) {
@@ -457,6 +494,7 @@ NoteInserter::getOffsetWithinRest(int staffNo,
 
     NotationStaff *staff = m_nParentView->getStaff(staffNo);
     NotationElement* el = static_cast<NotationElement*>(*i);
+    if (!el->getCanvasItem()) return 0;
     double offset = canvasX - el->getCanvasX();
 
     if (offset < 0) return 0;
@@ -531,9 +569,16 @@ Event *
 NoteInserter::doAddCommand(Segment &segment, timeT time, timeT endTime,
 			   const Note &note, int pitch, Accidental accidental)
 {
+    timeT noteEnd = time + note.getDuration();
+
+    // #1046934: make it possible to insert triplet at end of segment!
+    if (m_nParentView->isInTripletMode()) {
+	noteEnd = time + (note.getDuration() * 2 / 3);
+    }
+
     if (time < segment.getStartTime() ||
 	endTime > segment.getEndMarkerTime() ||
-	time + note.getDuration() > segment.getEndMarkerTime()) {
+	noteEnd > segment.getEndMarkerTime()) {
 	return 0;
     }
 
@@ -554,8 +599,9 @@ NoteInserter::doAddCommand(Segment &segment, timeT time, timeT endTime,
 	    !(*i)->has(BEAMED_GROUP_TUPLET_BASE)) {
 
 	    KMacroCommand *command = new KMacroCommand(insertionCommand->name());
-	    command->addCommand(new GroupMenuTupletCommand
-				(segment, time, note.getDuration()));
+	    command->addCommand(new AdjustMenuTupletCommand
+				(segment, time, note.getDuration(),
+				 3, 2, true)); // #1046934: "has timing already"
 	    command->addCommand(insertionCommand);
 	    activeCommand = command;
 	}
@@ -583,34 +629,13 @@ void NoteInserter::slotSetDots(unsigned int dots)
     if (dotsAction) dotsAction->setChecked(dots > 0);
 }
 
-void NoteInserter::slotSetAccidental(Rosegarden::Accidental accidental)
+void NoteInserter::slotSetAccidental(Rosegarden::Accidental accidental,
+				     bool follow)
 {
     NOTATION_DEBUG << "NoteInserter::setAccidental: accidental is "
 			 << accidental << endl;
     m_accidental = accidental;
-}
-
-void NoteInserter::slotSetAccidentalSync(Rosegarden::Accidental accidental)
-{
-    NOTATION_DEBUG << "NoteInserter::setAccidentalSync: accidental is "
-			 << accidental << endl;
-    slotSetAccidental(accidental);
-
-    int i;
-    for (i = 0; i < 6; ++i) {
-	if (accidental == m_actionsAccidental[i][4]) break;
-    }
-    if (i == 6) return;
-
-    // Get the parent view toolbar in sync - check the corresponding action
-    KAction* action = actionCollection()->action(m_actionsAccidental[i][2]);
-
-    KToggleAction* tAction = 0;
-    
-    if ((tAction = dynamic_cast<KToggleAction*>(action)))
-        tAction->setChecked(true);
-    else
-        KMessageBox::error(0, "NoteInserter::setAccidentalSync : couldn't find action");
+    m_followAccidental = follow;
 }
 
 void NoteInserter::slotNoAccidental()
@@ -618,6 +643,11 @@ void NoteInserter::slotNoAccidental()
     m_parentView->actionCollection()->action("no_accidental")->activate();
 }
 
+void NoteInserter::slotFollowAccidental()
+{
+    m_parentView->actionCollection()->action("follow_accidental")->activate();
+}
+
 void NoteInserter::slotSharp()
 {
     m_parentView->actionCollection()->action("sharp_accidental")->activate();
@@ -685,20 +715,22 @@ void NoteInserter::slotRestsSelected()
     }
 }
 
-const char* NoteInserter::m_actionsAccidental[][5] = 
+const char* NoteInserter::m_actionsAccidental[][4] = 
     {
-        { "No accidental",  "1slotNoAccidental()",  "no_accidental",
-          "accidental-none", Accidentals::NoAccidental.c_str() },
-        { "Sharp",          "1slotSharp()",         "sharp_accidental",
-          "accidental-sharp", Accidentals::Sharp.c_str() },
-        { "Flat",           "1slotFlat()",          "flat_accidental",
-          "accidental-flat", Accidentals::Flat.c_str() },
-        { "Natural",        "1slotNatural()",       "natural_accidental",
-          "accidental-natural", Accidentals::Natural.c_str() },
-        { "Double sharp",   "1slotDoubleSharp()",   "double_sharp_accidental",
-          "accidental-doublesharp", Accidentals::DoubleSharp.c_str() },
-        { "Double flat",    "1slotDoubleFlat()",    "double_flat_accidental",
-          "accidental-doubleflat", Accidentals::DoubleFlat.c_str() }
+        { i18n("No accidental"),  "1slotNoAccidental()",  "no_accidental",
+          "accidental-none" },
+        { i18n("Follow accidental"),  "1slotFollowAccidental()",  "follow_accidental",
+          "accidental-follow" },
+        { i18n("Sharp"),          "1slotSharp()",         "sharp_accidental",
+          "accidental-sharp" },
+        { i18n("Flat"),           "1slotFlat()",          "flat_accidental",
+          "accidental-flat" },
+        { i18n("Natural"),        "1slotNatural()",       "natural_accidental",
+          "accidental-natural" },
+        { i18n("Double sharp"),   "1slotDoubleSharp()",   "double_sharp_accidental",
+          "accidental-doublesharp" },
+        { i18n("Double flat"),    "1slotDoubleFlat()",    "double_flat_accidental",
+          "accidental-doubleflat" }
     };
 
 //------------------------------
@@ -762,7 +794,7 @@ RestInserter::doAddCommand(Segment &segment, timeT time, timeT endTime,
 	    !(*i)->has(BEAMED_GROUP_TUPLET_BASE)) {
 
 	    KMacroCommand *command = new KMacroCommand(insertionCommand->name());
-	    command->addCommand(new GroupMenuTupletCommand
+	    command->addCommand(new AdjustMenuTupletCommand
 				(segment, time, note.getDuration()));
 	    command->addCommand(insertionCommand);
 	    activeCommand = command;
@@ -1118,7 +1150,10 @@ void NotationSelector::handleLeftButtonPress(Rosegarden::timeT t,
     delete m_selectionToMerge;
     const EventSelection *selectionToMerge = 0;
     if (e->state() & ShiftButton) {
+	m_clickedShift = true;
 	selectionToMerge = m_nParentView->getCurrentSelection();
+    } else {
+	m_clickedShift = false;
     }
     m_selectionToMerge =
 	(selectionToMerge ? new EventSelection(*selectionToMerge) : 0);
@@ -1138,6 +1173,7 @@ void NotationSelector::handleLeftButtonPress(Rosegarden::timeT t,
 
     m_selectionRect->show();
     m_updateRect = true;
+    m_startedFineDrag = false;
 
     //m_parentView->setCursorPosition(p.x());
 }
@@ -1184,6 +1220,8 @@ void NotationSelector::handleMouseDoubleClick(Rosegarden::timeT,
 
     return;
 }
+
+
 void NotationSelector::handleMouseTripleClick(Rosegarden::timeT t,
 					      int height,
 					      int staffNo,
@@ -1225,14 +1263,20 @@ void NotationSelector::handleMouseTripleClick(Rosegarden::timeT t,
 int NotationSelector::handleMouseMove(timeT, int,
 				      QMouseEvent* e)
 {
-    if (!m_updateRect) return NoFollow;
+    if (!m_updateRect) return RosegardenCanvasView::NoFollow;
 
     int w = int(e->x() - m_selectionRect->x());
     int h = int(e->y() - m_selectionRect->y());
 
-    if (m_clickedElement && !m_clickedElement->isRest()) {
+    if (m_clickedElement /* && !m_clickedElement->isRest() */) {
 
-	if (w > 3 || w < -3 || h > 3 || h < -3) {
+	if (m_startedFineDrag) {
+	    dragFine(e->x(), e->y(), false);
+	} else if (m_clickedShift) {
+	    if (w > 2 || w < -2 || h > 2 || h < -2) {
+		dragFine(e->x(), e->y(), false);
+	    }
+	} else if (w > 3 || w < -3 || h > 3 || h < -3) {
 	    drag(e->x(), e->y(), false);
 	}
 
@@ -1247,7 +1291,7 @@ int NotationSelector::handleMouseMove(timeT, int,
 	m_nParentView->canvas()->update();
     }
 
-    return FollowHorizontal | FollowVertical;
+    return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
 }
 
 void NotationSelector::handleMouseRelease(timeT, int, QMouseEvent *e)
@@ -1262,10 +1306,11 @@ void NotationSelector::handleMouseRelease(timeT, int, QMouseEvent *e)
     // how big the rectangle is (if we were dragging an event, the
     // rectangle size will still be zero).
     
-    if ((e->x() - m_selectionRect->x()) > -3 &&
-        (e->x() - m_selectionRect->x()) <  3 &&
-        (e->y() - m_selectionRect->y()) > -3 &&
-        (e->y() - m_selectionRect->y()) <  3) {
+    if (((e->x() - m_selectionRect->x()) > -3 &&
+	 (e->x() - m_selectionRect->x()) <  3 &&
+	 (e->y() - m_selectionRect->y()) > -3 &&
+	 (e->y() - m_selectionRect->y()) <  3) &&
+	!m_startedFineDrag) {
 
 	if (m_clickedElement != 0 && m_selectedStaff) {
 		
@@ -1305,7 +1350,9 @@ void NotationSelector::handleMouseRelease(timeT, int, QMouseEvent *e)
 
     } else {
 
-	if (m_clickedElement && !m_clickedElement->isRest()) {
+	if (m_startedFineDrag) {
+	    dragFine(e->x(), e->y(), true);
+	} else if (m_clickedElement /* && !m_clickedElement->isRest() */) {
 	    drag(e->x(), e->y(), true);
 	} else {
 	    setViewCurrentSelection(false);
@@ -1325,9 +1372,10 @@ void NotationSelector::drag(int x, int y, bool final)
     if (!m_clickedElement || !m_selectedStaff) return;
 
     EventSelection *selection = m_nParentView->getCurrentSelection();
-    if (!selection) selection = new EventSelection(m_selectedStaff->getSegment());
-    if (!selection->contains(m_clickedElement->event()))
-	 selection->addEvent(m_clickedElement->event());
+    if (!selection || !selection->contains(m_clickedElement->event())) {
+	selection = new EventSelection(m_selectedStaff->getSegment());
+	selection->addEvent(m_clickedElement->event());
+    }
     m_nParentView->setCurrentSelection(selection);
 
     NotationStaff *targetStaff = dynamic_cast<NotationStaff *>
@@ -1338,12 +1386,16 @@ void NotationSelector::drag(int x, int y, bool final)
     
     timeT clickedTime = m_clickedElement->event()->getNotationAbsoluteTime();
 
-    Accidental clickedAccidental = Accidentals::NoAccidental;
+    Accidental clickedAccidental = Rosegarden::Accidentals::NoAccidental;
     (void)m_clickedElement->event()->get<String>(ACCIDENTAL, clickedAccidental);
 
     long clickedPitch = 0;
     (void)m_clickedElement->event()->get<Rosegarden::Int>(PITCH, clickedPitch);
 
+    long clickedHeight = 0;
+    (void)m_clickedElement->event()->get<Rosegarden::Int>
+	(NotationProperties::HEIGHT_ON_STAFF, clickedHeight);
+
     Event *clefEvt = 0, *keyEvt = 0;
     Rosegarden::Clef clef;
     Rosegarden::Key key;
@@ -1361,7 +1413,7 @@ void NotationSelector::drag(int x, int y, bool final)
 	dragTime = elt->getViewAbsoluteTime();
 	layoutX  = elt->getLayoutX();
 
-	if (elt->isRest() && duration > 0) {
+	if (elt->isRest() && duration > 0 && elt->getCanvasItem()) {
 
 	    double restX = 0, restWidth = 0;
 	    elt->getCanvasAirspace(restX, restWidth);
@@ -1388,8 +1440,12 @@ void NotationSelector::drag(int x, int y, bool final)
     if (keyEvt) key = Rosegarden::Key(*keyEvt);
     
     int height = targetStaff->getHeightAtCanvasCoords(x, y);
-    int pitch = Rosegarden::Pitch
-	(height, clef, key, clickedAccidental).getPerformancePitch();
+    int pitch = clickedPitch;
+
+    if (height != clickedHeight)
+	pitch = 
+	    Rosegarden::Pitch
+	    (height, clef, key, clickedAccidental).getPerformancePitch();
 
     if (pitch < clickedPitch) {
 	if (height < -10) {
@@ -1485,12 +1541,104 @@ void NotationSelector::drag(int x, int y, bool final)
     }
 }
 
+void NotationSelector::dragFine(int x, int y, bool final)
+{
+    NOTATION_DEBUG << "NotationSelector::drag " << x << ", " << y << endl;
+
+    if (!m_clickedElement || !m_selectedStaff) return;
+
+    EventSelection *selection = m_nParentView->getCurrentSelection();
+    if (!selection) selection = new EventSelection(m_selectedStaff->getSegment());
+    if (!selection->contains(m_clickedElement->event()))
+	 selection->addEvent(m_clickedElement->event());
+    m_nParentView->setCurrentSelection(selection);
+
+    // Fine drag modifies the DISPLACED_X and DISPLACED_Y properties on
+    // each event.  The modifications have to be relative to the previous
+    // values of these properties, not to zero, so for each event we need
+    // to store the previous value at the time the drag starts.
+
+    static Rosegarden::PropertyName xProperty("temporary-displaced-x");
+    static Rosegarden::PropertyName yProperty("temporary-displaced-y");
+
+    if (!m_startedFineDrag) {
+	// back up original properties
+
+	for (EventSelection::eventcontainer::iterator i =
+		selection->getSegmentEvents().begin();
+	     i != selection->getSegmentEvents().end(); ++i) {
+	    long prevX = 0, prevY = 0;
+	    (*i)->get<Int>(DISPLACED_X, prevX);
+	    (*i)->get<Int>(DISPLACED_Y, prevY);
+	    (*i)->setMaybe<Int>(xProperty, prevX);
+	    (*i)->setMaybe<Int>(yProperty, prevY);
+	}
+
+	m_startedFineDrag = true;
+    }
+
+    // We want the displacements in 1/1000ths of a staff space
+
+    double dx = x - m_selectionRect->x();
+    double dy = y - m_selectionRect->y();
+
+    double noteBodyWidth = m_nParentView->getNotePixmapFactory()->getNoteBodyWidth();
+    double lineSpacing = m_nParentView->getNotePixmapFactory()->getLineSpacing();
+    dx = (1000.0 * dx) / noteBodyWidth;
+    dy = (1000.0 * dy) / lineSpacing;
+
+    if (final) {
+
+	// reset original values (and remove backup values) before
+	// applying command
+
+	for (EventSelection::eventcontainer::iterator i =
+		 selection->getSegmentEvents().begin();
+	     i != selection->getSegmentEvents().end(); ++i) {
+	    long prevX = 0, prevY = 0;
+	    (*i)->get<Int>(xProperty, prevX);
+	    (*i)->get<Int>(yProperty, prevY);
+	    (*i)->setMaybe<Int>(DISPLACED_X, prevX);
+	    (*i)->setMaybe<Int>(DISPLACED_Y, prevY);
+	    (*i)->unset(xProperty);
+	    (*i)->unset(yProperty);
+	}
+
+	IncrementDisplacementsCommand *command = new IncrementDisplacementsCommand
+	    (*selection, long(dx), long(dy));
+	m_nParentView->addCommandToHistory(command);
+
+    } else {
+
+	Rosegarden::timeT startTime = 0, endTime = 0;
+
+	for (EventSelection::eventcontainer::iterator i =
+		 selection->getSegmentEvents().begin();
+	     i != selection->getSegmentEvents().end(); ++i) {
+	    long prevX = 0, prevY = 0;
+	    (*i)->get<Int>(xProperty, prevX);
+	    (*i)->get<Int>(yProperty, prevY);
+	    (*i)->setMaybe<Int>(DISPLACED_X, prevX + long(dx));
+	    (*i)->setMaybe<Int>(DISPLACED_Y, prevY + long(dy));
+	    if (i == selection->getSegmentEvents().begin()) {
+		startTime = (*i)->getAbsoluteTime();
+	    }
+	    endTime = (*i)->getAbsoluteTime() + (*i)->getDuration();
+	}
+	
+	if (startTime == endTime) ++endTime;
+	selection->getSegment().updateRefreshStatuses(startTime, endTime);
+	m_nParentView->update();
+    }
+}
+
+
 void NotationSelector::ready()
 {
     m_selectionRect = new QCanvasRectangle(m_nParentView->canvas());
     
     m_selectionRect->hide();
-    m_selectionRect->setPen(RosegardenGUIColours::SelectionRectangle);
+    m_selectionRect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SelectionRectangle));
 
     m_nParentView->setCanvasCursor(Qt::arrowCursor);
     m_nParentView->setHeightTracking(false);
@@ -1537,7 +1685,7 @@ EventSelection* NotationSelector::getSelection()
         m_selectionRect->height() > -3 &&
         m_selectionRect->height() <  3) return 0;
 
-    QCanvasItemList itemList = m_selectionRect->collisions(true);
+    QCanvasItemList itemList = m_selectionRect->collisions(false);
     QCanvasItemList::Iterator it;
 
     QRect rect = m_selectionRect->rect().normalize();
@@ -1545,21 +1693,44 @@ EventSelection* NotationSelector::getSelection()
 
     if (!m_selectedStaff) {
 
-	// Scan the list of collisions, looking for a valid notation
+        // Scan the list of collisions, looking for a valid notation
 	// element; if we find one, initialise m_selectedStaff from it.
 	// If we don't find one, we have no selection.  This is a little
 	// inefficient but we only do it for the first event in the
 	// selection.
-	
+
 	for (it = itemList.begin(); it != itemList.end(); ++it) {
         
 	    if ((sprite = dynamic_cast<QCanvasNotationSprite*>(*it))) {
 
-		if (!rect.contains(int((*it)->x()), int((*it)->y()), true)) 
-		    continue;
-            
 		NotationElement &el = sprite->getNotationElement();
-		m_selectedStaff = getStaffForElement(&el);
+
+		NotationStaff *staff = getStaffForElement(&el);
+		if (!staff) continue;
+	    
+		int x = (int)(*it)->x();
+		bool shifted = false;
+		int nbw = staff->getNotePixmapFactory(false).getNoteBodyWidth();
+
+		
+		// #957364 (Notation: Hard to select upper note in
+		// chords of seconds) -- adjust x-coord for shifted
+		// note head
+		if (el.event()->get<Rosegarden::Bool>
+		    (staff->getProperties().NOTE_HEAD_SHIFTED, shifted) && shifted) {
+		    x += nbw;
+		}
+
+		if (!rect.contains(x, int((*it)->y()), true)) {
+		    // #988217 (Notation: Special column of pixels
+		    // prevents sweep selection) -- for notes, test
+		    // again with centred x-coord
+		    if (!el.isNote() || !rect.contains(x + nbw/2, int((*it)->y()), true)) {
+			continue;
+		    }
+		}
+            
+		m_selectedStaff = staff;
 		break;
 	    }
 	}
@@ -1588,14 +1759,31 @@ EventSelection* NotationSelector::getSelection()
 
         if ((sprite = dynamic_cast<QCanvasNotationSprite*>(*it))) {
 
+	    NotationElement &el = sprite->getNotationElement();
+	    
+	    int x = (int)(*it)->x();
+	    bool shifted = false;
+	    int nbw = m_selectedStaff->getNotePixmapFactory(false).getNoteBodyWidth();
+
+	    // #957364 (Notation: Hard to select upper note in chords
+	    // of seconds) -- adjust x-coord for shifted note head
+	    if (el.event()->get<Rosegarden::Bool>
+		(m_selectedStaff->getProperties().NOTE_HEAD_SHIFTED, shifted) && shifted) {
+		x += nbw;
+	    }
+
             // check if the element's rect
             // is actually included in the selection rect.
             //
-            if (!rect.contains(int((*it)->x()), int((*it)->y()), true)) 
-		continue;
+            if (!rect.contains(x, int((*it)->y()), true))  {
+		// #988217 (Notation: Special column of pixels
+		// prevents sweep selection) -- for notes, test again
+		// with centred x-coord
+		if (!el.isNote() || !rect.contains(x + nbw/2, int((*it)->y()), true)) {
+		    continue;
+		}
+	    }
             
-            NotationElement &el = sprite->getNotationElement();
-
 	    // must be in the same segment as we first started on,
 	    // we can't select events across multiple segments
 	    if (selection->getSegment().findSingle(el.event()) !=
