@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -45,6 +45,7 @@ class NotationToolBox;
 class PositionCursor;
 class ActiveItem;
 class NoteActionData;
+class NoteChangeActionData;
 class MarkActionData;
 class ChordNameRuler;
 class RawNoteRuler;
@@ -368,6 +369,7 @@ public slots:
 
     /// accidental switch slots
     void slotNoAccidental();
+    void slotFollowAccidental();
     void slotSharp();
     void slotFlat();
     void slotNatural();
@@ -423,6 +425,7 @@ public slots:
     void slotGroupGrace();
     void slotGroupUnGrace();
     void slotGroupSlur();
+    void slotGroupPhrasingSlur();
     void slotGroupGlissando();
     void slotGroupCrescendo();
     void slotGroupDecrescendo();
@@ -444,8 +447,12 @@ public slots:
     void slotTransformsStemsUp();
     void slotTransformsStemsDown();
     void slotTransformsRestoreStems();
+    void slotTransformsSlursAbove();
+    void slotTransformsSlursBelow();
+    void slotTransformsRestoreSlurs();
     void slotTransformsQuantize();
     void slotTransformsFixQuantization();
+    void slotTransformsRemoveQuantization();
     void slotTransformsInterpret();
 
     void slotRespellDoubleFlat();
@@ -464,6 +471,7 @@ public slots:
     void slotInsertRest();
     void slotSwitchFromRestToNote();
     void slotSwitchFromNoteToRest();
+    void slotToggleDot();
 
     void slotAddMark();
     void slotMarksAddTextMark();
@@ -471,13 +479,33 @@ public slots:
     void slotMarksAddFingeringMarkFromAction();
     void slotMarksRemoveMarks();
     void slotMarksRemoveFingeringMarks();
+    void slotMakeOrnament();
+    void slotUseOrnament();
+    void slotRemoveOrnament();
+
+    void slotNoteChangeAction();
+    void slotSetNoteDurations(Rosegarden::Note::Type, bool notationOnly);
+    void slotAddDot();
+    void slotAddDotNotationOnly();
 
     void slotAddSlashes();
 
     void slotEditAddClef();
     void slotEditAddKeySignature();
+    void slotEditAddSustainDown();
+    void slotEditAddSustainUp();
+    void slotEditAddSustain(bool down);
     void slotEditElement(NotationStaff *, NotationElement *, bool advanced);
 
+    void slotFinePositionLeft();
+    void slotFinePositionRight();
+    void slotFinePositionUp();
+    void slotFinePositionDown();
+    void slotFinePositionRestore();
+
+    void slotMakeVisible();
+    void slotMakeInvisible();
+
     void slotDebugDump();
 
     /// Canvas actions slots
@@ -660,6 +688,9 @@ public slots:
     // Update notation view based on track/staff name change
     void slotUpdateStaffName();
 
+    // Open tempo editor
+    void slotEditTempos(Rosegarden::timeT t);
+
 signals:
     /**
      * Emitted when the note selected in the palette changes
@@ -669,7 +700,7 @@ signals:
     /**
      * Emitted when a new accidental has been choosen by the user
      */
-    void changeAccidental(Rosegarden::Accidental);
+    void changeAccidental(Rosegarden::Accidental, bool follow);
 
     /**
      * Emitted when the selection has been cut or copied
@@ -699,6 +730,8 @@ signals:
 
     void editMetadata(QString);
 
+    void editTriggerSegment(int);
+
     void staffLabelChanged(Rosegarden::TrackId id, QString label);
 
 protected:
@@ -843,6 +876,8 @@ protected:
      */
     bool canPreviewAnotherNote();
 
+    virtual void updateViewCaption();
+
     //--------------- Data members ---------------------------------
 
     NotationProperties m_properties;
@@ -889,7 +924,6 @@ protected:
     DeferredCursorMoveType m_deferredCursorMove;
     double m_deferredCursorScrollToX;
 
-    Rosegarden::Accidental m_currentAccidental;
     QString m_lastNoteAction;
 
     std::string m_fontName;
@@ -912,6 +946,9 @@ protected:
     typedef QMap<QString, NoteActionData> NoteActionDataMap;
     static NoteActionDataMap* m_noteActionDataMap;
 
+    typedef QMap<QString, NoteChangeActionData> NoteChangeActionDataMap;
+    static NoteChangeActionDataMap* m_noteChangeActionDataMap;
+
     typedef QMap<QString, MarkActionData> MarkActionDataMap;
     static MarkActionDataMap *m_markActionDataMap;
 
