@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -19,6 +19,7 @@
     COPYING included with this distribution for more information.
 */
 
+#include <cmath>
 #include "notationvlayout.h"
 #include "notationstaff.h"
 #include "rosestrings.h"
@@ -62,11 +63,12 @@ using Rosegarden::Staff;
 using namespace Rosegarden::BaseProperties;
 
 
-NotationVLayout::NotationVLayout(Rosegarden::Composition *c,
+NotationVLayout::NotationVLayout(Rosegarden::Composition *c, NotePixmapFactory *npf,
 				 const NotationProperties &properties,
                                  QObject* parent, const char* name) :
     ProgressReporter(parent, name),
     m_composition(c),
+    m_npf(npf),
     m_notationQuantizer(c->getNotationQuantizer()),
     m_properties(properties)
 {
@@ -116,7 +118,16 @@ NotationVLayout::scanStaff(Staff &staffBase, timeT, timeT)
     for (i = from; i != to; ++i) {
 
         NotationElement *el = static_cast<NotationElement*>(*i);
-        el->setLayoutY(0);
+
+	// Displaced Y will only be used for certain events -- in
+	// particular not for notes, whose y-coord is obviously kind
+	// of meaningful.
+	double displacedY = 0.0;
+	long dyRaw = 0;
+	el->event()->get<Int>(DISPLACED_Y, dyRaw);
+	displacedY = double(dyRaw * m_npf->getLineSpacing()) / 1000.0;
+
+        el->setLayoutY(staff.getLayoutYForHeight(-9) + displacedY);
 
         if (el->isRest()) {
 
@@ -124,13 +135,80 @@ NotationVLayout::scanStaff(Staff &staffBase, timeT, timeT)
             // aligned with the line above the middle line; the rest
             // are aligned with the middle line
 
-            int noteType = el->event()->get<Int>(NOTE_TYPE);
-            if (noteType > Note::Minim) {
-                el->setLayoutY(staff.getLayoutYForHeight(6));
+	    long noteType;
+	    bool hasNoteType = el->event()->get<Int>(NOTE_TYPE, noteType);
+            if (hasNoteType && noteType > Note::Minim) {
+                el->setLayoutY(staff.getLayoutYForHeight(6) + displacedY);
             } else {
-                el->setLayoutY(staff.getLayoutYForHeight(4));
+                el->setLayoutY(staff.getLayoutYForHeight(4) + displacedY);
             }
 
+	    // Fix for bug 1090767 Rests outside staves have wrong glyphs
+	    // by William <rosegarden4c AT orthoset.com>
+	    // We use a "rest-outside-stave" glyph for any minim/semibreve/breve
+	    // rest that has been displaced vertically e.g. by fine-positioning
+	    // outside the stave. For small vertical displacements that keep
+	    // the rest inside the stave, we use the "rest-inside-stave" glyph
+	    // and also discretise the displacement into multiples of the
+	    // stave-line spacing. The outside-stave glyphs match the character
+	    // numbers 1D13A, 1D13B and 1D13C in the Unicode 4.0 standard.
+
+	    if (hasNoteType && (displacedY > 0.1 || displacedY < -0.1)) {
+
+		// a fiddly check for transition from inside to outside:
+		
+		int min = -1, max = 1;
+
+		switch (noteType) {
+		case Note::Breve:              min = -1; max = 2; break;
+		case Note::Semibreve:          min = -1; max = 3; break;
+		case Note::Minim:              min = -2; max = 2; break;
+		case Note::Crotchet:           min = -1; max = 3; break;
+		case Note::Quaver:             min = -2; max = 3; break;
+		case Note::Semiquaver:         min = -3; max = 3; break;
+		case Note::Demisemiquaver:     min = -3; max = 4; break;
+		case Note::Hemidemisemiquaver: min = -4; max = 4; break;
+		}
+
+		bool outside = false;
+
+		if (noteType == Note::Breve) {
+		    if (nearbyint(displacedY) < min * m_npf->getLineSpacing() ||
+			nearbyint(displacedY) > max * m_npf->getLineSpacing()) {
+			outside = true;
+		    }
+		} else {
+		    if ((int)displacedY < min * m_npf->getLineSpacing() ||
+			(int)displacedY > max * m_npf->getLineSpacing()) {
+		        outside = true;
+  		    }
+	        }
+
+	        el->event()->setMaybe<Bool>(m_properties.REST_OUTSIDE_STAVE,
+					    outside);
+
+		if (!outside) {
+		    displacedY = (double)m_npf->getLineSpacing() *
+			(int(nearbyint((double)displacedY /
+				       m_npf->getLineSpacing())));
+                    if (noteType > Note::Minim)
+		        el->setLayoutY(staff.getLayoutYForHeight(6)+displacedY);
+		    else
+		        el->setLayoutY(staff.getLayoutYForHeight(4)+displacedY);
+		}
+
+//		if (displacedY != 0.0)
+//		    NOTATION_DEBUG << "REST_OUTSIDE_STAVE AFTER "
+//			       << " : displacedY : " << displacedY
+//			       << " line-spacing : " << m_npf->getLineSpacing()
+//			       << " time : " << (el->getViewAbsoluteTime())
+//			       << endl;
+	    }
+	    else {
+	        el->event()->setMaybe<Bool>(m_properties.REST_OUTSIDE_STAVE,
+					    false);
+	    }
+
         } else if (el->isNote()) {
 
             NotationChord chord(*notes, i, m_notationQuantizer, m_properties);
@@ -141,7 +219,7 @@ NotationVLayout::scanStaff(Staff &staffBase, timeT, timeT)
 		long height = 0;
 		if (!(*chord[j])->event()->get<Int>
 		    (m_properties.HEIGHT_ON_STAFF, height)) {
-		    std::cerr << QString(i18n("ERROR: Event in chord at %1 has no HEIGHT_ON_STAFF property!\nThis is a bug (the program would previously have crashed by now)").arg((*chord[j])->getViewAbsoluteTime())) << std::endl;
+		    std::cerr << QString("ERROR: Event in chord at %1 has no HEIGHT_ON_STAFF property!\nThis is a bug (the program would previously have crashed by now)").arg((*chord[j])->getViewAbsoluteTime()) << std::endl;
 		    (*chord[j])->event()->dump(std::cerr);
 		}
 		h.push_back(height);
@@ -258,35 +336,40 @@ NotationVLayout::scanStaff(Staff &staffBase, timeT, timeT)
 
 	    } else if (el->event()->isa(Text::EventType)) {
 
-		std::string type =
-		    el->event()->get<String>(Text::TextTypePropertyName);
+		std::string type = Text::UnspecifiedType;
+		el->event()->get<String>(Text::TextTypePropertyName, type);
 
 		if (type == Text::Dynamic ||
 		    type == Text::LocalDirection ||
 		    type == Text::UnspecifiedType) {
-		    el->setLayoutY(staff.getLayoutYForHeight(-7));
+		    el->setLayoutY(staff.getLayoutYForHeight(-7) + displacedY);
 		} else if (type == Text::Lyric ||
 			   type == Text::Annotation) {
-		    el->setLayoutY(staff.getLayoutYForHeight(-13));
+		    el->setLayoutY(staff.getLayoutYForHeight(-13) + displacedY);
 		} else {
-		    el->setLayoutY(staff.getLayoutYForHeight(22));
+		    el->setLayoutY(staff.getLayoutYForHeight(22) + displacedY);
 		}
 
             } else if (el->event()->isa(Indication::EventType)) {
 
-		std::string indicationType =
-		    el->event()->get<String>(Indication::IndicationTypePropertyName);
+		try {
+		    std::string indicationType =
+			el->event()->get<String>(Indication::IndicationTypePropertyName);
 
-		if (indicationType == Indication::Slur) {
-		    getSlurList(staff).push_back(i);
-		}
-
-		if (indicationType == Indication::OttavaUp ||
-		    indicationType == Indication::QuindicesimaUp) {
-		    el->setLayoutY(staff.getLayoutYForHeight(15));
-		} else {
-		    el->setLayoutY(staff.getLayoutYForHeight(-9));
-		}
+		    if (indicationType == Indication::Slur ||
+			indicationType == Indication::PhrasingSlur) {
+			getSlurList(staff).push_back(i);
+		    }
+		    
+		    if (indicationType == Indication::OttavaUp ||
+			indicationType == Indication::QuindicesimaUp) {
+			el->setLayoutY(staff.getLayoutYForHeight(15) + displacedY);
+		    } else {
+			el->setLayoutY(staff.getLayoutYForHeight(-9) + displacedY);
+		    }
+		} catch (...) {
+		    el->setLayoutY(staff.getLayoutYForHeight(-9) + displacedY);
+		}		    
 	    }
         }
     }
@@ -320,13 +403,13 @@ void
 NotationVLayout::positionSlur(NotationStaff &staff,
 			      NotationElementList::iterator i)
 {
+    bool phrasing = ((*i)->event()->get<String>(Indication::IndicationTypePropertyName)
+		     == Indication::PhrasingSlur);
 
     NotationElementList::iterator scooter = i;
 
-//!!!    timeT slurDuration = (*i)->event()->get<Int>(Indication::IndicationDurationPropertyName);
-
     timeT slurDuration = (*i)->event()->getDuration();
-    if (slurDuration == 0) {
+    if (slurDuration == 0 && (*i)->event()->has("indicationduration")) {
 	slurDuration = (*i)->event()->get<Int>("indicationduration"); // obs property
     }
     timeT endTime = (*i)->getViewAbsoluteTime() + slurDuration;
@@ -343,6 +426,7 @@ NotationVLayout::positionSlur(NotationStaff &staff,
     long startMarks = 0, endMarks = 0;
     bool startTied = false, endTied = false;
     bool beamAbove = false, beamBelow = false;
+    bool dynamic = false;
 
     std::vector<Event *> stemUpNotes, stemDownNotes;
 
@@ -353,15 +437,14 @@ NotationVLayout::positionSlur(NotationStaff &staff,
     while (scooter != staff.getViewElementList()->end()) {
 
 	if ((*scooter)->getViewAbsoluteTime() >= endTime) break;
+	Event *event = (*scooter)->event();
 
-	if (static_cast<NotationElement*>(*scooter)->isNote()) {
-
-	    Event *event = (*scooter)->event();
+	if (event->isa(Rosegarden::Note::EventType)) {
 
 	    long h = 0;
 	    if (!event->get<Int>(m_properties.HEIGHT_ON_STAFF, h)) {
 		KMessageBox::sorry
-		    ((QWidget *)parent(), QString(i18n("Spanned note at %1 has no HEIGHT_ON_STAFF property!\nThis is a bug (the program would previously have crashed by now)").arg((*scooter)->getViewAbsoluteTime())));
+		    ((QWidget *)parent(), i18n("Spanned note at %1 has no HEIGHT_ON_STAFF property!\nThis is a bug (the program would previously have crashed by now)").arg((*scooter)->getViewAbsoluteTime()));
 		event->dump(std::cerr);
 	    }
 
@@ -431,35 +514,66 @@ NotationVLayout::positionSlur(NotationStaff &staff,
 		if (stemUp) stemUpNotes.push_back(event);
 		else stemDownNotes.push_back(event);
 	    }
+
+	} else if (event->isa(Indication::EventType)) {
+
+	    try {
+		std::string indicationType =
+		    event->get<String>(Indication::IndicationTypePropertyName);
+		
+		if (indicationType == Indication::Crescendo ||
+		    indicationType == Indication::Decrescendo) dynamic = true;
+	    } catch (...) { }
 	}
 
 	++scooter;
     }
 
     bool above = true;
-    if (startStemUp == endStemUp) {
-	above = !startStemUp;
-    } else if (beamBelow) {
-	above = true;
-    } else if (beamAbove) {
-	above = false;
-    } else if (stemUpNotes.size() != stemDownNotes.size()) {
-	above = (stemUpNotes.size() < stemDownNotes.size());
+
+    if ((*i)->event()->has(NotationProperties::SLUR_ABOVE) &&
+	(*i)->event()->isPersistent<Bool>(NotationProperties::SLUR_ABOVE)) {
+
+	(*i)->event()->get<Bool>(NotationProperties::SLUR_ABOVE, above);
+
+    } else if (phrasing) {
+
+	int score = 0; // for "above"
+
+	if (dynamic) score += 2;
+
+	if (startStemUp == endStemUp) {
+	    if (startStemUp) score -= 2;
+	    else score += 2;
+	} else if (beamBelow != beamAbove) {
+	    if (beamAbove) score -= 2;
+	    else score += 2;
+	}
+
+	if (maxTopHeight < 6) score += 1;
+	else if (minBottomHeight > 2) score -= 1;
+
+	if (stemUpNotes.size() != stemDownNotes.size()) {
+	    if (stemUpNotes.size() < stemDownNotes.size()) score += 1;
+	    else score -= 1;
+	}
+
+	above = (score >= 0);
+
     } else {
-	above = ((startTopHeight - 4) + (endTopHeight - 4) +
-		 (4 - startBottomHeight) + (4 - endBottomHeight) <= 8);
-    }
 
-    // re-point the stems of any notes that will otherwise interfere
-    // with our slur (n.b. the only notes in the stemUpNotes and
-    // stemDownNotes arrays are the ones that are not beamed, so this
-    // shouldn't cause any trouble)
-
-    std::vector<Event *> *wrongStemNotes =
-	(above ? &stemUpNotes : &stemDownNotes);
-
-    for (unsigned int wsi = 0; wsi < wrongStemNotes->size(); ++wsi) {
-	(*wrongStemNotes)[wsi]->setMaybe<Bool>(m_properties.VIEW_LOCAL_STEM_UP, !above);
+	if (startStemUp == endStemUp) {
+	    above = !startStemUp;
+	} else if (beamBelow) {
+	    above = true;
+	} else if (beamAbove) {
+	    above = false;
+	} else if (stemUpNotes.size() != stemDownNotes.size()) {
+	    above = (stemUpNotes.size() < stemDownNotes.size());
+	} else {
+	    above = ((startTopHeight - 4) + (endTopHeight - 4) +
+		     (4 - startBottomHeight) + (4 - endBottomHeight) <= 8);
+	}
     }
 
     // now choose the actual y-coord of the slur based on the side
@@ -468,16 +582,17 @@ NotationVLayout::positionSlur(NotationStaff &staff,
     int startHeight, endHeight;
     int startOffset = 2, endOffset = 2;
 
-    startOffset += startMarks * 2;
-    endOffset += endMarks * 2;
-
-    // no need, I think:
-//    if (startTied) ++startOffset;
-//    if (endTied) ++endOffset;
-
     if (above) {
+
+	if (!startStemUp) startOffset += startMarks * 2;
+	else startOffset += 5;
+
+	if (!endStemUp) endOffset += startMarks * 2;
+	else endOffset += 5;
+
 	startHeight = startTopHeight + startOffset;
 	endHeight = endTopHeight + endOffset;
+
 	bool maxRelevant = ((maxTopHeight != endTopHeight) || (maxCount > 1));
 	if (maxRelevant) {
 	    int midHeight = (startHeight + endHeight)/2;
@@ -486,9 +601,18 @@ NotationVLayout::positionSlur(NotationStaff &staff,
 		endHeight   += maxTopHeight - midHeight + 1;
 	    }
 	}
+
     } else {
+
+	if (startStemUp) startOffset += startMarks * 2;
+	else startOffset += 5;
+
+	if (endStemUp) endOffset += startMarks * 2;
+	else endOffset += 5;
+
 	startHeight = startBottomHeight - startOffset;
 	endHeight = endBottomHeight - endOffset;
+
 	bool minRelevant = ((minBottomHeight != endBottomHeight) || (minCount > 1));
 	if (minRelevant) {
 	    int midHeight = (startHeight + endHeight)/2;
@@ -509,10 +633,21 @@ NotationVLayout::positionSlur(NotationStaff &staff,
     if (length > diff*3) length -= diff/2;
     startX += diff;
 
-    (*i)->event()->setMaybe<Bool>(m_properties.SLUR_ABOVE, above);
+    (*i)->event()->setMaybe<Bool>(NotationProperties::SLUR_ABOVE, above);
     (*i)->event()->setMaybe<Int>(m_properties.SLUR_Y_DELTA, dy);
     (*i)->event()->setMaybe<Int>(m_properties.SLUR_LENGTH, length);
-    (*i)->setLayoutX(startX);
-    (*i)->setLayoutY(y0);
+
+    double displacedX = 0.0, displacedY = 0.0;
+
+    long dxRaw = 0;
+    (*i)->event()->get<Int>(DISPLACED_X, dxRaw);
+    displacedX = double(dxRaw * m_npf->getNoteBodyWidth()) / 1000.0;
+
+    long dyRaw = 0;
+    (*i)->event()->get<Int>(DISPLACED_Y, dyRaw);
+    displacedY = double(dyRaw * m_npf->getLineSpacing()) / 1000.0;
+
+    (*i)->setLayoutX(startX + displacedX);
+    (*i)->setLayoutY(y0 + displacedY);
 }
 
