@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -60,9 +60,12 @@ const CharName FLAG_2 = "MUSICAL SYMBOL COMBINING FLAG-2";
 const CharName FLAG_3 = "MUSICAL SYMBOL COMBINING FLAG-3";
 const CharName FLAG_4 = "MUSICAL SYMBOL COMBINING FLAG-4";
 
-const CharName MULTI_REST = "MUSICAL SYMBOL MULTI REST";
-const CharName WHOLE_REST = "MUSICAL SYMBOL WHOLE REST";
-const CharName HALF_REST = "MUSICAL SYMBOL HALF REST";
+const CharName MULTI_REST = "MUSICAL SYMBOL MULTI REST";// Unicode-4 glyph 1D13A
+const CharName MULTI_REST_ON_STAFF = "MUSICAL SYMBOL MULTI REST ON STAFF";
+const CharName WHOLE_REST = "MUSICAL SYMBOL WHOLE REST";// Unicode-4 glyph 1D13B
+const CharName WHOLE_REST_ON_STAFF = "MUSICAL SYMBOL WHOLE REST ON STAFF";
+const CharName HALF_REST = "MUSICAL SYMBOL HALF REST";  // Unicode-4 glyph 1D13C
+const CharName HALF_REST_ON_STAFF = "MUSICAL SYMBOL HALF REST ON STAFF";
 const CharName QUARTER_REST = "MUSICAL SYMBOL QUARTER REST";
 const CharName EIGHTH_REST = "MUSICAL SYMBOL EIGHTH REST";
 const CharName SIXTEENTH_REST = "MUSICAL SYMBOL SIXTEENTH REST";
@@ -78,7 +81,17 @@ const CharName STACCATISSIMO = "MUSICAL SYMBOL COMBINING STACCATISSIMO";
 const CharName MARCATO = "MUSICAL SYMBOL COMBINING MARCATO";
 const CharName FERMATA = "MUSICAL SYMBOL FERMATA";
 const CharName TRILL = "MUSICAL SYMBOL TR";
+const CharName TRILL_LINE = "MUSICAL SYMBOL COMBINING TRILL LINE";
 const CharName TURN = "MUSICAL SYMBOL TURN";
+
+const CharName MORDENT = "MUSICAL SYMBOL MORDENT";
+const CharName MORDENT_INVERTED = "MUSICAL SYMBOL INVERTED MORDENT";
+const CharName MORDENT_LONG = "MUSICAL SYMBOL LONG MORDENT";
+const CharName MORDENT_LONG_INVERTED = "MUSICAL SYMBOL LONG INVERTED MORDENT";
+
+const CharName PEDAL_MARK = "MUSICAL SYMBOL PEDAL MARK";
+const CharName PEDAL_UP_MARK = "MUSICAL SYMBOL PEDAL UP MARK";
+
 const CharName UP_BOW = "MUSICAL SYMBOL COMBINING UP BOW";
 const CharName DOWN_BOW = "MUSICAL SYMBOL COMBINING DOWN BOW";
 
