@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -25,9 +25,6 @@
 #include "PropertyName.h"
 
 typedef Rosegarden::PropertyName CharName;
-typedef Rosegarden::PropertyNameHash CharNameHash;
-typedef Rosegarden::PropertyNamesEqual CharNamesEqual;
-
 
 /// A selection of Unicode character names for symbols in a note font
 
@@ -62,8 +59,11 @@ extern const CharName FLAG_3;
 extern const CharName FLAG_4;
 
 extern const CharName MULTI_REST;
+extern const CharName MULTI_REST_ON_STAFF;
 extern const CharName WHOLE_REST;
+extern const CharName WHOLE_REST_ON_STAFF;
 extern const CharName HALF_REST;
+extern const CharName HALF_REST_ON_STAFF;
 extern const CharName QUARTER_REST;
 extern const CharName EIGHTH_REST;
 extern const CharName SIXTEENTH_REST;
@@ -79,10 +79,19 @@ extern const CharName STACCATISSIMO;
 extern const CharName MARCATO;
 extern const CharName FERMATA;
 extern const CharName TRILL;
+extern const CharName TRILL_LINE;
 extern const CharName TURN;
 extern const CharName UP_BOW;
 extern const CharName DOWN_BOW;
 
+extern const CharName MORDENT;
+extern const CharName MORDENT_INVERTED;
+extern const CharName MORDENT_LONG;
+extern const CharName MORDENT_LONG_INVERTED;
+
+extern const CharName PEDAL_MARK;
+extern const CharName PEDAL_UP_MARK;
+
 extern const CharName C_CLEF;
 extern const CharName G_CLEF;
 extern const CharName F_CLEF;
