@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -29,6 +29,8 @@
 #include <qregexp.h>
 #include <qfontdatabase.h>
 
+#include <algorithm>
+
 #include <kglobal.h>
 #include <kstddirs.h>
 #include <klocale.h>
@@ -64,6 +66,7 @@ NoteFontMap::NoteFontMap(string name) :
     m_srcDirectory(name),
     m_characterDestination(0),
     m_hotspotCharName(""),
+    m_errorString(i18n("unknown error")),
     m_ok(true)
 {
     m_fontDirectory = KGlobal::dirs()->findResource("appdata", "fonts/");
@@ -249,29 +252,29 @@ NoteFontMap::startElement(const QString &, const QString &,
 	QString s;
 
         s = attributes.value("font-height");
-	if (s) fontHeight = s.toDouble();
+	if (s) fontHeight = qstrtodouble(s);
 	else {
 	    m_errorString = "font-height is a required attribute of font-scale";
 	    return false;
 	}
 
         s = attributes.value("staff-line-thickness");
-	if (s) staffLineThickness = s.toDouble();
+	if (s) staffLineThickness = qstrtodouble(s);
 
         s = attributes.value("leger-line-thickness");
-	if (s) legerLineThickness = s.toDouble();
+	if (s) legerLineThickness = qstrtodouble(s);
 
         s = attributes.value("stem-thickness");
-	if (s) stemThickness = s.toDouble();
+	if (s) stemThickness = qstrtodouble(s);
 
         s = attributes.value("beam-thickness");
-	if (s) beamThickness = s.toDouble();
+	if (s) beamThickness = qstrtodouble(s);
 
         s = attributes.value("stem-length");
-	if (s) stemLength = s.toDouble();
+	if (s) stemLength = qstrtodouble(s);
 
         s = attributes.value("flag-spacing");
-	if (s) flagSpacing = s.toDouble();
+	if (s) flagSpacing = qstrtodouble(s);
 
 	int fontId = 0;
 	s = attributes.value("font-id");
@@ -471,14 +474,14 @@ NoteFontMap::startElement(const QString &, const QString &,
 
 	QString s = attributes.value("x");
 	double x = -1.0;
-	if (s) x = s.toDouble();
+	if (s) x = qstrtodouble(s);
 
 	s = attributes.value("y");
 	if (!s) {
 	    m_errorString = "y is a required attribute of scaled";
 	    return false;
 	}
-	double y = s.toDouble();
+	double y = qstrtodouble(s);
 	
         HotspotDataMap::iterator i = m_hotspots.find(m_hotspotCharName);
         if (i == m_hotspots.end()) {
@@ -637,20 +640,22 @@ NoteFontMap::startElement(const QString &, const QString &,
 bool
 NoteFontMap::error(const QXmlParseException& exception)
 {
-    m_errorString = QString("%1 at line %2, column %3")
+    m_errorString = QString("%1 at line %2, column %3: %4")
 	.arg(exception.message())
 	.arg(exception.lineNumber())
-	.arg(exception.columnNumber());
+	.arg(exception.columnNumber())
+	.arg(m_errorString);
     return QXmlDefaultHandler::error(exception);
 }
 
 bool
 NoteFontMap::fatalError(const QXmlParseException& exception)
 {
-    m_errorString = QString("%1 at line %2, column %3")
+    m_errorString = QString("%1 at line %2, column %3: %4")
 	.arg(exception.message())
 	.arg(exception.lineNumber())
-	.arg(exception.columnNumber());
+	.arg(exception.columnNumber())
+	.arg(m_errorString);
     return QXmlDefaultHandler::fatalError(exception);
 }
 
@@ -1492,12 +1497,51 @@ NoteFont::getColouredPixmap(CharName baseCharName, QPixmap &pixmap,
     return ok;
 }
 
+bool
+NoteFont::getShadedPixmap(CharName baseCharName, QPixmap &pixmap,
+			  bool inverted) const
+{
+    CharName charName(getNameShaded(baseCharName));
+
+    QPixmap *found = 0;
+    bool ok = lookup(charName, inverted, found);
+    if (ok) {
+	if (found) {
+	    pixmap = *found;
+	    return true;
+	} else {
+	    pixmap = *m_blankPixmap;
+	    return false;
+	}
+    }
+
+    QPixmap basePixmap;
+    ok = getPixmap(baseCharName, basePixmap, inverted);
+
+    if (!ok) {
+	add(charName, inverted, 0);
+	pixmap = *m_blankPixmap;
+	return false;
+    }
+
+    found = new QPixmap(PixmapFunctions::shadePixmap(basePixmap));
+    add(charName, inverted, found);
+    pixmap = *found;
+    return ok;
+}
+
 CharName
 NoteFont::getNameWithColour(CharName base, int hue) const
 {
     return qstrtostr(QString("%1__%2").arg(hue).arg(strtoqstr(base)));
 }
 
+CharName
+NoteFont::getNameShaded(CharName base) const
+{
+    return qstrtostr(QString("shaded__%1").arg(strtoqstr(base)));
+}
+
 bool
 NoteFont::getDimensions(CharName charName, int &x, int &y, bool inverted) const
 {
@@ -1642,6 +1686,53 @@ NoteFont::getCharacterColoured(CharName charName,
     return character;
 }
 
+bool
+NoteFont::getCharacterShaded(CharName charName,
+			     NoteCharacter &character,
+			     CharacterType type,
+			     bool inverted)
+{
+    QPixmap pixmap;
+    if (!getShadedPixmap(charName, pixmap, inverted)) {
+	return false;
+    }
+
+    if (type == Screen) {
+
+	character = NoteCharacter(pixmap,
+				  getHotspot(charName, inverted),
+				  0);
+
+    } else {
+
+	// Get the pointer direct from cache (depends on earlier call
+	// to getPixmap to put it in the cache if available)
+
+	QPixmap *pmapptr = 0;
+	CharName cCharName(getNameShaded(charName));
+	bool found = lookup(cCharName, inverted, pmapptr);
+
+	NoteCharacterDrawRep *rep = 0;
+	if (found && pmapptr) rep = lookupDrawRep(pmapptr);
+
+	character = NoteCharacter(pixmap,
+				  getHotspot(charName, inverted),
+				  rep);
+    }
+
+    return true;
+}
+
+NoteCharacter
+NoteFont::getCharacterShaded(CharName charName,
+			     CharacterType type,
+			     bool inverted)
+{
+    NoteCharacter character;
+    getCharacterShaded(charName, character, type, inverted);
+    return character;
+}
+
 
 std::set<std::string>
 NoteFontFactory::getFontNames()
