@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -435,6 +435,17 @@ public:
 				       CharacterType type = Screen,
 				       bool inverted = false);
 
+    /// Returns false if not available
+    bool getCharacterShaded(CharName charName,
+			    NoteCharacter &character,
+			    CharacterType type = Screen,
+			    bool inverted = false);
+
+    /// Returns an empty character if not available
+    NoteCharacter getCharacterShaded(CharName charName,
+				     CharacterType type = Screen,
+				     bool inverted = false);
+
     /// Returns false + dimensions of blank pixmap if none found
     bool getDimensions(CharName charName, int &x, int &y,
                        bool inverted = false) const;
@@ -462,6 +473,10 @@ private:
                            int hue, int minValue,
 			   bool inverted = false) const;
 
+    /// Returns false + blank pixmap if it can't find the right one
+    bool getShadedPixmap(CharName charName, QPixmap &pixmap,
+			 bool inverted = false) const;
+
     friend class NoteFontFactory;
     NoteFont(std::string fontName, int size = 0);
     std::set<int> getSizes() const { return m_fontMap.getSizes(); }
@@ -472,7 +487,7 @@ private:
     NoteCharacterDrawRep *lookupDrawRep(QPixmap *pixmap) const;
 
     CharName getNameWithColour(CharName origName, int hue) const;
-    QPixmap *recolour(QPixmap in, int hue, int minValue) const;
+    CharName getNameShaded(CharName origName) const;
 
     typedef std::pair<QPixmap *, QPixmap *>    PixmapPair;
     typedef std::map<CharName, PixmapPair>     PixmapMap;
