@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -36,9 +36,7 @@
 
 
 using Rosegarden::Accidental;
-using Rosegarden::Accidentals;
 using Rosegarden::Mark;
-using Rosegarden::Marks;
 using Rosegarden::Clef;
 using Rosegarden::Note;
 using Rosegarden::String;
@@ -348,27 +346,37 @@ NoteStyle::getNoteHeadCharName(Note::Type type)
 CharName
 NoteStyle::getAccidentalCharName(const Accidental &a)
 {
-    if      (a == Accidentals::Sharp)        return NoteCharacterNames::SHARP;
-    else if (a == Accidentals::Flat)         return NoteCharacterNames::FLAT;
-    else if (a == Accidentals::Natural)      return NoteCharacterNames::NATURAL;
-    else if (a == Accidentals::DoubleSharp)  return NoteCharacterNames::DOUBLE_SHARP;
-    else if (a == Accidentals::DoubleFlat)   return NoteCharacterNames::DOUBLE_FLAT;
+    if      (a == Rosegarden::Accidentals::Sharp)        return NoteCharacterNames::SHARP;
+    else if (a == Rosegarden::Accidentals::Flat)         return NoteCharacterNames::FLAT;
+    else if (a == Rosegarden::Accidentals::Natural)      return NoteCharacterNames::NATURAL;
+    else if (a == Rosegarden::Accidentals::DoubleSharp)  return NoteCharacterNames::DOUBLE_SHARP;
+    else if (a == Rosegarden::Accidentals::DoubleFlat)   return NoteCharacterNames::DOUBLE_FLAT;
     return NoteCharacterNames::UNKNOWN;
 }
 
 CharName
 NoteStyle::getMarkCharName(const Mark &mark)
 {
-    if      (mark == Marks::Accent)    return NoteCharacterNames::ACCENT;
-    else if (mark == Marks::Tenuto)    return NoteCharacterNames::TENUTO;
-    else if (mark == Marks::Staccato)  return NoteCharacterNames::STACCATO;
-    else if (mark == Marks::Staccatissimo) return NoteCharacterNames::STACCATISSIMO;
-    else if (mark == Marks::Marcato)   return NoteCharacterNames::MARCATO;
-    else if (mark == Marks::Trill)     return NoteCharacterNames::TRILL;
-    else if (mark == Marks::Turn)      return NoteCharacterNames::TURN;
-    else if (mark == Marks::Pause)     return NoteCharacterNames::FERMATA;
-    else if (mark == Marks::UpBow)     return NoteCharacterNames::UP_BOW;
-    else if (mark == Marks::DownBow)   return NoteCharacterNames::DOWN_BOW;
+    if      (mark == Rosegarden::Marks::Accent)    return NoteCharacterNames::ACCENT;
+    else if (mark == Rosegarden::Marks::Tenuto)    return NoteCharacterNames::TENUTO;
+    else if (mark == Rosegarden::Marks::Staccato)  return NoteCharacterNames::STACCATO;
+    else if (mark == Rosegarden::Marks::Staccatissimo) return NoteCharacterNames::STACCATISSIMO;
+    else if (mark == Rosegarden::Marks::Marcato)   return NoteCharacterNames::MARCATO;
+    else if (mark == Rosegarden::Marks::Trill)     return NoteCharacterNames::TRILL;
+    else if (mark == Rosegarden::Marks::LongTrill) return NoteCharacterNames::TRILL;
+    else if (mark == Rosegarden::Marks::TrillLine) return NoteCharacterNames::TRILL_LINE;
+    else if (mark == Rosegarden::Marks::Turn)      return NoteCharacterNames::TURN;
+    else if (mark == Rosegarden::Marks::Pause)     return NoteCharacterNames::FERMATA;
+    else if (mark == Rosegarden::Marks::UpBow)     return NoteCharacterNames::UP_BOW;
+    else if (mark == Rosegarden::Marks::DownBow)   return NoteCharacterNames::DOWN_BOW;
+    else if (mark == Rosegarden::Marks::Mordent)
+	return NoteCharacterNames::MORDENT;
+    else if (mark == Rosegarden::Marks::MordentInverted)
+	return NoteCharacterNames::MORDENT_INVERTED;
+    else if (mark == Rosegarden::Marks::MordentLong)
+	return NoteCharacterNames::MORDENT_LONG;
+    else if (mark == Rosegarden::Marks::MordentLongInverted)
+	return NoteCharacterNames::MORDENT_LONG_INVERTED;
     // Things like "sf" and "rf" are generated from text fonts
     return NoteCharacterNames::UNKNOWN;
 }
@@ -388,7 +396,7 @@ NoteStyle::getClefCharName(const Clef &clef)
 }
 
 CharName
-NoteStyle::getRestCharName(Note::Type type)
+NoteStyle::getRestCharName(Note::Type type, bool restOutsideStave)
 {
     switch (type) {
     case Note::Hemidemisemiquaver:  return NoteCharacterNames::SIXTY_FOURTH_REST;
@@ -396,9 +404,15 @@ NoteStyle::getRestCharName(Note::Type type)
     case Note::Semiquaver:          return NoteCharacterNames::SIXTEENTH_REST;
     case Note::Quaver:              return NoteCharacterNames::EIGHTH_REST;
     case Note::Crotchet:            return NoteCharacterNames::QUARTER_REST;
-    case Note::Minim:               return NoteCharacterNames::HALF_REST;
-    case Note::Semibreve:           return NoteCharacterNames::WHOLE_REST;
-    case Note::Breve:               return NoteCharacterNames::MULTI_REST;
+    case Note::Minim:               return restOutsideStave ?
+					NoteCharacterNames::HALF_REST
+				      : NoteCharacterNames::HALF_REST_ON_STAFF;
+    case Note::Semibreve:           return restOutsideStave ?
+    					NoteCharacterNames::WHOLE_REST
+				      : NoteCharacterNames::WHOLE_REST_ON_STAFF;
+    case Note::Breve:               return restOutsideStave ?
+					NoteCharacterNames::MULTI_REST
+				      : NoteCharacterNames::MULTI_REST_ON_STAFF;
     default:
         return NoteCharacterNames::UNKNOWN;
     }
@@ -524,7 +538,7 @@ NoteStyle::setStemFixPoints(Note::Type note, HFixPoint hfix, VFixPoint vfix)
 
 
 NoteStyleFileReader::NoteStyleFileReader(std::string name) :
-    m_style(new NoteStyle),
+    m_style(new NoteStyle(name)),
     m_haveNote(false)
 {
     QString styleDirectory =
