@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -23,8 +23,8 @@
 #define _NOTE_STYLE_H_
 
 #include <vector>
+#include <map>
 
-#include "StringHash.h"
 #include "NotationTypes.h"
 #include "Exception.h"
 
@@ -51,7 +51,7 @@ public:
     typedef Rosegarden::Exception StyleUnavailable;
 
 private:
-    typedef Rosegarden::hash_string<NoteStyle*> StyleMap;
+    typedef std::map<std::string, NoteStyle *> StyleMap;
     static StyleMap m_styles;
 };
 
@@ -59,7 +59,6 @@ private:
 class NoteStyle
 {
 public:
-    NoteStyle() : m_baseStyle(0) { }
     virtual ~NoteStyle();
 
     typedef std::string NoteHeadShape;
@@ -78,6 +77,8 @@ public:
     enum HFixPoint { Normal, Central, Reversed };
     enum VFixPoint { Near, Middle, Far };
 
+    NoteStyleName getName() const { return m_name; }
+
     NoteHeadShape getShape     (Rosegarden::Note::Type);
     bool          isFilled     (Rosegarden::Note::Type);
     bool          hasStem      (Rosegarden::Note::Type);
@@ -87,7 +88,7 @@ public:
     typedef std::pair<CharName, bool> CharNameRec; // bool is "inverted"
     CharNameRec getNoteHeadCharName(Rosegarden::Note::Type);
 
-    CharName getRestCharName(Rosegarden::Note::Type);
+    CharName getRestCharName(Rosegarden::Note::Type, bool restOutsideStave);
     CharName getPartialFlagCharName(bool final);
     CharName getFlagCharName(int flagCount);
     CharName getAccidentalCharName(const Rosegarden::Accidental &);
@@ -130,13 +131,17 @@ protected:
 	    hfix(_hfix), vfix(_vfix) { }
     };
 
-    typedef __HASH_NS::hash_map<Rosegarden::Note::Type,
-				NoteDescription> NoteDescriptionMap;
+    typedef std::map<Rosegarden::Note::Type, NoteDescription> NoteDescriptionMap;
 
     NoteDescriptionMap m_notes;
     NoteStyle *m_baseStyle;
+    NoteStyleName m_name;
 
     void checkDescription(Rosegarden::Note::Type type);
+
+protected: // for use by NoteStyleFileReader
+    NoteStyle(NoteStyleName name) : m_baseStyle(0), m_name(name) { }
+    friend class NoteStyleFileReader;
 };
 
 
