@@ -4,7 +4,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -44,7 +44,7 @@ PianoKeyboard::PianoKeyboard(QWidget *parent)
       m_lastHoverHighlight(0)
 {
     m_hoverHighlight->hide();
-    m_hoverHighlight->setPaletteBackgroundColor(RosegardenGUIColours::MatrixKeyboardFocus);
+    m_hoverHighlight->setPaletteBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MatrixKeyboardFocus));
 
     setPaletteBackgroundColor(QColor(238, 238, 224));
 
