@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -121,6 +121,34 @@ PixmapFunctions::colourPixmap(const QPixmap &map, int hue, int minValue)
     return rmap;
 }
 
+QPixmap
+PixmapFunctions::shadePixmap(const QPixmap &map)
+{
+    QImage image = map.convertToImage();
+
+    int h, s, v;
+
+    for (int y = 0; y < image.height(); ++y) {
+        for (int x = 0; x < image.width(); ++x) {
+
+            QColor pixel(image.pixel(x, y));
+
+            pixel.hsv(&h, &s, &v);
+
+            image.setPixel
+                (x, y, QColor(h,
+                              s,
+			      255 - ((255 - v) / 2),
+                              QColor::Hsv).rgb());
+        }
+    }
+
+    QPixmap rmap;
+    rmap.convertFromImage(image);
+    if (map.mask()) rmap.setMask(*map.mask());
+    return rmap;
+}
+
 QPixmap
 PixmapFunctions::flipVertical(const QPixmap &map)
 {
