@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -63,6 +63,11 @@ public:
      */
     static QPixmap colourPixmap(const QPixmap &map, int hue, int minValue);
 
+    /**
+     * Make a pixmap grey, or otherwise reduce its intensity.
+     */
+    static QPixmap shadePixmap(const QPixmap &map);
+
     /// Return a QPixmap that is a mirror image of map (including mask)
     static QPixmap flipVertical(const QPixmap &map);
 
