@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -183,12 +183,7 @@ PlayList::~PlayList()
 void PlayList::slotOpenFiles()
 {
     KURL::List kurlList =
-        KFileDialog::getOpenURLs(
-#if KDE_VERSION >= 196614
-                                 ":ROSEGARDEN",
-#else
-                                 QString::null,
-#endif
+        KFileDialog::getOpenURLs(":ROSEGARDEN",
                                  "audio/x-rosegarden audio/x-midi audio/x-rosegarden21",
                                  this,
                                  i18n("Select one or more Rosegarden files"));
