@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -35,13 +35,13 @@
 using Rosegarden::RulerScale;
 using Rosegarden::Composition;
 using Rosegarden::Segment;
-using Rosegarden::BaseProperties;
 using Rosegarden::BaseProperties::PITCH;
 using Rosegarden::Event;
 using Rosegarden::Note;
 using Rosegarden::Int;
 using Rosegarden::timeT;
 
+//#define DEBUG_RAW_NOTE_RULER 1
 
 RawNoteRuler::RawNoteRuler(RulerScale *rulerScale,
 			   Segment *segment,
@@ -57,7 +57,7 @@ RawNoteRuler::RawNoteRuler(RulerScale *rulerScale,
     m_segment(segment),
     m_rulerScale(rulerScale)
 {
-    setBackgroundColor(RosegardenGUIColours::RawNoteRulerBackground);
+    setBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerBackground));
 }
 
 RawNoteRuler::~RawNoteRuler()
@@ -126,7 +126,7 @@ RawNoteRuler::getExtents(Segment::iterator i)
     return std::pair<timeT, timeT>(t0, t1);
 }
 
-void
+Segment::iterator
 RawNoteRuler::addChildren(Segment *s,
 			  Segment::iterator to,
 			  timeT rightBound,
@@ -136,20 +136,39 @@ RawNoteRuler::addChildren(Segment *s,
 
     std::pair<timeT, timeT> iex = getExtents(i);
     Segment::iterator j = i;
+    Segment::iterator rightmost = to;
+
+#ifdef DEBUG_RAW_NOTE_RULER
+    RG_DEBUG << "addChildren called for extents " << iex.first << "->" << iex.second << ", rightBound " << rightBound << endl;
+#endif
     
     for (++j; j != to && s->isBeforeEndMarker(j); ) {
 	
 	if (!(*j)->isa(Note::EventType)) { ++j; continue; }
-	
 	std::pair<timeT, timeT> jex = getExtents(j);
+
+#ifdef DEBUG_RAW_NOTE_RULER
+	RG_DEBUG << "addChildren: event at " << (*j)->getAbsoluteTime() << ", extents " << jex.first << "->" << jex.second << endl;
+#endif
+
 	if (jex.first == jex.second) { ++j; continue; }
 	if (jex.first >= iex.second || jex.first >= rightBound) break;
+	
+#ifdef DEBUG_RAW_NOTE_RULER
+	RG_DEBUG << "addChildren: adding" << endl;
+#endif
 
 	EventTreeNode *subnode = new EventTreeNode(j);
-	addChildren(s, to, rightBound, subnode);
+
+	Segment::iterator subRightmost = addChildren(s, to, rightBound, subnode);
+	if (subRightmost != to) rightmost = subRightmost;
+	else rightmost = j;
+
 	node->children.push_back(subnode);
 	j = s->findTime(jex.second);
     }
+
+    return rightmost;
 }    
 	
 void
@@ -167,25 +186,31 @@ RawNoteRuler::buildForest(Segment *s,
 		     s->getEndMarkerTime());
 
     for (Segment::iterator i = from; i != to && s->isBeforeEndMarker(i); ) {
+
 	if (!(*i)->isa(Note::EventType)) { ++i; continue; }
 
 	std::pair<timeT, timeT> iex = getExtents(i);
 
 #ifdef DEBUG_RAW_NOTE_RULER
-	RG_DEBUG << "buildForest: event at " << (*i)->getAbsoluteTime() << ", extents " << iex.first << "->" << iex.second << std::endl;
+	RG_DEBUG << "buildForest: event at " << (*i)->getAbsoluteTime() << ", extents " << iex.first << "->" << iex.second << endl;
 #endif
 	
 	if (iex.first == iex.second) { ++i; continue; }
 	if (iex.first >= endTime) break;
 
 	EventTreeNode *node = new EventTreeNode(i);
-	addChildren(s, to, iex.second, node);
+	Segment::iterator rightmost = addChildren(s, to, iex.second, node);
 	m_forest.push_back(node);
 
-	i = s->findTime(iex.second);
+	if (rightmost != to) {
+	    i = rightmost; 
+	    ++i;
+	} else {
+	    i = s->findTime(iex.second);
+	}
 
 #ifdef DEBUG_RAW_NOTE_RULER
-	RG_DEBUG << "findTime " << iex.second << " returned iterator at " << (i == s->end() ? -1 : (*i)->getAbsoluteTime()) << std::endl;
+	RG_DEBUG << "findTime " << iex.second << " returned iterator at " << (i == s->end() ? -1 : (*i)->getAbsoluteTime()) << endl;
 #endif
     }
 }
@@ -195,15 +220,15 @@ RawNoteRuler::dumpSubtree(EventTreeNode *node, int depth)
 {
     if (!node) return;
 #ifdef DEBUG_RAW_NOTE_RULER
-    for (int i = 0; i < depth; ++i) RG_DEBUG << "  ";
-    if (depth > 0) RG_DEBUG << "->";
-    RG_DEBUG << (*node->node)->getAbsoluteTime() << ","
+    for (int i = 0; i < depth; ++i) std::cerr << "  ";
+    if (depth > 0) std::cerr << "->";
+    std::cerr << (*node->node)->getAbsoluteTime() << ","
 	      << (*node->node)->getDuration() << " [";
     long pitch = 0;
     if ((*node->node)->get<Int>(PITCH, pitch)) {
-	RG_DEBUG << pitch << "]" << std::endl;
+	std::cerr << pitch << "]" << std::endl;
     } else {
-	RG_DEBUG << "no-pitch]" << std::endl;
+	std::cerr << "no-pitch]" << std::endl;
     }
     for (EventTreeNode::NodeList::iterator i = node->children.begin();
 	 i != node->children.end(); ++i) {
@@ -217,15 +242,15 @@ void
 RawNoteRuler::dumpForest(EventTreeNode::NodeList *forest)
 {
 #ifdef DEBUG_RAW_NOTE_RULER
-    RG_DEBUG << "\nFOREST:\n" << std::endl;
+    std::cerr << "\nFOREST:\n" << std::endl;
 
     for (unsigned int i = 0; i < forest->size(); ++i) {
 
-	RG_DEBUG << "\nTREE " << i << ":\n" << std::endl;
+	std::cerr << "\nTREE " << i << ":\n" << std::endl;
 	dumpSubtree((*forest)[i], 0);
     }
 
-    RG_DEBUG << std::endl;
+    std::cerr << std::endl;
 #endif
     (void)forest; // avoid warnings
 }
@@ -285,10 +310,11 @@ RawNoteRuler::drawNode(QPainter &paint, DefaultVelocityColour &vc,
     if (heightPer < 2) heightPer = 2;
 
     double myOrigin = yorigin + (heightPer * above);
-    int myPitch = (*node->node)->get<Int>(PITCH);
+    long myPitch = 60;
+    (*node->node)->get<Int>(PITCH, myPitch);
 
     long velocity = 100;
-    (*node->node)->get<Int>(BaseProperties::VELOCITY, velocity);
+    (*node->node)->get<Int>(Rosegarden::BaseProperties::VELOCITY, velocity);
     QColor colour = vc.getColour(velocity);
     
     timeT start = (*node->node)->getAbsoluteTime();
@@ -335,8 +361,8 @@ RawNoteRuler::drawNode(QPainter &paint, DefaultVelocityColour &vc,
     paint.setBrush(colour);
     paint.drawRect(ui0 + 1, iy + 1, uiw, ih - 1);
     
-    paint.setPen(RosegardenGUIColours::RawNoteRulerForeground);
-    paint.setBrush(RosegardenGUIColours::RawNoteRulerForeground);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
+    paint.setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
     paint.drawLine(qi0,     iy,      qi1 - 1, iy);
     paint.drawLine(qi0,     iy + ih, qi1 - 1, iy + ih);
     paint.drawLine(ui0,     iy + 1,  ui0,     iy + ih - 1);
@@ -345,10 +371,10 @@ RawNoteRuler::drawNode(QPainter &paint, DefaultVelocityColour &vc,
     for (EventTreeNode::NodeList::iterator i = node->children.begin();
 	 i != node->children.end(); ++i) {
 
-	//!!! need to ensure nothing goes in the tree that doesn't
-	// have a pitch, or else make these calls safe
+	long nodePitch = myPitch;
+	(*(*i)->node)->get<Int>(PITCH, nodePitch);
 
-	if ((*(*i)->node)->get<Int>(PITCH) < myPitch) {
+	if (nodePitch < myPitch) {
 
 	    drawNode(paint, vc, *i,
 		     height - heightPer - myOrigin, myOrigin + heightPer);
@@ -379,8 +405,8 @@ RawNoteRuler::paintEvent(QPaintEvent* e)
     timeT   to = m_rulerScale->getTimeForX
 	(clipRect.x() + clipRect.width() - m_currentXOffset + 100 - m_xorigin);
 
-    paint.setPen(RosegardenGUIColours::RawNoteRulerForeground);
-    paint.setBrush(RosegardenGUIColours::RawNoteRulerForeground);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
+    paint.setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
     paint.drawLine(0, 0, width(), 0);
 
     // draw the bar divisions
@@ -404,8 +430,8 @@ RawNoteRuler::paintEvent(QPaintEvent* e)
 	timeT base = timeSig.getBarDuration();
 	timeT barEnd = barStart + base;
 
-	paint.setPen(RosegardenGUIColours::RawNoteRulerForeground);
-	paint.setBrush(RosegardenGUIColours::RawNoteRulerForeground);
+	paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
+	paint.setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RawNoteRulerForeground));
 
 	int x = int(m_rulerScale->getXForTime(barStart) +
 		    m_currentXOffset + m_xorigin);
