@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -30,9 +30,11 @@
 #include "progressreporter.h"
 #include "NotationTypes.h"
 
-class Rosegarden::Studio;
-class Rosegarden::Composition;
-class Rosegarden::Segment;
+namespace Rosegarden {
+    class Studio;
+    class Composition;
+    class Segment;
+}
 
 /**
  * Rosegarden 2.1 file import
