@@ -23,19 +23,9 @@
 */
 
 /*************************************************************************
- * $Id: rgled.cpp 6694 2005-07-17 11:42:36Z cannam $
- *
- * $Log$
- * Revision 1.2  2005/07/17 11:42:36  cannam
- * * fixes from stg branch:
- *   - fix failure to recolour led when switching track from audio to midi or vice versa
- *   - fix colour number / menu index confusion in spb colour menu
- *
- * Revision 1.1.10.1  2005/07/17 00:00:30  cannam
- *
- * * merge across from HEAD to 17/07/2005
- * * appearance fixes
+ * $Id: rgled.cpp,v 1.1 2004/04/05 09:54:05 cannam Exp $
  *
+ * $Log: rgled.cpp,v $
  * Revision 1.1  2004/04/05 09:54:05  cannam
  *
  * * antialiased LEDs
@@ -658,10 +648,6 @@ KLed::setColor(const QColor& col)
   if(led_color!=col) {
     led_color = col;
     d->offcolor = col.dark(d->dark_factor);
-    delete d->on_map;
-    d->on_map = 0;
-    delete d->off_map;
-    d->off_map = 0;
     update();
   }
 }
