@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -33,6 +33,8 @@
 
 #include "config.h"
 
+#include <algorithm>
+
 #include <qspinbox.h>
 #include <qslider.h>
 #include <qcombobox.h>
@@ -51,6 +53,7 @@
 #include <qheader.h>
 #include <klineeditdlg.h>
 
+#include <kdeversion.h>
 #include <kcombobox.h>
 #include <klistview.h>
 #include <klocale.h>
@@ -58,7 +61,12 @@
 #include <kmessagebox.h>
 #include <kprocess.h>
 #include <kcolordialog.h>
-
+#include <kdiskfreesp.h>
+#if KDE_VERSION >= KDE_MAKE_VERSION(3,2,0)
+#include <kfontrequester.h>
+#else
+#include "kde32_kfontrequester.h"
+#endif
 #include "constants.h"
 #include "colours.h"
 #include "rosestrings.h"
@@ -83,7 +91,6 @@
 #include "widgets.h"
 #include "colourwidgets.h"
 #include "audiopluginmanager.h"
-#include "diskspace.h"
 #include "segmentcommands.h"
 #include "rgapplication.h"
 
@@ -332,11 +339,12 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
 
     QFrame *frame = new QFrame(m_tabWidget);
     QGridLayout *layout = new QGridLayout(frame,
-                                          6, 2, // nbrow, nbcol
+                                          8, 2, // nbrow, nbcol
                                           10, 5);
 
     layout->addWidget
-        (new QLabel(i18n("Notation font"), frame), 0, 0);
+        (new QLabel(i18n("Notation font"), frame),
+         0, 0);
     layout->addWidget
         (new QLabel(i18n("Font size for single-staff views"), frame),
          2, 0);
@@ -346,6 +354,12 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
     layout->addWidget
         (new QLabel(i18n("Font size for printing (pt)"), frame),
          4, 0);
+    layout->addWidget
+        (new QLabel(i18n("Text font"), frame),
+         5, 0);
+    layout->addWidget
+        (new QLabel(i18n("Time Signature font"), frame),
+         6, 0);
 
     QFrame *subFrame = new QFrame(frame);
     QGridLayout *subLayout = new QGridLayout(subFrame,
@@ -429,10 +443,33 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
     layout->addWidget(m_multiStaffSize, 3, 1);
     layout->addWidget(m_printingSize, 4, 1);
 
+#if KDE_VERSION < KDE_MAKE_VERSION(3,2,0)
+    m_textFont = new KDE32Backport::KFontRequester(frame);
+    m_timeSigFont = new KDE32Backport::KFontRequester(frame);
+#else
+    m_textFont = new KFontRequester(frame);
+    m_timeSigFont = new KFontRequester(frame);
+#endif
+
+    QFont defaultTextFont(NotePixmapFactory::defaultSerifFontFamily),
+        defaultTimeSigFont(NotePixmapFactory::defaultTimeSigFontFamily);
+    
+    QFont textFont    = m_cfg->readFontEntry("textfont", &defaultTextFont);
+    QFont timeSigFont = m_cfg->readFontEntry("timesigfont", &defaultTimeSigFont);
+
+    m_textFont->setFont(textFont);
+    m_timeSigFont->setFont(timeSigFont);
+
+    layout->addWidget(m_textFont, 5, 1);
+    layout->addWidget(m_timeSigFont, 6, 1);
+    
+
     addTab(frame, i18n("Font"));
 
+
+
     frame = new QFrame(m_tabWidget);
-    layout = new QGridLayout(frame, 6, 2, 10, 5);
+    layout = new QGridLayout(frame, 7, 2, 10, 5);
 
     layout->addWidget(new QLabel(i18n("Default layout mode"), frame), 0, 0);
 
@@ -442,7 +479,7 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
     m_layoutMode->insertItem(i18n("Continuous page layout"));
     m_layoutMode->insertItem(i18n("Multiple page layout"));
     int defaultLayoutMode = m_cfg->readNumEntry("layoutmode", 0);
-    if (defaultLayoutMode >= 0 && defaultLayoutMode <= 1) {
+    if (defaultLayoutMode >= 0 && defaultLayoutMode <= 2) {
         m_layoutMode->setCurrentItem(defaultLayoutMode);
     }
     layout->addWidget(m_layoutMode, 0, 1);
@@ -468,7 +505,7 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
 
     layout->addWidget(m_spacing, 1, 1);
     
-    layout->addWidget(new QLabel(i18n("Default proportion"), frame), 2, 0);
+    layout->addWidget(new QLabel(i18n("Default duration factor"), frame), 2, 0);
 
     m_proportion = new KComboBox(frame);
     m_proportion->setEditable(false);
@@ -498,13 +535,21 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
     layout->addWidget(m_showUnknowns, 3, 1);
 
     m_colourQuantize = new QCheckBox
-        (i18n("Show notation-quantized notes in a different colour"), frame);
+        (i18n("Show notation-quantized notes in a different color"), frame);
     bool defaultColourQuantize = m_cfg->readBoolEntry("colourquantize", false);
     m_colourQuantize->setChecked(defaultColourQuantize);
     layout->addWidget(m_colourQuantize, 4, 1);
 
+    m_showInvisibles = new QCheckBox
+        (i18n("Show \"invisible\" events in grey"), frame);
+    bool defaultShowInvisibles = m_cfg->readBoolEntry("showinvisibles", true);
+    m_showInvisibles->setChecked(defaultShowInvisibles);
+    layout->addWidget(m_showInvisibles, 5, 1);
+
     addTab(frame, i18n("Layout"));
 
+
+
     frame = new QFrame(m_tabWidget);
     layout = new QGridLayout(frame, 6, 2, 10, 5);
 
@@ -584,6 +629,48 @@ NotationConfigurationPage::NotationConfigurationPage(KConfig *cfg,
 
     addTab(frame, i18n("Editing"));
 
+
+
+    frame = new QFrame(m_tabWidget);
+    layout = new QGridLayout(frame, 4, 2, 10, 5);
+
+    layout->addWidget(new QLabel(i18n("Accidentals in one octave..."), frame), 0, 0);
+    m_accOctavePolicy = new KComboBox(frame);
+    m_accOctavePolicy->insertItem(i18n("Affect only that octave"));
+    m_accOctavePolicy->insertItem(i18n("Require cautionaries in other octaves"));
+    m_accOctavePolicy->insertItem(i18n("Affect all subsequent octaves"));
+    int accOctaveMode = m_cfg->readNumEntry("accidentaloctavemode", 1);
+    if (accOctaveMode >= 0 && accOctaveMode < 3) {
+	m_accOctavePolicy->setCurrentItem(accOctaveMode);
+    }
+    layout->addWidget(m_accOctavePolicy, 0, 1);
+
+    layout->addWidget(new QLabel(i18n("Accidentals in one bar..."), frame), 1, 0);
+    m_accBarPolicy = new KComboBox(frame);
+    m_accBarPolicy->insertItem(i18n("Affect only that bar"));
+    m_accBarPolicy->insertItem(i18n("Require cautionary resets in following bar"));
+    m_accBarPolicy->insertItem(i18n("Require explicit resets in following bar"));
+    int accBarMode = m_cfg->readNumEntry("accidentalbarmode", 0);
+    if (accBarMode >= 0 && accBarMode < 3) {
+	m_accBarPolicy->setCurrentItem(accBarMode);
+    }
+    layout->addWidget(m_accBarPolicy, 1, 1);
+
+    layout->addWidget(new QLabel(i18n("Key signature cancellation style:"), frame), 2, 0);
+    m_keySigCancelMode = new KComboBox(frame);
+    m_keySigCancelMode->insertItem(i18n("Cancel only when entering C major or A minor"));
+    m_keySigCancelMode->insertItem(i18n("Cancel whenever removing sharps or flats"));
+    m_keySigCancelMode->insertItem(i18n("Cancel always"));
+    int cancelMode = m_cfg->readNumEntry("keysigcancelmode", 1);
+    if (cancelMode >= 0 && cancelMode < 3) {
+	m_keySigCancelMode->setCurrentItem(cancelMode);
+    }
+    layout->addWidget(m_keySigCancelMode, 2, 1);
+
+    addTab(frame, i18n("Accidentals"));
+
+
+
     QString preamble =
         (i18n("Rosegarden can apply automatic quantization to recorded "
               "or imported MIDI data for notation purposes only. "
@@ -697,6 +784,10 @@ NotationConfigurationPage::apply()
                       m_multiStaffSize->currentText().toUInt());
     m_cfg->writeEntry("printingnotesize",
                       m_printingSize->currentText().toUInt());
+    m_cfg->writeEntry("textfont",
+                      m_textFont->font());
+    m_cfg->writeEntry("timesigfont",
+                      m_timeSigFont->font());
 
     std::vector<int> s = NotationHLayout::getAvailableSpacings();
     m_cfg->writeEntry("spacing", s[m_spacing->currentItem()]);
@@ -707,11 +798,15 @@ NotationConfigurationPage::apply()
     m_cfg->writeEntry("layoutmode", m_layoutMode->currentItem());
     m_cfg->writeEntry("colourquantize", m_colourQuantize->isChecked());
     m_cfg->writeEntry("showunknowns", m_showUnknowns->isChecked());
+    m_cfg->writeEntry("showinvisibles", m_showInvisibles->isChecked());
     m_cfg->writeEntry("style", m_untranslatedNoteStyle[m_noteStyle->currentItem()]);
     m_cfg->writeEntry("inserttype", m_insertType->currentItem());
     m_cfg->writeEntry("autobeam", m_autoBeam->isChecked());
     m_cfg->writeEntry("collapse", m_collapseRests->isChecked());
     m_cfg->writeEntry("pastetype", m_pasteType->currentItem());
+    m_cfg->writeEntry("accidentaloctavemode", m_accOctavePolicy->currentItem());
+    m_cfg->writeEntry("accidentalbarmode", m_accBarPolicy->currentItem());
+    m_cfg->writeEntry("keysigcancelmode", m_keySigCancelMode->currentItem());
 
     (void)m_quantizeFrame->getQuantizer(); // this also writes to the config
 }
@@ -976,6 +1071,8 @@ SequencerConfigurationPage::SequencerConfigurationPage(
     layout = new QGridLayout(frame, 4, 4, 10, 5);
 
 #ifdef HAVE_LIBJACK
+#define OFFER_JACK_START_OPTION 1
+#ifdef OFFER_JACK_START_OPTION
     label = new QLabel(i18n("Rosegarden can start the JACK audio daemon (jackd) for you\nautomatically if it isn't already running when Rosegarden starts.\n\nThis is recommended for beginners and those who use Rosegarden as their main\naudio application, but it might not be to the liking of advanced users.\n\nIf you want to start JACK automatically, make sure the command includes a full\npath where necessary as well as any command-line arguments you want to use.\n\nFor example: /usr/local/bin/jackd -d alsa -d hw -r44100 -p 2048 -n 2\n"), frame);
 
     layout->addMultiCellWidget(label, 1, 1, 0, 3);
@@ -1005,6 +1102,7 @@ SequencerConfigurationPage::SequencerConfigurationPage(
     // set the initial state
     slotJackToggled();
 
+#endif // OFFER_JACK_START_OPTION
 #endif // HAVE_LIBJACK
 
     addTab(frame, i18n("Startup"));
@@ -1150,6 +1248,8 @@ SequencerConfigurationPage::SequencerConfigurationPage(
 
     m_smallFile->setMinValue(5);
     m_smallFile->setMaxValue(15);
+    m_smallFile->setLineStep(1);
+    m_smallFile->setPageStep(1);
 
     int smallFileValue = m_cfg->readLongNumEntry("smallaudiofilekbytes", 128);
     int powerOfTwo = 1;
@@ -1174,24 +1274,35 @@ SequencerConfigurationPage::SequencerConfigurationPage(
             SIGNAL(valueChanged(int)),
             SLOT(slotSmallFileChanged(int)));
 
+    frame->hide();//!!!
+/*
     addTab(frame, i18n("Buffers"));
-
+*/
 
     // ------------------ Record tab ---------------------
     //
     frame = new QFrame(m_tabWidget);
-    layout = new QGridLayout(frame, 4, 2, 10, 5);
+    layout = new QGridLayout(frame, 5, 2, 10, 5);
 
     int increment = 0;
 
 #ifdef HAVE_LIBJACK
 
+    label = new QLabel(i18n("Audio mix and monitor mode:"), frame);
+    m_lowLatencyMode = new KComboBox(frame);
+    m_lowLatencyMode->insertItem(i18n("Low latency"));
+    m_lowLatencyMode->insertItem(i18n("Buffered"));
+    m_lowLatencyMode->setCurrentItem(m_cfg->readBoolEntry("audiolowlatencymonitoring", true) ? 0 : 1);
+    layout->addWidget(label, 0, 0);
+    layout->addWidget(m_lowLatencyMode, 0, 1);
+    ++increment;
+
     label = new QLabel(i18n("Create post-fader outputs for audio instruments"), frame);
     m_createFaderOuts = new QCheckBox(frame);
     m_createFaderOuts->setChecked(m_cfg->readBoolEntry("audiofaderouts", false));
 
-    layout->addWidget(label, 0, 0);
-    layout->addWidget(m_createFaderOuts, 0, 1);
+    layout->addWidget(label, 1, 0);
+    layout->addWidget(m_createFaderOuts, 1, 1);
     ++increment;
 
     label = new QLabel(i18n("Create post-fader outputs for submasters"), frame);
@@ -1199,12 +1310,15 @@ SequencerConfigurationPage::SequencerConfigurationPage(
     m_createSubmasterOuts->setChecked(m_cfg->readBoolEntry("audiosubmasterouts",
 							   false));
 
-    layout->addWidget(label, 1, 0);
-    layout->addWidget(m_createSubmasterOuts, 1, 1);
+    layout->addWidget(label, 2, 0);
+    layout->addWidget(m_createSubmasterOuts, 2, 1);
     ++increment;
 
 #endif // HAVE_LIBJACK
 
+/*  #1045380 ("minutes of audio recording" just insanely confusing)
+    Remove this option.
+
     label = new QLabel(i18n("Minutes of audio recording:"), frame);
     m_audioRecordMinutes = new QSpinBox(frame);
 
@@ -1216,7 +1330,7 @@ SequencerConfigurationPage::SequencerConfigurationPage(
     m_audioRecordMinutes->setValue(audioRecordMinutes);
     m_audioRecordMinutes->setMinValue(1);
     m_audioRecordMinutes->setMaxValue(60);
-
+*/
     addTab(frame, i18n("Record and Mix"));
 
     //  -------------- Synchronisation tab -----------------
@@ -1233,8 +1347,8 @@ SequencerConfigurationPage::SequencerConfigurationPage(
     layout->addWidget(m_timer, 0, 1); //, Qt::AlignHCenter);
 
     QStringList timers = m_doc->getTimers();
-    QString currentTimer = m_doc->getCurrentTimer();
-    currentTimer = m_cfg->readEntry("timer", currentTimer);
+    m_origTimer = m_doc->getCurrentTimer();
+    QString currentTimer = m_cfg->readEntry("timer", m_origTimer);
 
     for (unsigned int i = 0; i < timers.size(); ++i) {
 	m_timer->insertItem(timers[i]);
@@ -1261,21 +1375,25 @@ SequencerConfigurationPage::SequencerConfigurationPage(
 
     m_jackTransport->insertItem(i18n("Ignore JACK transport"));
     m_jackTransport->insertItem(i18n("Sync"));
+/*!!! Removed as not yet implemented
     m_jackTransport->insertItem(i18n("Sync, and offer timebase master"));
+*/
 
     bool jackMaster = m_cfg->readBoolEntry("jackmaster", false);
     bool jackTransport = m_cfg->readBoolEntry("jacktransport", false);
-
+/*!!!
     if (jackMaster && jackTransport)
         m_jackTransport->setCurrentItem(2);
     else 
     {
+*/
         if (jackTransport)
             m_jackTransport->setCurrentItem(1);
         else
             m_jackTransport->setCurrentItem(0);
-    }
+//!!!    }
 
+    /*
     // MMC Transport
     //
     label = new QLabel(i18n("MMC transport mode"), frame);
@@ -1301,6 +1419,21 @@ SequencerConfigurationPage::SequencerConfigurationPage(
             m_mmcTransport->setCurrentItem(0);
     }
 
+    // MTC transport (send only for the moment)
+    //
+    label = new QLabel(i18n("MTC send"), frame);
+    layout->addWidget(label, 4, 0);
+
+    m_mtcTransport = new KComboBox(frame);
+    layout->addWidget(m_mtcTransport, 4, 1);
+
+    m_mtcTransport->insertItem(i18n("Off"));
+    m_mtcTransport->insertItem(i18n("MTC Master"));
+
+    bool mtcMaster = m_cfg->readBoolEntry("mtcmaster", false);
+
+    if (mtcMaster) m_mtcTransport->setCurrentItem(1);
+    */
 
     addTab(frame, i18n("Synchronisation"));
 }
@@ -1490,19 +1623,24 @@ SequencerConfigurationPage::apply()
 
 #ifdef HAVE_LIBJACK
 
+#ifdef OFFER_JACK_START_OPTION
     // Jack control
     //
     m_cfg->writeEntry("jackstart", m_startJack->isChecked());
     m_cfg->writeEntry("jackcommand", m_jackPath->text());
+#endif // OFFER_JACK_START_OPTION
 
     // Jack audio inputs
     //
+    m_cfg->writeEntry("audiolowlatencymonitoring", m_lowLatencyMode->currentItem() == 0);
     m_cfg->writeEntry("audiofaderouts", m_createFaderOuts->isChecked());
     m_cfg->writeEntry("audiosubmasterouts", m_createSubmasterOuts->isChecked());
 
     // Audio record minutes
     //
+/*  #1045380 ("minutes of audio recording" just insanely confusing) -- remove
     m_cfg->writeEntry("audiorecordminutes", m_audioRecordMinutes->value());
+*/
 
     Rosegarden::MidiByte ports = 0;
     if (m_createFaderOuts->isChecked()) {
@@ -1519,9 +1657,9 @@ SequencerConfigurationPage::apply()
     Rosegarden::StudioControl::sendMappedEvent(mEports);
 
     m_cfg->writeEntry("timer", m_timer->currentText());
-    m_doc->setCurrentTimer(m_timer->currentText());
-    
-
+    if (m_timer->currentText() != m_origTimer) {
+	m_doc->setCurrentTimer(m_timer->currentText());
+    }
 
     // Write the JACK entry
     //
@@ -1563,6 +1701,7 @@ SequencerConfigurationPage::apply()
     Rosegarden::StudioControl::sendMappedEvent(mEjackValue);
 #endif // HAVE_LIBJACK
 
+	/*
     // Now write the MMC entry
     //
     bool mmcTransport, mmcMaster;
@@ -1590,10 +1729,15 @@ SequencerConfigurationPage::apply()
 
     }
 
+    bool mtcMaster = false;
+    if (m_mtcTransport->currentItem() == 1) mtcMaster = true;
+
     // Write the entries
     //
     m_cfg->writeEntry("mmctransport", mmcTransport);
     m_cfg->writeEntry("mmcmaster", mmcMaster);
+    m_cfg->writeEntry("mtcmaster", mtcMaster);
+    
 
     // Now send it
     //
@@ -1602,20 +1746,34 @@ SequencerConfigurationPage::apply()
                                        Rosegarden::MidiByte(mmcValue));
 
     Rosegarden::StudioControl::sendMappedEvent(mEmccValue);
-
+	*/
 
     // ------------- MIDI Clock and System messages ------------
     //
     bool midiClock = m_midiClockEnabled->isChecked();
     m_cfg->writeEntry("midiclock", midiClock);
 
-    // Now send it
+    // Now send it (OLD METHOD - to be removed)
     //
     Rosegarden::MappedEvent mEMIDIClock(Rosegarden::MidiInstrumentBase, // InstrumentId
                                         Rosegarden::MappedEvent::SystemMIDIClock,
                                         Rosegarden::MidiByte(midiClock));
 
     Rosegarden::StudioControl::sendMappedEvent(mEMIDIClock);
+
+
+    // Now update the metronome mapped segment with new clock ticks
+    // if needed.
+    //
+    Rosegarden::Studio &studio = m_doc->getStudio();
+    const Rosegarden::MidiMetronome *metronome = studio.
+        getMetronomeFromDevice(studio.getMetronomeDevice());
+
+    if (metronome)
+    {
+        Rosegarden::InstrumentId instrument = metronome->getInstrument();
+        m_doc->getSequenceManager()->metronomeChanged(instrument, true);
+    }
 }
 
 // ---
@@ -1658,7 +1816,18 @@ DocumentMetaConfigurationPage::DocumentMetaConfigurationPage(RosegardenGUIDoc *d
     TabbedConfigurationPage(doc, parent, name)
 {
     QFrame *frame = new QFrame(m_tabWidget);
-    QGridLayout *layout = new QGridLayout(frame, 2, 2, 10, 5);
+    QGridLayout *layout = new QGridLayout(frame, 3, 2, 10, 5);
+
+    m_fixed = new KListView(frame);
+    m_fixed->addColumn(i18n("Name"));
+    m_fixed->addColumn(i18n("Value"));
+    m_fixed->setFullWidth();
+    m_fixed->setItemsRenameable(true);
+    m_fixed->setRenameable(1);
+    m_fixed->setItemMargin(5);
+    m_fixed->setSorting(-1);
+    m_fixed->setDefaultRenameAction(QListView::Accept);
+    m_fixed->setShowSortIndicator(false);
 
     m_metadata = new KListView(frame);
     m_metadata->addColumn(i18n("Name"));
@@ -1668,29 +1837,65 @@ DocumentMetaConfigurationPage::DocumentMetaConfigurationPage(RosegardenGUIDoc *d
     m_metadata->setRenameable(0);
     m_metadata->setRenameable(1);
     m_metadata->setItemMargin(5);
-    //m_metadata->setSelectionModeExt(KListView::NoSelection);
     m_metadata->setDefaultRenameAction(QListView::Accept);
     m_metadata->setShowSortIndicator(true);
 
     Rosegarden::Configuration &metadata = doc->getComposition().getMetadata();
-    std::vector<std::string> names(metadata.getPropertyNames());
-    for (unsigned int i = 0; i < names.size(); ++i) {
-        QString name(strtoqstr(names[i]));
-        // property names stored in lower case
-        name = name.left(1).upper() + name.right(name.length()-1);
-        new KListViewItem(m_metadata, name,
-                          strtoqstr(metadata.get<String>(names[i])));
+
+    std::set<std::string> shown;
+
+    std::vector<Rosegarden::PropertyName> fixedKeys =
+	CompositionMetadataKeys::getFixedKeys();
+
+    // do these in reverse order, as the list appears to insert at the start
+    for (unsigned int i = fixedKeys.size(); i > 0; --i) {
+	Rosegarden::PropertyName pn = fixedKeys[i-1];
+	QString trName;
+	if (pn == CompositionMetadataKeys::Copyright) {
+	    trName = i18n("Copyright");
+	} else if (pn == CompositionMetadataKeys::Composer) {
+	    trName = i18n("Composer");
+	} else if (pn == CompositionMetadataKeys::Title) {
+	    trName = i18n("Title");
+	} else if (pn == CompositionMetadataKeys::Subtitle) {
+	    trName = i18n("Subtitle");
+	} else if (pn == CompositionMetadataKeys::Arranger) {
+	    trName = i18n("Arranger");
+	} else {
+	    trName = strtoqstr(pn.getName());
+	    trName = trName.left(1).upper() + trName.right(trName.length()-1);
+	}
+	new KListViewItem(m_fixed, trName, strtoqstr(metadata.get<String>(pn, "")));
+	shown.insert(pn.getName());
     }
 
-    layout->addMultiCellWidget(m_metadata, 0, 0, 0, 1);
+    std::vector<std::string> names(metadata.getPropertyNames());
+
+    for (unsigned int i = 0; i < names.size(); ++i) {
+
+	if (shown.find(names[i]) != shown.end()) continue;
+
+        QString name(strtoqstr(names[i]));
+
+        // property names stored in lower case
+        name = name.left(1).upper() + name.right(name.length()-1);
+
+        new KListViewItem(m_metadata, name,
+                          strtoqstr(metadata.get<String>(names[i])));
+
+	shown.insert(names[i]);
+    }
+
+    layout->addMultiCellWidget(m_fixed, 0, 0, 0, 1);
+    layout->addMultiCellWidget(m_metadata, 1, 1, 0, 1);
 
     QPushButton* addPropButton = new QPushButton(i18n("Add New Property"),
                                                  frame);
-    layout->addWidget(addPropButton, 1, 0, Qt::AlignHCenter);
+    layout->addWidget(addPropButton, 2, 0, Qt::AlignHCenter);
 
     QPushButton* deletePropButton = new QPushButton(i18n("Delete Property"),
                                                     frame);
-    layout->addWidget(deletePropButton, 1, 1, Qt::AlignHCenter);
+    layout->addWidget(deletePropButton, 2, 1, Qt::AlignHCenter);
 
     connect(addPropButton, SIGNAL(clicked()),
             this, SLOT(slotAddNewProperty()));
@@ -1742,6 +1947,8 @@ DocumentMetaConfigurationPage::DocumentMetaConfigurationPage(RosegardenGUIDoc *d
                                  .arg(internalSegments + audioSegments),
                                  frame), 4, 1);
     
+    layout->setRowStretch(5, 2);
+
     addTab(frame, i18n("Statistics"));
 
     frame = new QFrame(m_tabWidget);
@@ -1793,7 +2000,7 @@ DocumentMetaConfigurationPage::DocumentMetaConfigurationPage(RosegardenGUIDoc *d
 	QPixmap colourPixmap(16, 16);
 	Rosegarden::Colour colour =
 	    comp.getSegmentColourMap().getColourByIndex(s->getColourIndex());
-	colourPixmap.fill(RosegardenGUIColours::convertColour(colour));
+	colourPixmap.fill(Rosegarden::GUIPalette::convertColour(colour));
 
 	table->setItem(i, 2,
 		       new QTableItem(table, QTableItem::Never,
@@ -1930,8 +2137,23 @@ DocumentMetaConfigurationPage::apply()
     metadata.clear();
     
     // If one of the items still has focus, it won't remember edits
+    m_fixed->setFocus();
     m_metadata->setFocus();
 
+    std::vector<Rosegarden::PropertyName> fixedKeys =
+	CompositionMetadataKeys::getFixedKeys();
+    std::vector<Rosegarden::PropertyName>::iterator i = fixedKeys.begin();
+
+    for (QListViewItem *item = m_fixed->firstChild();
+         item != 0; item = item->nextSibling()) {
+
+	if (i == fixedKeys.end()) break;
+
+        metadata.set<String>(*i, qstrtostr(item->text(1)));
+
+	++i;
+    }
+
     for (QListViewItem *item = m_metadata->firstChild();
          item != 0; item = item->nextSibling()) {
 
@@ -1945,6 +2167,24 @@ DocumentMetaConfigurationPage::apply()
 void
 DocumentMetaConfigurationPage::selectMetadata(QString name)
 {
+    std::vector<Rosegarden::PropertyName> fixedKeys =
+	CompositionMetadataKeys::getFixedKeys();
+    std::vector<Rosegarden::PropertyName>::iterator i = fixedKeys.begin();
+
+    for (QListViewItem *item = m_fixed->firstChild();
+         item != 0; item = item->nextSibling()) {
+
+	if (i == fixedKeys.end()) break;
+
+	if (name == strtoqstr(i->getName())) {
+	    m_fixed->setSelected(item, true);
+	    m_fixed->setCurrentItem(item);
+	    return;
+	}
+
+	++i;
+    }
+
     for (QListViewItem *item = m_metadata->firstChild();
          item != 0; item = item->nextSibling()) {
 
@@ -1967,8 +2207,7 @@ AudioConfigurationPage::AudioConfigurationPage(RosegardenGUIDoc *doc,
     Rosegarden::AudioFileManager &afm = doc->getAudioFileManager();
 
     QFrame *frame = new QFrame(m_tabWidget);
-    QGridLayout *layout = new QGridLayout(frame, 4, 3,
-                                          10, 5);
+    QGridLayout *layout = new QGridLayout(frame, 4, 3, 10, 5);
     layout->addWidget(new QLabel(i18n("Audio file path:"), frame), 0, 0);
     m_path = new QLabel(QString(afm.getAudioPath().c_str()), frame);
     layout->addWidget(m_path, 0, 1);
@@ -1989,6 +2228,8 @@ AudioConfigurationPage::AudioConfigurationPage(RosegardenGUIDoc *doc,
 
     layout->addWidget(m_minutesAtStereo, 2, 1, AlignCenter);
 
+    layout->setRowStretch(3, 2);
+
     calculateStats();
 
     connect(m_changePathButton, SIGNAL(released()),
@@ -2000,47 +2241,59 @@ AudioConfigurationPage::AudioConfigurationPage(RosegardenGUIDoc *doc,
 void
 AudioConfigurationPage::calculateStats()
 {
-    Rosegarden::DiskSpace *space;
-    try
-    {
-        space = new Rosegarden::DiskSpace(m_path->text());
-    }
-    catch (QString e)
-    {
-        KMessageBox::error(this, e);
-        return;
-    }
-    
-    float mbSize = float(space->getFreeKBytes())/1024.0;
-    QString mbSizeStr;
-    mbSizeStr.sprintf("%10.3f", mbSize);
-    m_diskSpace->setText(QString("%1 MB").arg(mbSizeStr));
-
-    // Work out minutes of recordable stereo from centralised sample rate value
+    // This stolen from KDE libs kfile/kpropertiesdialog.cpp
     //
+    QString mountPoint = KIO::findPathMountPoint(m_path->text());
+    KDiskFreeSp * job = new KDiskFreeSp;
+    connect(job, SIGNAL(foundMountPoint(const QString&, unsigned long, unsigned long,
+                                        unsigned long)),
+            this, SLOT(slotFoundMountPoint(const QString&, unsigned long, unsigned long,
+                                           unsigned long)));
+    job->readDF(mountPoint);
+}
+
+void
+AudioConfigurationPage::slotFoundMountPoint(const QString&,
+					    unsigned long kBSize,
+					    unsigned long /*kBUsed*/,
+					    unsigned long kBAvail )
+{
+    m_diskSpace->setText(i18n("%1 out of %2 (%3% used)")
+                         .arg(KIO::convertSizeFromKB(kBAvail))
+                         .arg(KIO::convertSizeFromKB(kBSize))
+                         .arg( 100 - (int)(100.0 * kBAvail / kBSize) ));
+
+
     Rosegarden::AudioPluginManager *apm = m_doc->getPluginManager();
 
-    if (apm == 0 || apm->getSampleRate() == 0)
-    {
-        m_minutesAtStereo->setText(i18n("<sample rate not available>"));
-        return;
+    int sampleRate = 48000;
+    QCString replyType;
+    QByteArray replyData;
+
+    if (rgapp->sequencerCall("getSampleRate()", replyType, replyData)) {
+
+        QDataStream streamIn(replyData, IO_ReadOnly);
+        unsigned int result;
+        streamIn >> result;
+        sampleRate = result;
     }
 
     // Work out total bytes and divide this by the sample rate times the
     // number of channels (2) times the number of bytes per sample (2)
     // times 60 seconds.
     //
-    float stereoMins = ( float(space->getFreeKBytes()) * 1024.0 ) / 
-                       ( float(apm->getSampleRate()) * 2.0 * 2.0 * 60.0 );
+    float stereoMins = ( float(kBAvail) * 1024.0 ) / 
+                       ( float(sampleRate) * 2.0 * 2.0 * 60.0 );
     QString minsStr;
     minsStr.sprintf("%8.1f", stereoMins);
 
     m_minutesAtStereo->
         setText(QString("%1 %2 %3Hz").arg(minsStr)
                                      .arg(i18n("minutes at"))
-                                     .arg(apm->getSampleRate()));
+                                     .arg(sampleRate));
 }
 
+
 void
 AudioConfigurationPage::slotFileDialog()
 {
@@ -2131,7 +2384,7 @@ ColourConfigurationPage::slotTextChanged(unsigned int index, QString string)
 void
 ColourConfigurationPage::slotColourChanged(unsigned int index, QColor color)
 {
-    m_map.modifyColourByIndex(m_listmap[index], RosegardenGUIColours::convertColour(color));
+    m_map.modifyColourByIndex(m_listmap[index], Rosegarden::GUIPalette::convertColour(color));
     m_colourtable->populate_table(m_map, m_listmap);
 }
 
@@ -2160,7 +2413,7 @@ ColourConfigurationPage::slotAddNew()
 
         if (result == KColorDialog::Accepted)
         {
-            Rosegarden::Colour temp2 = RosegardenGUIColours::convertColour(temp);
+            Rosegarden::Colour temp2 = Rosegarden::GUIPalette::convertColour(temp);
             m_map.addItem(temp2, qstrtostr(newName));
             m_colourtable->populate_table(m_map, m_listmap);
         }
@@ -2194,8 +2447,9 @@ static inline QPixmap loadIcon(const char *name)
 }
 
 ConfigureDialogBase::ConfigureDialogBase(QWidget *parent,
+					 QString label,
                                          const char *name):
-    KDialogBase(IconList, i18n("Configure"), Help|Apply|Ok|Cancel,
+    KDialogBase(IconList, label ? label : i18n("Configure"), Help|Apply|Ok|Cancel,
                 Ok, parent, name, true) // modal
 {
     setWFlags(WDestructiveClose);
@@ -2236,7 +2490,7 @@ ConfigureDialog::ConfigureDialog(RosegardenGUIDoc *doc,
                                  KConfig* cfg,
                                  QWidget *parent,
                                  const char *name)
-    : ConfigureDialogBase(parent, name)
+    : ConfigureDialogBase(parent, i18n("Configure Rosegarden"), name)
 {
     QWidget *pageWidget = 0;
     QVBoxLayout *vlay = 0;
@@ -2305,7 +2559,7 @@ ConfigureDialog::ConfigureDialog(RosegardenGUIDoc *doc,
 DocumentConfigureDialog::DocumentConfigureDialog(RosegardenGUIDoc *doc,
                                                  QWidget *parent,
                                                  const char *name)
-    : ConfigureDialogBase(parent, name)
+    : ConfigureDialogBase(parent, i18n("Document Properties"), name)
 {
     QWidget *pageWidget = 0;
     QVBoxLayout *vlay = 0;
