@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -34,6 +34,7 @@
 
 #include "rosegardendcop.h"
 #include "rosegardengui.h"
+#include "audiopreviewthread.h"
 
 #include "Composition.h"
 #include "MappedComposition.h"
@@ -96,6 +97,10 @@ private:
     RosegardenGUIDoc& operator=(const RosegardenGUIDoc &doc);
 
 public:
+    static int FILE_FORMAT_VERSION_MAJOR;
+    static int FILE_FORMAT_VERSION_MINOR;
+    static int FILE_FORMAT_VERSION_POINT;
+
     /**
      * Destructor for the fileclass of the application
      */
@@ -156,7 +161,7 @@ public:
      * "save modified" - asks the user for saving if the document is
      * modified
      */
-    bool saveIfModified();	
+    bool saveIfModified();
 
     /**
      * get the autosave interval in seconds
@@ -181,8 +186,10 @@ public:
 
     /**
      * saves the document under filename and format.
+     *
+     * errMsg will be set to a user-readable error message if save fails
      */	
-    bool saveDocument(const QString &filename, const char *format=0,
+    bool saveDocument(const QString &filename, QString& errMsg,
 		      bool autosave = false);
 
     /**
@@ -243,6 +250,15 @@ public:
 
     const Rosegarden::Studio& getStudio() const { return m_studio;}
 
+    /*
+     * return the AudioPreviewThread
+     */
+    AudioPreviewThread& getAudioPreviewThread()
+        { return m_audioPreviewThread; }
+
+    const AudioPreviewThread& getAudioPreviewThread() const
+        { return m_audioPreviewThread; }
+
     /*
      * return the AudioFileManager
      */
@@ -317,6 +333,12 @@ public:
      */
     void setLoop(Rosegarden::timeT, Rosegarden::timeT);
 
+    /**
+     * Cause the document to use the given time as the origin
+     * when inserting any subsequent recorded data
+     */
+    void setRecordStartTime(Rosegarden::timeT t) { m_recordStartTime = t; }
+
     /*
     * Sync device information with sequencer
     */
@@ -514,7 +536,9 @@ protected:
     /**
      * Save one segment to the given text stream
      */
-    void saveSegment(QTextStream&, Rosegarden::Segment*, KProgress* = 0, int totalNbOfEvents = 0);
+    void saveSegment(QTextStream&, Rosegarden::Segment*, KProgress*,
+                     long totalNbOfEvents, long &count,
+                     QString extraAttributes = QString::null);
 
     //--------------- Data members ---------------------------------
 
@@ -558,12 +582,30 @@ protected:
      */
     Rosegarden::AudioFileManager m_audioFileManager;
 
+    /**
+     * calculates AudioFile previews
+     */
+    AudioPreviewThread m_audioPreviewThread;
+
     /**
      * a Segment onto which we can record events
      */
     Rosegarden::Segment *m_recordSegment;
 
-    typedef std::map<int, Rosegarden::Segment::iterator> NoteOnMap;
+    /**
+     * a map[Pitch] of Rosegarden::Event elements, for NoteOn calculations
+     */
+    typedef std::map<int, Rosegarden::Segment::iterator>	PitchMap;
+    
+    /**
+     * a map[Channel] of PitchMap
+     */
+    typedef std::map<int, PitchMap>				ChanMap;
+    
+    /**
+     * a map[Port] of ChanMap
+     */
+    typedef std::map<int, ChanMap>				NoteOnMap;
 
     /**
      * During recording, we collect note-ons that haven't yet had a note-off
@@ -591,6 +633,8 @@ protected:
 
     Rosegarden::RealTime m_audioRecordLatency;
 
+    Rosegarden::timeT m_recordStartTime;
+
     // Autosave period for this document in seconds
     //
     int m_autoSavePeriod;
