@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -81,16 +81,14 @@ k_dcop:
     //
     virtual void alive() = 0;
 
-    // the sequencer has skipped so many slices
-    //
-    virtual void skippedSlices(unsigned int slices) = 0;
-
     // The sequencer requests that a new audio file is created - the
     // gui does so and returns the path of the new file so that the
     // sequencer can use it.
     //
     virtual QString createNewAudioFile() = 0;
 
+    virtual void showError(QString error) = 0;
+
     // Actions proxy
     //
     DCOPRef action( const QCString &name );
