@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -102,25 +102,6 @@ RosegardenGUIView::RosegardenGUIView(bool showTrackLabels,
         Rosegarden::TimeSignature(4, 4).getBarDuration() / barWidth44;
     m_rulerScale = new SimpleRulerScale(comp, 0, unitsPerPixel);
 
-//     QHBox *hbox = new QHBox(this);
-
-//     m_vbox = new QFrame(parametersDock /*hbox*/);
-//     parametersDock->setWidget(m_vbox);
-
-//     m_vboxLayout = new QVBoxLayout(m_vbox, 5);
-
-    // Segment and Instrument Parameter Boxes [rwb]
-    //
-//     m_segmentParameterBox = new SegmentParameterBox(this, m_vbox);
-//     m_vboxLayout->addWidget(m_segmentParameterBox);
-
-//     connect(m_segmentParameterBox, SIGNAL(documentModified()),
-//             doc, SLOT(slotDocumentModified()));
-
-//     m_instrumentParameterBox = new InstrumentParameterBox(getDocument(), m_vbox);
-//     m_vboxLayout->addWidget(m_instrumentParameterBox);
-//     m_vboxLayout->addStretch();
-
     // Construct the trackEditor first so we can then
     // query it for placement information
     //
@@ -179,21 +160,6 @@ RosegardenGUIView::RosegardenGUIView(bool showTrackLabels,
             this,
             SLOT(slotChangeInstrumentLabel(Rosegarden::InstrumentId, QString)));
 
-    connect(m_instrumentParameterBox,
-            SIGNAL(setMute(Rosegarden::InstrumentId, bool)),
-            this,
-            SLOT(slotSetMute(Rosegarden::InstrumentId, bool)));
-
-    connect(m_instrumentParameterBox,
-            SIGNAL(setSolo(Rosegarden::InstrumentId, bool)),
-            this,
-            SLOT(slotSetSolo(Rosegarden::InstrumentId, bool)));
-
-    connect(m_instrumentParameterBox,
-            SIGNAL(setRecord(Rosegarden::InstrumentId, bool)),
-            this,
-            SLOT(slotSetRecord(Rosegarden::InstrumentId, bool)));
-
     if (doc) {
         m_trackEditor->setupSegments();
 	connect(doc, SIGNAL(recordingSegmentUpdated(Rosegarden::Segment *,
@@ -460,8 +426,12 @@ RosegardenGUIView::createNotationView(std::vector<Rosegarden::Segment *> segment
 	    this, SLOT(slotEditTempos(Rosegarden::timeT)));
     connect(notationView, SIGNAL(editMetadata(QString)),
 	    this, SLOT(slotEditMetadata(QString)));
+    connect(notationView, SIGNAL(editTriggerSegment(int)),
+	    this, SLOT(slotEditTriggerSegment(int)));
     connect(notationView, SIGNAL(staffLabelChanged(Rosegarden::TrackId, QString)),
 	    this, SLOT(slotChangeTrackLabel(Rosegarden::TrackId, QString)));
+    connect(notationView, SIGNAL(toggleSolo(bool)),
+            RosegardenGUIApp::self(), SLOT(slotToggleSolo(bool)));
 
     Rosegarden::SequenceManager *sM = getDocument()->getSequenceManager();
 
@@ -474,6 +444,10 @@ RosegardenGUIView::createNotationView(std::vector<Rosegarden::Segment *> segment
 	    this, SIGNAL(stepByStepTargetRequested(QObject *)));
     connect(this, SIGNAL(stepByStepTargetRequested(QObject *)),
 	    notationView, SLOT(slotStepByStepTargetRequested(QObject *)));
+    connect(RosegardenGUIApp::self(), SIGNAL(compositionStateUpdate()),
+	    notationView, SLOT(slotCompositionStateUpdate()));
+    connect(this, SIGNAL(compositionStateUpdate()),
+	    notationView, SLOT(slotCompositionStateUpdate()));
 
     // Encourage the notation view window to open to the same
     // interval as the current segment view
@@ -523,8 +497,13 @@ void RosegardenGUIView::slotEditSegmentMatrix(Rosegarden::Segment* p)
 
 void RosegardenGUIView::slotEditSegmentsMatrix(std::vector<Rosegarden::Segment *> segmentsToEdit)
 {
-    MatrixView *view = createMatrixView(segmentsToEdit);
-    if (view) view->show();
+    for (std::vector<Rosegarden::Segment *>::iterator i = segmentsToEdit.begin();
+	 i != segmentsToEdit.end(); ++i) {
+	std::vector<Rosegarden::Segment *> tmpvec;
+	tmpvec.push_back(*i);
+	MatrixView *view = createMatrixView(tmpvec);
+	if (view) view->show();
+    }
 }
 
 MatrixView *
@@ -567,6 +546,10 @@ RosegardenGUIView::createMatrixView(std::vector<Rosegarden::Segment *> segmentsT
 	    this, SLOT(slotEditSegmentsMatrix(std::vector<Rosegarden::Segment *>)));
     connect(matrixView, SIGNAL(openInEventList(std::vector<Rosegarden::Segment *>)),
 	    this, SLOT(slotEditSegmentsEventList(std::vector<Rosegarden::Segment *>)));
+    connect(matrixView, SIGNAL(editTriggerSegment(int)),
+	    this, SLOT(slotEditTriggerSegment(int)));
+    connect(matrixView, SIGNAL(toggleSolo(bool)),
+            RosegardenGUIApp::self(), SLOT(slotToggleSolo(bool)));
 
     Rosegarden::SequenceManager *sM = getDocument()->getSequenceManager();
 
@@ -579,6 +562,16 @@ RosegardenGUIView::createMatrixView(std::vector<Rosegarden::Segment *> segmentsT
 	    this, SIGNAL(stepByStepTargetRequested(QObject *)));
     connect(this, SIGNAL(stepByStepTargetRequested(QObject *)),
 	    matrixView, SLOT(slotStepByStepTargetRequested(QObject *)));
+    connect(RosegardenGUIApp::self(), SIGNAL(compositionStateUpdate()),
+	    matrixView, SLOT(slotCompositionStateUpdate()));
+    connect(this, SIGNAL(compositionStateUpdate()),
+	    matrixView, SLOT(slotCompositionStateUpdate()));
+    connect(this,
+	    SIGNAL(instrumentLevelsChanged(Rosegarden::InstrumentId,
+					   const Rosegarden::LevelInfo &)),
+	    matrixView,
+	    SLOT(slotInstrumentLevelsChanged(Rosegarden::InstrumentId,
+					     const Rosegarden::LevelInfo &)));
 
     // Encourage the matrix view window to open to the same
     // interval as the current segment view
@@ -628,6 +621,17 @@ void RosegardenGUIView::slotEditSegmentEventList(Rosegarden::Segment *p)
 }
 
 void RosegardenGUIView::slotEditSegmentsEventList(std::vector<Rosegarden::Segment *> segmentsToEdit)
+{
+    for (std::vector<Rosegarden::Segment *>::iterator i = segmentsToEdit.begin();
+	 i != segmentsToEdit.end(); ++i) {
+	std::vector<Rosegarden::Segment *> tmpvec;
+	tmpvec.push_back(*i);
+	EventView *view = createEventView(tmpvec);
+	if (view) view->show();
+    }
+}
+
+EventView *RosegardenGUIView::createEventView(std::vector<Rosegarden::Segment *> segmentsToEdit)
 {
     EventView *eventView = new EventView(getDocument(),
                                          segmentsToEdit,
@@ -645,6 +649,14 @@ void RosegardenGUIView::slotEditSegmentsEventList(std::vector<Rosegarden::Segmen
 	    this, SLOT(slotEditSegmentsMatrix(std::vector<Rosegarden::Segment *>)));
     connect(eventView, SIGNAL(openInEventList(std::vector<Rosegarden::Segment *>)),
 	    this, SLOT(slotEditSegmentsEventList(std::vector<Rosegarden::Segment *>)));
+    connect(eventView, SIGNAL(editTriggerSegment(int)),
+	    this, SLOT(slotEditTriggerSegment(int)));
+    connect(this, SIGNAL(compositionStateUpdate()),
+	    eventView, SLOT(slotCompositionStateUpdate()));
+    connect(RosegardenGUIApp::self(), SIGNAL(compositionStateUpdate()),
+	    eventView, SLOT(slotCompositionStateUpdate()));
+    connect(eventView, SIGNAL(toggleSolo(bool)),
+            RosegardenGUIApp::self(), SLOT(slotToggleSolo(bool)));
 
     // create keyboard accelerators on view
     //
@@ -654,7 +666,24 @@ void RosegardenGUIView::slotEditSegmentsEventList(std::vector<Rosegarden::Segmen
         par->plugAccelerators(eventView, eventView->getAccelerators());
     }
 
-    eventView->show();
+    return eventView;
+}
+
+void RosegardenGUIView::slotEditTriggerSegment(int id)
+{
+    SetWaitCursor waitCursor;
+
+    std::vector<Rosegarden::Segment *> segmentsToEdit;
+
+    Rosegarden::Segment *s = getDocument()->getComposition().getTriggerSegment(id);
+
+    if (s) {
+	segmentsToEdit.push_back(s);
+    } else {
+	return;
+    }
+
+    slotEditSegmentsEventList(segmentsToEdit);
 }
 
 void RosegardenGUIView::slotSegmentAutoSplit(Rosegarden::Segment *segment)
@@ -686,6 +715,11 @@ void RosegardenGUIView::slotEditSegmentAudio(Rosegarden::Segment *segment)
         std::cerr << "RosegardenGUIView::slotEditSegmentAudio() - "
                   << "external editor \"" << application.data()
                   << "\" not found" << std::endl;
+
+
+        KMessageBox::sorry(this, 
+                i18n("You've not yet defined an audio editor for Rosegarden to use.\nSee Settings -> Configure Rosegarden -> General -> External Editors."));
+
         return;
     }
 
@@ -741,7 +775,7 @@ void RosegardenGUIView::setZoomSize(double size)
 
     QWMatrix zoomMatrix;
     double xScale = duration44/(size * barWidth44);
-    //RG_DEBUG << "RosegardenGUIView::setZoomSize - xScale =  " << xScale << endl;
+    RG_DEBUG << "RosegardenGUIView::setZoomSize - xScale =  " << xScale << endl;
 
     zoomMatrix.scale(xScale, 1.0);
     m_trackEditor->getSegmentCanvas()->setWorldMatrix(zoomMatrix);
@@ -819,25 +853,9 @@ void RosegardenGUIView::slotSelectTrackSegments(int trackId)
 
     slotSetSelectedSegments(segments);
 
-    // Select track for recording if the current one is MIDI
-    // and the one we're going to is MIDI.
-    //
-    Rosegarden::Studio &studio = getDocument()->getStudio();
-
-    if (track)
-    {
-        Rosegarden::Instrument *instr =
-            studio.getInstrumentById(track->getInstrument());
-
-        if (instr && instr->getType() == Rosegarden::Instrument::Midi)
-        {
-            getTrackEditor()->getTrackButtons()->
-                slotSetRecordTrack(track->getPosition());
-        }
-    }
-    
     // inform
     emit segmentsSelected(segments);
+    emit compositionStateUpdate();
 }
 
 void RosegardenGUIView::slotSetSelectedSegments(
@@ -938,6 +956,7 @@ void RosegardenGUIView::slotUpdateInstrumentParameterBox(int id)
 
     // Then do this instrument/track fiddling
     //
+/*
     if (track && instrument &&
             instrument->getType() == Rosegarden::Instrument::Audio)
     {
@@ -952,7 +971,7 @@ void RosegardenGUIView::slotUpdateInstrumentParameterBox(int id)
         m_instrumentParameterBox->setSolo(
                 comp.isSolo() && (track->getId() == comp.getSelectedTrack()));
     }
-
+*/
     emit checkTrackAssignments();
 }
 
@@ -1014,22 +1033,34 @@ void RosegardenGUIView::showVisuals(const Rosegarden::MappedEvent *mE)
 void
 RosegardenGUIView::updateMeters(SequencerMapper *mapper)
 {
-    std::set<Rosegarden::InstrumentId> doneInstruments;
+    const int unknownState = 0, oldState = 1, newState = 2;
+
+    typedef std::map<Rosegarden::InstrumentId, int> StateMap;
+    static StateMap states;
+
+    typedef std::map<Rosegarden::InstrumentId, Rosegarden::LevelInfo> LevelMap;
+    static LevelMap levels;
+
+    for (StateMap::iterator i = states.begin(); i != states.end(); ++i) {
+	i->second = unknownState;
+    }
 
     for (Rosegarden::Composition::trackcontainer::iterator i =
 	     getDocument()->getComposition().getTracks().begin();
 	 i != getDocument()->getComposition().getTracks().end(); ++i) {
 
 	Rosegarden::Track *track = i->second;
-
-	Rosegarden::LevelInfo info;
-	bool isNew = mapper->getInstrumentLevel(track->getInstrument(), info);
+	if (!track) continue;
 
 	Rosegarden::InstrumentId instrumentId = track->getInstrument();
-	if (doneInstruments.find(instrumentId) != doneInstruments.end()) {
-	    continue;
+
+	if (states[instrumentId] == unknownState) {
+	    bool isNew =
+		mapper->getInstrumentLevel(instrumentId, levels[instrumentId]);
+	    states[instrumentId] = (isNew ? newState : oldState);
 	}
-	doneInstruments.insert(instrumentId);
+
+	if (states[instrumentId] == oldState) continue;
 
 	Rosegarden::Instrument *instrument =
 	    getDocument()->getStudio().getInstrumentById(instrumentId);
@@ -1043,9 +1074,10 @@ RosegardenGUIView::updateMeters(SequencerMapper *mapper)
 	}
         */
 
-	if (!isNew) continue;
-	
-	if (instrument->getType() == Rosegarden::Instrument::Audio) {
+	Rosegarden::LevelInfo &info = levels[instrumentId];
+
+	if (instrument->getType() == Rosegarden::Instrument::Audio ||
+	    instrument->getType() == Rosegarden::Instrument::SoftSynth) {
 
             // Don't send a 0 to an audio meter
             //
@@ -1063,13 +1095,8 @@ RosegardenGUIView::updateMeters(SequencerMapper *mapper)
 		m_instrumentParameterBox->setAudioMeter(dBleft, dBright);
 	    }
 
-	    if (instrument->getAudioChannels() > 1) {
-		m_trackEditor->getTrackButtons()->slotSetTrackMeter
-		    ((info.level + info.levelRight) / 254.0, track->getId());
-	    } else {
-		m_trackEditor->getTrackButtons()->slotSetTrackMeter
-		    ((info.level + info.levelRight) / 127.0, track->getId());
-	    }
+	    m_trackEditor->getTrackButtons()->slotSetTrackMeter
+		((info.level + info.levelRight) / 254.0, track->getPosition());
 
 	} else {
 
@@ -1086,6 +1113,12 @@ RosegardenGUIView::updateMeters(SequencerMapper *mapper)
 		(info.level / 127.0, instrumentId);
 	}
     }
+
+    for (StateMap::iterator i = states.begin(); i != states.end(); ++i) {
+	if (i->second == newState) {
+	    emit instrumentLevelsChanged(i->first, levels[i->first]);
+	}
+    }
 }    
 
 void
@@ -1157,6 +1190,14 @@ void RosegardenGUIView::slotShowPreviews(bool v)
     m_trackEditor->getSegmentCanvas()->repaint();
 }
 
+void RosegardenGUIView::slotUpdateAudioPreviews(Rosegarden::InstrumentId id)
+{
+    if (!m_trackEditor->getSegmentCanvas()->isShowingPreviews()) return;
+    RG_DEBUG << "RosegardenGUIView::slotUpdateAudioPreviews" << endl;
+    m_trackEditor->getSegmentCanvas()->canvas()->setAllChanged();
+    m_trackEditor->getSegmentCanvas()->canvas()->update();
+}
+	
 void RosegardenGUIView::slotAddTracks(unsigned int nbTracks,
                                       Rosegarden::InstrumentId id)
 {
@@ -1414,7 +1455,7 @@ RosegardenGUIView::slotSetMuteButton(Rosegarden::TrackId track, bool value)
     m_trackEditor->getTrackButtons()->setMuteButton(track, value);
     Rosegarden::Track *trackObj = getDocument()->
         getComposition().getTrackById(track);
-
+/*
     // to fix 739544
     if (m_instrumentParameterBox->getSelectedInstrument() &&
         m_instrumentParameterBox->getSelectedInstrument()->getId() ==
@@ -1422,7 +1463,7 @@ RosegardenGUIView::slotSetMuteButton(Rosegarden::TrackId track, bool value)
     {
         m_instrumentParameterBox->setMute(value);
     }
-
+*/
     // set the value in the composition
     trackObj->setMuted(value);
 
@@ -1455,11 +1496,11 @@ RosegardenGUIView::slotSetRecord(Rosegarden::InstrumentId id, bool value)
     RG_DEBUG << "RosegardenGUIView::slotSetRecord - "
              << "id = " << id
              << ",value = " << value << endl;
-
+/*
     // IPB
     //
     m_instrumentParameterBox->setRecord(value);
-
+*/
     Rosegarden::Composition &comp = getDocument()->getComposition();
     Rosegarden::Composition::trackcontainer &tracks = comp.getTracks();
     Rosegarden::Composition::trackiterator it;
