@@ -1,8 +1,9 @@
+// -*- c-basic-offset: 4 -*-
 /*
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -28,5 +29,7 @@
 extern QString strtoqstr(const std::string &);
 extern QString strtoqstr(const Rosegarden::PropertyName &);
 extern std::string qstrtostr(const QString &);
+extern double strtodouble(const std::string &);
+extern double qstrtodouble(const QString &);
 
 #endif
