@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -411,7 +411,35 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 
     } else if (lcName == "rosegarden-data") {
 
-        // validate top level open
+	// FILE FORMAT VERSIONING -- see comments in
+	// rosegardenguidoc.cpp.  We only care about major and minor
+	// here, not point.
+
+	QString version = atts.value("version");
+	QString smajor = atts.value("format-version-major");
+	QString sminor = atts.value("format-version-minor");
+
+	if (smajor) {
+
+	    int major = smajor.toInt();
+	    int minor = sminor.toInt();
+
+	    if (major > RosegardenGUIDoc::FILE_FORMAT_VERSION_MAJOR) {
+		m_errorString = i18n("This file was written by Rosegarden %1, and it uses\na different file format that cannot be read by this version.").arg(version);
+		return false;
+	    }
+
+	    if (major == RosegardenGUIDoc::FILE_FORMAT_VERSION_MAJOR &&
+		minor >  RosegardenGUIDoc::FILE_FORMAT_VERSION_MINOR) {
+
+                CurrentProgressDialog::freeze();
+	        KStartupLogo::hideIfStillThere();
+
+		KMessageBox::information(0, i18n("This file was written by Rosegarden %1, which is more recent than this version.\nThere may be some incompatibilities with the file format.").arg(version));
+
+		CurrentProgressDialog::thaw();
+	    }
+	}
 
     } else if (lcName == "studio") {
 
@@ -496,8 +524,12 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 	QString hiddenStr = atts.value("hidden");
 	if (hiddenStr) hidden = (hiddenStr == "true");
 
+	bool hiddenBars = false;
+	QString hiddenBarsStr = atts.value("hiddenbars");
+	if (hiddenBarsStr) hiddenBars = (hiddenBarsStr == "true");
+
 	getComposition().addTimeSignature
-	    (t, Rosegarden::TimeSignature(num, denom, common, hidden));
+	    (t, Rosegarden::TimeSignature(num, denom, common, hidden, hiddenBars));
 
     } else if (lcName == "tempo") {
 
@@ -549,7 +581,7 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         double tempo = 120.0;
         QString tempoStr = atts.value("defaultTempo");
         if (tempoStr) {
-            tempo = tempoStr.toDouble();
+            tempo = qstrtodouble(tempoStr);
         }
 
         getComposition().setDefaultTempo(tempo);
@@ -601,7 +633,7 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
                getComposition().setPlayMetronome(false);
         }
 
-        QString recMetStr = atts.value("playmetronome");
+        QString recMetStr = atts.value("recordmetronome");
         if (recMetStr)
         {
             if (recMetStr.toInt())
@@ -610,6 +642,11 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
                getComposition().setRecordMetronome(false);
         }
 
+	QString nextTriggerIdStr = atts.value("nexttriggerid");
+	if (nextTriggerIdStr) {
+	    getComposition().setNextTriggerSegmentId(nextTriggerIdStr.toInt());
+	}
+
 	QString copyrightStr = atts.value("copyright");
 	if (copyrightStr)
 	{
@@ -793,8 +830,33 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         }
 
 	m_currentTime = startTime;
-        getComposition().addSegment(m_currentSegment);
-        getComposition().setSegmentStartTime(m_currentSegment, startTime);
+
+	QString triggerIdStr = atts.value("triggerid");
+	QString triggerPitchStr = atts.value("triggerbasepitch");
+	QString triggerVelocityStr = atts.value("triggerbasevelocity");
+	QString triggerRetuneStr = atts.value("triggerretune");
+	QString triggerAdjustTimeStr = atts.value("triggeradjusttimes");
+
+	if (triggerIdStr) {
+	    int pitch = -1;
+	    if (triggerPitchStr) pitch = triggerPitchStr.toInt();
+	    int velocity = -1;
+	    if (triggerVelocityStr) velocity = triggerVelocityStr.toInt();
+	    Rosegarden::TriggerSegmentRec *rec =
+		getComposition().addTriggerSegment(m_currentSegment,
+						   triggerIdStr.toInt(),
+						   pitch, velocity);
+	    if (rec) {
+		if (triggerRetuneStr)
+		    rec->setDefaultRetune(triggerRetuneStr.lower() == "true");
+		if (triggerAdjustTimeStr)
+		    rec->setDefaultTimeAdjust(qstrtostr(triggerAdjustTimeStr));
+	    }
+	    m_currentSegment->setStartTimeDataMember(startTime);
+	} else {
+	    getComposition().addSegment(m_currentSegment);
+	    getComposition().setSegmentStartTime(m_currentSegment, startTime);
+	}
 
 	QString endMarkerStr = atts.value("endmarker");
 	if (endMarkerStr) {
@@ -979,7 +1041,8 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         getAudioFileManager().setAudioPath(qstrtostr(search));
 
     } else if (lcName == "begin") {
-        float marker = atts.value("index").toFloat();
+
+        double marker = qstrtodouble(atts.value("index"));
 
         if (!m_currentSegment)
         {
@@ -997,12 +1060,13 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 
         // convert to RealTime from float
         int sec = (int)marker;
-        int usec = (int)((marker - ((float)sec)) * 1000000.0);
+        int usec = (int)((marker - ((double)sec)) * 1000000.0);
         m_currentSegment->setAudioStartTime(Rosegarden::RealTime(sec, usec * 1000));
 
 
     } else if (lcName == "end") {
-        float marker = atts.value("index").toFloat();
+
+        double marker = qstrtodouble(atts.value("index"));
 
         if (!m_currentSegment)
         {
@@ -1019,7 +1083,7 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         }
 
         int sec = (int)marker;
-        int usec = (int)((marker - ((float)sec)) * 1000000.0);
+        int usec = (int)((marker - ((double)sec)) * 1000000.0);
         Rosegarden::RealTime markerTime(sec, usec * 1000);
 
         if (markerTime < m_currentSegment->getAudioStartTime())
@@ -1040,6 +1104,55 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         timeT absEnd = getComposition().getElapsedTimeForRealTime(realEndTime);
 	m_currentSegment->setEndTime(absEnd);
 
+    } else if (lcName == "fadein") {
+
+        if (!m_currentSegment)
+        {
+            // Don't fail - as this segment could be defunct if we
+            // skipped loading the audio file
+            //
+            return true;
+        }
+
+        if (m_currentSegment->getType() != Rosegarden::Segment::Audio)
+        {
+            m_errorString = "found fade in time in non audio segment";
+            return false;
+        }
+
+        double marker = qstrtodouble(atts.value("time"));
+        int sec = (int)marker;
+        int usec = (int)((marker - ((double)sec)) * 1000000.0);
+        Rosegarden::RealTime markerTime(sec, usec * 1000);
+
+        m_currentSegment->setFadeInTime(markerTime);
+        m_currentSegment->setAutoFade(true);
+
+
+    } else if (lcName == "fadeout") {
+
+        if (!m_currentSegment)
+        {
+            // Don't fail - as this segment could be defunct if we
+            // skipped loading the audio file
+            //
+            return true;
+        }
+
+        if (m_currentSegment->getType() != Rosegarden::Segment::Audio)
+        {
+            m_errorString = "found fade out time in non audio segment";
+            return false;
+        }
+
+        double marker = qstrtodouble(atts.value("time"));
+        int sec = (int)marker;
+        int usec = (int)((marker - ((double)sec)) * 1000000.0);
+        Rosegarden::RealTime markerTime(sec, usec * 1000);
+
+        m_currentSegment->setFadeOutTime(markerTime);
+        m_currentSegment->setAutoFade(true);
+
     } else if (lcName == "device") {
 
         if (m_section != InStudio)
@@ -1095,7 +1208,7 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 		if (vstr == "lsb") {
 		    variation = Rosegarden::MidiDevice::VariationFromLSB;
 		} else if (vstr == "msb") {
-		    variation = Rosegarden::MidiDevice::VariationFromLSB;
+		    variation = Rosegarden::MidiDevice::VariationFromMSB;
 		} else if (vstr == "") {
 		    variation = Rosegarden::MidiDevice::NoVariations;
 		}
@@ -1106,6 +1219,13 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 		md->setVariationType(variation);
 	    }
 	}
+	else if (type == "softsynth")
+        {
+            m_device = getStudio().getDevice(id);
+
+            if (m_device && m_device->getType() == Rosegarden::Device::SoftSynth)
+                m_device->setName(qstrtostr(nameStr));
+        }
 	else if (type == "audio")
         {
             m_device = getStudio().getDevice(id);
@@ -1423,7 +1543,8 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
 
         if (m_instrument)
         {
-	    if (m_instrument->getType() == Rosegarden::Instrument::Audio) {
+	    if (m_instrument->getType() == Rosegarden::Instrument::Audio ||
+		m_instrument->getType() == Rosegarden::Instrument::SoftSynth) {
 		// Backward compatibility: "volume" was in a 0-127
 		// range and we now store "level" (float dB) instead.
 		// Note that we have no such compatibility for
@@ -1443,12 +1564,13 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         if (m_section != InBuss &&
 	    (m_section != InInstrument ||
 	     (m_instrument &&
-	      m_instrument->getType() != Rosegarden::Instrument::Audio))) {
+	      m_instrument->getType() != Rosegarden::Instrument::Audio &&
+	      m_instrument->getType() != Rosegarden::Instrument::SoftSynth))) {
             m_errorString = "Found Level outside (audio) Instrument or Buss";
             return false;
         }
 
-	float value = atts.value("value").toFloat();
+        double value = qstrtodouble(atts.value("value"));
 
 	if (m_section == InBuss) {
 	    if (m_buss) m_buss->setLevel(value);
@@ -1472,14 +1594,17 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
             m_instrument->setControllerValue(type, value);
         }
 
-#ifdef HAVE_LADSPA
-
-    } else if (lcName == "plugin") {
+    } else if (lcName == "plugin" || lcName == "synth") {
 
         if (m_section != InInstrument)
         {
-            m_errorString = "Found Plugin outside Instrument";
-            return false;
+	    if (m_section == InBuss) {
+		m_instrument = 0;
+		// and continue, effectively ignoring the plugin
+	    } else {
+		m_errorString = "Found Plugin outside Instrument";
+		return false;
+	    }
         }
 
         // Despite being InInstrument we might not actually have a valid
@@ -1488,40 +1613,84 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
         if (m_instrument)
         {
             // Get the details
-            int position = atts.value("position").toInt();
-            unsigned long id = atts.value("id").toULong();
-            QString bpStr = atts.value("bypassed");
-            bool bypassed = false;
+	    int position;
+	    if (lcName == "synth") {
+		position = Rosegarden::Instrument::SYNTH_PLUGIN_POSITION;
+	    } else {
+		position = atts.value("position").toInt();
+	    }
 
-            if (bpStr.lower() == "true")
-                bypassed = true;
+	    bool bypassed = false;
+	    QString bpStr = atts.value("bypassed");
+	    if (bpStr.lower() == "true")
+		bypassed = true;
+
+	    std::string program = "";
+	    QString progStr = atts.value("program");
+	    if (progStr) {
+		program = qstrtostr(progStr);
+	    }
+
+	    // Plugins are identified by a structured identifier
+	    // string, but we will accept a LADSPA UniqueId if there's
+	    // no identifier, for backward compatibility
+
+	    QString identifier = atts.value("identifier");
 
-            // Check that ID exists
-            //
             Rosegarden::AudioPlugin *plugin = 0;
-            if (getAudioPluginManager())
-                plugin = getAudioPluginManager()->getPluginByUniqueId(id);
+	    Rosegarden::AudioPluginManager *apm = getAudioPluginManager();
+
+	    if (!identifier) {
+		if (atts.value("id")) {
+		    unsigned long id = atts.value("id").toULong();
+		    if (apm) plugin = apm->getPluginByUniqueId(id);
+		}
+	    } else {
+		if (apm) plugin = apm->getPluginByIdentifier(identifier);
+	    }
 
             // If we find the plugin all is well and good but if
             // we don't we just skip it.
             //
-	
             if (plugin)
             {
                 m_plugin = m_instrument->getPlugin(position);
-                m_plugin->setAssigned(true);
-                m_plugin->setBypass(bypassed);
-                //RG_DEBUG << "SETTING PLUGIN ASSIGNED AT " << position 
-                     //<< " ON INS " << m_instrument->getId() << endl;
+		if (!m_plugin) {
+		    RG_DEBUG << "WARNING: RoseXmlHandler: instrument "
+			     << m_instrument->getId() << " has no plugin position "
+			     << position << endl;
+		} else {
+		    m_plugin->setAssigned(true);
+		    m_plugin->setBypass(bypassed);
+		    m_plugin->setIdentifier(plugin->getIdentifier().data());
+		    if (program != "") {
+			m_plugin->setProgram(program);
+		    }
+		}
+            } else {
+		// we shouldn't be halting import of the RG file just because
+		// we can't match a plugin
+		//
+		if (identifier) {
+		    RG_DEBUG << "WARNING: RoseXmlHandler: plugin " << identifier << " not found" << endl;
+		    m_pluginsNotFound.insert(identifier);
+		} else if (atts.value("id")) {
+		    RG_DEBUG << "WARNING: RoseXmlHandler: plugin uid " << atts.value("id") << " not found" << endl;
+		} else {
+		    m_errorString = "No plugin identifier or uid specified";
+		    return false;
+		}
+            }
+        } else { // no instrument
 
-                m_plugin->setId(id);
-            }
-            else
-            {
-                m_errorString = "Can't find Plugin";
-                return false;
-            }
-        }
+	    if (lcName == "synth") {
+		QString identifier = atts.value("identifier");
+		if (identifier) {
+		    RG_DEBUG << "WARNING: RoseXmlHandler: no instrument for plugin " << identifier << endl;
+		    m_pluginsNotFound.insert(identifier);
+		}
+	    }
+	}
 
         m_section = InPlugin;
 
@@ -1533,27 +1702,28 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
             return false;
         }
         unsigned long portId = atts.value("id").toULong();
-        Rosegarden::MappedObjectValue value = atts.value("value").toFloat();
+	double value = qstrtodouble(atts.value("value"));
 
         if (m_plugin)
         {
             m_plugin->addPort(portId, value);
-            //RG_DEBUG << "ADDING PORT " << portId << " VALUE = " << value << endl;
         }
 
-#else
+    } else if (lcName == "configure") {
 
-        // just ignore them
+        if (m_section != InPlugin)
+        {
+            m_errorString = "Found Configure outside Plugin";
+            return false;
+        }
 
-    } else if (lcName == "plugin") {
+        QString key = atts.value("key");
+        QString value = atts.value("value");
 
-        ;
-
-    } else if (lcName == "port") {
-
-        ;
-
-#endif // HAVE_LADSPA
+        if (m_plugin)
+        {
+            m_plugin->setConfigurationValue(qstrtostr(key), qstrtostr(value));
+        }
 
     } else if (lcName == "metronome") {
 
@@ -1605,6 +1775,8 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
             type = Rosegarden::Instrument::Midi;
         else if (stringType == "audio")
             type = Rosegarden::Instrument::Audio;
+        else if (stringType == "softsynth")
+            type = Rosegarden::Instrument::SoftSynth;
         else
         {
             m_errorString = "Found unknown Instrument type";
@@ -1686,7 +1858,7 @@ RoseXmlHandler::startElement(const QString& namespaceURI,
             return false;
         }
 
-        float value = atts.value("value").toFloat();
+	double value = qstrtodouble(atts.value("value"));
 
 	// if the value retrieved is greater than (say) 15 then we
 	// must have an old-style 0-127 value instead of a shiny new
@@ -1833,7 +2005,11 @@ RoseXmlHandler::endElement(const QString& namespaceURI,
 
     QString lcName = qName.lower();
 
-    if (lcName == "event") {
+    if (lcName == "rosegarden-data") {
+
+	getComposition().updateTriggerSegmentReferences();
+
+    } else if (lcName == "event") {
 
         if (m_currentSegment && m_currentEvent) {
             m_currentSegment->insert(m_currentEvent);
