@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -27,6 +27,10 @@
 #include <qfont.h>
 #include <qfontmetrics.h>
 #include <qbitmap.h>
+#include <qtimer.h>
+#include <qobject.h>
+
+#include <algorithm>
 
 #include <klocale.h>
 #include <kmessagebox.h>
@@ -39,6 +43,8 @@
 #include "NotationTypes.h"
 #include "BaseProperties.h"
 #include "RulerScale.h"
+#include "AudioLevel.h"
+#include "Profiler.h"
 
 #include "rosestrings.h"
 #include "rosegardenguidoc.h"
@@ -94,10 +100,10 @@ SegmentRepeatRectangle::SegmentRepeatRectangle(QCanvas *canvas,
       m_snapGrid(snapGrid),
       m_doc(doc)
 {
-    setBrush(RosegardenGUIColours::convertColour
+    setBrush(Rosegarden::GUIPalette::convertColour
 	     (m_doc->getComposition().getSegmentColourMap().getColourByIndex
 	      (m_segment->getColourIndex())).light(150));
-    setPen(RosegardenGUIColours::RepeatSegmentBorder);
+    setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RepeatSegmentBorder));
 }
 
 Rosegarden::timeT SegmentRepeatRectangle::getRepeatStartTime(int ex)
@@ -124,10 +130,10 @@ void SegmentRepeatRectangle::drawShape(QPainter& painter)
     int rWidth = int(m_snapGrid->getRulerScale()->
 		     getXForTime(m_repeatInterval));
 
-    setBrush(RosegardenGUIColours::convertColour
+    setBrush(Rosegarden::GUIPalette::convertColour
 	     (m_doc->getComposition().getSegmentColourMap().getColourByIndex
 	      (m_segment->getColourIndex())).light(150));
-    painter.setPen(RosegardenGUIColours::RepeatSegmentBorder);
+    painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RepeatSegmentBorder));
 
     while (pos < width + rWidth)
     {
@@ -147,7 +153,7 @@ SegmentItemPreview::SegmentItemPreview(SegmentItem& parent,
     : m_parent(parent),
       m_segment(parent.getSegment()),
       m_rulerScale(scale),
-      m_previewIsCurrent(false)
+      m_previewState(PreviewChanged)
 {
 }
 
@@ -161,19 +167,23 @@ QRect SegmentItemPreview::rect()
 }
 
 
-class SegmentAudioPreview : public SegmentItemPreview
+class SegmentAudioPreview : public QObject, public SegmentItemPreview
 {
 public:
     SegmentAudioPreview(SegmentItem& parent, Rosegarden::RulerScale* scale);
+    virtual ~SegmentAudioPreview();
 
     virtual void drawShape(QPainter&);
+    virtual void clearPreview();
 
 protected:
 
+    virtual bool event(QEvent *);
     virtual void updatePreview(const QWMatrix &matrix);
 
     //--------------- Data members ---------------------------------
 
+    int                  m_previewToken;
     std::vector<float>   m_values;
     bool                 m_showMinima;
     unsigned int         m_channels;
@@ -182,11 +192,23 @@ protected:
 SegmentAudioPreview::SegmentAudioPreview(SegmentItem& parent,
                                          Rosegarden::RulerScale* scale)
     : SegmentItemPreview(parent, scale),
-    m_showMinima(false),
-    m_channels(0)
+      m_previewToken(-1),
+      m_showMinima(false),
+      m_channels(0)
 {
 }
 
+SegmentAudioPreview::~SegmentAudioPreview()
+{
+    if (m_previewToken >= 0) {
+	RosegardenGUIDoc *doc = m_parent.getDocument();
+	if (doc && !doc->isBeingDestroyed()) {
+	    AudioPreviewThread &thread = doc->getAudioPreviewThread();
+	    thread.cancelPreview(m_previewToken);
+	}
+    }
+}	
+
 void SegmentAudioPreview::drawShape(QPainter& painter)
 {
     // Fetch new set of values
@@ -195,16 +217,36 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
 
     // If there's nothing to draw then don't draw it
     //
-    if (m_values.size() == 0)
+    if (m_values.size() == 0) {
         return;
+    }
+
+    QRect clipRect = painter.clipRegion().boundingRect();
 
     painter.save();
-    //painter.translate(rect().x(), rect().y());
 
     // perhaps antialias this somehow at some point
-    int height = rect().height()/2 - 3;
+    int height = rect().height()/2 - 2;
     int halfRectHeight = rect().height()/2;
 
+    float gain[2] = { 1.0, 1.0 };
+    if (m_segment && m_parent.getDocument()) {
+	Rosegarden::TrackId trackId = m_segment->getTrack();
+	Rosegarden::Track *track =
+	    m_parent.getDocument()->getComposition().getTrackById(trackId);
+	if (track) {
+	    Rosegarden::Instrument *instrument =
+		m_parent.getDocument()->getStudio().getInstrumentById
+		(track->getInstrument());
+	    if (instrument) {
+		float level = Rosegarden::AudioLevel::dB_to_multiplier(instrument->getLevel());
+		float pan = instrument->getPan() - 100.0;
+		gain[0] = level * ((pan > 0.0) ? (1.0 - (pan / 100.0)) : 1.0);
+		gain[1] = level * ((pan < 0.0) ? ((pan + 100.0) / 100.0) : 1.0);
+	    }
+	}
+    }
+
     // Sometimes our audio file fails to get recorded properly and
     // the number of channels fails too.  Guard against this.
     //
@@ -212,7 +254,7 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
     {
         std::cerr << "SegmentAudioPreview::drawShape - m_channels == 0 "
                   << "problem with audio file" <<std::endl;
-	//painter.restore();
+	painter.restore();
         return;
     }
 
@@ -226,38 +268,50 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
 
     // The visible width of the rectangle
     //
-    QWMatrix matrix = painter.worldMatrix();
     QRect tRect = painter.worldMatrix().map(rect());
     double sampleScaleFactor = samplePoints / double(tRect.width());
-    double drawScaleFactor = double(rect().width())/double(tRect.width());
 
     bool state = painter.hasWorldXForm();
     painter.setWorldXForm(false);
 
-    for (int i = 0; i < tRect.width(); ++i)
+    int i0 = clipRect.x() - tRect.x();
+    i0 = std::max(i0, 0);
+
+    int i1 = i0 + clipRect.width();
+    i1 = std::min(i1, tRect.width());
+    
+    for (int i = i0; i < i1; ++i)
     {
         // For each i work get the sample starting point
         //
         int position = int(m_channels * i * sampleScaleFactor);
+	if (position < 0) continue;
+	if (position >= m_values.size() - m_channels) break;
 
         if (m_channels == 1) {
 
             h1 = m_values[position++];
             h2 = h1;
 
+	    h1 *= gain[0];
+	    h2 *= gain[1];
+
             if (m_showMinima)
             {
                 l1 = m_values[position++];
                 l2 = l1;
+
+		l1 *= gain[0];
+		l2 *= gain[1];
             }
         }
         else {
 
-            h1 = m_values[position++];
-            if (m_showMinima) l1 = m_values[position++];
+            h1 = m_values[position++] * gain[0];
+            if (m_showMinima) l1 = m_values[position++] * gain[0];
 
-            h2 = m_values[position++];
-            if (m_showMinima) l2 = m_values[position++];
+            h2 = m_values[position++] * gain[1];
+            if (m_showMinima) l2 = m_values[position++] * gain[1];
             
         }
 
@@ -266,11 +320,23 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
                   << ", y = " << rect().y() << std::endl;
                   */
 
-        painter.setPen(RosegardenGUIColours::SegmentAudioPreview);
-        painter.drawLine(tRect.x() + i,
-                         tRect.y() + int(halfRectHeight - h1 * height),
-                         tRect.x() + i,
-                         tRect.y() + int(halfRectHeight + h2 * height));
+        painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentAudioPreview));
+
+	if (h1 >= 1.0) { h1 = 1.0; painter.setPen(Qt::red); }
+	else { painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentAudioPreview)); }
+
+	painter.drawLine(tRect.x() + i,
+			 tRect.y() + int(halfRectHeight - h1 * height + 0.5),
+			 tRect.x() + i,
+			 tRect.y() + int(halfRectHeight));
+
+	if (h2 >= 1.0) { h2 = 1.0; painter.setPen(Qt::red); }
+	else { painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentAudioPreview)); }
+
+	painter.drawLine(tRect.x() + i,
+			 tRect.y() + int(halfRectHeight),
+			 tRect.x() + i,
+			 tRect.y() + int(halfRectHeight + h2 * height + 0.5));
 
         // For the moment draw it the same colour - the resolution on the
         // segmentcanvas doens't allow us to tell the difference between
@@ -287,9 +353,36 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
                          */
     }
 
+    // Draw an autofade line if we're applying it
+    //
+    //
+    if (m_segment->isAutoFading())
+    {
+        Rosegarden::Composition &comp = m_parent.getDocument()->
+            getComposition();
+
+        int audioFadeInEnd = 
+            m_rulerScale->getXForTime(comp.
+                    getElapsedTimeForRealTime(m_segment->getFadeInTime()) +
+		    m_segment->getStartTime()) -
+	    m_rulerScale->getXForTime(m_segment->getStartTime());
+
+	// Convert by matrix
+        int mappedFadeInEnd, y;
+        painter.worldMatrix().map(audioFadeInEnd, 0, &mappedFadeInEnd, &y);
+
+	painter.setPen(Qt::blue);
+	painter.drawLine(tRect.x(),
+			 tRect.y() + tRect.height() - 1,
+			 tRect.x() + mappedFadeInEnd,
+			 tRect.y());
+
+    }
+
+
     // perhaps draw an XOR'd label at some point
     /*
-      painter.setPen(RosegardenGUIColours::SegmentLabel);
+      painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentLabel));
       painter.setFont(*m_font);
       QRect labelRect = rect();
       labelRect.setX(labelRect.x() + 3);
@@ -300,14 +393,19 @@ void SegmentAudioPreview::drawShape(QPainter& painter)
     painter.restore();
 }
 
+void SegmentAudioPreview::clearPreview()
+{
+    m_values.clear();
+    setPreviewCurrent(false);
+}
+
 void SegmentAudioPreview::updatePreview(const QWMatrix &matrix)
 {
-    if (isPreviewCurrent()) return;
+    if (getPreviewState() != PreviewChanged) return;
 
-    // Fetch vector of floats adjusted to our resolution
-    //
+//    Rosegarden::Profiler profiler("SegmentAudioPreview::updatePreview", true);
 
-    Rosegarden::AudioFileManager &aFM = m_parent.getDocument()->getAudioFileManager();
+    m_values.clear();
 
     Rosegarden::Composition &comp = m_parent.getDocument()->getComposition();
 
@@ -318,50 +416,61 @@ void SegmentAudioPreview::updatePreview(const QWMatrix &matrix)
         comp.getElapsedRealTime(m_parent.getEndTime()) -
         comp.getElapsedRealTime(m_parent.getStartTime()) ;
 
-    try
-    {
-        RG_DEBUG << "SegmentAudioPreview::updatePreview() - for file id "
-                 << m_segment->getAudioFileId() << " fetching values" <<endl;
+    QRect tRect = matrix.map(rect());
+    QRect uRect = matrix.map(rect());
 
-        QRect tRect = matrix.map(rect());
-        QRect uRect = matrix.map(rect());
+//    RG_DEBUG << "SegmentAudioPreview::updatePreview() - for file id "
+//	     << m_segment->getAudioFileId() << " requesting values" <<endl;
 
-        RG_DEBUG << "SegmentAudioPreview::updatePreview "
-                 << "rect().width() = " << rect().width()
-                 << ", mapped width = " << tRect.width()
-                 << ", inverse mapped width = " <<  uRect.width()
-                 << endl;
+    AudioPreviewThread &thread = m_parent.getDocument()->getAudioPreviewThread();
+    AudioPreviewThread::Request request;
+    request.audioFileId = m_segment->getAudioFileId();
+    request.audioStartTime = audioStartTime;
+    request.audioEndTime = audioEndTime;
+    request.width = tRect.width();
+    request.showMinima = m_showMinima;
+    request.notify = this;
+    if (m_previewToken >= 0) thread.cancelPreview(m_previewToken);
+    m_previewToken = thread.requestPreview(request);
 
-        m_values =
-            aFM.getPreview(m_segment->getAudioFileId(),
-                           audioStartTime,
-                           audioEndTime,
-                           tRect.width(),
-                           m_showMinima); // do we get and show the minima too?
-    }
-    catch (std::string e)
-    {
-        // empty all values out
-        RG_DEBUG << "SegmentAudioPreview::updatePreview : "
-                 << e.c_str() << endl;
-        
-        m_values.clear();
-    }
-
-
-    // If we haven't inserted the audio file yet we're probably
-    // just still recording it.
-    //
-    if (m_values.size() == 0) {
-        RG_DEBUG << "SegmentAudioPreview::updatePreview : no values\n";
-        return;
-    }
-
-    m_channels = aFM.getAudioFile(m_segment->getAudioFileId())->getChannels();
-
-    setPreviewCurrent(true);
+    m_previewState = PreviewCalculating;
 }
 
+bool
+SegmentAudioPreview::event(QEvent *e)
+{
+    RG_DEBUG << "SegmentAudioPreview::event" <<endl;
+
+    if (e->type() == QEvent::User + 1) {
+	QCustomEvent *ev = dynamic_cast<QCustomEvent *>(e);
+	if (ev) {
+	    int token = (int)ev->data();
+	    AudioPreviewThread &thread = m_parent.getDocument()->getAudioPreviewThread();
+
+	    RG_DEBUG << "SegmentAudioPreview::token " << token << ", my token " << m_previewToken <<endl;
+
+	    if (m_previewToken >= 0 && token >= m_previewToken) {
+		m_previewToken = -1;
+		thread.getPreview(token, m_channels, m_values);
+		setPreviewCurrent(true);
+		if (m_parent.canvas()) {
+		    m_parent.canvas()->setChanged(m_parent.rect());
+		    m_parent.canvas()->update();
+		}
+	    } else {
+		// this one is out of date already
+		std::vector<float> tmp;
+		thread.getPreview(token, m_channels, tmp);
+	    }
+	    return true;
+	}
+    }
+
+    return QObject::event(e);
+}
+
+
+
 //////////////////////////////////////////////////////////////////////
 //                SegmentNotationPreview
 //////////////////////////////////////////////////////////////////////
@@ -374,6 +483,8 @@ public:
 
     virtual void drawShape(QPainter&);
 
+    virtual void clearPreview();
+
 protected:
     virtual void updatePreview(const QWMatrix &matrix);
 
@@ -400,13 +511,19 @@ SegmentNotationPreview::SegmentNotationPreview(SegmentItem& parent,
 {
 }
 
+void SegmentNotationPreview::clearPreview()
+{
+    m_previewInfo.clear();
+    setPreviewCurrent(false);
+}
+
 void SegmentNotationPreview::drawShape(QPainter& painter)
 {
     updatePreview(painter.worldMatrix());
     painter.save();
 
     painter.translate(rect().x(), rect().y());
-    painter.setPen(RosegardenGUIColours::SegmentInternalPreview);
+    painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentInternalPreview));
     QRect viewportRect = painter.xFormDev(painter.viewport());
 
     double scaleFactor = m_rulerScale->getXForTime(Rosegarden::Note(Rosegarden::Note::Crotchet).
@@ -454,30 +571,30 @@ void SegmentNotationPreview::drawShape(QPainter& painter)
 
 void SegmentNotationPreview::updatePreview(const QWMatrix & /*matrix*/)
 {
-    if (isPreviewCurrent()) return;
-
-    //RG_DEBUG << "SegmentNotationItem::updatePreview() "
-                         //<< this << endl;
+    if (getPreviewState() != PreviewChanged) return;
 
     if (!m_haveSegmentRefreshStatus) {
 	m_segmentRefreshStatus = m_segment->getNewRefreshStatusId();
 	m_haveSegmentRefreshStatus = true;
     }
-
+	
     Rosegarden::SegmentRefreshStatus &status =
 	m_segment->getRefreshStatus(m_segmentRefreshStatus);
-    if (!status.needsRefresh()) return;
+    if (!status.needsRefresh() && !m_previewInfo.empty()) return;
+
+    Rosegarden::timeT fromTime = 0, toTime = 0;
+
+    if (status.needsRefresh() && !m_previewInfo.empty()) {
+	fromTime = status.from();
+	toTime = status.to();
+    }
 
-    Rosegarden::timeT fromTime = status.from(), toTime = status.to();
     if (fromTime == toTime) {
 	fromTime = m_segment->getStartTime();
 	  toTime = m_segment->getEndMarkerTime();
     }
 
-    int fromX = (int)m_rulerScale->getXForTime(fromTime) - rect().x();
-    int   toX = (int)m_rulerScale->getXForTime(  toTime) - rect().x();
-
-    QRect fromRect(fromX, 0, 10, 10), toRect(toX, 0, 10, 10);
+    QRect fromRect(fromTime, 0, 10, 10), toRect(toTime, 0, 10, 10);
     RectList::iterator fromi = m_previewInfo.lower_bound(fromRect);
     RectList::iterator   toi = m_previewInfo.lower_bound(  toRect);
     m_previewInfo.erase(fromi, toi);
@@ -551,7 +668,7 @@ SegmentItem::SegmentItem(TrackId trackPosition, timeT startTime, timeT endTime,
     m_selected(false),
     m_snapGrid(snapGrid),
     m_repeatRectangle(0),
-    m_colour(RosegardenGUIColours::convertColour
+    m_colour(Rosegarden::GUIPalette::convertColour
 	     (doc->getComposition().getSegmentColourMap().getColourByIndex(0))),
     m_overrideColour(false),
     m_preview(0),
@@ -575,7 +692,7 @@ SegmentItem::SegmentItem(Segment *segment,
     m_selected(false),
     m_snapGrid(snapGrid),
     m_repeatRectangle(0),
-    m_colour(RosegardenGUIColours::convertColour
+    m_colour(Rosegarden::GUIPalette::convertColour
 	     (doc->getComposition().getSegmentColourMap().getColourByIndex(0))),
     m_overrideColour(false),
     m_preview(0),
@@ -608,6 +725,11 @@ void SegmentItem::setShowPreview(bool preview)
     m_showPreview = preview;
 }
 
+bool SegmentItem::getShowPreview() const
+{
+    return m_showPreview;
+}
+
 void SegmentItem::setPreview()
 {
     delete m_preview;
@@ -665,7 +787,7 @@ void SegmentItem::drawShape(QPainter& painter)
         {
             if (m_segment) {
                 painter.setBrush
-		    (RosegardenGUIColours::convertColour
+		    (Rosegarden::GUIPalette::convertColour
 		     (m_doc->getComposition().getSegmentColourMap().getColourByIndex
 		      (m_segment->getColourIndex())).dark(125));
 		colourset = true;
@@ -684,32 +806,29 @@ void SegmentItem::drawShape(QPainter& painter)
         painter.drawRect(intersection);
     }
 
+    bool showText = (m_showPreview && m_segment &&
+		     m_segment->getType() != Rosegarden::Segment::Audio);
+    QString text = m_label;
+    QFont font = *m_font;
+
     if (m_preview && m_showPreview) {
-/*!!!
-	Rosegarden::Colour col;
-
-	if (m_segment && !m_overrideColour) {
-	    col = (m_doc->getComposition().getSegmentColourMap().getColourByIndex
-		   (m_segment->getColourIndex()));
-	} else {
-	    col = RosegardenGUIColours::convertColour(m_colour);
-	}
-
-	painter.setPen(RosegardenGUIColours::convertColour
-		       (col.getContrastingColour()));
-*/
 	m_preview->drawShape(painter);
+
+//	if (m_preview->getPreviewState() ==
+//	    SegmentItemPreview::PreviewCalculating) {
+//	    showText = true;
+//	    text = i18n("loading...");
+//	    font.setBold(false);
+//	    font.setItalic(true);
+//	}
     }
 
     // draw label
     if (m_segment) 
     {
-
         //RG_DEBUG << "SegmentItem::drawShape - has world Xform = " << painter.hasWorldXForm() << endl;
 
-        // Don't show label if we're showing the preview
-        //
-        if (m_showPreview && m_segment->getType() == Rosegarden::Segment::Audio) {
+        if (!showText) {
 	    painter.restore();
             return;
 	}
@@ -722,7 +841,7 @@ void SegmentItem::drawShape(QPainter& painter)
         //
         bool state = painter.hasWorldXForm();
         painter.setWorldXForm(false);
-        painter.setFont(*m_font);
+        painter.setFont(font);
 
         QRect labelRect = matrix.mapRect(rect()); // map the rectangle to the transform
         int x = labelRect.x() + 3;
@@ -732,6 +851,7 @@ void SegmentItem::drawShape(QPainter& painter)
         // we've got vertical zooming too - for the moment it's nice
         // to have it for audio segments too.  [rwb]
         //
+	/*!!!
         if(m_segment->getType() == Rosegarden::Segment::Audio)
         {
             painter.setPen(Qt::white);
@@ -743,14 +863,15 @@ void SegmentItem::drawShape(QPainter& painter)
 
                     painter.drawText(labelRect,
                                      Qt::AlignLeft|Qt::AlignVCenter,
-                                     m_label);
+                                     text);
                 }
         }
+	*/
 
         labelRect.setX(x);
         labelRect.setY(y);
-        painter.setPen(RosegardenGUIColours::SegmentLabel);
-        painter.drawText(labelRect, Qt::AlignLeft|Qt::AlignVCenter, m_label);
+        painter.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentLabel));
+        painter.drawText(labelRect, Qt::AlignLeft|Qt::AlignVCenter, text);
 
         // Reenable the transform state
         //
@@ -770,7 +891,7 @@ void SegmentItem::recalculateRectangle(bool inheritFromSegment)
     // Get our segment colour
     QColor brush;
     if (m_segment && !m_overrideColour) {
-        brush = RosegardenGUIColours::convertColour
+        brush = Rosegarden::GUIPalette::convertColour
 	    (m_doc->getComposition().getSegmentColourMap().getColourByIndex
 	     (m_segment->getColourIndex()));
     } else {
@@ -916,19 +1037,6 @@ void SegmentItem::setSelected(bool select, const QBrush &brush)
 
 void SegmentItem::showRepeatRect(bool s)
 {
-    //!!! This doesn't really work terribly well... need a better fix
-    // for the remaining preview trails when you move a segment
-/*!!!
-    m_suspendPreview = !s;
-    if (m_showPreview) {
-	if (m_suspendPreview) {
-	    delete m_preview;
-	    m_preview = 0;
-	} else {
-	    setPreview();
-	}
-    }
-*/
     if (m_repeatRectangle) {
     
 	if (s)
@@ -950,8 +1058,8 @@ SegmentSplitLine::SegmentSplitLine(int x, int y, int height,
                                    m_rulerScale(rulerScale),
                                    m_height(height)
 {
-    setPen(RosegardenGUIColours::SegmentSplitLine);
-    setBrush(RosegardenGUIColours::SegmentSplitLine);
+    setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentSplitLine));
+    setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentSplitLine));
     setZ(3);
     moveLine(x, y);
 }
@@ -968,6 +1076,86 @@ void SegmentSplitLine::hideLine()
 }
 
 
+//////////////////////////////////////////////////////////////////////
+// SegmentCanvasTextFloat
+//
+// - based on a QCanvasRectangle
+// - a version of RosegardenTextFloat to be used on the main canvas
+//   (we could probably use that class in preference but this allows
+//    us to make some canvas specific tweaks)
+//
+//////////////////////////////////////////////////////////////////////
+SegmentCanvasTextFloat::SegmentCanvasTextFloat(QCanvas *canvas):
+    QCanvasRectangle(canvas)
+    /*,
+    m_text(new QCanvasText(canvas))
+    */
+
+{
+    setBrush(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryFloatBackground));
+    setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryFloatForeground));
+    setZ(10000);
+    hide();
+
+    /*
+    m_text->setColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryFloatForeground));
+    m_text->setZ(10001);
+    m_text->hide();
+    */
+}
+
+void
+SegmentCanvasTextFloat::setText(int x, int y, const QString &text)
+{
+    setX(x);
+    setY(y);
+    m_text = text;
+
+    /*
+    m_text->setX(x + 2);
+    m_text->setY(y + 2);
+    m_text->setText(text);
+    QRect bound = m_text->boundingRect();
+    setSize(bound.width() + 4, bound.height() + 4);
+    */
+
+    show();
+    //m_text->show();
+}
+
+void
+SegmentCanvasTextFloat::hideText()
+{
+    hide();
+    //m_text->hide();
+}
+
+void 
+SegmentCanvasTextFloat::drawShape(QPainter & p)
+{
+    QRect bound = p.boundingRect(0, 0, 300, 40, AlignAuto, m_text);
+    bound.setLeft(bound.left() - 2);
+    bound.setRight(bound.right() + 2);
+    bound.setTop(bound.top() - 2);
+    bound.setBottom(bound.bottom() + 2);
+
+    QRect mappedBound = p.worldMatrix().invert().mapRect(bound);
+    setSize(mappedBound.width(), mappedBound.height());
+
+    QCanvasRectangle::drawShape(p);
+
+    bool state = p.hasWorldXForm();
+    p.setWorldXForm(false);
+
+    QPoint mapPos = p.worldMatrix().map(QPoint(int(x()), int(y())));
+
+    p.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryFloatForeground));
+    p.drawText(mapPos.x() + 2, mapPos.y() + 14, m_text);
+
+    p.setWorldXForm(state);
+}
+
+
 
 //////////////////////////////////////////////////////////////////////
 //                SegmentCanvas
@@ -984,7 +1172,7 @@ SegmentCanvas::SegmentCanvas(RosegardenGUIDoc *doc,
     m_currentItem(0),
     m_recordingSegment(0),
     m_splitLine(0),
-    m_pen(RosegardenGUIColours::SegmentBorder),
+    m_pen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentBorder)),
     m_fineGrain(false),
     m_showPreviews(true),
     m_doc(doc),
@@ -992,14 +1180,19 @@ SegmentCanvas::SegmentCanvas(RosegardenGUIDoc *doc,
     m_selectionRect(0)
 {
     m_toolBox = new SegmentToolBox(this, m_doc);
+    m_textFloat = new SegmentCanvasTextFloat(canvas());
 
     QWhatsThis::add(this, i18n("Segments Canvas - Create and manipulate your segments here"));
 
     // prepare selection rectangle
     m_selectionRect = new QCanvasRectangle(canvas());
-    m_selectionRect->setPen(RosegardenGUIColours::SelectionRectangle);
+    m_selectionRect->setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SelectionRectangle));
     m_selectionRect->setZ(1000);  // Always in front
     m_selectionRect->hide();
+
+    QObject::connect(&m_showTimer, SIGNAL(timeout()),
+                    this, SLOT(slotTextFloatTimeout()));
+
 }
 
 SegmentCanvas::~SegmentCanvas()
@@ -1017,6 +1210,26 @@ SegmentCanvas::getSelectionRectangle()
     return m_selectionRect;
 }
 
+void 
+SegmentCanvas::slotTextFloatTimeout() 
+{ 
+    hideTextFloat();
+}
+
+void
+SegmentCanvas::setTextFloat(int x, int y, const QString &text)
+{
+    m_textFloat->setText(x, y, text);
+}
+
+void
+SegmentCanvas::hideTextFloat()
+{
+    m_textFloat->hideText();
+}
+
+
+
 void SegmentCanvas::slotSetTool(const QString& toolName)
 {
     RG_DEBUG << "SegmentCanvas::slotSetTool(" << toolName << ")"
@@ -1104,8 +1317,6 @@ void SegmentCanvas::updateAllSegmentItems()
 
 void SegmentCanvas::updateSegmentItem(Segment *segment)
 {
-    RG_DEBUG << "SegmentCanvas::updateSegmentItem" << endl;
-
     SegmentItem *item = findSegmentItem(segment);
     if (!item) {
 	addSegmentItem(segment);
@@ -1177,19 +1388,24 @@ SegmentCanvas::findRepeatClickedOn(QPoint pos)
 void SegmentCanvas::contentsMousePressEvent(QMouseEvent* e)
 {
     switch (e->button()) {
+
     case LeftButton:
     case MidButton:
+	startAutoScroll();
+
         if (m_tool) m_tool->handleMouseButtonPress(e);
         else
             RG_DEBUG << "SegmentCanvas::contentsMousePressEvent() :"
                      << this << " no tool\n";
         break;
+
     case RightButton:
         if (m_tool) m_tool->handleRightButtonPress(e);
         else
             RG_DEBUG << "SegmentCanvas::contentsMousePressEvent() :"
                      << this << " no tool\n";
         break;
+
     default:
         break;
     }
@@ -1206,10 +1422,10 @@ void SegmentCanvas::contentsMouseDoubleClickEvent(QMouseEvent* e)
         m_currentItem = item;
 	emit editSegment(item->getSegment());
     } else {
-	SegmentRepeatRectangle *rect = findRepeatClickedOn(
-                inverseWorldMatrix().map(e->pos()));
+	QPoint mpos = inverseWorldMatrix().map(e->pos());
+	SegmentRepeatRectangle *rect = findRepeatClickedOn(mpos);
         if (rect) {
-	    Rosegarden::timeT time = rect->getRepeatStartTime(e->x());
+	    Rosegarden::timeT time = rect->getRepeatStartTime(mpos.x());
 
 //	    RG_DEBUG << "editRepeat at time " << time << endl;
 	
@@ -1220,6 +1436,8 @@ void SegmentCanvas::contentsMouseDoubleClickEvent(QMouseEvent* e)
 
 void SegmentCanvas::contentsMouseReleaseEvent(QMouseEvent *e)
 {
+    stopAutoScroll();
+
     if (!m_tool) return;
 
     if (e->button() == LeftButton ||
@@ -1231,14 +1449,18 @@ void SegmentCanvas::contentsMouseMoveEvent(QMouseEvent* e)
     if (!m_tool) return;
 
     int follow = m_tool->handleMouseMove(e);
+    setScrollDirectionConstraint(follow);
     
-    if (follow != SegmentTool::NoFollow) {
+    if (follow != RosegardenCanvasView::NoFollow) {
+        doAutoScroll();
 
-        if (follow & SegmentTool::FollowHorizontal)
-            slotScrollHorizSmallSteps(e->pos().x());
+//         if (follow & EditTool::FollowHorizontal) {
+//             slotScrollHorizSmallSteps(e->x());
+//         }
 
-        if (follow & SegmentTool::FollowVertical)
-            slotScrollVertSmallSteps(e->pos().y());
+//         if (follow & EditTool::FollowVertical) {
+//             slotScrollVertSmallSteps(e->y());
+//         }
     }
 }
 
@@ -1281,7 +1503,7 @@ SegmentCanvas::addSegmentItem(TrackId track, timeT startTime, timeT endTime)
 //    RG_DEBUG << "addSegmentItem from data: new item is " << newItem << endl;
 
     newItem->setPen(m_pen);
-//    newItem->setBrush(RosegardenGUIColours::convertColour(m_doc->getComposition().getSegmentColourMap().getColourByIndex(0)));
+//    newItem->setBrush(Rosegarden::GUIPalette::convertColour(m_doc->getComposition().getSegmentColourMap().getColourByIndex(0)));
     newItem->setVisible(true);     
     newItem->setZ(1);           // Segment at Z=1, Pointer at Z=10 [rwb]
 
@@ -1297,7 +1519,7 @@ SegmentCanvas::addSegmentItem(Segment *segment)
 //    RG_DEBUG << "addSegmentItem from segment: new item is " << newItem << endl;
 
     newItem->setPen(m_pen);
-//    newItem->setBrush(RosegardenGUIColours::convertColour(m_doc->getComposition().getSegmentColourMap().getColourByIndex(segment->getColourIndex())));
+//    newItem->setBrush(Rosegarden::GUIPalette::convertColour(m_doc->getComposition().getSegmentColourMap().getColourByIndex(segment->getColourIndex())));
     newItem->setVisible(true);     
     newItem->setZ(1);           // Segment at Z=1, Pointer at Z=10 [rwb]
 
@@ -1323,8 +1545,15 @@ void SegmentCanvas::showRecordingSegmentItem(Segment *segment, timeT endTime)
 	
 	m_recordingSegment = addSegmentItem(segment);
 	m_recordingSegment->setEndTime(endTime);
-	m_recordingSegment->setPen(RosegardenGUIColours::RecordingSegmentBorder);
-        m_recordingSegment->setColour(RosegardenGUIColours::RecordingSegmentBlock);
+	m_recordingSegment->setPen(Rosegarden::GUIPalette::getColour
+				   (Rosegarden::GUIPalette::RecordingSegmentBorder));
+	if (segment && segment->getType() == Segment::Audio) {
+	    m_recordingSegment->setColour(Rosegarden::GUIPalette::getColour
+					  (Rosegarden::GUIPalette::RecordingAudioSegmentBlock));
+	} else {
+	    m_recordingSegment->setColour(Rosegarden::GUIPalette::getColour
+					  (Rosegarden::GUIPalette::RecordingInternalSegmentBlock));
+	}
 	m_recordingSegment->setZ(2);
 
 //	RG_DEBUG << "(recording segment now is " << m_recordingSegment << ")" << endl;
