@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -30,6 +30,7 @@
 #include "BaseProperties.h"
 #include "Composition.h"
 #include "PeakFile.h"
+#include "Clipboard.h"
 #include "Sets.h"
 
 using Rosegarden::Composition;
@@ -640,22 +641,28 @@ SegmentReconfigureCommand::swap()
 
 	// set the segment's values from the record, but set the
 	// previous values back in to the record for use in the
-	// next iteration of the execute/unexecute cycle
+	// next iteration of the execute/unexecute cycle.
 
-	timeT currentEndMarkerTime = i->segment->getEndMarkerTime();
-	timeT currentStartTime = i->segment->getStartTime();
-	TrackId currentTrack = i->segment->getTrack();
+	// #1083496: look up both of the "old" values before we set
+	// anything, as setting the start time is likely to change the
+	// end marker time.
 
-	if (currentStartTime != i->startTime) {
+	timeT prevStartTime = i->segment->getStartTime();
+	timeT prevEndMarkerTime = i->segment->getEndMarkerTime();
+
+	if (i->segment->getStartTime() != i->startTime) {
 	    i->segment->setStartTime(i->startTime);
-	    i->startTime = currentStartTime;
 	}
 
-	if (currentEndMarkerTime != i->endMarkerTime) {
+	if (i->segment->getEndMarkerTime() != i->endMarkerTime) {
 	    i->segment->setEndMarkerTime(i->endMarkerTime);
-	    i->endMarkerTime = currentEndMarkerTime;
 	}
 
+	i->startTime = prevStartTime;
+	i->endMarkerTime = prevEndMarkerTime;
+
+	TrackId currentTrack = i->segment->getTrack();
+
 	if (currentTrack != i->track) {
 	    i->segment->setTrack(i->track);
 	    i->track = currentTrack;
@@ -664,6 +671,58 @@ SegmentReconfigureCommand::swap()
 }
 
 
+// SegmentResizeFromStartCommand
+//
+//
+
+SegmentResizeFromStartCommand::SegmentResizeFromStartCommand(Segment *s,
+							     timeT time) :
+    BasicCommand(i18n("Resize Segment"), *s,
+		 std::min(time, s->getStartTime()),
+		 std::max(time, s->getStartTime())),
+    m_segment(s),
+    m_oldStartTime(s->getStartTime()),
+    m_newStartTime(time)
+{
+    // nothing else 
+}
+
+SegmentResizeFromStartCommand::~SegmentResizeFromStartCommand()
+{
+    // nothing
+}
+
+void
+SegmentResizeFromStartCommand::modifySegment()
+{
+    if (m_newStartTime < m_oldStartTime) {
+	m_segment->fillWithRests(m_newStartTime, m_oldStartTime);
+    } else {
+
+	for (Segment::iterator i = m_segment->begin();
+	     m_segment->isBeforeEndMarker(i); ) {
+
+	    Segment::iterator j = i;
+	    ++j;
+
+	    if ((*i)->getAbsoluteTime() >= m_newStartTime) break;
+
+	    if ((*i)->getAbsoluteTime() + (*i)->getDuration() <= m_newStartTime) {
+		m_segment->erase(i);
+	    } else {
+		Event *e = new Event
+		    (**i, m_newStartTime,
+		     (*i)->getAbsoluteTime() + (*i)->getDuration() - m_newStartTime);
+		m_segment->erase(i);
+		m_segment->insert(e);
+	    }
+
+	    i = j;
+	}
+    }
+}
+
+
 // --------- Audio Split Segment -----------
 //
 //
@@ -857,6 +916,8 @@ SegmentSplitCommand::execute()
 				      (strtoqstr(m_segmentLabel))));
         m_newSegment->setLabel(m_segment->getLabel());
 	m_newSegment->setColourIndex(m_segment->getColourIndex());
+        m_newSegment->setTranspose(m_segment->getTranspose());
+        m_newSegment->setDelay(m_segment->getDelay());
     }
 
     // Resize left hand Segment
@@ -1169,7 +1230,7 @@ SegmentAutoSplitCommand::unexecute()
 }
 
 
-SegmentMergeCommand::SegmentMergeCommand(const Rosegarden::SegmentSelection &
+SegmentMergeCommand::SegmentMergeCommand(Rosegarden::SegmentSelection &
 					 segments) :
     KNamedCommand(getGlobalName()),
     m_newSegment(0),
@@ -1196,12 +1257,17 @@ SegmentMergeCommand::~SegmentMergeCommand()
 void
 SegmentMergeCommand::execute()
 {
-    Composition *composition = 0;
+    Composition *composition = m_oldSegments[0]->getComposition();
+    if (!composition) {
+	std::cerr
+	    << "SegmentMergeCommand::execute: ERROR: old segments are not in composition!"
+	    << std::endl;
+	return;
+    }
 
     if (!m_newSegment) {
 
 	m_newSegment = new Segment(*m_oldSegments[0]);
-	composition = m_oldSegments[0]->getComposition();
 
 	// that duplicated segment 0; now do the rest
 
@@ -1249,9 +1315,6 @@ SegmentMergeCommand::execute()
 		    (m_oldSegments[i]->getEndMarkerTime());
 	    }
 	}
-	
-    } else {
-	composition = m_newSegment->getComposition();
     }
 
     composition->addSegment(m_newSegment);
@@ -1507,7 +1570,7 @@ AddTimeSignatureAndNormalizeCommand::AddTimeSignatureAndNormalizeCommand
 	// also does a rest normalize.  It's what we normally want
 	// when adding a time signature.
 
-	addCommand(new TransformsMenuMakeRegionViableCommand
+	addCommand(new AdjustMenuMakeRegionViableCommand
 		   (**i,
 		    std::max(startTime, time),
 		    std::min(endTime, nextTimeSigTime)));
@@ -1768,8 +1831,7 @@ void DeleteTracksCommand::execute()
             m_oldTracks.push_back(track);
             if (m_composition->detachTrack(track) == false)
             {
-                RG_DEBUG << "DeleteTracksCommand::execute - "
-                         << "can't detach track" << endl;
+                RG_DEBUG << "DeleteTracksCommand::execute - can't detach track" << endl;
             }
         }
     }
@@ -1779,19 +1841,15 @@ void DeleteTracksCommand::execute()
     {
         for (tit = tracks.begin(); tit != tracks.end(); ++tit)
         {
-            if ((*tit).second->getId() > (*otIt)->getId())
+            if ((*tit).second->getPosition() > (*otIt)->getPosition())
             {
                 // If the track we've removed was after the current
                 // track then decrement the track position.
                 //
-                int newPosition = (*tit).second->getPosition();
-                if ((*tit).second->getPosition() > (*otIt)->getPosition())
-                    --newPosition;
+                int newPosition = (*tit).second->getPosition() - 1;
 
-                m_composition->resetTrackIdAndPosition
-                    ((*tit).second->getId(),
-                     (*tit).second->getId() - 1,
-                     newPosition);
+                (*tit).second->setPosition(newPosition);
+                
             }
         }
     }
@@ -1821,19 +1879,14 @@ void DeleteTracksCommand::unexecute()
         {
             --tit;
 
-            if ((*tit).second->getId() >= (*otIt)->getId())
+            if ((*tit).second->getPosition() >= (*otIt)->getPosition())
             {
                 // If the track we're adding has position after the
                 // current track then increment position.
                 //
-                int newPosition = (*tit).second->getPosition();
-                if ((*tit).second->getPosition() >= (*otIt)->getPosition())
-                   ++newPosition;
+                int newPosition = (*tit).second->getPosition() + 1;
 
-                m_composition->resetTrackIdAndPosition
-                    ((*tit).second->getId(),
-                     (*tit).second->getId() + 1,
-                     newPosition);
+                (*tit).second->setPosition(newPosition);
             }
 
             if (tit == tracks.begin()) break;
@@ -2195,7 +2248,7 @@ SegmentSplitByPitchCommand::getSplitPitchAt(Segment::iterator i,
 
 
 SegmentLabelCommand::SegmentLabelCommand(
-        const Rosegarden::SegmentSelection &segments,
+        Rosegarden::SegmentSelection &segments,
         const QString &label):
     KNamedCommand(i18n("Label Segments")),
     m_newLabel(label)
@@ -2233,7 +2286,7 @@ SegmentLabelCommand::unexecute()
 // Alter colour
 
 SegmentColourCommand::SegmentColourCommand(
-        const Rosegarden::SegmentSelection &segments,
+        Rosegarden::SegmentSelection &segments,
         const unsigned int index):
     KNamedCommand(i18n("Change Segment Color")),
     m_newColourIndex(index)
@@ -2293,3 +2346,450 @@ SegmentColourMapCommand::unexecute()
     m_doc->slotDocColoursChanged();
 }
 
+
+AddTriggerSegmentCommand::AddTriggerSegmentCommand(RosegardenGUIDoc *doc,
+						   Rosegarden::timeT duration,
+						   int basePitch,
+						   int baseVelocity) :
+    KNamedCommand(i18n("Add Triggered Segment")),
+    m_composition(&doc->getComposition()),
+    m_duration(duration),
+    m_basePitch(basePitch),
+    m_baseVelocity(baseVelocity),
+    m_id(0),
+    m_segment(0),
+    m_detached(false)
+{
+    // nothing else
+}
+
+AddTriggerSegmentCommand::~AddTriggerSegmentCommand()
+{
+    if (m_detached) delete m_segment;
+}
+
+Rosegarden::TriggerSegmentId
+AddTriggerSegmentCommand::getId() const
+{
+    return m_id;
+}
+
+void
+AddTriggerSegmentCommand::execute()
+{
+    if (m_segment) {
+	m_composition->addTriggerSegment(m_segment, m_id, m_basePitch, m_baseVelocity);
+    } else {
+	m_segment = new Rosegarden::Segment();
+	m_segment->setEndMarkerTime(m_duration);
+	Rosegarden::TriggerSegmentRec *rec = m_composition->addTriggerSegment
+	    (m_segment, m_basePitch, m_baseVelocity);
+	if (rec) m_id = rec->getId();
+    }
+    m_detached = false;
+}
+
+void
+AddTriggerSegmentCommand::unexecute()
+{
+    if (m_segment) m_composition->detachTriggerSegment(m_id);
+    m_detached = true;
+}
+
+
+DeleteTriggerSegmentCommand::DeleteTriggerSegmentCommand(RosegardenGUIDoc *doc,
+							 Rosegarden::TriggerSegmentId id) :
+    KNamedCommand(i18n("Delete Triggered Segment")),
+    m_composition(&doc->getComposition()),
+    m_id(id),
+    m_segment(0),
+    m_basePitch(-1),
+    m_baseVelocity(-1),
+    m_detached(true)
+{
+    // nothing else
+}
+
+DeleteTriggerSegmentCommand::~DeleteTriggerSegmentCommand()
+{
+    if (m_detached) delete m_segment;
+}
+
+void
+DeleteTriggerSegmentCommand::execute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    m_segment = rec->getSegment();
+    m_basePitch = rec->getBasePitch();
+    m_baseVelocity = rec->getBaseVelocity();
+    m_composition->detachTriggerSegment(m_id);
+    m_detached = true;
+}
+
+void
+DeleteTriggerSegmentCommand::unexecute()
+{
+    if (m_segment)
+	m_composition->addTriggerSegment(m_segment, m_id, m_basePitch, m_baseVelocity);
+    m_detached = false;
+}
+
+
+PasteToTriggerSegmentCommand::PasteToTriggerSegmentCommand(Rosegarden::Composition *composition,
+							   Rosegarden::Clipboard *clipboard,
+							   QString label,
+							   int basePitch,
+							   int baseVelocity) :
+    KNamedCommand(i18n("Paste as New Triggered Segment")),
+    m_composition(composition),
+    m_clipboard(clipboard),
+    m_label(label),
+    m_basePitch(basePitch),
+    m_baseVelocity(baseVelocity),
+    m_segment(0),
+    m_detached(false)
+{
+    // nothing else
+}
+
+PasteToTriggerSegmentCommand::~PasteToTriggerSegmentCommand()
+{
+    if (m_detached) delete m_segment;
+}
+
+void
+PasteToTriggerSegmentCommand::execute()
+{
+    if (m_segment) {
+
+	m_composition->addTriggerSegment(m_segment, m_id, m_basePitch, m_baseVelocity);
+
+    } else {
+	
+	if (m_clipboard->isEmpty()) return;
+	
+	m_segment = new Rosegarden::Segment();
+	
+	timeT earliestStartTime = 0;
+	timeT latestEndTime = 0;
+	
+	for (Rosegarden::Clipboard::iterator i = m_clipboard->begin();
+	     i != m_clipboard->end(); ++i) {
+
+	    if (i == m_clipboard->begin() ||
+		(*i)->getStartTime() < earliestStartTime) {
+		earliestStartTime = (*i)->getStartTime();
+	    }
+
+	    if ((*i)->getEndMarkerTime() > latestEndTime)
+		latestEndTime = (*i)->getEndMarkerTime();
+	}
+
+	for (Rosegarden::Clipboard::iterator i = m_clipboard->begin();
+	     i != m_clipboard->end(); ++i) {
+	    
+	    for (Rosegarden::Segment::iterator si = (*i)->begin();
+		 (*i)->isBeforeEndMarker(si); ++si) {
+		if (!(*si)->isa(Rosegarden::Note::EventRestType)) {
+		    m_segment->insert
+			(new Event(**si, 
+				   (*si)->getAbsoluteTime() - earliestStartTime));
+		}
+	    }
+	}
+	
+	m_segment->setLabel(qstrtostr(m_label));
+
+	Rosegarden::TriggerSegmentRec *rec = 
+	    m_composition->addTriggerSegment(m_segment, m_basePitch, m_baseVelocity);
+	if (rec) m_id = rec->getId();
+    }
+
+    m_composition->getTriggerSegmentRec(m_id)->updateReferences();
+    
+    m_detached = false;
+}
+
+void
+PasteToTriggerSegmentCommand::unexecute()
+{
+    if (m_segment) m_composition->detachTriggerSegment(m_id);
+    m_detached = true;
+}
+    
+
+SetTriggerSegmentBasePitchCommand::SetTriggerSegmentBasePitchCommand(Rosegarden::Composition *composition,
+								     Rosegarden::TriggerSegmentId id,
+								     int newPitch) :
+    KNamedCommand(i18n("Set Base Pitch")),
+    m_composition(composition),
+    m_id(id),
+    m_newPitch(newPitch),
+    m_oldPitch(-1)
+{
+    // nothing
+}
+
+SetTriggerSegmentBasePitchCommand::~SetTriggerSegmentBasePitchCommand()
+{
+    // nothing
+}
+
+void
+SetTriggerSegmentBasePitchCommand::execute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    if (m_oldPitch == -1) {
+	m_oldPitch = rec->getBasePitch();
+    }
+    rec->setBasePitch(m_newPitch);
+}
+
+void
+SetTriggerSegmentBasePitchCommand::unexecute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    rec->setBasePitch(m_oldPitch);
+}
+
+
+SetTriggerSegmentBaseVelocityCommand::SetTriggerSegmentBaseVelocityCommand(Rosegarden::Composition *composition,
+								     Rosegarden::TriggerSegmentId id,
+								     int newVelocity) :
+    KNamedCommand(i18n("Set Base Velocity")),
+    m_composition(composition),
+    m_id(id),
+    m_newVelocity(newVelocity),
+    m_oldVelocity(-1)
+{
+    // nothing
+}
+
+SetTriggerSegmentBaseVelocityCommand::~SetTriggerSegmentBaseVelocityCommand()
+{
+    // nothing
+}
+
+void
+SetTriggerSegmentBaseVelocityCommand::execute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    if (m_oldVelocity == -1) {
+	m_oldVelocity = rec->getBaseVelocity();
+    }
+    rec->setBaseVelocity(m_newVelocity);
+}
+
+void
+SetTriggerSegmentBaseVelocityCommand::unexecute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    rec->setBaseVelocity(m_oldVelocity);
+}
+
+
+SetTriggerSegmentDefaultTimeAdjustCommand::SetTriggerSegmentDefaultTimeAdjustCommand(Rosegarden::Composition *composition,
+										     Rosegarden::TriggerSegmentId id,
+										     std::string newDefaultTimeAdjust) :
+    KNamedCommand(i18n("Set Default Time Adjust")),
+    m_composition(composition),
+    m_id(id),
+    m_newDefaultTimeAdjust(newDefaultTimeAdjust),
+    m_oldDefaultTimeAdjust("")
+{
+    // nothing
+}
+
+SetTriggerSegmentDefaultTimeAdjustCommand::~SetTriggerSegmentDefaultTimeAdjustCommand()
+{
+    // nothing
+}
+
+void
+SetTriggerSegmentDefaultTimeAdjustCommand::execute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    if (m_oldDefaultTimeAdjust == "") {
+	m_oldDefaultTimeAdjust = rec->getDefaultTimeAdjust();
+    }
+    rec->setDefaultTimeAdjust(m_newDefaultTimeAdjust);
+}
+
+void
+SetTriggerSegmentDefaultTimeAdjustCommand::unexecute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    rec->setDefaultTimeAdjust(m_oldDefaultTimeAdjust);
+}
+
+
+
+
+SetTriggerSegmentDefaultRetuneCommand::SetTriggerSegmentDefaultRetuneCommand(Rosegarden::Composition *composition,
+									     Rosegarden::TriggerSegmentId id,
+									     bool newDefaultRetune) :
+    KNamedCommand(i18n("Set Default Retune")),
+    m_composition(composition),
+    m_id(id),
+    m_newDefaultRetune(newDefaultRetune),
+    m_oldDefaultRetune(false),
+    m_haveOldDefaultRetune(false)
+{
+    // nothing
+}
+
+SetTriggerSegmentDefaultRetuneCommand::~SetTriggerSegmentDefaultRetuneCommand()
+{
+    // nothing
+}
+
+void
+SetTriggerSegmentDefaultRetuneCommand::execute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    if (!m_haveOldDefaultRetune) {
+	m_oldDefaultRetune = rec->getDefaultRetune();
+    }
+    rec->setDefaultRetune(m_newDefaultRetune);
+}
+
+void
+SetTriggerSegmentDefaultRetuneCommand::unexecute()
+{
+    Rosegarden::TriggerSegmentRec *rec = m_composition->getTriggerSegmentRec(m_id);
+    if (!rec) return;
+    rec->setDefaultRetune(m_oldDefaultRetune);
+}
+
+
+CreateTempoMapFromSegmentCommand::CreateTempoMapFromSegmentCommand(Segment *groove) :
+    KNamedCommand(i18n("Set Tempos from Beat Segment")),
+    m_composition(groove->getComposition())
+{
+    initialise(groove);
+}
+
+CreateTempoMapFromSegmentCommand::~CreateTempoMapFromSegmentCommand()
+{
+    // nothing
+}
+
+void
+CreateTempoMapFromSegmentCommand::execute()
+{
+    for (TempoMap::iterator i = m_oldTempi.begin(); i != m_oldTempi.end(); ++i) {
+	int n = m_composition->getTempoChangeNumberAt(i->first);
+	if (n < m_composition->getTempoChangeCount()) {
+	    m_composition->removeTempoChange(n);
+	}
+    }
+
+    for (TempoMap::iterator i = m_newTempi.begin(); i != m_newTempi.end(); ++i) {
+	m_composition->addRawTempo(i->first, i->second);
+    }
+}
+
+void
+CreateTempoMapFromSegmentCommand::unexecute()
+{
+    for (TempoMap::iterator i = m_newTempi.begin(); i != m_newTempi.end(); ++i) {
+	int n = m_composition->getTempoChangeNumberAt(i->first);
+	if (n < m_composition->getTempoChangeCount()) {
+	    m_composition->removeTempoChange(n);
+	}
+    }
+
+    for (TempoMap::iterator i = m_oldTempi.begin(); i != m_oldTempi.end(); ++i) {
+	m_composition->addRawTempo(i->first, i->second);
+    }
+}
+
+void
+CreateTempoMapFromSegmentCommand::initialise(Rosegarden::Segment *s)
+{
+    m_oldTempi.clear();
+    m_newTempi.clear();
+
+    //!!! need an additional option: per-chord, per-beat, per-bar.
+    // Let's work per-beat for the moment.  Even for this, we should
+    // probably use TimeSignature.getDivisions()
+
+    std::vector<Rosegarden::timeT> beatTimeTs;
+    std::vector<Rosegarden::RealTime> beatRealTimes;
+
+    int startBar = m_composition->getBarNumber(s->getStartTime());
+    int barNo = startBar;
+    int beat = 0;
+
+    for (Segment::iterator i = s->begin(); s->isBeforeEndMarker(i); ++i) {
+	if ((*i)->isa(Rosegarden::Note::EventType)) {
+
+	    bool isNew;
+	    Rosegarden::TimeSignature sig =
+		m_composition->getTimeSignatureInBar(barNo, isNew);
+
+	    beatTimeTs.push_back(m_composition->getBarStart(barNo) +
+				 beat * sig.getBeatDuration());
+
+	    if (++beat >= sig.getBeatsPerBar()) {
+		++barNo;
+		beat = 0;
+	    }
+
+	    beatRealTimes.push_back(s->getComposition()->getElapsedRealTime
+				    ((*i)->getAbsoluteTime()));
+	}
+    }
+
+    if (beatTimeTs.size() < 2) return;
+
+    long prevRawTempo = -1;
+
+    // set up m_oldTempi and prevRawTempo
+
+    for (int i = m_composition->getTempoChangeNumberAt(*beatTimeTs.begin()-1) + 1;
+	 i <= m_composition->getTempoChangeNumberAt(*beatTimeTs.end()-1); ++i) {
+
+	std::pair<Rosegarden::timeT, long> tempoChange =
+	    m_composition->getRawTempoChange(i);
+	m_oldTempi[tempoChange.first] = tempoChange.second;
+	if (prevRawTempo == -1) prevRawTempo = tempoChange.second;
+    }
+
+    RG_DEBUG << "starting raw tempo: " << prevRawTempo << endl;
+
+    Rosegarden::timeT quarter = Rosegarden::Note(Rosegarden::Note::Crotchet).getDuration();
+
+    for (int beat = 1; beat < beatTimeTs.size(); ++beat) {
+
+	Rosegarden::timeT beatTime = beatTimeTs[beat] - beatTimeTs[beat-1];
+	Rosegarden::RealTime beatRealTime = beatRealTimes[beat] - beatRealTimes[beat-1];
+
+	// Calculate tempo in quarter notes per hour.
+	// This is 3600 / {quarter note duration in seconds}
+	// = 3600 / ( {beat in seconds} * {quarter in ticks} / { beat in ticks} )
+	// = ( 3600 * {beat in ticks} ) / ( {beat in seconds} * {quarter in ticks} )
+
+	double beatSec = double(beatRealTime.sec) +
+	    double(beatRealTime.usec() / 1000000.0);
+	double qph = (3600.0 * beatTime) / (beatSec * quarter);
+	long rawTempo = long(qph + 0.000001);
+
+	RG_DEBUG << "prev beat: " << beatTimeTs[beat] << ", prev beat real time " << beatRealTimes[beat] << endl;
+	RG_DEBUG << "time " << beatTime << ", rt " << beatRealTime << ", beatSec " << beatSec << ", rawTempo " << rawTempo << endl;
+
+	if (rawTempo != prevRawTempo) {
+	    m_newTempi[beatTimeTs[beat-1]] = rawTempo;
+	    prevRawTempo = rawTempo;
+	}
+    }
+	
+}
