@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -27,13 +27,15 @@
 #include <kcommand.h>
 
 #include "Segment.h"
-#include "Composition.h"
 #include "NotationTypes.h"
 #include "rosegardenguidoc.h"
 #include "AudioFileManager.h"
 #include "Selection.h"
+#include "TriggerSegment.h"
 #include "basiccommand.h"
 
+namespace Rosegarden { class Composition; }
+
 /**
  * Base class for commands from the SegmentParameterBox
  */
@@ -280,6 +282,31 @@ private:
 };
 
 
+/**
+ * SegmentResizeFromStartCommand moves the start time of a segment
+ * leaving the events in it at the same absolute times as before, so
+ * padding with rests or deleting events as appropriate.  (Distinct
+ * from Segment::setStartTime, as used by SegmentReconfigureCommand,
+ * which moves all the events in the segment.)
+ */
+
+class SegmentResizeFromStartCommand : public BasicCommand
+{
+public:
+    SegmentResizeFromStartCommand(Rosegarden::Segment *segment,
+				  Rosegarden::timeT newStartTime);
+    virtual ~SegmentResizeFromStartCommand();
+
+protected:
+    virtual void modifySegment();
+
+private:
+    Rosegarden::Segment *m_segment;
+    Rosegarden::timeT m_oldStartTime;
+    Rosegarden::timeT m_newStartTime;
+};
+
+
 class AudioSegmentSplitCommand : public KNamedCommand
 {
 public:
@@ -363,7 +390,7 @@ private:
 class SegmentMergeCommand : public KNamedCommand
 {
 public:
-    SegmentMergeCommand(const Rosegarden::SegmentSelection &segments);
+    SegmentMergeCommand(Rosegarden::SegmentSelection &segments);
     virtual ~SegmentMergeCommand();
 
     virtual void execute();
@@ -714,7 +741,7 @@ private:
 class SegmentLabelCommand : public KNamedCommand
 {
 public:
-    SegmentLabelCommand(const Rosegarden::SegmentSelection &segments,
+    SegmentLabelCommand(Rosegarden::SegmentSelection &segments,
                         const QString &label);
     virtual ~SegmentLabelCommand();
 
@@ -734,7 +761,7 @@ protected:
 class SegmentColourCommand : public KNamedCommand
 {
 public:
-    SegmentColourCommand(const Rosegarden::SegmentSelection &segments,
+    SegmentColourCommand(Rosegarden::SegmentSelection &segments,
                          const unsigned int index);
     virtual ~SegmentColourCommand();
 
@@ -769,4 +796,184 @@ protected:
 };
 
 
+// Trigger Segment commands.  These are the commands that create
+// and manage the triggered segments themselves.  See editcommands.h
+// for SetTriggerCommand and ClearTriggersCommand which manipulate
+// the events that do the triggering.
+
+class AddTriggerSegmentCommand : public KNamedCommand
+{
+public:
+    AddTriggerSegmentCommand(RosegardenGUIDoc *doc,
+			     Rosegarden::timeT duration, // start time always 0
+			     int basePitch = -1,
+			     int baseVelocity = -1);
+    virtual ~AddTriggerSegmentCommand();
+
+    Rosegarden::TriggerSegmentId getId() const; // after invocation
+
+    virtual void execute();
+    virtual void unexecute();
+
+private:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::timeT m_duration;
+    int m_basePitch;
+    int m_baseVelocity;
+    Rosegarden::TriggerSegmentId m_id;
+    Rosegarden::Segment *m_segment;
+    bool m_detached;
+};
+
+
+class DeleteTriggerSegmentCommand : public KNamedCommand
+{
+public:
+    DeleteTriggerSegmentCommand(RosegardenGUIDoc *doc,
+				Rosegarden::TriggerSegmentId);
+    virtual ~DeleteTriggerSegmentCommand();
+
+    virtual void execute();
+    virtual void unexecute();
+
+private:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::TriggerSegmentId m_id;
+    Rosegarden::Segment *m_segment;
+    int m_basePitch;
+    int m_baseVelocity;
+    bool m_detached;
+};
+
+
+class PasteToTriggerSegmentCommand : public KNamedCommand
+{
+public:
+    /// If basePitch is -1, the first pitch in the selection will be used
+    PasteToTriggerSegmentCommand(Rosegarden::Composition *composition,
+				 Rosegarden::Clipboard *clipboard,
+				 QString label,
+				 int basePitch = -1,
+				 int baseVelocity = -1);
+    virtual ~PasteToTriggerSegmentCommand();
+
+    virtual void execute();
+    virtual void unexecute();
+
+protected:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::Clipboard *m_clipboard;
+    QString m_label;
+    int m_basePitch;
+    int m_baseVelocity;
+    Rosegarden::Segment *m_segment;
+    Rosegarden::TriggerSegmentId m_id;
+    bool m_detached;
+};
+
+
+class SetTriggerSegmentBasePitchCommand : public KNamedCommand
+{
+public:
+    SetTriggerSegmentBasePitchCommand(Rosegarden::Composition *composition,
+				      Rosegarden::TriggerSegmentId id,
+				      int newPitch);
+    virtual ~SetTriggerSegmentBasePitchCommand();
+    
+    virtual void execute();
+    virtual void unexecute();
+
+protected:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::TriggerSegmentId m_id;
+    int m_newPitch;
+    int m_oldPitch;
+};
+
+
+class SetTriggerSegmentBaseVelocityCommand : public KNamedCommand
+{
+public:
+    SetTriggerSegmentBaseVelocityCommand(Rosegarden::Composition *composition,
+				      Rosegarden::TriggerSegmentId id,
+				      int newVelocity);
+    virtual ~SetTriggerSegmentBaseVelocityCommand();
+    
+    virtual void execute();
+    virtual void unexecute();
+
+protected:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::TriggerSegmentId m_id;
+    int m_newVelocity;
+    int m_oldVelocity;
+};
+
+
+class SetTriggerSegmentDefaultTimeAdjustCommand : public KNamedCommand
+{
+public:
+    SetTriggerSegmentDefaultTimeAdjustCommand(Rosegarden::Composition *composition,
+					      Rosegarden::TriggerSegmentId id,
+					      std::string newDefaultTimeAdjust);
+    virtual ~SetTriggerSegmentDefaultTimeAdjustCommand();
+
+    virtual void execute();
+    virtual void unexecute();
+
+protected:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::TriggerSegmentId m_id;
+    std::string m_newDefaultTimeAdjust;
+    std::string m_oldDefaultTimeAdjust;
+};
+
+
+class SetTriggerSegmentDefaultRetuneCommand : public KNamedCommand
+{
+public:
+    SetTriggerSegmentDefaultRetuneCommand(Rosegarden::Composition *composition,
+					  Rosegarden::TriggerSegmentId id,
+					  bool newDefaultRetune);
+    virtual ~SetTriggerSegmentDefaultRetuneCommand();
+
+    virtual void execute();
+    virtual void unexecute();
+
+protected:
+    Rosegarden::Composition *m_composition;
+    Rosegarden::TriggerSegmentId m_id;
+    bool m_newDefaultRetune;
+    bool m_oldDefaultRetune;
+    bool m_haveOldDefaultRetune;
+};
+
+
+/**
+ * CreateTempoMapFromSegment applies timings found in a reference
+ * segment to the composition as a whole via the tempo map.
+ */
+
+class CreateTempoMapFromSegmentCommand : public KNamedCommand
+{
+public:
+    CreateTempoMapFromSegmentCommand(Rosegarden::Segment *grooveSegment);
+    virtual ~CreateTempoMapFromSegmentCommand();
+
+    virtual void execute();
+    virtual void unexecute();
+
+private:
+    void initialise(Rosegarden::Segment *s);
+    
+    Rosegarden::Composition *m_composition;
+
+    typedef std::map<Rosegarden::timeT, long> TempoMap;
+    TempoMap m_oldTempi;
+    TempoMap m_newTempi;
+};
+    
+    
+
+
 #endif  // _SEGMENTCOMMANDS_H_
