@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -73,7 +73,8 @@ SegmentParameterBox::initBox()
     // magic numbers: 13 is the height of the menu pixmaps, 10 is just 10
     int comboHeight = std::max(fontMetrics.height(), 13) + 10;
 
-    QGridLayout *gridLayout = new QGridLayout(this, 6, 2, 8, 1);
+//    QFrame *frame = new QFrame(this);
+    QGridLayout *gridLayout = new QGridLayout(this, 6, 4, 4, 2);
 
     QLabel *label = new QLabel(i18n("Label"), this);
     QLabel *repeatLabel    = new QLabel(i18n("Repeat"), this);
@@ -81,6 +82,9 @@ SegmentParameterBox::initBox()
     QLabel *transposeLabel = new QLabel(i18n("Transpose"), this);
     QLabel *delayLabel     = new QLabel(i18n("Delay"), this);
     QLabel *colourLabel    = new QLabel(i18n("Color"), this);
+    m_autoFadeLabel        = new QLabel(i18n("Audio auto-fade"), this);
+    m_fadeInLabel          = new QLabel(i18n("Fade in"), this);
+    m_fadeOutLabel         = new QLabel(i18n("Fade out"), this);
 
     // HBox for label
     //
@@ -89,12 +93,13 @@ SegmentParameterBox::initBox()
     // Label ..
     m_label = new QLabel(hbox);
     m_label->setFont(font);
+    m_label->setFixedWidth(120);
     m_label->setFrameStyle(QFrame::Panel | QFrame::Sunken);
-    m_label->setFixedWidth(100);
 
     // .. and edit button
     m_labelButton = new QPushButton("...", hbox);
     m_labelButton->setFont(font);
+    m_labelButton->setFixedWidth(25);
 
     connect(m_labelButton, SIGNAL(released()),
             SLOT(slotEditSegmentLabel()));
@@ -154,7 +159,27 @@ SegmentParameterBox::initBox()
     connect(m_colourValue, SIGNAL(activated(int)),
             SLOT(slotColourSelected(int)));
 
+    // Audio autofade enabled
+    //
+    m_autoFadeBox = new QCheckBox(this);
+    connect(m_autoFadeBox, SIGNAL(stateChanged(int)),
+            this, SLOT(slotAudioFadeChanged(int)));
 
+    // Fade in and out times
+    //
+    m_fadeInSpin = new QSpinBox(this);
+    m_fadeInSpin->setMinValue(0);
+    m_fadeInSpin->setMaxValue(5000);
+    m_fadeInSpin->setSuffix(i18n(" ms"));
+    connect(m_fadeInSpin, SIGNAL(valueChanged(int)),
+            this, SLOT(slotFadeInChanged(int)));
+
+    m_fadeOutSpin = new QSpinBox(this);
+    m_fadeOutSpin->setMinValue(0);
+    m_fadeOutSpin->setMaxValue(5000);
+    m_fadeOutSpin->setSuffix(i18n(" ms"));
+    connect(m_fadeOutSpin, SIGNAL(valueChanged(int)),
+            this, SLOT(slotFadeOutChanged(int)));
 
     label->setFont(font);
     repeatLabel->setFont(font);
@@ -162,26 +187,38 @@ SegmentParameterBox::initBox()
     transposeLabel->setFont(font);
     delayLabel->setFont(font);
     colourLabel->setFont(font);
+    m_autoFadeLabel->setFont(font);
+    m_fadeInLabel->setFont(font);
+    m_fadeOutLabel->setFont(font);
 
-    gridLayout->addRowSpacing(0, 8);
+    gridLayout->addRowSpacing(0, 12);
 
-    gridLayout->addWidget(label,         1, 0, AlignLeft);
-    gridLayout->addWidget(hbox,       1, 1, AlignLeft);
+    gridLayout->addWidget(label, 1, 0, AlignRight);
+    gridLayout->addMultiCellWidget(hbox, 1, 1, 1, 3, AlignLeft);
 
-    gridLayout->addWidget(repeatLabel,   2, 0, AlignLeft);
+    gridLayout->addWidget(repeatLabel, 2, 0, AlignRight);
     gridLayout->addWidget(m_repeatValue, 2, 1, AlignLeft);
 
-    gridLayout->addWidget(quantizeLabel,   3, 0, AlignLeft);
+    gridLayout->addWidget(transposeLabel, 2, 2, AlignRight);
+    gridLayout->addWidget(m_transposeValue, 2, 3);
+
+    gridLayout->addWidget(quantizeLabel, 3, 0, AlignRight);
     gridLayout->addWidget(m_quantizeValue, 3, 1);
 
-    gridLayout->addWidget(transposeLabel,   4, 0, AlignLeft);
-    gridLayout->addWidget(m_transposeValue, 4, 1);
+    gridLayout->addWidget(delayLabel, 3, 2, AlignRight);
+    gridLayout->addWidget(m_delayValue, 3, 3);
 
-    gridLayout->addWidget(delayLabel,   5, 0, AlignLeft);
-    gridLayout->addWidget(m_delayValue, 5, 1);
+    gridLayout->addWidget(colourLabel, 4, 0, AlignRight);
+    gridLayout->addMultiCellWidget(m_colourValue, 4, 4, 1, 3);
 
-    gridLayout->addWidget(colourLabel,   6, 0, AlignLeft);
-    gridLayout->addWidget(m_colourValue, 6, 1);
+    m_autoFadeLabel->hide();
+    m_autoFadeBox->hide();
+
+    gridLayout->addWidget(m_fadeInLabel,   5, 0, AlignRight);
+    gridLayout->addWidget(m_fadeInSpin,  5, 1);
+
+    gridLayout->addWidget(m_fadeOutLabel,  5, 2, AlignRight);
+    gridLayout->addWidget(m_fadeOutSpin, 5, 3);
 
     // populate the quantize combo
     //
@@ -210,7 +247,7 @@ SegmentParameterBox::initBox()
 
     m_delays.clear();
 
-    for (int i = 0; i < 9; i++)
+    for (int i = 0; i < 6; i++)
     {
 	Rosegarden::timeT time = 0;
 	if (i > 0 && i < 6) {
@@ -243,7 +280,6 @@ SegmentParameterBox::initBox()
 
     // populate m_colourValue
     slotDocColoursChanged();
-
 }
 
 void
@@ -289,7 +325,7 @@ SegmentParameterBox::slotDocColoursChanged()
     for (Rosegarden::RCMap::const_iterator it=temp.begin(); it != temp.end(); ++it)
     {
         QPixmap colour(15,15);
-        colour.fill(RosegardenGUIColours::convertColour(it->second.first));
+        colour.fill(Rosegarden::GUIPalette::convertColour(it->second.first));
         if (it->second.second == std::string(""))
             m_colourValue->insertItem(colour, i18n("Default Color"), i);
         else
@@ -604,6 +640,74 @@ SegmentParameterBox::populateBoxFromSegments()
 
     m_colourValue->setEnabled(diffcolours != NotApplicable);
 
+    // Enable or disable the fade in/out params
+    if (m_segments.size() == 1 && 
+        (*(m_segments.begin()))->getType() == Rosegarden::Segment::Audio)
+    {
+        m_autoFadeBox->blockSignals(true);
+        m_fadeInSpin->blockSignals(true);
+        m_fadeOutSpin->blockSignals(true);
+
+/*!!! No, not setting up autofade widgets.  The implementation's too
+      incomplete to finish for this release.
+
+        m_fadeInLabel->show();
+        m_fadeInSpin->show();
+        m_fadeOutLabel->show();
+        m_fadeOutSpin->show();
+
+	instead:
+*/
+        m_fadeInLabel->hide();
+        m_fadeInSpin->hide();
+        m_fadeOutLabel->hide();
+        m_fadeOutSpin->hide();
+
+        m_autoFadeLabel->setEnabled(true);
+        m_autoFadeBox->setEnabled(true);
+        m_fadeInLabel->setEnabled(true);
+        m_fadeInSpin->setEnabled(true);
+        m_fadeOutLabel->setEnabled(true);
+        m_fadeOutSpin->setEnabled(true);
+
+        Rosegarden::Segment *seg = *(m_segments.begin());
+
+        int fadeInTime = seg->getFadeInTime().sec * 1000 +
+            seg->getFadeInTime().msec();
+        m_fadeInSpin->setValue(fadeInTime);
+
+        int fadeOutTime = seg->getFadeOutTime().sec * 1000 +
+            seg->getFadeOutTime().msec();
+        m_fadeOutSpin->setValue(fadeOutTime);
+
+        m_autoFadeBox->setChecked(seg->isAutoFading());
+
+        m_autoFadeBox->blockSignals(false);
+        m_fadeInSpin->blockSignals(false);
+        m_fadeOutSpin->blockSignals(false);
+    }
+    else
+    {
+        m_autoFadeLabel->setEnabled(false);
+        m_autoFadeBox->setEnabled(false);
+        m_fadeInLabel->setEnabled(false);
+        m_fadeInSpin->setEnabled(false);
+        m_fadeOutLabel->setEnabled(false);
+        m_fadeOutSpin->setEnabled(false);
+
+        m_autoFadeLabel->hide();
+        m_autoFadeBox->hide();
+        m_fadeInLabel->hide();
+        m_fadeInSpin->hide();
+        m_fadeOutLabel->hide();
+        m_fadeOutSpin->hide();
+
+        m_autoFadeBox->setChecked(false);
+        m_fadeInSpin->setValue(0);
+        m_fadeOutSpin->setValue(0);
+    }
+
+
 }
 
 void SegmentParameterBox::slotRepeatPressed()
@@ -779,7 +883,7 @@ SegmentParameterBox::slotColourSelected(int value)
 
             if (result == KColorDialog::Accepted)
             {
-                Rosegarden::Colour newRColour = RosegardenGUIColours::convertColour(newColour);
+                Rosegarden::Colour newRColour = Rosegarden::GUIPalette::convertColour(newColour);
                 newMap.addItem(newRColour, qstrtostr(newName));
                 SegmentColourMapCommand *command = new SegmentColourMapCommand(m_doc, newMap);
                 addCommandToHistory(command);
@@ -841,3 +945,72 @@ SegmentParameterBox::slotEditSegmentLabel()
     }
 }
 
+
+void
+SegmentParameterBox::slotAudioFadeChanged(int value)
+{
+    RG_DEBUG << "SegmentParameterBox::slotAudioFadeChanged - value = "
+             << value << endl;
+
+    if (m_segments.size() == 0)
+        return;
+
+    bool state = false;
+    if (value == QButton::On) state = true;
+
+    std::vector<Rosegarden::Segment*>::iterator it;
+    for (it = m_segments.begin(); it != m_segments.end(); it++) {
+        (*it)->setAutoFade(state);
+    }
+
+}
+
+
+void
+SegmentParameterBox::slotFadeInChanged(int value)
+{
+    RG_DEBUG << "SegmentParameterBox::slotFadeInChanged - value = "
+             << value << endl;
+
+    if (m_segments.size() == 0)
+        return;
+
+    if (value == 0 && m_fadeOutSpin->value() == 0) slotAudioFadeChanged(QButton::Off);
+    else slotAudioFadeChanged(QButton::On);
+
+    // Convert from ms
+    //
+    Rosegarden::RealTime fadeInTime(value/1000, (value % 1000) * 1000000);
+
+    std::vector<Rosegarden::Segment*>::iterator it;
+    for (it = m_segments.begin(); it != m_segments.end(); it++) {
+        (*it)->setFadeInTime(fadeInTime);
+    }
+
+    emit documentModified();
+}
+
+void
+SegmentParameterBox::slotFadeOutChanged(int value)
+{
+    RG_DEBUG << "SegmentParameterBox::slotFadeOutChanged - value = "
+             << value << endl;
+
+    if (m_segments.size() == 0)
+        return;
+
+    if (value == 0 && m_fadeInSpin->value() == 0) slotAudioFadeChanged(QButton::Off);
+    else slotAudioFadeChanged(QButton::On);
+
+    // Convert from ms
+    //
+    Rosegarden::RealTime fadeOutTime(value/1000000, (value % 1000) * 10000000);
+
+    std::vector<Rosegarden::Segment*>::iterator it;
+    for (it = m_segments.begin(); it != m_segments.end(); it++) {
+        (*it)->setFadeOutTime(fadeOutTime);
+    }
+
+    emit documentModified();
+}
+
