@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -42,7 +42,7 @@ using Rosegarden::timeT;
 using Rosegarden::SnapGrid;
 using Rosegarden::Note;
 using Rosegarden::SegmentSelection;
-using RosegardenGUIColours::convertColour;
+using Rosegarden::GUIPalette;
 
 //////////////////////////////////////////////////////////////////////
 //                 Segment Tools
@@ -204,8 +204,26 @@ SegmentPencil::SegmentPencil(SegmentCanvas *c, RosegardenGUIDoc *d)
 void SegmentPencil::ready()
 {
     m_canvas->viewport()->setCursor(Qt::ibeamCursor);
+    connect(m_canvas, SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotCanvasScrolled(int, int)));
+
 }
 
+void SegmentPencil::stow()
+{
+    disconnect(m_canvas, SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotCanvasScrolled(int, int)));
+}
+
+void SegmentPencil::slotCanvasScrolled(int newX, int newY)
+{
+    QMouseEvent tmpEvent(QEvent::MouseMove,
+                         m_canvas->viewport()->mapFromGlobal(QCursor::pos()) + QPoint(newX, newY),
+                         Qt::NoButton, Qt::NoButton);
+    handleMouseMove(&tmpEvent);
+}
+
+
 void SegmentPencil::handleMouseButtonPress(QMouseEvent *e)
 {
     if (e->button() == RightButton) return;
@@ -297,12 +315,16 @@ void SegmentPencil::handleMouseButtonRelease(QMouseEvent* e)
 
 int SegmentPencil::handleMouseMove(QMouseEvent *e)
 {
-    if (!m_currentItem) return NoFollow;
+    QPoint pos = e->pos();
+
+    if (!m_currentItem) return RosegardenCanvasView::NoFollow;
 
     m_canvas->setSnapGrain(false);
 
-    QWMatrix matrix = m_canvas->worldMatrix().invert();
-    QPoint tPos = matrix.map(e->pos());
+    QPoint tPos = m_canvas->inverseMapPoint(pos);
+
+//     RG_DEBUG << "SegmentPencil::handleMouseMove: pos " << pos << ", tPos "
+// 	     << tPos << endl;
 
     SnapGrid::SnapDirection direction = SnapGrid::SnapRight;
     if (tPos.x() < m_currentItem->x()) direction = SnapGrid::SnapLeft;
@@ -332,7 +354,7 @@ int SegmentPencil::handleMouseMove(QMouseEvent *e)
 
     m_canvas->slotUpdate();
 
-    return FollowHorizontal;
+    return RosegardenCanvasView::FollowHorizontal;
 }
 
 //////////////////////////////
@@ -373,7 +395,7 @@ void SegmentEraser::handleMouseButtonRelease(QMouseEvent*)
 
 int SegmentEraser::handleMouseMove(QMouseEvent*)
 {
-    return NoFollow;
+    return RosegardenCanvasView::NoFollow;
 }
 
 //////////////////////////////
@@ -385,14 +407,14 @@ SegmentMover::SegmentMover(SegmentCanvas *c, RosegardenGUIDoc *d)
     m_foreGuide(new QCanvasRectangle(m_canvas->canvas())),
     m_topGuide(new QCanvasRectangle(m_canvas->canvas()))
 {
-    m_foreGuide->setPen(RosegardenGUIColours::MovementGuide);
-    m_foreGuide->setBrush(RosegardenGUIColours::MovementGuide);
+    m_foreGuide->setPen(GUIPalette::getColour(GUIPalette::MovementGuide));
+    m_foreGuide->setBrush(GUIPalette::getColour(GUIPalette::MovementGuide));
     m_foreGuide->setSize(1, m_canvas->canvas()->height());
     m_foreGuide->setZ(10);
     m_foreGuide->hide();
 
-    m_topGuide->setPen(RosegardenGUIColours::MovementGuide);
-    m_topGuide->setBrush(RosegardenGUIColours::MovementGuide);
+    m_topGuide->setPen(GUIPalette::getColour(GUIPalette::MovementGuide));
+    m_topGuide->setBrush(GUIPalette::getColour(GUIPalette::MovementGuide));
     m_topGuide->setSize(m_canvas->canvas()->width(), 1);
     m_topGuide->setZ(10);
     m_topGuide->hide();
@@ -403,8 +425,26 @@ SegmentMover::SegmentMover(SegmentCanvas *c, RosegardenGUIDoc *d)
 void SegmentMover::ready()
 {
     m_canvas->viewport()->setCursor(Qt::sizeAllCursor);
+    connect(m_canvas, SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotCanvasScrolled(int, int)));
+
 }
 
+void SegmentMover::stow()
+{
+    disconnect(m_canvas, SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotCanvasScrolled(int, int)));
+}
+
+void SegmentMover::slotCanvasScrolled(int newX, int newY)
+{
+    QMouseEvent tmpEvent(QEvent::MouseMove,
+                         m_canvas->viewport()->mapFromGlobal(QCursor::pos()) + QPoint(newX, newY),
+                         Qt::NoButton, Qt::NoButton);
+    handleMouseMove(&tmpEvent);
+}
+
+
 void SegmentMover::handleMouseButtonPress(QMouseEvent *e)
 {
     QWMatrix matrix = m_canvas->worldMatrix().invert();
@@ -414,6 +454,14 @@ void SegmentMover::handleMouseButtonPress(QMouseEvent *e)
     SegmentSelector* selector = dynamic_cast<SegmentSelector*>
             (getToolBox()->getTool("segmentselector"));
 
+    // #1027303: Segment move issue
+    // Clear selection if we're clicking on an item that's not in it
+    // and we're not in add mode
+    if (selector && item &&
+	!item->isSelected() && !selector->isSegmentAdding()) {
+	selector->clearSelected();
+    }
+
     if (item) {
 
         m_currentItem = item;
@@ -436,10 +484,9 @@ void SegmentMover::handleMouseButtonPress(QMouseEvent *e)
         }
 
 	m_passedInertiaEdge = false;
-        // Don't update until the move
-        //
-        //m_canvas->canvas()->update();
+
     } else {
+
         // check for addmode - clear the selection if not
         selector->clearSelected();
     }
@@ -450,13 +497,6 @@ void SegmentMover::handleMouseButtonRelease(QMouseEvent*)
 {
     if (m_currentItem)
     {
-        /*
-        Rosegarden::Composition &comp = m_doc->getComposition();
-
-        Rosegarden::Track *track = comp.getTrackByPosition(
-                m_currentItem->getTrackPosition());
-                */
-
 	bool haveChange = false;
 
         SegmentSelector* 
@@ -503,16 +543,15 @@ void SegmentMover::handleMouseButtonRelease(QMouseEvent*)
 	}
 
         addCommandToHistory(command);
-        //m_currentItem->showRepeatRect(true);
 
         m_foreGuide->hide();
         m_topGuide->hide();
+        m_canvas->hideTextFloat();
 
         selector->clearSelected();
         for (SegmentSelection::const_iterator nIt = newSelection.begin();
                 nIt != newSelection.end(); ++nIt)
         {
-            selector->addToSelection(*nIt);
             selector->slotSelectSegmentItem(m_canvas->getSegmentItem(*nIt));
         }
     }
@@ -522,69 +561,6 @@ void SegmentMover::handleMouseButtonRelease(QMouseEvent*)
 
 int SegmentMover::handleMouseMove(QMouseEvent *e)
 {
-    /*
-    if (m_currentItem) {
-
-	m_canvas->setSnapGrain(true);
-
-        QWMatrix matrix = m_canvas->worldMatrix().invert();
-        QPoint tPos = matrix.map(e->pos());
-
-        // prevent item from being dragged out of screen.
-        //
-	int newX = tPos.x() - m_clickPoint.x() + int(m_currentItemStartX);
-
-        RG_DEBUG << "SegmentMover::handleMouseMove : newX = "
-                 << newX
-                 << " - canvas width : " << m_canvas->canvas()->width()
-                 << endl;
-
-        if ((newX + m_currentItem->width()) < 20) {
-            newX = 20 - m_currentItem->width();
-        }
-        
-        if (newX >= m_canvas->canvas()->width() - 20)
-            newX = m_canvas->canvas()->width() - 20;
-
-        int newY = tPos.y();
-        if (newY < 0) newY = 0;
-        if ((newY + m_currentItem->height()) >= m_canvas->canvas()->height())
-            newY = m_canvas->canvas()->height() - m_currentItem->height();
-
-	timeT newStartTime = m_canvas->grid().snapX(newX);
-
-        // We shouldn't have the initial ratio reset around these modifiers
-        // otherwise the rulers refresh to the wrong Ratio
-        //
-	m_currentItem->setEndTime(m_currentItem->getEndTime() + newStartTime -
-				  m_currentItem->getStartTime());
-	m_currentItem->setStartTime(newStartTime);
-
-	TrackId track = m_canvas->grid().getYBin(newY);
-
-        int nbTracks = m_doc->getComposition().getNbTracks();
-
-        if (track >= ((unsigned int)nbTracks)) {
-            // Make sure the item isn't dragged to below the last track
-            track = nbTracks - 1;
-        }
-
-        // Don't use proper TrackPosition yet - just visual position
-        // until the release.
-        //
-        m_currentItem->setTrackPosition(track);
-
-        m_foreGuide->setX(int(m_canvas->grid().getRulerScale()->
-                            getXForTime(newStartTime)) - 2);
-
-        m_topGuide->setY(m_canvas->grid().getYBinCoordinate(track));
-
-        m_canvas->canvas()->update();
-
-	return FollowHorizontal | FollowVertical;
-    }
-    */
-
     m_canvas->setSnapGrain(true);
     QWMatrix matrix = m_canvas->worldMatrix().invert();
     QPoint tPos = matrix.map(e->pos());
@@ -602,7 +578,7 @@ int SegmentMover::handleMouseMove(QMouseEvent *e)
         if (selector)
             m_selectedItems = selector->getSegmentItemList();
         else
-            return NoFollow;
+            return RosegardenCanvasView::NoFollow;
 	
 	for (it = m_selectedItems->begin();
 	     it != m_selectedItems->end();
@@ -630,6 +606,7 @@ int SegmentMover::handleMouseMove(QMouseEvent *e)
 
 	    TrackId track;
             int newY=it->first.y() + y;
+
             // Make sure we don't set a non-existing track
             if (newY < 0) { newY = 0; }
             track = m_canvas->grid().getYBin(newY);
@@ -647,25 +624,6 @@ int SegmentMover::handleMouseMove(QMouseEvent *e)
             if (track >= TrackId(m_doc->getComposition().getNbTracks()))
                 track  = TrackId(m_doc->getComposition().getNbTracks() - 1);
 
-            /*
-            if (it == m_selectedItems->begin())
-            {
-                guideX = int(m_canvas->grid().getRulerScale()->
-                    getXForTime(newStartTime));
-
-                guideY = m_canvas->grid().getYBinCoordinate(track);
-            }
-            else
-            {
-                if (x < guideX)
-                    guideX = int(m_canvas->grid().getRulerScale()->
-                        getXForTime(newStartTime));
-
-                if (y < guideY)
-                    guideY = m_canvas->grid().getYBinCoordinate(track);
-            }
-            */
-
             // This is during a "mover" so don't use the normalised (i.e.
             // proper) TrackPosition value yet.
             //
@@ -677,12 +635,28 @@ int SegmentMover::handleMouseMove(QMouseEvent *e)
 
         m_foreGuide->setX(guideX);
         m_topGuide->setY(guideY);
+
+        Rosegarden::Composition &comp = m_doc->getComposition();
+        Rosegarden::RealTime time = 
+            comp.getElapsedRealTime(m_currentItem->getStartTime());
+        QString ms;
+        ms.sprintf("%03d", time.msec());
+
+        int bar, beat, fraction, remainder;
+        comp.getMusicalTimeForAbsoluteTime(
+                m_currentItem->getStartTime(), bar, beat, fraction, remainder);
+
+        QString posString = QString("%1.%2s (%3, %4, %5)")
+            .arg(time.sec).arg(ms)
+            .arg(bar+1).arg(beat).arg(fraction);
+
+        m_canvas->setTextFloat(guideX + 10, guideY - 30, posString);
 	m_canvas->canvas()->update();
 
-	return FollowHorizontal | FollowVertical;
+	return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
     }
 
-    return NoFollow;
+    return RosegardenCanvasView::NoFollow;
 }
 
 //////////////////////////////
@@ -700,8 +674,26 @@ SegmentResizer::SegmentResizer(SegmentCanvas *c, RosegardenGUIDoc *d,
 void SegmentResizer::ready()
 {
     m_canvas->viewport()->setCursor(Qt::sizeHorCursor);
+    connect(m_canvas, SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotCanvasScrolled(int, int)));
+
 }
 
+void SegmentResizer::stow()
+{
+    disconnect(m_canvas, SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotCanvasScrolled(int, int)));
+}
+
+void SegmentResizer::slotCanvasScrolled(int newX, int newY)
+{
+    QMouseEvent tmpEvent(QEvent::MouseMove,
+                         m_canvas->viewport()->mapFromGlobal(QCursor::pos()) + QPoint(newX, newY),
+                         Qt::NoButton, Qt::NoButton);
+    handleMouseMove(&tmpEvent);
+}
+
+
 void SegmentResizer::handleMouseButtonPress(QMouseEvent *e)
 {
     RG_DEBUG << "SegmentResizer::handleMouseButtonPress" << endl;
@@ -719,31 +711,56 @@ void SegmentResizer::handleMouseButtonPress(QMouseEvent *e)
         m_currentItem = item;
 
         if (selector) selector->slotSelectSegmentItem(item);
+
+	// Are we resizing from start or end?
+	if (item->rect().x() + item->rect().width()/2 > tPos.x()) {
+	    m_resizeStart = true;
+	} else {
+	    m_resizeStart = false;
+	}
     }
+
+    m_previewSuspended = false;
 }
 
 void SegmentResizer::handleMouseButtonRelease(QMouseEvent*)
 {
     if (!m_currentItem) return;
-    m_currentItem->normalize();
 
-    // normalisation may mean start time has changed as well as duration
-    SegmentReconfigureCommand *command =
-                new SegmentReconfigureCommand("Resize Segment");
+    if (m_resizeStart &&
+	(m_currentItem->getStartTime() < m_currentItem->getEndTime())) {
 
-    Rosegarden::Composition &comp = m_doc->getComposition();
-    Rosegarden::Track *track =
-        comp.getTrackByPosition(m_currentItem->getTrackPosition());
+	addCommandToHistory(new SegmentResizeFromStartCommand
+			    (m_currentItem->getSegment(),
+			     m_currentItem->getStartTime()));
 
-    command->addSegment(m_currentItem->getSegment(),
-                        m_currentItem->getStartTime(),
-                        m_currentItem->getEndTime(),
-                        track->getId());
-    addCommandToHistory(command);
+    } else {
 
-    // update preview
-    if (m_currentItem->getPreview())
-        m_currentItem->getPreview()->setPreviewCurrent(false);
+	m_currentItem->normalize();
+
+	// normalisation may mean start time has changed as well as duration
+	SegmentReconfigureCommand *command =
+	    new SegmentReconfigureCommand("Resize Segment");
+	
+	Rosegarden::Composition &comp = m_doc->getComposition();
+	Rosegarden::Track *track =
+	    comp.getTrackByPosition(m_currentItem->getTrackPosition());
+	
+	command->addSegment(m_currentItem->getSegment(),
+			    m_currentItem->getStartTime(),
+			    m_currentItem->getEndTime(),
+			    track->getId());
+	addCommandToHistory(command);
+    }
+
+    if (m_previewSuspended) { 
+	m_currentItem->setShowPreview(true);
+	m_previewSuspended = false;
+    }
+
+    if (m_currentItem->getPreview()) {
+	m_currentItem->getPreview()->clearPreview();
+    }
 
     m_canvas->canvas()->update();
 
@@ -752,7 +769,7 @@ void SegmentResizer::handleMouseButtonRelease(QMouseEvent*)
 
 int SegmentResizer::handleMouseMove(QMouseEvent *e)
 {
-    if (!m_currentItem) return NoFollow;
+    if (!m_currentItem) return RosegardenCanvasView::NoFollow;
 
     // Don't allow Audio segments to resize yet
     //
@@ -761,7 +778,7 @@ int SegmentResizer::handleMouseMove(QMouseEvent *e)
         m_currentItem = 0;
         KMessageBox::information(m_canvas,
                 i18n("You can't yet resize an audio segment!"));
-        return NoFollow;
+        return RosegardenCanvasView::NoFollow;
     }
 
     m_canvas->setSnapGrain(true);
@@ -770,34 +787,65 @@ int SegmentResizer::handleMouseMove(QMouseEvent *e)
     QPoint tPos = matrix.map(e->pos());
 
     timeT time = m_canvas->grid().snapX(tPos.x());
-    timeT duration = time - m_currentItem->getStartTime();
-
     timeT snap = m_canvas->grid().getSnapTime(double(tPos.x()));
-
     if (snap == 0) snap = Note(Note::Shortest).getDuration();
 
-    if ((duration > 0 && duration <  snap) ||
-	(duration < 0 && duration > -snap)) {
-	m_currentItem->setEndTime((duration < 0 ? -snap : snap) +
-				  m_currentItem->getStartTime());
-    } else {
-	m_currentItem->setEndTime(duration +
-				  m_currentItem->getStartTime());
-    }
+    if (m_resizeStart) {
 
-    // update preview
-    if (m_currentItem->getPreview())
-        m_currentItem->getPreview()->setPreviewCurrent(false);
+	timeT duration = m_currentItem->getEndTime() - time;
+
+	if ((duration > 0 && duration <  snap) ||
+	    (duration < 0 && duration > -snap)) {
+	    m_currentItem->setStartTime(m_currentItem->getEndTime() -
+					(duration < 0 ? -snap : snap));
+	} else {
+	    m_currentItem->setStartTime(m_currentItem->getEndTime() -
+					duration);
+	}
+
+	// avoid updating preview, as it will update incorrectly
+	// (moving the events rather than leaving them alone and
+	// truncating if appropriate)
+	if (m_currentItem->getShowPreview()) {
+	    m_previewSuspended = true;
+	    m_currentItem->setShowPreview(false);
+	}
+
+    } else {
+
+	timeT duration = time - m_currentItem->getStartTime();
+
+	if ((duration > 0 && duration <  snap) ||
+	    (duration < 0 && duration > -snap)) {
+	    m_currentItem->setEndTime((duration < 0 ? -snap : snap) +
+				      m_currentItem->getStartTime());
+	} else {
+	    m_currentItem->setEndTime(duration +
+				      m_currentItem->getStartTime());
+	}
+
+	// update preview
+	if (m_currentItem->getPreview())
+	    m_currentItem->getPreview()->setPreviewCurrent(false);
+    }
 
     m_canvas->canvas()->update();
 
-    return FollowHorizontal;
+    return RosegardenCanvasView::FollowHorizontal;
 }
 
 bool SegmentResizer::cursorIsCloseEnoughToEdge(SegmentItem* p, const QPoint &coord,
-					       int edgeThreshold)
+					       int edgeThreshold, bool &start)
 {
-    return (abs(p->rect().x() + p->rect().width() - coord.x()) < edgeThreshold);
+    if (abs(p->rect().x() + p->rect().width() - coord.x()) < edgeThreshold) {
+	start = false;
+	return true;
+    } else if (abs(p->rect().x() - coord.x()) < edgeThreshold) {
+	start = true;
+	return true;
+    } else {
+	return false;
+    }
 }
 
 //////////////////////////////
@@ -815,14 +863,14 @@ SegmentSelector::SegmentSelector(SegmentCanvas *c, RosegardenGUIDoc *d)
 {
     RG_DEBUG << "SegmentSelector()\n";
 
-    m_foreGuide->setPen(RosegardenGUIColours::MovementGuide);
-    m_foreGuide->setBrush(RosegardenGUIColours::MovementGuide);
+    m_foreGuide->setPen(GUIPalette::getColour(GUIPalette::MovementGuide));
+    m_foreGuide->setBrush(GUIPalette::getColour(GUIPalette::MovementGuide));
     m_foreGuide->setSize(1, m_canvas->canvas()->height());
     m_foreGuide->setZ(10);
     m_foreGuide->hide();
 
-    m_topGuide->setPen(RosegardenGUIColours::MovementGuide);
-    m_topGuide->setBrush(RosegardenGUIColours::MovementGuide);
+    m_topGuide->setPen(GUIPalette::getColour(GUIPalette::MovementGuide));
+    m_topGuide->setBrush(GUIPalette::getColour(GUIPalette::MovementGuide));
     m_topGuide->setSize(m_canvas->canvas()->width(), 1);
     m_topGuide->setZ(10);
     m_topGuide->hide();
@@ -830,7 +878,14 @@ SegmentSelector::SegmentSelector(SegmentCanvas *c, RosegardenGUIDoc *d)
 
 SegmentSelector::~SegmentSelector()
 {
-    //delete m_dispatchTool;
+}
+
+void SegmentSelector::ready()
+{
+    m_canvas->viewport()->setCursor(Qt::arrowCursor);
+    connect(m_canvas, SIGNAL(contentsMoving (int, int)),
+            this, SLOT(slotCanvasScrolled(int, int)));
+
 }
 
 void SegmentSelector::stow()
@@ -839,6 +894,17 @@ void SegmentSelector::stow()
     // switch back to the selector tool
     //
     // clearSelected();
+    disconnect(m_canvas, SIGNAL(contentsMoving (int, int)),
+               this, SLOT(slotCanvasScrolled(int, int)));
+}
+
+void SegmentSelector::slotCanvasScrolled(int newX, int newY)
+{
+    if (!m_canvas->getSelectionRectangle()) return;
+    QMouseEvent tmpEvent(QEvent::MouseMove,
+                         m_canvas->viewport()->mapFromGlobal(QCursor::pos()) + QPoint(newX, newY),
+                         Qt::NoButton, Qt::NoButton);
+    handleMouseMove(&tmpEvent);
 }
 
 void
@@ -858,20 +924,20 @@ SegmentSelector::removeFromSelection(Rosegarden::Segment *segment)
     }
 }
 
-void
+bool
 SegmentSelector::addToSelection(Rosegarden::Segment *segment)
 {
     SegmentItem *item = m_canvas->getSegmentItem(segment);
-    if (!item) return;
+    if (!item) return false;
 
-    addToSelection(item); 
+    return addToSelection(item); 
 }
 
-void
+bool
 SegmentSelector::addToSelection(SegmentItem* item)
 {
-    RG_DEBUG << "SegmentSelector::addToSelection() SegmentItem = "
-             << item << endl;
+//    RG_DEBUG << "SegmentSelector::addToSelection() SegmentItem = "
+//             << item << endl;
 
     // Check that the segment isn't already selected
     for (SegmentItemList::iterator i = m_selectedItems.begin();
@@ -879,7 +945,7 @@ SegmentSelector::addToSelection(SegmentItem* item)
 //         RG_DEBUG << "SegmentSelector::addToSelection() SegmentItem already in selection\n";
 	if (i->second == item) {
 	    i->first = QPoint(int(item->x()), int(item->y()));
-	    return;
+	    return false;
 	}
     }
 
@@ -888,6 +954,8 @@ SegmentSelector::addToSelection(SegmentItem* item)
 
     connect(item, SIGNAL(destroyed(QObject*)),
             this, SLOT(slotDestroyedSegmentItem(QObject*)));
+
+    return true;
 }
 
 void
@@ -902,7 +970,7 @@ SegmentSelector::clearSelected()
     {
         it->second->disconnect(this);
         it->second->setSelected(false, 
-          convertColour(m_doc->getComposition().getSegmentColourMap()
+                                GUIPalette::convertColour(m_doc->getComposition().getSegmentColourMap()
                         .getColourByIndex(it->second->getSegment()->getColourIndex())));
     }
 
@@ -935,7 +1003,6 @@ SegmentSelector::handleMouseButtonPress(QMouseEvent *e)
     //
     if ((!m_segmentAddMode && !item) || 
         (!m_segmentAddMode && !(item->isSelected()))) {
-    //if (!item || (!m_segmentAddMode && !(item->isSelected()))) {
         clearSelected();
     }
 
@@ -947,8 +1014,11 @@ SegmentSelector::handleMouseButtonPress(QMouseEvent *e)
         if (threshold  == 0) threshold = 1;
         if (threshold > 10) threshold = 10;
 
+	bool start = false;
+
 	if (!m_segmentAddMode &&
-	    SegmentResizer::cursorIsCloseEnoughToEdge(item, tPos, threshold)) {
+	    SegmentResizer::cursorIsCloseEnoughToEdge(item, tPos, threshold, start)) {
+
             SegmentResizer* resizer = 
                 dynamic_cast<SegmentResizer*>(getToolBox()->
                     getTool(SegmentResizer::ToolName));
@@ -963,6 +1033,7 @@ SegmentSelector::handleMouseButtonPress(QMouseEvent *e)
 
 	    m_dispatchTool = resizer;
             
+	    m_dispatchTool->ready(); // set mouse cursor
 	    m_dispatchTool->handleMouseButtonPress(e);
 	    return;
 	}
@@ -1002,8 +1073,10 @@ SegmentSelector::handleMouseButtonPress(QMouseEvent *e)
 	if (e->button() == MidButton) {
 	    m_dispatchTool =  getToolBox()->getTool(SegmentPencil::ToolName);
 
-            if (m_dispatchTool)
+            if (m_dispatchTool) {
+		m_dispatchTool->ready(); // set mouse cursor
                 m_dispatchTool->handleMouseButtonPress(e);
+	    }
 
 	    return;
 	}
@@ -1058,11 +1131,17 @@ SegmentSelector::slotSelectSegmentItem(SegmentItem *selectedItem)
     // If we're selecting a Segment through this method
     // then don't set the m_currentItem
     //
-    selectedItem->setSelected(true, 
-        convertColour(m_doc->getComposition().getSegmentColourMap()
-        .getColourByIndex(selectedItem->getSegment()->getColourIndex())).dark(200));
-    addToSelection(selectedItem);
-    m_canvas->canvas()->update();
+    if (addToSelection(selectedItem)) {
+
+	selectedItem->setSelected
+	    (true, 
+	     GUIPalette::convertColour
+	     (m_doc->getComposition().getSegmentColourMap()
+	      .getColourByIndex(selectedItem->getSegment()->getColourIndex()))
+	     .dark(200));
+	
+	m_canvas->canvas()->update();
+    }
 }
 
 void
@@ -1098,11 +1177,11 @@ SegmentSelector::handleMouseButtonRelease(QMouseEvent *e)
     //
     m_foreGuide->hide();
     m_topGuide->hide();
+    m_canvas->hideTextFloat();
     m_canvas->canvas()->update();
 
     if (m_dispatchTool) {
 	m_dispatchTool->handleMouseButtonRelease(e);
-	//delete m_dispatchTool;
 	m_dispatchTool = 0;
 	m_canvas->viewport()->setCursor(Qt::arrowCursor);
 	return;
@@ -1146,11 +1225,6 @@ SegmentSelector::handleMouseButtonRelease(QMouseEvent *e)
 
 	    SegmentItem *item = it->second;
 
-/*
-            if (item->getSegment()->isRepeating())
-                item->showRepeatRect(true);
-*/
-
             Rosegarden::Composition &comp = m_doc->getComposition();
             Rosegarden::Track *track = 
                 comp.getTrackByPosition(item->getTrackPosition());
@@ -1183,7 +1257,6 @@ SegmentSelector::handleMouseButtonRelease(QMouseEvent *e)
         for (SegmentSelection::const_iterator nIt = newSelection.begin();
                 nIt != newSelection.end(); ++nIt)
         {
-            addToSelection(*nIt);
             slotSelectSegmentItem(m_canvas->getSegmentItem(*nIt));
         }
 
@@ -1212,12 +1285,18 @@ SegmentSelector::handleMouseMove(QMouseEvent *e)
 	return m_dispatchTool->handleMouseMove(e);
     }
 
+    if (!m_currentItem && 
+	(!m_canvas->getSelectionRectangle() ||
+	 !m_canvas->getSelectionRectangle()->visible())) {
+	return RosegardenCanvasView::NoFollow;
+    }
+
     QWMatrix matrix = m_canvas->worldMatrix().invert();
     QPoint tPos = matrix.map(e->pos());
 
     if (!m_currentItem)  {
 
-	RG_DEBUG << "SegmentSelector::handleMouseMove: no current item" << endl;
+// 	RG_DEBUG << "SegmentSelector::handleMouseMove: no current item\n";
 
         // do a bounding box
         QCanvasRectangle *selectionRect  = m_canvas->getSelectionRectangle();
@@ -1231,82 +1310,10 @@ SegmentSelector::handleMouseMove(QMouseEvent *e)
             if (w > 0) ++w; else --w;
             if (h > 0) ++h; else --h;
 
-            // Translate these points
-            //
-            selectionRect->setSize(w, h);
+            updateSelectionRect(w, h);
 
-	    m_canvas->canvas()->update();
-
-            // Get collisions and do selection (true for exact collisions)
-            //
-            QCanvasItemList l = selectionRect->collisions(true);
-
-            // selection management
-            SegmentSelection oldSelection = getSelectedSegments();
-            SegmentSelection newSelection;
-
-            int segCount = 0;
-
-            if (l.count()) {
-                for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it)
-                {
-                    if (SegmentItem *item = dynamic_cast<SegmentItem*>(*it))
-                    {
-                        if (m_segmentAddMode)
-                        {
-                            slotSelectSegmentItem(item);
-                            addToSelection(item);
-                        }
-                        else
-                        {
-                            segCount++;
-                            slotSelectSegmentItem(item);
-                            newSelection.insert(item->getSegment());
-                        }
-                    }
-                }
-            }
-
-            if (m_segmentAddMode)
-            { 
-                emit selectedSegments(getSelectedSegments());
-                return FollowHorizontal | FollowVertical;
-            }
-
-            // Check for unselected items with this piece of crap
-            //
-            bool found = false;
-
-            for (SegmentSelection::const_iterator oIt = oldSelection.begin();
-                  oIt != oldSelection.end(); oIt++)
-            {
-                found = false;
-                for (SegmentSelection::const_iterator nIt = newSelection.begin();
-                     nIt != newSelection.end(); nIt++)
-                {
-                    if (*oIt == *nIt)
-                    {
-                        found = true;
-                        break;
-                    }
-                }
-                if (found == false)
-                {
-                    removeFromSelection(*oIt);
-                    m_canvas->getSegmentItem(*oIt)->
-                        setSelected(false, convertColour(
-                            m_doc->getComposition().getSegmentColourMap().
-                            getColourByIndex(m_canvas->getSegmentItem(*oIt)
-                            ->getSegment()->getColourIndex())));
-                }
-            }
-
-            if (segCount)
-            {
-                emit selectedSegments(getSelectedSegments());
-            }
         }
-        return FollowHorizontal | FollowVertical;
+        return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
     }
 
     m_canvas->viewport()->setCursor(Qt::sizeAllCursor);
@@ -1395,31 +1402,10 @@ SegmentSelector::handleMouseMove(QMouseEvent *e)
             if (track >= TrackId(m_doc->getComposition().getNbTracks()))
                 track  = TrackId(m_doc->getComposition().getNbTracks() - 1);
 
-            /*
-            if (it == m_selectedItems.begin())
-            {
-                guideX = int(m_canvas->grid().getRulerScale()->
-                    getXForTime(newStartTime));
-
-                guideY = m_canvas->grid().getYBinCoordinate(track);
-            }
-            else
-            {
-                if (x < guideX)
-                    guideX = int(m_canvas->grid().getRulerScale()->
-                        getXForTime(newStartTime));
-
-                if (y < guideY)
-                    guideY = m_canvas->grid().getYBinCoordinate(track);
-            }
-            */
-
-
             // This is during a "mover" so don't use the normalised (i.e.
             // proper) TrackPosition value yet.
             //
 	    it->second->setTrackPosition(track);
-
 	}
 
         guideX = int(m_currentItem->x());
@@ -1428,12 +1414,96 @@ SegmentSelector::handleMouseMove(QMouseEvent *e)
         m_foreGuide->setX(guideX);
         m_topGuide->setY(guideY);
 
+        Rosegarden::Composition &comp = m_doc->getComposition();
+        Rosegarden::RealTime time = 
+            comp.getElapsedRealTime(m_currentItem->getStartTime());
+        QString ms;
+        ms.sprintf("%03d", time.msec());
+
+        int bar, beat, fraction, remainder;
+        comp.getMusicalTimeForAbsoluteTime(
+                m_currentItem->getStartTime(), bar, beat, fraction, remainder);
+
+        QString posString = QString("%1.%2s (%3, %4, %5)")
+            .arg(time.sec).arg(ms)
+            .arg(bar+1).arg(beat).arg(fraction);
+
+        m_canvas->setTextFloat(guideX + 10, guideY - 30, posString);
 	m_canvas->canvas()->update();
     }
 
-    return FollowHorizontal | FollowVertical;
+    return RosegardenCanvasView::FollowHorizontal | RosegardenCanvasView::FollowVertical;
 }
 
+void SegmentSelector::updateSelectionRect(int w, int h) 
+{
+    QCanvasRectangle *selectionRect  = m_canvas->getSelectionRectangle();
+    if (!selectionRect) return;
+
+    selectionRect->setSize(w, h);
+
+    m_canvas->canvas()->update();
+
+    // Get collisions and do selection (true for exact collisions)
+    //
+    QCanvasItemList l = selectionRect->collisions(true);
+
+    // selection management
+    SegmentSelection oldSelection = getSelectedSegments();
+    SegmentSelection newSelection;
+
+    int segCount = 0;
+
+    if (l.count()) {
+        for (QCanvasItemList::Iterator it=l.begin(); it!=l.end(); ++it) {
+            if (SegmentItem *item = dynamic_cast<SegmentItem*>(*it)) {
+                if (m_segmentAddMode) {
+                    slotSelectSegmentItem(item);
+                } else {
+                    segCount++;
+                    slotSelectSegmentItem(item);
+                    newSelection.insert(item->getSegment());
+                }
+            }
+        }
+    }
+
+    if (m_segmentAddMode) { 
+        emit selectedSegments(getSelectedSegments());
+    } else {
+
+        // Check for unselected items with this piece of crap
+        //
+        bool found = false;
+
+        for (SegmentSelection::const_iterator oIt = oldSelection.begin();
+             oIt != oldSelection.end(); oIt++) {
+            found = false;
+            for (SegmentSelection::const_iterator nIt = newSelection.begin();
+                 nIt != newSelection.end(); nIt++) {
+                if (*oIt == *nIt) {
+                    found = true;
+                    break;
+                }
+            }
+            if (found == false) {
+                removeFromSelection(*oIt);
+                m_canvas->getSegmentItem(*oIt)->
+                    setSelected(false, GUIPalette::convertColour(m_doc->getComposition().getSegmentColourMap().
+                                                                 getColourByIndex(m_canvas->getSegmentItem(*oIt)
+                                                                                  ->getSegment()->getColourIndex())));
+            }
+        }
+
+        if (segCount) {
+            emit selectedSegments(getSelectedSegments());
+        }
+    }
+            
+    
+}
+
+
 //////////////////////////////
 //
 // SegmentSplitter
@@ -1525,13 +1595,13 @@ SegmentSplitter::handleMouseMove(QMouseEvent *e)
     {
         m_canvas->viewport()->setCursor(Qt::blankCursor);
         drawSplitLine(e);
-	return FollowHorizontal;
+	return RosegardenCanvasView::FollowHorizontal;
     }
     else
     {
         m_canvas->viewport()->setCursor(Qt::splitHCursor);
         m_canvas->slotHideSplitLine();
-	return NoFollow;
+	return RosegardenCanvasView::NoFollow;
     }
 }
 
@@ -1595,7 +1665,7 @@ SegmentJoiner::handleMouseButtonRelease(QMouseEvent*)
 int
 SegmentJoiner::handleMouseMove(QMouseEvent*)
 {
-    return NoFollow;
+    return RosegardenCanvasView::NoFollow;
 }
 
 void
