@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -25,19 +25,23 @@
 #include "rosedebug.h"
 #include "rosegardendcop.h"
 
+#include "Profiler.h"
+
 namespace Rosegarden
 {
 
 MappedObjectId
 StudioControl::createStudioObject(MappedObject::MappedObjectType type)
 {
-    Rosegarden::MappedObjectId value = -1;
+    Rosegarden::Profiler profiler("StudioControl::createStudioObject", true);
+
+    int value = -1;
     QByteArray data;
     QCString replyType;
     QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << type;
+    streamOut << (int)type;
 
     if (!rgapp->sequencerCall("createMappedObject(int)",
                               replyType, replyData, data))
@@ -58,6 +62,8 @@ StudioControl::createStudioObject(MappedObject::MappedObjectType type)
 bool
 StudioControl::destroyStudioObject(MappedObjectId id)
 {
+    Rosegarden::Profiler profiler("StudioControl::destroyStudioObject", true);
+
     int value = 0;
     QByteArray data;
     QCString replyType;
@@ -89,6 +95,8 @@ MappedObjectPropertyList
 StudioControl::getStudioObjectProperty(MappedObjectId id,
                         const MappedObjectProperty &property)
 {
+    Rosegarden::Profiler profiler("StudioControl::getStudioObjectProperty", true);
+
     MappedObjectPropertyList list;
 
     QByteArray data;
@@ -96,7 +104,7 @@ StudioControl::getStudioObjectProperty(MappedObjectId id,
     QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << id;
+    streamOut << (int)id;
     streamOut << QString(property);
 
     if (!rgapp->sequencerCall("getPropertyList(int, QString)",
@@ -120,14 +128,14 @@ StudioControl::setStudioObjectProperty(MappedObjectId id,
                         const MappedObjectProperty &property,
                         MappedObjectValue value)
 {
+    Rosegarden::Profiler profiler("StudioControl::setStudioObjectProperty(float)", true);
+
     QByteArray data;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    // Use new MappedEvent interface
-    //
-    streamOut << id;
-    streamOut << property;
-    streamOut << value;
+    streamOut << (int)id;
+    streamOut << (QString)property;
+    streamOut << (float)value;
 
     rgapp->sequencerSend("setMappedProperty(int, QString, float)", data);
 
@@ -135,20 +143,60 @@ StudioControl::setStudioObjectProperty(MappedObjectId id,
 }
 
 bool
-StudioControl::setStudioObjectProperty(MappedObjectId id,
-                        const MappedObjectProperty &property,
-                        MappedObjectValueList value)
+StudioControl::setStudioObjectProperties(const MappedObjectIdList &ids,
+					 const MappedObjectPropertyList &properties,
+					 const MappedObjectValueList &values)
 {
+    Rosegarden::Profiler profiler("StudioControl::setStudioObjectProperties(floats)", true);
+
     QByteArray data;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    // Use new MappedEvent interface
-    //
-    streamOut << id;
-    streamOut << property;
-    streamOut << value;
+    streamOut << ids;
+    streamOut << properties;
+    streamOut << values;
 
-    rgapp->sequencerSend("setMappedProperty(int, QString, std::vector<float>)",
+    rgapp->sequencerSend("setMappedProperties(Rosegarden::MappedObjectIdList, Rosegarden::MappedObjectPropertyList, Rosegarden::MappedObjectValueList)", data);
+
+    return true;
+}
+
+bool
+StudioControl::setStudioObjectProperty(MappedObjectId id,
+                        const MappedObjectProperty &property,
+                        const QString &value)
+{
+    Rosegarden::Profiler profiler("StudioControl::setStudioObjectProperty(string)", true);
+
+    QByteArray data;
+    QDataStream streamOut(data, IO_WriteOnly);
+
+    streamOut << (int)id;
+    streamOut << (QString)property;
+    streamOut << (QString)value;
+
+    rgapp->sequencerSend("setMappedProperty(int, QString, QString)", data);
+
+    return true;
+}
+
+bool
+StudioControl::setStudioObjectPropertyList(MappedObjectId id,
+                        const MappedObjectProperty &property,
+                        const MappedObjectPropertyList &values)
+{
+    Rosegarden::Profiler profiler("StudioControl::setStudioObjectPropertyList", true);
+
+    QByteArray data;
+    QDataStream streamOut(data, IO_WriteOnly);
+
+    streamOut << (int)id;
+    streamOut << (QString)property;
+    streamOut << values;
+
+    RG_DEBUG << "StudioControl::setStudioObjectPropertyList: " << values.size() << " values for property " << property << endl;
+
+    rgapp->sequencerSend("setMappedPropertyList(int, QString, Rosegarden::MappedObjectPropertyList)",
                          data);
     
     return true;
@@ -157,13 +205,15 @@ StudioControl::setStudioObjectProperty(MappedObjectId id,
 MappedObjectId
 StudioControl::getStudioObjectByType(MappedObject::MappedObjectType type)
 {
-    Rosegarden::MappedObjectId value = -1;
+    Rosegarden::Profiler profiler("StudioControl::getStudioObjectByType", true);
+
+    int value = -1;
     QByteArray data;
     QCString replyType;
     QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << type;
+    streamOut << (int)type;
 
     if (!rgapp->sequencerCall("getMappedObjectId(int)",
                               replyType, replyData, data))
@@ -186,33 +236,152 @@ StudioControl::setStudioPluginPort(MappedObjectId pluginId,
                                    unsigned long portId,
                                    MappedObjectValue value)
 {
+    Rosegarden::Profiler profiler("StudioControl::setStudioPluginPort", true);
+
     QByteArray data;
     QDataStream streamOut(data, IO_WriteOnly);
 
     // Use new MappedEvent interface
     //
-    streamOut << pluginId;
-    streamOut << portId;
-    streamOut << value;
+    streamOut << (int)pluginId;
+    streamOut << (unsigned long)portId;
+    streamOut << (float)value;
 
     rgapp->sequencerSend("setMappedPort(int, unsigned long int, float)", data);
 }
 
+MappedObjectValue
+StudioControl::getStudioPluginPort(MappedObjectId pluginId,
+                                   unsigned long portId)
+{
+    Rosegarden::Profiler profiler("StudioControl::getStudioPluginPort", true);
+
+    float value = 0.0;
+    QByteArray data;
+    QCString replyType;
+    QByteArray replyData;
+    QDataStream streamOut(data, IO_WriteOnly);
+
+    streamOut << (int)pluginId;
+    streamOut << (unsigned long)portId;
+
+    if (!rgapp->sequencerCall("getMappedPort(int, unsigned long int)",
+                              replyType, replyData, data))
+    {
+        SEQMAN_DEBUG << "getStudioPluginPort - "
+                     << "failed to contact Rosegarden sequencer"
+                     << endl;
+    }
+    else
+    {
+        QDataStream streamIn(replyData, IO_ReadOnly);
+        streamIn >> value;
+    }
+
+    return value;
+}
+
+
+MappedObjectPropertyList
+StudioControl::getPluginInformation()
+{
+    MappedObjectPropertyList list;
+
+    QByteArray data;
+    QCString replyType;
+    QByteArray replyData;
+
+    if (!rgapp->sequencerCall("getPluginInformation()",
+                              replyType, replyData, data))
+    {
+        SEQMAN_DEBUG << "getPluginInformation - "
+                     << "failed to contact Rosegarden sequencer"
+                     << endl;
+    }
+    else
+    {
+        QDataStream streamIn(replyData, IO_ReadOnly);
+        streamIn >> list;
+    }
+
+    return list;
+}
+
+
+QString
+StudioControl::getPluginProgram(MappedObjectId id, int bank, int program)
+{
+    QByteArray data;
+    QCString replyType;
+    QByteArray replyData;
+    QDataStream streamOut(data, IO_WriteOnly);
+
+    streamOut << (int)id;
+    streamOut << (int)bank;
+    streamOut << (int)program;
+
+    QString programName;
+
+    if (!rgapp->sequencerCall("getPluginProgram(int, int, int)",
+                              replyType, replyData, data))
+    {
+        SEQMAN_DEBUG << "getPluginProgram - "
+                     << "failed to contact Rosegarden sequencer"
+                     << endl;
+    }
+    else
+    {
+	QDataStream streamIn(replyData, IO_ReadOnly);
+	streamIn >> programName;
+    }
+
+    return programName;
+}
+
+
+unsigned long
+StudioControl::getPluginProgram(MappedObjectId id, QString name)
+{
+    QByteArray data;
+    QCString replyType;
+    QByteArray replyData;
+    QDataStream streamOut(data, IO_WriteOnly);
+
+    streamOut << (int)id;
+    streamOut << name;
+
+    unsigned long rv;
+
+    if (!rgapp->sequencerCall("getPluginProgram(int, QString)",
+                              replyType, replyData, data))
+    {
+        SEQMAN_DEBUG << "getPluginProgram - "
+                     << "failed to contact Rosegarden sequencer"
+                     << endl;
+    }
+    else
+    {
+	QDataStream streamIn(replyData, IO_ReadOnly);
+	streamIn >> rv;
+    }
+
+    return rv;
+}
+
 
 void
 StudioControl::connectStudioObjects(MappedObjectId id1,
 				    MappedObjectId id2)
 {
+    Rosegarden::Profiler profiler("StudioControl::connectStudioObjects", true);
+
     QByteArray data;
-    QCString replyType;
-    QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << id1;
-    streamOut << id2;
+    streamOut << (int)id1;
+    streamOut << (int)id2;
 
-    if (!rgapp->sequencerCall("connectMappedObjects(int, int)",
-                              replyType, replyData, data))
+    if (!rgapp->sequencerSend("connectMappedObjects(int, int)", data))
     {
         SEQMAN_DEBUG << "connectStudioObjects - "
                      << "failed to contact Rosegarden sequencer"
@@ -226,16 +395,15 @@ void
 StudioControl::disconnectStudioObjects(MappedObjectId id1,
 				       MappedObjectId id2)
 {
+    Rosegarden::Profiler profiler("StudioControl::disconnectStudioObjects", true);
+
     QByteArray data;
-    QCString replyType;
-    QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << id1;
-    streamOut << id2;
+    streamOut << (int)id1;
+    streamOut << (int)id2;
 
-    if (!rgapp->sequencerCall("disconnectMappedObjects(int, int)",
-                              replyType, replyData, data))
+    if (!rgapp->sequencerSend("disconnectMappedObjects(int, int)", data))
     {
         SEQMAN_DEBUG << "disconnectStudioObjects - "
                      << "failed to contact Rosegarden sequencer"
@@ -248,15 +416,14 @@ StudioControl::disconnectStudioObjects(MappedObjectId id1,
 void
 StudioControl::disconnectStudioObject(MappedObjectId id)
 {
+    Rosegarden::Profiler profiler("StudioControl::disconnectStudioObject", true);
+
     QByteArray data;
-    QCString replyType;
-    QByteArray replyData;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << id;
+    streamOut << (int)id;
 
-    if (!rgapp->sequencerCall("disconnectMappedObject(int)",
-                              replyType, replyData, data))
+    if (!rgapp->sequencerSend("disconnectMappedObject(int)", data))
     {
         SEQMAN_DEBUG << "disconnectStudioObject - "
                      << "failed to contact Rosegarden sequencer"
@@ -269,6 +436,8 @@ StudioControl::disconnectStudioObject(MappedObjectId id)
 void
 StudioControl::sendMappedEvent(const Rosegarden::MappedEvent &mE)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendMappedEvent", true);
+
     static Rosegarden::MappedEvent mEs;
     
     mEs = mE; // just in case the passed mapped event has dubious
@@ -284,13 +453,15 @@ StudioControl::sendMappedEvent(const Rosegarden::MappedEvent &mE)
 void
 StudioControl::sendMappedComposition(const Rosegarden::MappedComposition &mC)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendMappedComposition", true);
+
     if (mC.size() == 0)
         return;
 
     QCString replyType;
     QByteArray replyData;
 
-    MappedComposition::iterator it = mC.begin();
+    MappedComposition::const_iterator it = mC.begin();
 
     for (; it != mC.end(); it++)
     {
@@ -305,12 +476,14 @@ StudioControl::sendMappedComposition(const Rosegarden::MappedComposition &mC)
 void
 StudioControl::sendMappedInstrument(const Rosegarden::MappedInstrument &mI)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendMappedInstrument", true);
+
     QByteArray data;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << mI.getType();
-    streamOut << mI.getChannel();
-    streamOut << mI.getId();
+    streamOut << (int)mI.getType();
+    streamOut << (unsigned char)mI.getChannel();
+    streamOut << (unsigned int)mI.getId();
 
     rgapp->sequencerSend("setMappedInstrument(int, unsigned char, unsigned int)", data);
 }
@@ -319,11 +492,13 @@ StudioControl::sendMappedInstrument(const Rosegarden::MappedInstrument &mI)
 void
 StudioControl::sendQuarterNoteLength(const Rosegarden::RealTime &length)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendQuarterNoteLength", true);
+
     QByteArray data;
     QDataStream streamOut(data, IO_WriteOnly);
 
-    streamOut << length.sec;
-    streamOut << length.nsec;
+    streamOut << (long)length.sec;
+    streamOut << (long)length.nsec;
 
     rgapp->sequencerSend("setQuarterNoteLength(long int, long int)", data);
 }
@@ -336,6 +511,8 @@ StudioControl::sendRPN(Rosegarden::InstrumentId instrumentId,
                        Rosegarden::MidiByte /* controller */,
                        Rosegarden::MidiByte value)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendRPN", true);
+
     Rosegarden::MappedComposition mC;
     Rosegarden::MappedEvent *mE =
         new Rosegarden::MappedEvent(instrumentId,
@@ -382,6 +559,8 @@ StudioControl::sendNRPN(Rosegarden::InstrumentId instrumentId,
                         Rosegarden::MidiByte /* controller */,
                         Rosegarden::MidiByte value)
 {
+    Rosegarden::Profiler profiler("StudioControl::sendNRPN", true);
+
     Rosegarden::MappedComposition mC;
     Rosegarden::MappedEvent *mE =
         new Rosegarden::MappedEvent(instrumentId,
@@ -419,5 +598,5 @@ StudioControl::sendNRPN(Rosegarden::InstrumentId instrumentId,
 }
 
 
-};
+}
 
