@@ -3,7 +3,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -58,18 +58,44 @@ public:
                                         const MappedObjectProperty &property,
                                         MappedObjectValue value);
 
-    // Set a value to a list
+    // Set many values to values
+    //
+    static bool setStudioObjectProperties(const MappedObjectIdList &ids,
+					  const MappedObjectPropertyList &properties,
+					  const MappedObjectValueList &values);
+
+    // Set a value to a string 
     //
     static bool setStudioObjectProperty(MappedObjectId id,
                                         const MappedObjectProperty &property,
-                                        MappedObjectValueList value);
+					const QString &value);
 
-    // cheat so we can avoid making calls() during playback
+    // Set a value to a string list
     //
+    static bool setStudioObjectPropertyList(MappedObjectId id,
+					    const MappedObjectProperty &property,
+					    const MappedObjectPropertyList &values);
+
     static void setStudioPluginPort(MappedObjectId pluginId,
                                     unsigned long portId,
                                     MappedObjectValue value);
 
+    static MappedObjectValue getStudioPluginPort(MappedObjectId pluginId,
+						 unsigned long portId);
+
+    // Get all plugin information
+    //
+    static MappedObjectPropertyList getPluginInformation();
+
+    // Get program name for a given program
+    // 
+    static QString getPluginProgram(MappedObjectId, int bank, int program);
+
+    // Get program numbers for a given name (rv is bank << 16 + program)
+    // This is one of the nastiest hacks in the whole application
+    // 
+    static unsigned long getPluginProgram(MappedObjectId, QString name);
+
     // Connection
     //
     static void connectStudioObjects(MappedObjectId id1,
