@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -19,6 +19,8 @@
     COPYING included with this distribution for more information.
 */
 
+#include "config.h"
+
 #include <kapp.h>
 #include <klocale.h>
 #include <kstddirs.h>
@@ -40,6 +42,8 @@
 #include "Studio.h"
 #include "MappedStudio.h"
 #include "studiocontrol.h"
+#include "rosegardengui.h"
+#include "audiopluginoscgui.h"
 
 #include <cmath>
 
@@ -76,6 +80,8 @@ RosegardenFader::RosegardenFader(Rosegarden::AudioLevel::FaderType type,
 	m_sliderMax = width() - m_sliderMin;
     }	
 
+    m_outlineColour = colorGroup().mid();
+
     calculateGroovePixmap();
     setFader(0.0);
 
@@ -111,6 +117,8 @@ RosegardenFader::RosegardenFader(int min, int max, int deflt,
 	m_sliderMax = width() - m_sliderMin;
     }	
 
+    m_outlineColour = colorGroup().mid();
+
     calculateGroovePixmap();
     setFader(deflt);
 
@@ -140,6 +148,8 @@ RosegardenFader::RosegardenFader(int min, int max, int deflt,
 	m_sliderMax = width() - m_sliderMin;
     }	
 
+    m_outlineColour = colorGroup().mid();
+
     calculateGroovePixmap();
     setFader(deflt);
 
@@ -151,19 +161,33 @@ RosegardenFader::~RosegardenFader()
 {
 }
 
+void
+RosegardenFader::setOutlineColour(QColor c)
+{
+    m_outlineColour = c;
+    calculateGroovePixmap();
+}
+
 QPixmap *
 RosegardenFader::groovePixmap()
 {
     PixmapCache::iterator i = m_pixmapCache.find(SizeRec(width(), height()));
-    if (i != m_pixmapCache.end()) return i->second.first;
-    else return 0;
+    if (i != m_pixmapCache.end()) {
+	ColourPixmapRec::iterator j = i->second.first.find(m_outlineColour.pixel());
+	if (j != i->second.first.end()) {
+	    return j->second;
+	}
+    }
+    return 0;
 }
 
 QPixmap *
 RosegardenFader::buttonPixmap()
 {
     PixmapCache::iterator i = m_pixmapCache.find(SizeRec(width(), height()));
-    if (i != m_pixmapCache.end()) return i->second.second;
+    if (i != m_pixmapCache.end()) {
+	return i->second.second;
+    }
     else return 0;
 }
 
@@ -270,7 +294,7 @@ RosegardenFader::paintEvent(QPaintEvent *)
 			 buttonPixmap()->height());
 
     } else {
-//!!!update
+//... update
 	int leftOfButton =
 	    (m_sliderMax - m_sliderMin) - position - buttonPixmap()->width()/2;
 
@@ -421,11 +445,9 @@ RosegardenFader::slotFloatTimeout()
 void
 RosegardenFader::calculateGroovePixmap()
 {
-    PixmapCache::iterator i = m_pixmapCache.find(SizeRec(width(), height()));
-    if (i != m_pixmapCache.end() && i->second.first) return;
-    
-    QPixmap *& map = m_pixmapCache[SizeRec(width(), height())].first;
+    QPixmap *& map = m_pixmapCache[SizeRec(width(), height())].first[m_outlineColour.pixel()];
 
+    delete map;
     map = new QPixmap(width(), height());
     map->fill(colorGroup().background());
     QPainter paint(map);
@@ -433,11 +455,11 @@ RosegardenFader::calculateGroovePixmap()
 
     if (m_vertical) {
 
-	paint.setPen(colorGroup().mid());
+	paint.setPen(m_outlineColour);
 	paint.drawRect(0, 0, width(), height());
  
 	if (m_integral) {
-	    //!!!
+	    //...
 	} else {
 	    for (int dB = -70; dB <= 10; ) {
 		int position = value_to_position(float(dB));
@@ -459,7 +481,7 @@ RosegardenFader::calculateGroovePixmap()
 		       6, m_sliderMax - m_sliderMin);
 	paint.end();
     } else {
-	//!!!
+	//...
     }
 }
 
@@ -533,7 +555,7 @@ RosegardenFader::calculateButtonPixmap()
 
 	paint.end();
     } else {
-	//!!!
+	//...
     }
 }
 
@@ -943,17 +965,14 @@ AudioRouteMenu::slotEntrySelected(int i)
 
 
 
-// ---------------- AudioFaderWidget ------------------
+// ---------------- AudioFaderBox ------------------
 //
 
 
-AudioFaderWidget::AudioFaderWidget(QWidget *parent,
-				   LayoutType type,
-				   QString id,
-				   bool haveInOut,
-				   bool havePlugins,
-				   bool haveMiscButtons,
-                                   const char *name):
+AudioFaderBox::AudioFaderBox(QWidget *parent,
+			     QString id,
+			     bool haveInOut,
+			     const char *name):
     QFrame(parent, name),
     m_signalMapper(new QSignalMapper(this)),
     m_id(id),
@@ -963,103 +982,67 @@ AudioFaderWidget::AudioFaderWidget(QWidget *parent,
     //
     QPushButton *plugin;
     QVBox *pluginVbox = 0;
-
-    if (havePlugins) {
-
-	pluginVbox = new QVBox(this);
-	pluginVbox->setSpacing(2);
-
-	for (int i = 0; i < 5; i++)
-	{
-	    plugin = new QPushButton(pluginVbox);
-	    if (type == FaderStrip) {
-		plugin->setText(i18n("<none>"));
-	    } else {
-		plugin->setText(i18n("<no plugin>"));
-	    }
-
-	    QToolTip::add(plugin, i18n("Audio plugin button"));
-
-	    m_plugins.push_back(plugin);
-	    m_signalMapper->setMapping(plugin, i);
-	    connect(plugin, SIGNAL(clicked()),
-		    m_signalMapper, SLOT(map()));
-	}
+    
+    pluginVbox = new QVBox(this);
+    pluginVbox->setSpacing(2);
+    
+    for (int i = 0; i < 5; i++)
+    {
+	plugin = new QPushButton(pluginVbox);
+	plugin->setText(i18n("<no plugin>"));
+	
+	QToolTip::add(plugin, i18n("Audio plugin button"));
+	
+	m_plugins.push_back(plugin);
+	m_signalMapper->setMapping(plugin, i);
+	connect(plugin, SIGNAL(clicked()),
+		m_signalMapper, SLOT(map()));
     }
 
+    m_synthButton = new QPushButton(this);
+    m_synthButton->setText(i18n("<no synth>"));
+    QToolTip::add(m_synthButton, i18n("Synth plugin button"));
+
     // VU meter and fader
     //
-    if (type == FaderBox) {
-	m_vuMeter = new AudioVUMeter(this);
-    } else {
-	m_vuMeter = new AudioVUMeter(this,
-				     VUMeter::AudioPeakHoldLong,
-				     true,
-				     14,
-				     240);
-    }
-
-    m_fader = new RosegardenFader(type == FaderBox ?
-				  Rosegarden::AudioLevel::ShortFader :
-				  Rosegarden::AudioLevel::LongFader,
-				  20, m_vuMeter->height(), this);
+    QHBox *faderHbox = new QHBox(this);
     
-    if (type == FaderBox) {
-	m_recordFader = new RosegardenFader(Rosegarden::AudioLevel::ShortFader,
-					    20, m_vuMeter->height(), this);
-    } else {
-	m_recordFader = 0;
-    }
+    m_vuMeter = new AudioVUMeter(faderHbox);
+    
+    m_recordFader = new RosegardenFader(Rosegarden::AudioLevel::ShortFader,
+					20, m_vuMeter->height(), faderHbox);
+
+    m_recordFader->setOutlineColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RecordFaderOutline));
+
+    delete m_vuMeter; // only used the first one to establish height,
+		      // actually want it after the record fader in
+		      // hbox
+    m_vuMeter = new AudioVUMeter(faderHbox);
+
+    m_fader = new RosegardenFader(Rosegarden::AudioLevel::ShortFader,
+				  20, m_vuMeter->height(), faderHbox);
+
+    m_fader->setOutlineColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::PlaybackFaderOutline));
 
-    // Stereo, solo, mute and pan
-    //
     QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
     m_monoPixmap.load(QString("%1/misc/mono.xpm").arg(pixmapDir));
     m_stereoPixmap.load(QString("%1/misc/stereo.xpm").arg(pixmapDir));
 
-    m_pan = new RosegardenRotary(this, -100.0, 100.0, 1.0, 5.0, 0.0, 22);
-
-//    QLabel *panLabel = 0;
-
-    if (type == FaderBox) {
-//	panLabel = new QLabel(i18n("Pan"), this);
-    }
+    m_pan = new RosegardenRotary(this, -100.0, 100.0, 1.0, 5.0, 0.0, 22,
+				 RosegardenRotary::NoTicks, false, true);
 
     // same as the knob colour on the MIDI pan
-    m_pan->setKnobColour(RosegardenGUIColours::RotaryPastelGreen);
+    m_pan->setKnobColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryPastelGreen));
 
-    m_muteButton = new QPushButton(this);
-    m_muteButton->setText("M");
-    m_muteButton->setToggleButton(true);
+    m_stereoButton = new QPushButton(this);
+    m_stereoButton->setPixmap(m_monoPixmap); // default is mono
+    m_stereoButton->setFixedSize(24, 24);
+    
+    connect(m_stereoButton, SIGNAL(clicked()),
+	    this, SLOT(slotChannelStateChanged()));
 
-    if (haveMiscButtons) {
-	m_stereoButton = new QPushButton(this);
-	m_stereoButton->setPixmap(m_monoPixmap); // default is mono
-	m_stereoButton->setFixedSize(24, 24);
-
-	connect(m_stereoButton, SIGNAL(clicked()),
-		this, SLOT(slotChannelStateChanged()));
-
-	m_soloButton = new QPushButton(this);
-	m_soloButton->setText("S");
-	m_soloButton->setToggleButton(true);
-
-	m_recordButton = new QPushButton(this);
-	m_recordButton->setText("R");
-	m_recordButton->setToggleButton(true);
-
-	m_muteButton->setFixedWidth(m_stereoButton->width());
-	m_soloButton->setFixedWidth(m_stereoButton->width());
-	m_recordButton->setFixedWidth(m_stereoButton->width());
-	m_muteButton->setFixedHeight(m_stereoButton->height());
-	m_soloButton->setFixedHeight(m_stereoButton->height());
-	m_recordButton->setFixedHeight(m_stereoButton->height());
-
-    } else {
-	m_stereoButton = 0;
-	m_soloButton = 0;
-	m_recordButton = 0;
-    }
+    m_synthGUIButton = new QPushButton(this);
+    m_synthGUIButton->setText(i18n("Editor"));
 
     if (haveInOut) {
 	m_audioInput  = new AudioRouteMenu(this,
@@ -1069,156 +1052,94 @@ AudioFaderWidget::AudioFaderWidget(QWidget *parent,
 					   AudioRouteMenu::Out,
 					   AudioRouteMenu::Regular);
     } else {
-	m_pan->setKnobColour(RosegardenGUIColours::RotaryPastelOrange);
+	m_pan->setKnobColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RotaryPastelOrange));
 
 	m_audioInput = 0;
 	m_audioOutput = 0;
     }
 
-    // Tooltips.  Use more verbose ones for the fader box than the
-    // strip.
-    if (type == FaderBox) {
-	QToolTip::add(m_pan, i18n("Set the audio pan position in the stereo field"));
-	if (haveMiscButtons) {
-	    QToolTip::add(m_recordButton, i18n("Arm recording for this audio Instrument"));
-	    QToolTip::add(m_soloButton, i18n("Solo the Track to which this Instrument is attached."));
-	    QToolTip::add(m_stereoButton, i18n("Mono or Stereo Audio Instrument"));
-	}
-	QToolTip::add(m_muteButton, i18n("Mute the Track to which this Instrument is attached."));
-	QToolTip::add(m_vuMeter, i18n("Audio level"));
-    } else {
-	QToolTip::add(m_pan, i18n("Pan"));
-	if (haveMiscButtons) {
-	    QToolTip::add(m_recordButton, i18n("Arm recording"));
-	    QToolTip::add(m_soloButton, i18n("Solo"));
-	    QToolTip::add(m_stereoButton, i18n("Mono or stereo"));
-	}
-	QToolTip::add(m_muteButton, i18n("Mute"));
-	QToolTip::add(m_vuMeter, i18n("Audio level"));
-    }
+    QToolTip::add(m_pan, i18n("Set the audio pan position in the stereo field"));
+    QToolTip::add(m_synthGUIButton, i18n("Open synth plugin's native editor"));
+    QToolTip::add(m_stereoButton, i18n("Mono or Stereo Instrument"));
+    QToolTip::add(m_recordFader, i18n("Record level"));
+    QToolTip::add(m_fader, i18n("Playback level"));
+    QToolTip::add(m_vuMeter, i18n("Audio level"));
+
+    QGridLayout *grid = new QGridLayout(this, 3, 6, 4, 4);
     
-    // Sort out the layout accordingly
-    //
-    QGridLayout *grid;
-   
-    if (type == FaderStrip)
-    {
-	setFrameStyle(Box | Sunken);
-	setLineWidth(1);
+    grid->addMultiCellWidget(m_synthButton, 0, 0, 0, 2);
 
-	int rows = 3;
-	if (haveInOut) rows += 2;
-	if (havePlugins) ++rows;
-	if (haveMiscButtons) ++rows;
-
-        grid = new QGridLayout(this, rows, 2, 0, 1);
-
-	int row = 0;
-	if (haveInOut) {
-	    grid->addMultiCellWidget(m_audioInput->getWidget(), 0, 0, 0, 1, AlignCenter);
-	    grid->addMultiCellWidget(m_audioOutput->getWidget(), 1, 1, 0, 1, AlignCenter);
-	    row = 2;
-	}
-
-	if (id != "") {
-	    QFont f(font());
-	    f.setBold(true);
-	    QLabel *idLabel = new QLabel(id, this);
-	    idLabel->setFont(f);
-	    grid->addWidget(idLabel, row, 0, AlignCenter);
-	    grid->addWidget(m_pan, row, 1, AlignCenter);
-	} else {
-	    grid->addMultiCellWidget(m_pan, row, row, 0, 1, AlignCenter);
-	}
-
-	++row;
-
-	grid->addMultiCellWidget(m_fader, row, row, 0, 0, AlignRight);
-	grid->addMultiCellWidget(m_vuMeter, row, row, 1, 1, AlignLeft);
-
-	++row;
-	if (haveMiscButtons) {
-	    grid->addWidget(m_muteButton, row, 0, AlignCenter);
-	    grid->addWidget(m_soloButton, row, 1, AlignCenter);
-	} else {
-	    grid->addMultiCellWidget(m_muteButton, row, row, 0, 1, AlignCenter);
-	}
-
-	if (haveMiscButtons) {
-	    ++row;
-	    grid->addWidget(m_recordButton, row, 0, AlignCenter);	
-	    grid->addWidget(m_stereoButton, row, 1, AlignCenter);
-	}
-
-	if (havePlugins) {
-	    ++row;
-	    grid->addMultiCellWidget(pluginVbox, row, row, 0, 1, AlignCenter); 
-	}
+    if (haveInOut) {
+	m_inputLabel = new QLabel(i18n("In:"), this);
+	grid->addWidget(m_inputLabel, 0, 0, AlignRight);
+	grid->addMultiCellWidget(m_audioInput->getWidget(), 0, 0, 1, 2);
+	m_outputLabel = new QLabel(i18n("Out:"), this);
+	grid->addWidget(m_outputLabel, 0, 3, AlignRight);
+	grid->addMultiCellWidget(m_audioOutput->getWidget(), 0, 0, 4, 5);
     }
-    else
-    {
-        grid = new QGridLayout(this, 6, 6, 6, 6);
 
-	if (havePlugins) {
-	    grid->addMultiCellWidget(pluginVbox,    2, 6, 0, 1, AlignCenter);
-	}
+    grid->addMultiCellWidget(pluginVbox,   2, 2, 0, 2);
+    grid->addMultiCellWidget(faderHbox,    1, 2, 3, 5);
 
-        grid->addMultiCellWidget(m_vuMeter,     2, 6, 3, 3, AlignCenter);
-        grid->addMultiCellWidget(m_fader,       2, 6, 2, 2, AlignCenter);
-        grid->addMultiCellWidget(m_recordFader, 2, 6, 4, 4, AlignCenter);
-
-        grid->addWidget(m_muteButton,           2, 5, AlignLeft);
-        grid->addWidget(m_soloButton,           3, 5, AlignLeft);
-
-        grid->addWidget(m_recordButton,         4, 5, AlignLeft);
-
-//        grid->addMultiCellWidget(panLabel,      8, 8, 0, 1, AlignCenter);
-        grid->addWidget(m_pan,                  5, 5, AlignLeft);
-        grid->addWidget(m_stereoButton,         6, 5, AlignLeft);
-
-	if (haveInOut) {
-
-	    QLabel *inputLabel = new QLabel(i18n("In:"), this);
-	    grid->addWidget(inputLabel,             0, 1, AlignRight);
-	    grid->addMultiCellWidget(m_audioInput->getWidget(),  0, 0, 2, 5, AlignLeft);
-
-	    QLabel *outputLabel = new QLabel(i18n("Out:"), this);
-	    grid->addWidget(outputLabel,             1, 1, AlignRight);
-	    grid->addMultiCellWidget(m_audioOutput->getWidget(),  1, 1, 2, 5, AlignLeft);
-	}
-    }
+    grid->addWidget(m_synthGUIButton,       1, 0);
+    grid->addWidget(m_pan,                  1, 2);
+    grid->addWidget(m_stereoButton,         1, 1);
 
     for (int i = 0; i < 5; ++i) {
-        // Force width
-	if (type == FaderStrip) {
-	    if (havePlugins) {
-		m_plugins[i]->setMaximumWidth(50);
-	    }
-	    if (haveInOut) {
-		m_audioInput->getWidget()->setMaximumWidth(50);
-		m_audioOutput->getWidget()->setMaximumWidth(50);
-	    }
-	} else {
-	    if (havePlugins) {
-		m_plugins[i]->setFixedWidth(m_plugins[i]->width());
-	    }
-	} 
+	// Force width
+	m_plugins[i]->setFixedWidth(m_plugins[i]->width());
     }
-
+    m_synthButton->setFixedWidth(m_plugins[0]->width());
+    
+    m_synthButton->hide();
+    m_synthGUIButton->hide();
 }
 
+void
+AudioFaderBox::setIsSynth(bool isSynth)
+{
+    if (isSynth) {
+	m_inputLabel->hide();
+	m_synthButton->show();
+	m_synthGUIButton->show();
+	m_audioInput->getWidget()->hide();
+	m_recordFader->hide();
+    } else {
+	m_synthButton->hide();
+	m_synthGUIButton->hide();
+	m_inputLabel->show();
+	m_audioInput->getWidget()->show();
+	m_recordFader->show();
+    }
+}
 
 void
-AudioFaderWidget::slotSetInstrument(Rosegarden::Studio *studio,
-				    Rosegarden::Instrument *instrument)
+AudioFaderBox::slotSetInstrument(Rosegarden::Studio *studio,
+				 Rosegarden::Instrument *instrument)
 {
     if (m_audioInput) m_audioInput->slotSetInstrument(studio, instrument);
     if (m_audioOutput) m_audioOutput->slotSetInstrument(studio, instrument);
     if (instrument) setAudioChannels(instrument->getAudioChannels());
+    if (instrument) {
+
+	RG_DEBUG << "AudioFaderBox::slotSetInstrument(" << instrument->getId() << ")" << endl;
+
+	setIsSynth(instrument->getType() == Rosegarden::Instrument::SoftSynth);
+	if (instrument->getType() == Rosegarden::Instrument::SoftSynth) {
+	    bool gui = false;
+	    RG_DEBUG << "AudioFaderBox::slotSetInstrument(" << instrument->getId() << "): is soft synth" << endl;
+#ifdef HAVE_LIBLO
+	    gui = RosegardenGUIApp::self()->getPluginGUIManager()->hasGUI
+		(instrument->getId(), Rosegarden::Instrument::SYNTH_PLUGIN_POSITION);
+	    RG_DEBUG << "AudioFaderBox::slotSetInstrument(" << instrument->getId() << "): has gui = " << gui << endl;
+#endif
+	    m_synthGUIButton->setEnabled(gui);
+	}	    
+    }
 }
 
 bool
-AudioFaderWidget::owns(const QObject *object)
+AudioFaderBox::owns(const QObject *object)
 {
     return (object &&
 	    ((object->parent() == this) ||
@@ -1226,7 +1147,7 @@ AudioFaderWidget::owns(const QObject *object)
 }
 
 void
-AudioFaderWidget::setAudioChannels(int channels)
+AudioFaderBox::setAudioChannels(int channels)
 {
     m_isStereo = (channels > 1);
 
@@ -1242,7 +1163,7 @@ AudioFaderWidget::setAudioChannels(int channels)
             m_isStereo = true;
             break;
         default:
-            RG_DEBUG << "AudioFaderWidget::setAudioChannels - "
+            RG_DEBUG << "AudioFaderBox::setAudioChannels - "
                      << "unsupported channel numbers (" << channels
                      << ")" << endl;
 	    return;
@@ -1253,7 +1174,7 @@ AudioFaderWidget::setAudioChannels(int channels)
 }
 
 void
-AudioFaderWidget::slotChannelStateChanged()
+AudioFaderBox::slotChannelStateChanged()
 {
     if (m_isStereo)
     {
