@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -37,6 +37,8 @@ namespace Rosegarden {
     class Studio;
 }
 
+class QGridLayout;
+
 
 class RosegardenFader : public QWidget
 {
@@ -65,6 +67,8 @@ public:
 		    bool vertical, QWidget *parent);
 
     virtual ~RosegardenFader();
+    
+    void setOutlineColour(QColor);
 
     float getFaderLevel() const;
 
@@ -109,8 +113,11 @@ protected:
     QPixmap *groovePixmap();
     QPixmap *buttonPixmap();
 
+    QColor m_outlineColour;
+
     typedef std::pair<int, int> SizeRec;
-    typedef std::pair<QPixmap *, QPixmap *> PixmapRec;
+    typedef std::map<unsigned int, QPixmap *> ColourPixmapRec; // key is QColor::pixel()
+    typedef std::pair<ColourPixmapRec, QPixmap *> PixmapRec;
     typedef std::map<SizeRec, PixmapRec> PixmapCache;
     static PixmapCache m_pixmapCache;
 };
@@ -254,45 +261,29 @@ public:
 };
 
 
-class AudioFaderWidget : public QFrame
+class AudioFaderBox : public QFrame
 {
     Q_OBJECT
 
 public:
-    enum LayoutType {
-	FaderStrip,
-	FaderBox
-    };
-
-    //!!! We've rather abandoned the FaderStrip mode -- the mixer no
-    //longer uses it.  Maybe we should rip it out again rather than
-    //just let it rot.  This class is a complete mess anyway, not
-    //least because it now needs to know the studio &c (in order to
-    //construct the audio route menus) yet still expects its owner to
-    //set instrument-related data on all the other widgets.
-    
-    AudioFaderWidget(QWidget *parent,
-                     LayoutType type,
-		     QString id = "",
-		     bool haveInOut = true,
-		     bool havePlugins = true,
-		     bool haveMiscButtons = true, // solo, record, mono/stereo
-                     const char *name = 0);
+    AudioFaderBox(QWidget *parent,
+		  QString id = "",
+		  bool haveInOut = true,
+		  const char *name = 0);
 
     void setAudioChannels(int);
 
+    void setIsSynth(bool);
+
     bool owns(const QObject *object);
 
+    QPushButton               *m_synthButton;
     std::vector<QPushButton*>  m_plugins;
 
     AudioVUMeter              *m_vuMeter;
 
     RosegardenFader           *m_fader;
     RosegardenFader           *m_recordFader;
-
-    QPushButton               *m_muteButton;
-    QPushButton               *m_soloButton;
-    QPushButton               *m_recordButton;
     RosegardenRotary          *m_pan;
 
     QPixmap                    m_monoPixmap;
@@ -300,9 +291,14 @@ public:
 
     QSignalMapper             *m_signalMapper;
 
+    QLabel                    *m_inputLabel;
+    QLabel                    *m_outputLabel;
+
     AudioRouteMenu            *m_audioInput; 
     AudioRouteMenu            *m_audioOutput; 
 
+    QPushButton               *m_synthGUIButton;
+
     QString                    m_id;
 
     bool                       isStereo() const { return m_isStereo; }
