@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -27,7 +27,8 @@ QColor
 TempoColour::getColour(double tempo)
 {
     int h, s, v;
-    RosegardenGUIColours::TempoBase.hsv(&h, &s, &v);
+    QColor c = Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::TempoBase);
+    c.hsv(&h, &s, &v);
     v += 20;
     if (v > 255) v = 255;
 
