@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -20,6 +20,7 @@
 */
 
 #include <qpainter.h>
+#include <qtooltip.h>
 
 #include "temporuler.h"
 #include "colours.h"
@@ -64,11 +65,13 @@ TempoRuler::TempoRuler(RulerScale *rulerScale,
     m_boldFont.setBold(true);
     m_fontMetrics = QFontMetrics(m_boldFont);
 
-    setBackgroundColor(RosegardenGUIColours::TextRulerBackground);
+    setBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::TextRulerBackground));
 
     QObject::connect
 	(doc->getCommandHistory(), SIGNAL(commandExecuted()),
 	 this, SLOT(update()));
+
+    QToolTip::add(this, i18n("Tempo and Time Signature Ruler.\nDouble click to insert an event."));
 }
 
 TempoRuler::~TempoRuler()
@@ -133,7 +136,7 @@ void
 TempoRuler::paintEvent(QPaintEvent* e)
 {
     QPainter paint(this);
-    paint.setPen(RosegardenGUIColours::TextRulerForeground);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::TextRulerForeground));
 
     paint.setClipRegion(e->region());
     paint.setClipRect(e->rect().normalize());
@@ -245,7 +248,11 @@ TempoRuler::paintEvent(QPaintEvent* e)
 
 	    QString tempoString;
 	    if (frac) {
-		tempoString = QString("%1.%2").arg(bpm).arg(frac);
+		if (frac < 10) {
+		    tempoString = QString("%1.0%2").arg(bpm).arg(frac);
+		} else {
+		    tempoString = QString("%1.%2").arg(bpm).arg(frac);
+		}
 	    } else {
 		tempoString = QString("%1").arg(bpm);
 	    }
