@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -141,8 +141,7 @@ TempoView::TempoView(RosegardenGUIDoc *doc, QWidget *parent, Rosegarden::timeT o
 
     m_grid->addWidget(m_list, 2, 1);
 
-    setCaption(i18n("%1 - Tempo and Time Signature Editor")
-	       .arg(doc->getTitle()));
+    updateViewCaption();
 
     doc->getComposition().addObserver(this);
 
@@ -364,6 +363,13 @@ TempoView::makeInitialSelection(Rosegarden::timeT time)
     }
 }
 
+Rosegarden::Segment *
+TempoView::getCurrentSegment()
+{
+    if (m_segments.empty()) return 0;
+    else return *m_segments.begin();
+}
+
 QString
 TempoView::makeTimeString(Rosegarden::timeT time, int timeMode)
 {
@@ -448,10 +454,14 @@ TempoView::slotEditDelete()
     int itemIndex = -1;
 
     m_ignoreUpdates = true;
-    KMacroCommand *command = new KMacroCommand
-	(i18n("Delete Tempo or Time Signature"));
     bool haveSomething = false;
 
+    // We want the Remove commands to be in reverse order, because
+    // removing one item by index will affect the indices of
+    // subsequent items.  So we'll stack them onto here and then pull
+    // them off again.
+    std::vector<KCommand *> commands;
+
     while ((listItem = it.current()) != 0)
     {
         item = dynamic_cast<TempoListItem*>((*it));
@@ -461,21 +471,29 @@ TempoView::slotEditDelete()
         if (item)
         {
 	    if (item->getType() == TempoListItem::TimeSignature) {
-		command->addCommand(new RemoveTimeSignatureCommand
-				    (item->getComposition(),
-				     item->getIndex()));
+		commands.push_back(new RemoveTimeSignatureCommand
+				   (item->getComposition(),
+				    item->getIndex()));
 		haveSomething = true;
 	    } else {
-		command->addCommand(new RemoveTempoChangeCommand
-				    (item->getComposition(),
-				     item->getIndex()));
+		commands.push_back(new RemoveTempoChangeCommand
+				   (item->getComposition(),
+				    item->getIndex()));
 		haveSomething = true;
 	    }		
         }
         ++it;
     }
 
-    if (haveSomething) addCommandToHistory(command);
+    if (haveSomething) {
+	KMacroCommand *command = new KMacroCommand
+	    (i18n("Delete Tempo or Time Signature"));
+	for (std::vector<KCommand *>::iterator i = commands.end();
+	     i != commands.begin();) {
+	    command->addCommand(*--i);
+	}
+	addCommandToHistory(command);
+    }
 
     applyLayout();
     m_ignoreUpdates = false;
@@ -586,19 +604,22 @@ TempoView::setupActions()
                 SLOT(slotEditInsertTempo()), actionCollection(),
                 "insert_tempo");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-insert-timesig.xpm"));
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/event-insert-timesig.xpm");
+    icon = QIconSet(pixmap);
     
     new KAction(AddTimeSignatureCommand::getGlobalName(), icon, Key_G, this,
                 SLOT(slotEditInsertTimeSignature()), actionCollection(),
                 "insert_timesig");
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-delete.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/event-delete.xpm");
+    icon = QIconSet(pixmap);
     
     new KAction(i18n("&Delete"), icon, Key_Delete, this,
                 SLOT(slotEditDelete()), actionCollection(),
                 "delete");
     
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/event-edit.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/event-edit.xpm");
+    icon = QIconSet(pixmap);
 
     new KAction(i18n("&Edit Item"), icon, Key_E, this,
                 SLOT(slotEdit()), actionCollection(),
@@ -617,7 +638,8 @@ TempoView::setupActions()
 
     KRadioAction *action;
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-musical.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-musical.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Musical Times"), icon, 0, this,
 			      SLOT(slotMusicalTime()),
@@ -625,7 +647,8 @@ TempoView::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 0) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-real.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-real.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Real Times"), icon, 0, this,
 			      SLOT(slotRealTime()),
@@ -633,7 +656,8 @@ TempoView::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 1) action->setChecked(true);
 
-    icon = QIconSet(QCanvasPixmap(pixmapDir + "/toolbar/time-raw.xpm"));
+    pixmap.load(pixmapDir + "/toolbar/time-raw.xpm");
+    icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("Ra&w Times"), icon, 0, this,
 			      SLOT(slotRawTime()),
@@ -823,6 +847,16 @@ TempoView::slotPopupEditor(QListViewItem *qitem)
     }
 }
 
+
+void
+TempoView::updateViewCaption()
+{
+    setCaption(i18n("%1 - Tempo and Time Signature Editor")
+	       .arg(getDocument()->getTitle()));
+}
+
+
+
 const char* const TempoView::LayoutConfigGroupName = "TempoView Layout";
 const char* const TempoView::ConfigGroup = "TempoView Options";
 
