@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -53,7 +53,7 @@ TextRuler::TextRuler(RulerScale *rulerScale,
       m_fontMetrics(m_font)
 {
     m_mySegmentMaybe = (m_segment->getComposition() != 0);
-    setBackgroundColor(RosegardenGUIColours::TextRulerBackground);
+    setBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::TextRulerBackground));
 
     m_font.setPixelSize(10);
 }
@@ -110,7 +110,7 @@ void
 TextRuler::paintEvent(QPaintEvent* e)
 {
     QPainter paint(this);
-    paint.setPen(RosegardenGUIColours::TextRulerForeground);
+    paint.setPen(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::TextRulerForeground));
 
     paint.setClipRegion(e->region());
     paint.setClipRect(e->rect().normalize());
