@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -35,6 +35,9 @@
 #include "Studio.h"
 #include "colours.h"
 #include "tracklabel.h"
+#include "AudioPluginInstance.h"
+#include "PluginIdentifier.h"
+#include "AudioLevel.h"
 
 #include "segmentcommands.h"
 #include "rosestrings.h"
@@ -263,10 +266,10 @@ QFrame* TrackButtons::makeButton(Rosegarden::TrackId trackId)
     //
     // 'mute' and 'record' leds
     //
-    mute = new KLedButton(RosegardenGUIColours::MuteTrackLED, trackHBox);
+    mute = new KLedButton(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::MuteTrackLED), trackHBox);
     QToolTip::add(mute, i18n("Mute track"));
     hblayout->addWidget(mute);
-    record = new KLedButton(RosegardenGUIColours::RecordTrackLED, trackHBox);
+    record = new KLedButton(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::RecordTrackLED), trackHBox);
     QToolTip::add(record, i18n("Record on this track"));
     hblayout->addWidget(record);
 
@@ -279,8 +282,8 @@ QFrame* TrackButtons::makeButton(Rosegarden::TrackId trackId)
             m_recordSigMapper, SLOT(map()));
     connect(mute, SIGNAL(stateChanged(bool)),
             m_muteSigMapper, SLOT(map()));
-    m_recordSigMapper->setMapping(record, trackId);
-    m_muteSigMapper->setMapping(mute, trackId);
+    m_recordSigMapper->setMapping(record, track->getPosition());
+    m_muteSigMapper->setMapping(mute, track->getPosition());
 
     // Store the KLedButton
     //
@@ -448,18 +451,15 @@ TrackButtons::mutedTracks()
 //
 //
 void
-TrackButtons::slotToggleMutedTrack(int mutedTrack)
+TrackButtons::slotToggleMutedTrack(int mutedTrackPos)
 {
-    RG_DEBUG << "TrackButtons::slotToggleMutedTrack(" << mutedTrack << ")\n";
+    RG_DEBUG << "TrackButtons::slotToggleMutedTrack(" << mutedTrackPos << ")\n";
 
-    if (mutedTrack < 0 || mutedTrack > (int)m_tracks )
+    if (mutedTrackPos < 0 || mutedTrackPos > (int)m_tracks )
         return;
 
     Rosegarden::Track *track = 
-        m_doc->getComposition().getTrackByPosition(mutedTrack);
-
-    RG_DEBUG << "TrackButtons::slotToggleMutedTrack - track = " << mutedTrack
-             << " is " << !track->isMuted() << endl;
+        m_doc->getComposition().getTrackByPosition(mutedTrackPos);
 
     emit muteButton(track->getId(), !track->isMuted()); // will set the value
 }
@@ -536,8 +536,7 @@ TrackButtons::slotUpdateTracks()
                 }
             }
             else
-                RG_DEBUG << "TrackButtons::slotUpdateTracks - "
-                         << "can't find TrackId for position " << i << endl;
+                RG_DEBUG << "TrackButtons::slotUpdateTracks - can't find TrackId for position " << i << endl;
         }
     }
 
@@ -561,9 +560,12 @@ TrackButtons::slotUpdateTracks()
 		    trackLabel->setText(i18n("<untitled>"));
 		}
 	    }
-	    else
+	    else {
 		trackLabel->setText(strtoqstr(track->getLabel()));
-	    
+            }
+            
+//             RG_DEBUG << "TrackButtons::slotUpdateTracks - set button mapping at pos "
+//                      << i << " to track id " << track->getId() << endl;
 	    setButtonMapping(m_trackLabels[i], track->getId());
 	}
     }
@@ -619,11 +621,14 @@ TrackButtons::setRecordButtonDown(int position)
 void
 TrackButtons::selectLabel(int position)
 {
-    if (m_lastSelected >= 0)
+    if (m_lastSelected >= 0 && m_lastSelected < m_trackLabels.size()) {
         m_trackLabels[m_lastSelected]->setSelected(false);
-    m_trackLabels[position]->setSelected(true);
+    }
 
-    m_lastSelected = position;
+    if (position >= 0 && position < m_trackLabels.size()) {
+	m_trackLabels[position]->setSelected(true);
+	m_lastSelected = position;
+    }
 }
 
 
@@ -658,13 +663,17 @@ TrackButtons::slotRenameTrack(QString newName, Rosegarden::TrackId trackId)
 
 
 void
-TrackButtons::slotSetTrackMeter(double value, int position)
+TrackButtons::slotSetTrackMeter(float value, int position)
 {
+    Rosegarden::Composition &comp = m_doc->getComposition();
+    Rosegarden::Studio &studio = m_doc->getStudio();
+    Rosegarden::Track *track;
+
     for (unsigned int i = 0; i < m_trackMeters.size(); ++i)
     {
         if (i == ((unsigned int)position))
         {
-            m_trackMeters[i]->setLevel(value);
+	    m_trackMeters[i]->setLevel(value);
             return;
         }
     }
@@ -675,25 +684,20 @@ TrackButtons::slotSetTrackMeter(double value, int position)
 // equate that to the Instrument we're setting.
 //
 void
-TrackButtons::slotSetMetersByInstrument(double value,
+TrackButtons::slotSetMetersByInstrument(float value,
                                         Rosegarden::InstrumentId id)
 {
     Rosegarden::Composition &comp = m_doc->getComposition();
+    Rosegarden::Studio &studio = m_doc->getStudio();
     Rosegarden::Track *track;
 
     for (unsigned int i = 0; i < m_trackMeters.size(); ++i)
     {
         track = comp.getTrackByPosition(i);
 
-        if (track !=0 && track->getInstrument() == id)
+        if (track != 0 && track->getInstrument() == id)
         {
-            m_trackMeters[i]->setLevel(value);
-
-            /*
-            RG_DEBUG << "TrackButtons::slotSetMetersByInstrument - "
-                     << "setting level = " << value 
-                     << " on instrument " << id << endl;
-                     */
+	    m_trackMeters[i]->setLevel(value);
         }
     }
 }
@@ -798,6 +802,37 @@ TrackButtons::populateInstrumentPopup(Rosegarden::Instrument *thisTrackInstr, QP
 	QString pname(strtoqstr((*it)->getProgramName()));
 	Rosegarden::Device *device = (*it)->getDevice();
         Rosegarden::DeviceId devId = device->getId();
+	bool connected = false;
+
+	if ((*it)->getType() == Rosegarden::Instrument::SoftSynth) {
+	    pname = "";
+	    Rosegarden::AudioPluginInstance *plugin = (*it)->getPlugin
+		(Rosegarden::Instrument::SYNTH_PLUGIN_POSITION);
+	    if (plugin) {
+		pname = strtoqstr(plugin->getProgram());
+		QString identifier = strtoqstr(plugin->getIdentifier());
+		if (identifier != "") {
+		    connected = true;
+		    QString type, soName, label;
+		    Rosegarden::PluginIdentifier::parseIdentifier
+			(identifier, type, soName, label);
+		    if (pname == "") {
+			pname = strtoqstr(plugin->getDistinctiveConfigurationText());
+		    }
+		    if (pname != "") {
+			pname = QString("%1: %2").arg(label).arg(pname);
+		    } else {
+			pname = label;
+		    }
+		} else {
+		    connected = false;
+		}
+	    }
+	} else if ((*it)->getType() == Rosegarden::Instrument::Audio) {
+	    connected = true;
+	} else {
+	    connected = (device->getConnection() != "");
+	}
 
 	bool instrUsedByMe = false;
 	bool instrUsedByAnyone = false;
@@ -832,11 +867,11 @@ TrackButtons::populateInstrumentPopup(Rosegarden::Instrument *thisTrackInstr, QP
 	    }
 
 	    QIconSet iconSet
-		(device->getConnection() == "" ?
+		(connected ?
 		 (deviceUsedByAnyone ? 
-		  unconnectedUsedPixmap : unconnectedPixmap) :
+		  connectedUsedPixmap : connectedPixmap) :
 		 (deviceUsedByAnyone ? 
-		  connectedUsedPixmap : connectedPixmap));
+		  unconnectedUsedPixmap : unconnectedPixmap));
 
             currentDevId = int(devId);
 
@@ -864,15 +899,15 @@ TrackButtons::populateInstrumentPopup(Rosegarden::Instrument *thisTrackInstr, QP
 	}
 
 	QIconSet iconSet
-	    (device->getConnection() == "" ?
-	     (instrUsedByAnyone ? 
-	      instrUsedByMe ?
-	      unconnectedSelectedPixmap :
-	      unconnectedUsedPixmap : unconnectedPixmap) :
+	    (connected ?
 	     (instrUsedByAnyone ? 
 	      instrUsedByMe ?
 	      connectedSelectedPixmap :
-	      connectedUsedPixmap : connectedPixmap));
+	      connectedUsedPixmap : connectedPixmap) :
+	     (instrUsedByAnyone ? 
+	      instrUsedByMe ?
+	      unconnectedSelectedPixmap :
+	      unconnectedUsedPixmap : unconnectedPixmap));
 
 	if (pname != "") iname += " (" + pname + ")";
 
@@ -922,7 +957,7 @@ TrackButtons::slotInstrumentPopupActivated(int item)
             // Ensure that we set a record track properly
             //
             if (track->getId() == comp.getRecordTrack())
-                slotSetRecordTrack(track->getId());
+                slotSetRecordTrack(track->getPosition());
 
         }
         else
@@ -1001,9 +1036,9 @@ TrackButtons::slotSynchroniseWithComposition()
         if (track)
         {
             if (track->isMuted())
-                m_muteLeds[i]->on();
-            else
                 m_muteLeds[i]->off();
+            else
+                m_muteLeds[i]->on();
 
             Rosegarden::Instrument *ins = studio.
                 getInstrumentById(track->getInstrument());
@@ -1038,7 +1073,12 @@ TrackButtons::setMuteButton(TrackId track, bool value)
     Rosegarden::Track *trackObj = m_doc->getComposition().getTrackById(track);
     if (trackObj == 0) return;
 
-    m_muteLeds[trackObj->getPosition()]->setState(value ? KLed::Off : KLed::On);
+    int pos = trackObj->getPosition();
+
+    RG_DEBUG << "TrackButtons::setMuteButton() trackId = "
+             << track << ", pos = " << pos << endl;
+    
+    m_muteLeds[pos]->setState(value ? KLed::Off : KLed::On);
 }
 
 
