@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -55,6 +55,7 @@
 #include "segmenttool.h"
 #include "qdeferscrollview.h"
 #include "rosegardenguiview.h"
+#include "sequencemanager.h"
 
 #include "rosedebug.h"
 
@@ -64,7 +65,7 @@ using Rosegarden::timeT;
 using Rosegarden::Segment;
 using Rosegarden::TrackId;
 
-static double _pointerWidth = 3.0;
+const double TrackEditor::PointerWidth = 3.0;
 
 TrackEditor::TrackEditor(RosegardenGUIDoc* doc,
                          QWidget* rosegardenguiview,
@@ -112,14 +113,14 @@ TrackEditor::init(QWidget* rosegardenguiview)
 
     QCanvas *canvas = new QCanvas(this);
     canvas->resize(100, 100); // call slotReadjustCanvasSize later
-    canvas->setBackgroundColor(RosegardenGUIColours::SegmentCanvas);
+    canvas->setBackgroundColor(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::SegmentCanvas));
     //canvas->retune(128);
 
     kapp->config()->setGroup(Rosegarden::GeneralOptionsConfigGroup);
     if (kapp->config()->readBoolEntry("backgroundtextures", false)) {
 	QPixmap background;
 	QString pixmapDir = KGlobal::dirs()->findResource("appdata", "pixmaps/");
-	if (background.load(QString("%1/misc/bg-paper-grey.xpm").
+	if (background.load(QString("%1/misc/bg-segmentcanvas.xpm").
 			    arg(pixmapDir))) {
 	    canvas->setBackgroundPixmap(background);
 	}
@@ -226,6 +227,19 @@ TrackEditor::init(QWidget* rosegardenguiview)
     connect(m_trackButtons, SIGNAL(newRecordButton()),
             m_doc, SLOT(slotNewRecordButton()));
 
+    // connect loop rulers' follow-scroll signals
+    connect(m_topBarButtons->getLoopRuler(), SIGNAL(startMouseMove(int)),
+            m_segmentCanvas, SLOT(startAutoScroll(int)));
+    connect(m_topBarButtons->getLoopRuler(), SIGNAL(stopMouseMove()),
+            m_segmentCanvas, SLOT(stopAutoScroll()));
+    connect(m_bottomBarButtons->getLoopRuler(), SIGNAL(startMouseMove(int)),
+            m_segmentCanvas, SLOT(startAutoScroll(int)));
+    connect(m_bottomBarButtons->getLoopRuler(), SIGNAL(stopMouseMove()),
+            m_segmentCanvas, SLOT(stopAutoScroll()));
+
+    connect(m_segmentCanvas, SIGNAL(contentsMoving(int, int)),
+            this, SLOT(slotCanvasScrolled(int, int)));
+
     // Synchronize bar buttons' scrollview with segment canvas' scrollbar
     //
     connect(m_segmentCanvas->verticalScrollBar(), SIGNAL(valueChanged(int)),
@@ -274,14 +288,12 @@ TrackEditor::init(QWidget* rosegardenguiview)
 	    this, SLOT(slotSetPointerPosition(Rosegarden::timeT)));
  
     connect(m_doc, SIGNAL(loopChanged(Rosegarden::timeT,
-					   Rosegarden::timeT)),
+                                      Rosegarden::timeT)),
 	    this, SLOT(slotSetLoop(Rosegarden::timeT, Rosegarden::timeT)));
 
     // create the position pointer
-    m_pointer = new QCanvasRectangle(canvas);
-    m_pointer->setPen(RosegardenGUIColours::Pointer);
-    m_pointer->setBrush(RosegardenGUIColours::Pointer);
-    m_pointer->setSize(3, canvas->height());
+    m_pointer = new CanvasCursor(canvas, int(PointerWidth));
+    m_pointer->updateHeight();
     m_pointer->setX(-2);
     m_pointer->setY(0);
     m_pointer->setZ(10);
@@ -304,21 +316,28 @@ void TrackEditor::slotReadjustCanvasSize()
 //                                 //QApplication::desktop()->height()
 //                                 );
 
-    RG_DEBUG << "TrackEditor::slotReadjustCanvasSize() : nbTracks = "
+    RG_DEBUG << "TrackEditor::slotReadjustCanvasSize() : width = "
+	     << m_canvasWidth << ", nbTracks = "
              << comp.getNbTracks() << endl;
 
     int canvasHeight = getTrackCellHeight() * std::max(40u, comp.getNbTracks());
 
+    int chunkSize = 100;
+    if (m_canvasWidth > 20000) {
+	chunkSize = m_canvasWidth / 1000;
+    }
+    m_segmentCanvas->canvas()->retune(chunkSize);
+
     m_segmentCanvas->canvas()->resize(m_canvasWidth, canvasHeight);
 
 
     Rosegarden::SimpleRulerScale *sRuler = 
         dynamic_cast<Rosegarden::SimpleRulerScale*>(m_rulerScale);
 
-    int width = int(rint((_pointerWidth * sRuler->getUnitsPerPixel())/ m_initialUnitsPerPixel));
-    width = int(_pointerWidth);
+    int width = int(rint((PointerWidth * sRuler->getUnitsPerPixel())/ m_initialUnitsPerPixel));
+    width = int(PointerWidth);
 
-    m_pointer->setSize(width, m_segmentCanvas->canvas()->height());
+    m_pointer->updateHeight();
 
     RG_DEBUG << "TrackEditor::slotReadjustCanvasSize - done" << endl;
 }
@@ -370,7 +389,7 @@ TrackEditor::setupSegments()
 	RG_DEBUG << "TrackEditor::setupSegments() add segment"
 			     << " - start idx : " << (*i)->getStartTime()
 			     << " - nb time steps : " << ((*i)->getEndTime() - (*i)->getStartTime())
-			     << " - track : " << (*i)->getTrack()
+			     << " - track id : " << (*i)->getTrack()
 			     << endl;
 
 	m_segmentCanvas->addSegmentItem((*i));
@@ -489,12 +508,29 @@ void TrackEditor::slotSegmentOrderChanged(int section, int fromIdx, int toIdx)
     emit needUpdate();
 }
 
+void
+TrackEditor::slotCanvasScrolled(int x, int y)
+{
+    if ((m_topBarButtons && m_topBarButtons->getLoopRuler() &&
+	 m_topBarButtons->getLoopRuler()->hasActiveMousePress()) ||
+	(m_bottomBarButtons && m_bottomBarButtons->getLoopRuler() &&
+	 m_bottomBarButtons->getLoopRuler()->hasActiveMousePress())) {
+
+	int mx = m_segmentCanvas->viewport()->mapFromGlobal(QCursor::pos()).x();
+	timeT t = m_segmentCanvas->grid().getRulerScale()->getTimeForX(x + mx);
+
+	slotSetPointerPosition(t);
+    }
+}
+
+
 // Move the position pointer
 void
 TrackEditor::slotSetPointerPosition(Rosegarden::timeT position)
 {
 
-    //RG_DEBUG << "TrackEditor::setPointerPosition: time is " << position << endl;
+//     RG_DEBUG << "TrackEditor::setPointerPosition: time is " << position << endl;
+
     if (!m_pointer) return;
 
     Rosegarden::SimpleRulerScale *ruler = 
@@ -506,10 +542,8 @@ TrackEditor::slotSetPointerPosition(Rosegarden::timeT position)
     /*
     RG_DEBUG << "TrackEditor::setPointerPosition - scale = " << ruler->getUnitsPerPixel() << endl;
     */
-    int width = int(rint((_pointerWidth * ruler->getUnitsPerPixel())/ m_initialUnitsPerPixel));
-    width = int(_pointerWidth);
 
-    m_pointer->setSize(width, m_segmentCanvas->canvas()->height());
+    m_pointer->updateHeight();
 
     double pos = m_segmentCanvas->grid().getRulerScale()->getXForTime(position);
     double distance = pos - m_pointer->x();
@@ -519,9 +553,19 @@ TrackEditor::slotSetPointerPosition(Rosegarden::timeT position)
 
 	m_pointer->setX(pos);
 
-	if (m_playTracking) {
+// 	if (m_playTracking) {
+// 	    getSegmentCanvas()->slotScrollHoriz(int(double(position) / ruler->getUnitsPerPixel()));
+// 	}
+	
+	if (m_doc && m_doc->getSequenceManager() &&
+	    (m_doc->getSequenceManager()->getTransportStatus() != STOPPED)) {
+
+	    if (m_playTracking) {
+		getSegmentCanvas()->slotScrollHoriz(int(double(position) / ruler->getUnitsPerPixel()));
+	    }
+	} else if (!getSegmentCanvas()->isAutoScrolling()) {
 	    getSegmentCanvas()->slotScrollHoriz(int(double(position) / ruler->getUnitsPerPixel()));
-	}
+        }
 
 	emit needUpdate();
     }
