@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2006
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -34,7 +34,6 @@
 #include <qlabel.h>
 #include <qaccel.h>
 #include <qiconset.h>
-#include <qcanvas.h>
 
 #include "triggermanager.h"
 #include "rosegardenguidoc.h"
@@ -44,7 +43,6 @@
 #include "segmentcommands.h"
 #include "widgets.h"
 #include "midipitchlabel.h"
-#include "dialogs.h"
 
 #include "CompositionTimeSliceAdapter.h"
 #include "BaseProperties.h"
@@ -295,9 +293,11 @@ TriggerSegmentManager::slotUpdate()
 	QString label = strtoqstr((*it)->getSegment()->getLabel());
 	if (label == "") label = i18n("<no label>");
 
-	QString used = i18n("%1 on 1 track",
-			    "%1 on %n tracks",
-			    tracks.size()).arg(uses);
+	QString used;
+	if (tracks.size() == 1)
+	    used = i18n("%1 on %2 track").arg(uses).arg(tracks.size());
+	else
+	    used = i18n("%1 on %2 tracks").arg(uses).arg(tracks.size());
 
 	QString pitch = QString("%1 (%2)")
 	    .arg(Rosegarden::MidiPitchLabel((*it)->getBasePitch()).getQString())
@@ -383,8 +383,7 @@ TriggerSegmentManager::slotDelete()
     if (!item) return;
 
     if (item->getUsage() > 0) {
-	if (KMessageBox::warningContinueCancel(this, i18n("This triggered segment is used 1 time in the current composition.  Are you sure you want to remove it?",
-                                                          "This triggered segment is used %n times in the current composition.  Are you sure you want to remove it?", item->getUsage())) != KMessageBox::Continue) return;
+	if (KMessageBox::warningContinueCancel(this, i18n("This triggered segment is used %1 times in the current composition.  Are you sure you want to remove it?").arg(item->getUsage())) != KMessageBox::Continue) return;
     }
 
     DeleteTriggerSegmentCommand *command =
@@ -455,7 +454,7 @@ TriggerSegmentManager::setupActions()
 
     KRadioAction *action;
 
-    QCanvasPixmap pixmap(pixmapDir + "/toolbar/time-musical.png");
+    QCanvasPixmap pixmap(pixmapDir + "/toolbar/time-musical.xpm");
     QIconSet icon(pixmap);
 
     action = new KRadioAction(i18n("&Musical Times"), icon, 0, this,
@@ -464,7 +463,7 @@ TriggerSegmentManager::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 0) action->setChecked(true);
 
-    pixmap.load(pixmapDir + "/toolbar/time-real.png");
+    pixmap.load(pixmapDir + "/toolbar/time-real.xpm");
     icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("&Real Times"), icon, 0, this,
@@ -473,7 +472,7 @@ TriggerSegmentManager::setupActions()
     action->setExclusiveGroup("timeMode");
     if (timeMode == 1) action->setChecked(true);
 
-    pixmap.load(pixmapDir + "/toolbar/time-raw.png");
+    pixmap.load(pixmapDir + "/toolbar/time-raw.xpm");
     icon = QIconSet(pixmap);
 
     action = new KRadioAction(i18n("Ra&w Times"), icon, 0, this,
@@ -588,8 +587,7 @@ TriggerSegmentManager::makeDurationString(Rosegarden::timeT time,
 	Rosegarden::RealTime rt =
 	    m_doc->getComposition().getRealTimeDifference
 	    (time, time + duration);
-//	return QString("%1  ").arg(rt.toString().c_str());
-	return QString("%1  ").arg(rt.toText().c_str());
+	return QString("%1  ").arg(rt.toString().c_str());
     }
 
     default:
@@ -624,4 +622,3 @@ TriggerSegmentManager::slotRawTime()
 
 
 const char* const TriggerSegmentManager::TriggerManagerConfigGroup = "Trigger Editor";
-#include "triggermanager.moc"
