@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -124,4 +124,14 @@ DefaultVelocityColour* DefaultVelocityColour::getInstance()
     return m_instance;
 }
 
+DefaultVelocityColour::DefaultVelocityColour()
+    : VelocityColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterRed),
+                     Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterOrange),
+                     Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterGreen),
+                     127, // max knee
+                     115, // red knee
+                     75,  // orange knee
+                     25)  // green knee
+{}
+
 DefaultVelocityColour* DefaultVelocityColour::m_instance = 0;
