@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -63,6 +63,8 @@ VUMeter::VUMeter(QWidget *parent,
         case PeakHold:
         case AudioPeakHoldShort:
         case AudioPeakHoldLong:
+        case AudioPeakHoldIEC:
+        case AudioPeakHoldIECLong:
         case FixedHeightVisiblePeakHold:
             m_showPeakLevel = true;
             break;
@@ -119,9 +121,19 @@ VUMeter::VUMeter(QWidget *parent,
 	green  = AudioLevel::dB_to_fader( -10.0, max, AudioLevel::ShortFader);
 	m_background = QColor(50, 50, 50);
     } else if (m_type == AudioPeakHoldLong) {
-	red    = AudioLevel::dB_to_fader(   0.0, max, AudioLevel::ShortFader);
-	orange = AudioLevel::dB_to_fader(  -2.0, max, AudioLevel::ShortFader);
-	green  = AudioLevel::dB_to_fader( -10.0, max, AudioLevel::ShortFader);
+	red    = AudioLevel::dB_to_fader(   0.0, max, AudioLevel::LongFader);
+	orange = AudioLevel::dB_to_fader(  -2.0, max, AudioLevel::LongFader);
+	green  = AudioLevel::dB_to_fader( -10.0, max, AudioLevel::LongFader);
+	m_background = QColor(50, 50, 50);
+    } else if (m_type == AudioPeakHoldIEC) {
+	red    = AudioLevel::dB_to_fader(  -0.1, max, AudioLevel::IEC268Meter);
+	orange = AudioLevel::dB_to_fader(  -6.0, max, AudioLevel::IEC268Meter);
+	green  = AudioLevel::dB_to_fader( -10.0, max, AudioLevel::IEC268Meter);
+	m_background = QColor(50, 50, 50);
+    } else if (m_type == AudioPeakHoldIECLong) {
+	red    = AudioLevel::dB_to_fader(   0.0, max, AudioLevel::IEC268LongMeter);
+	orange = AudioLevel::dB_to_fader(  -6.0, max, AudioLevel::IEC268LongMeter);
+	green  = AudioLevel::dB_to_fader( -10.0, max, AudioLevel::IEC268LongMeter);
 	m_background = QColor(50, 50, 50);
     } else {
 	red    = max * 92 / 100;
@@ -131,17 +143,19 @@ VUMeter::VUMeter(QWidget *parent,
     }
 
     if (m_type == AudioPeakHoldLong ||
-	m_type == AudioPeakHoldShort) {
+	m_type == AudioPeakHoldShort ||
+	m_type == AudioPeakHoldIEC ||
+	m_type == AudioPeakHoldIECLong) {
 	m_velocityColour =
-	    new VelocityColour(RosegardenGUIColours::LevelMeterSolidRed,
-			       RosegardenGUIColours::LevelMeterSolidOrange,
-			       RosegardenGUIColours::LevelMeterSolidGreen,
+	    new VelocityColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterSolidRed),
+			       Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterSolidOrange),
+			       Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterSolidGreen),
 			       max, red, orange, green);
     } else {
 	m_velocityColour =
-	    new VelocityColour(RosegardenGUIColours::LevelMeterRed,
-			       RosegardenGUIColours::LevelMeterOrange,
-			       RosegardenGUIColours::LevelMeterGreen,
+	    new VelocityColour(Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterRed),
+			       Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterOrange),
+			       Rosegarden::GUIPalette::getColour(Rosegarden::GUIPalette::LevelMeterGreen),
 			       max, red, orange, green);
     }
 }
@@ -175,6 +189,20 @@ VUMeter::setLevel(double leftLevel, double rightLevel)
 	    (rightLevel, m_maxLevel, AudioLevel::LongFader);
 	break;
 
+    case AudioPeakHoldIEC:
+	m_levelLeft = AudioLevel::dB_to_fader
+	    (leftLevel, m_maxLevel, AudioLevel::IEC268Meter);
+	m_levelRight = AudioLevel::dB_to_fader
+	    (rightLevel, m_maxLevel, AudioLevel::IEC268Meter);
+	break;
+
+    case AudioPeakHoldIECLong:
+	m_levelLeft = AudioLevel::dB_to_fader
+	    (leftLevel, m_maxLevel, AudioLevel::IEC268LongMeter);
+	m_levelRight = AudioLevel::dB_to_fader
+	    (rightLevel, m_maxLevel, AudioLevel::IEC268LongMeter);
+	break;
+
     default:
 	m_levelLeft = (int)(double(m_maxLevel) * leftLevel);
 	m_levelRight = (int)(double(m_maxLevel) * rightLevel);
@@ -239,7 +267,9 @@ VUMeter::paintEvent(QPaintEvent*)
     QPainter paint(this);
 
     if (m_type == VUMeter::AudioPeakHoldShort ||
-	m_type == VUMeter::AudioPeakHoldLong)
+	m_type == VUMeter::AudioPeakHoldLong ||
+	m_type == VUMeter::AudioPeakHoldIEC ||
+	m_type == VUMeter::AudioPeakHoldIECLong)
     {
 	paint.setPen(m_background);
 	paint.setBrush(m_background);
@@ -292,12 +322,13 @@ VUMeter::drawColouredBar(QPainter *paint, int channel,
 			 int x, int y, int w, int h)
 {
     if (m_type == AudioPeakHoldLong ||
-	m_type == AudioPeakHoldShort) {
+	m_type == AudioPeakHoldShort ||
+	m_type == AudioPeakHoldIEC ||
+	m_type == AudioPeakHoldIECLong) {
 
 	Qt::BrushStyle style = Qt::SolidPattern;
 
-	int quiet = m_velocityColour->getQuietKnee(),
-	    medium = m_velocityColour->getMediumKnee(),
+	int medium = m_velocityColour->getMediumKnee(),
 	    loud = m_velocityColour->getLoudKnee();
 
 	if (m_alignment == Vertical) {
@@ -373,8 +404,9 @@ VUMeter::drawColouredBar(QPainter *paint, int channel,
 void
 VUMeter::drawMeterLevel(QPainter* paint)
 {
-    // Get the colour from the VelocityColour helper, if appropriate.
-    //
+    int medium = m_velocityColour->getMediumKnee(),
+	loud = m_velocityColour->getLoudKnee();
+
     if (m_stereo)
     {
         if (m_alignment == VUMeter::Vertical)
@@ -393,11 +425,16 @@ VUMeter::drawMeterLevel(QPainter* paint)
 
             if (m_showPeakLevel)
             {
+		int h = (m_peakLevelLeft * height()) / m_maxLevel;
+                y = height() - h;
+
+		if (h > loud) {
+		    paint->setPen(Qt::red); // brighter than the red meter bar
+		    paint->drawLine(0, y-1, hW - 2, y-1); 
+		    paint->drawLine(0, y+1, hW - 2, y+1);
+		}		    
+
                 paint->setPen(Qt::white);
-                paint->setBrush(Qt::white);
-
-                y = height() - (m_peakLevelLeft * height()) / m_maxLevel;
-
                 paint->drawLine(0, y, hW - 2, y);
             }
 
@@ -412,11 +449,18 @@ VUMeter::drawMeterLevel(QPainter* paint)
 
             if (m_showPeakLevel)
             {
+		int h = (m_peakLevelRight * height()) / m_maxLevel;
+                y = height() - h;
+
+		if (h > loud) {
+		    paint->setPen(Qt::red); // brighter than the red meter bar
+		    paint->drawLine(hW + 1, y-1, width(), y-1); 
+		    paint->drawLine(hW + 1, y+1, width(), y+1);
+		}		    
+
                 paint->setPen(Qt::white);
                 paint->setBrush(Qt::white);
 
-                y = height() - (m_peakLevelRight * height()) / m_maxLevel;
-
                 paint->drawLine(hW + 1, y, width(), y);
             }
         }
@@ -427,7 +471,7 @@ VUMeter::drawMeterLevel(QPainter* paint)
 	    paint->drawRect(0, 0, width(), height());
 
             int x = (m_levelLeft * width()) / m_maxLevel;
-            paint->drawRect(0, 0, x, height());
+	    if (x > 0) paint->drawRect(0, 0, x, height());
 
             if (m_showPeakLevel)
             {
@@ -476,7 +520,7 @@ VUMeter::drawMeterLevel(QPainter* paint)
         else
         {
             int x = (m_levelLeft * width()) / m_maxLevel;
-	    drawColouredBar(paint, 0, 0, 0, x, height());
+	    if (x > 0) drawColouredBar(paint, 0, 0, 0, x, height());
 
 	    paint->setPen(m_background);
 	    paint->setBrush(m_background);
