@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -40,6 +40,8 @@ public:
         PeakHold,
 	AudioPeakHoldShort, 
         AudioPeakHoldLong,
+	AudioPeakHoldIEC,
+	AudioPeakHoldIECLong,
         FixedHeightVisiblePeakHold
     } VUMeterType;
 
@@ -49,9 +51,8 @@ public:
         Vertical
     } VUAlignment;
 
-    // Mono and stereo level setting.  The AudioPeakHold meter type
-    // expects its levels in dB; other types expect levels between 0
-    // and 1.
+    // Mono and stereo level setting.  The AudioPeakHold meter types
+    // expect levels in dB; other types expect levels between 0 and 1.
     //
     void setLevel(double level);
     void setLevel(double leftLevel, double rightLevel);
