@@ -4,7 +4,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -35,6 +35,8 @@
 #include "rosegardensequencer.h"
 #include "rosedebug.h"
 
+#include "config.h"
+
 using std::cout;
 using std::cerr;
 using std::endl;
@@ -94,21 +96,15 @@ int main(int argc, char *argv[])
     KAboutData aboutData( "rosegardensequencer",
                           I18N_NOOP("RosegardenSequencer"),
                           VERSION, description, KAboutData::License_GPL,
-                          "(c) 2000-2004, Guillaume Laurent, Chris Cannam, Richard Bown");
+                          "(c) 2000-2005, Guillaume Laurent, Chris Cannam, Richard Bown");
     aboutData.addAuthor("Guillaume Laurent, Chris Cannam, Richard Bown",0, "glaurent@telegraph-road.org, cannam@all-day-breakfast.com, bownie@bownie.com");
     KCmdLineArgs::init( argc, argv, &aboutData );
     KCmdLineArgs::addCmdLineOptions( options ); // Add our own options.
 
     // Parse cmd line args
     //
-    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
+    /*KCmdLineArgs *args =*/ KCmdLineArgs::parsedArgs();
     KApplication app;
-    std::vector<std::string> jackArgs;
-
-    // Construct JACK args
-    //
-    for (int i = 0; i < args->count(); i++)
-        jackArgs.push_back(std::string(args->arg(i)));
 
     if (app.isRestored())
     {
@@ -116,12 +112,7 @@ int main(int argc, char *argv[])
     }
     else
     {
-        roseSeq = new RosegardenSequencerApp(jackArgs);
-
-        // we don't show() the sequencer application as we're just taking
-        // advantage of DCOP/KApplication and there's nothing to show().
-
-        args->clear();
+        roseSeq = new RosegardenSequencerApp;
     }
 
     QObject::connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));
