@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -21,6 +21,9 @@
 #include "mmappedsegment.h"
 #include "rosedebug.h"
 
+#include "MappedComposition.h"
+#include "Midi.h"
+
 using std::cerr;
 using std::endl;
 using std::cout;
@@ -124,10 +127,10 @@ bool MmappedSegment::remap(size_t newSize)
     }
 
 #ifdef linux
-    void *oldBuffer = m_mmappedRegion;
-    m_mmappedRegion = (MappedEvent*)::mremap(m_mmappedBuffer, m_mmappedSize, newSize, MREMAP_MAYMOVE);
-    if (m_mmappedRegion != oldBuffer) {
-	SEQUENCER_DEBUG << "NOTE: buffer moved from " << oldBuffer <<
+    void *oldRegion = m_mmappedRegion;
+    m_mmappedRegion = (MappedEvent*)::mremap(m_mmappedRegion, m_mmappedSize, newSize, MREMAP_MAYMOVE);
+    if (m_mmappedRegion != oldRegion) {
+	SEQUENCER_DEBUG << "NOTE: buffer moved from " << oldRegion <<
 	    " to " << (void *)m_mmappedRegion << endl;
     }
 #else
@@ -359,20 +362,38 @@ bool MmappedSegmentsMetaIterator::acceptEvent(MappedEvent *evt, bool evtIsFromMe
     if (evt->getType() == 0) return false; // discard those right away
 
     if (evtIsFromMetronome)
+    {
+        if (evt->getType() == MappedEvent::MidiSystemMessage && 
+                evt->getData1() == Rosegarden::MIDI_TIMING_CLOCK)
+        {
+            /*
+            std::cout << "MmappedSegmentsMetaIterator::acceptEvent - " 
+                      << "found clock" << std::endl;
+                      */
+            return true;
+        }
+
         return !m_controlBlockMmapper->isMetronomeMuted();
+    }
         
     // else, evt is not from metronome : first check if we're soloing (i.e. playing only the selected track)
     if (m_controlBlockMmapper->isSolo())
         return (evt->getTrackId() == m_controlBlockMmapper->getSelectedTrack());
 
     // finally we're not soloing, so check if track is muted
-    return !m_controlBlockMmapper->isTrackMuted(evt->getTrackId());
-                     
+    Rosegarden::TrackId track = evt->getTrackId();
+    bool muted = m_controlBlockMmapper->isTrackMuted(evt->getTrackId());
+
+#ifdef DEBUG_META_ITERATOR
+    SEQUENCER_DEBUG << "track " << track << " muted status: " << muted << endl;
+#endif
+    
+    return !muted;
 }
 
 
 bool
-MmappedSegmentsMetaIterator::fillCompositionWithEventsUntil(bool firstFetch,
+MmappedSegmentsMetaIterator::fillCompositionWithEventsUntil(bool /*firstFetch*/,
                                                             Rosegarden::MappedComposition* c,
                                                             const Rosegarden::RealTime& startTime,
                                                             const Rosegarden::RealTime& endTime)
@@ -459,7 +480,6 @@ MmappedSegmentsMetaIterator::fillCompositionWithEventsUntil(bool firstFetch,
                                 << " - data1: " << (unsigned int)evt->getData1()
                                 << " - data2: " << (unsigned int)evt->getData2()
                                 << " - metronome event: " << evtIsFromMetronome
-                                << " - firstFetch : " << firstFetch
                                 << endl;
 #endif
 
@@ -475,17 +495,40 @@ MmappedSegmentsMetaIterator::fillCompositionWithEventsUntil(bool firstFetch,
 
 		    c->insert(evt);
 
+                } else if (evt->getType() == MappedEvent::MidiSystemMessage &&
+
+			   // #1048388:
+			   // Ensure sysex heeds mute status, but ensure
+			   // clocks etc still get through
+			   evt->getData1() != Rosegarden::MIDI_SYSTEM_EXCLUSIVE) {
+
+                    c->insert(evt);
+
                 } else if (acceptEvent(evt, evtIsFromMetronome) &&
 
-			   (evt->getEventTime() + evt->getDuration() > startTime)) {
-                    //SEQUENCER_DEBUG << "inserting event\n";
+			   ((evt->getEventTime() + evt->getDuration() > startTime) ||
+			    (evt->getDuration() == Rosegarden::RealTime::zeroTime &&
+			     evt->getEventTime() == startTime))) {
+
+//                    std::cout << "inserting event" << std::endl;
+
+                    /*
+                    std::cout << "Inserting event (type = "
+                        << evt->getType() << ")" << std::endl;
+                        */
 
 
                     c->insert(evt);
 
                 } else {
                     
-                    //SEQUENCER_DEBUG << "skipping event\n";
+#ifdef DEBUG_META_ITERATOR
+                    std::cout << "skipping event"
+                        << " - event time = " << evt->getEventTime()
+                        << ", duration = " << evt->getDuration()
+                        << ", startTime = " << startTime << std::endl;
+#endif
+
 		    delete evt;
                 }
             
@@ -531,6 +574,41 @@ void MmappedSegmentsMetaIterator::resetIteratorForSegment(const QString& filenam
     }
 }
 
+void
+MmappedSegmentsMetaIterator::getAudioEvents(std::vector<MappedEvent> &v)
+{
+    v.clear();
+    
+    for (mmappedsegments::iterator i = m_segments.begin();
+	 i != m_segments.end(); ++i) {
+
+        MmappedSegment::iterator itr(i->second);
+
+        while (!itr.atEnd()) {
+	    
+	    if ((*itr).getType() != MappedEvent::Audio) {
+		++itr;
+		continue;
+	    }
+	    
+            MappedEvent evt(*itr);
+	    ++itr;
+
+            if (m_controlBlockMmapper->isTrackMuted(evt.getTrackId())) {
+		continue;
+            }
+
+            if (m_controlBlockMmapper->isSolo() == true && 
+                evt.getTrackId() != m_controlBlockMmapper->getSelectedTrack()) {
+                continue;
+            }
+
+	    v.push_back(evt);
+        }
+    }
+}
+
+
 std::vector<MappedEvent>& 
 MmappedSegmentsMetaIterator::getPlayingAudioFiles(const Rosegarden::RealTime &
 						  songPosition)
@@ -549,6 +627,7 @@ MmappedSegmentsMetaIterator::getPlayingAudioFiles(const Rosegarden::RealTime &
 
         bool found = false;
 
+	//!!! any point to this loop at all? can found ever fail?
         for (segmentiterators::iterator sI = m_iterators.begin();
 	     sI != m_iterators.end(); ++sI) {
             if ((*sI)->getSegment() == iter.getSegment())
