@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -37,7 +37,6 @@
 #include <qbuffer.h>
 
 #include "mmappedcontrolblock.h"
-#include "Sequencer.h"
 #include "MappedInstrument.h"
 
 using std::cerr;
@@ -45,6 +44,8 @@ using std::endl;
 using std::cout;
 using Rosegarden::MappedEvent;
 
+namespace Rosegarden { class MappedComposition; }
+
 // Seems not to be properly defined under some gcc 2.95 setups
 #ifndef MREMAP_MAYMOVE
 # define MREMAP_MAYMOVE        1
@@ -145,11 +146,14 @@ public:
     void addSegment(MmappedSegment*);
     void deleteSegment(MmappedSegment*);
 
+    void getAudioEvents(std::vector<MappedEvent> &);
+
     // Manipulate a vector of currently mapped audio segments so that we
     // can cross check them against PlayableAudioFiles (and stop if
     // necessary).  This will account for muting/soloing too I should
     // hope.
     //
+    //!!! to be obsoleted, hopefully
     std::vector<MappedEvent>& getPlayingAudioFiles
     (const Rosegarden::RealTime &songPosition);
 
