@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -42,20 +42,20 @@
 #include "mmappedsegment.h"
 #include "rosegardendcop.h"
 #include "ControlBlock.h"
-#include "Sequencer.h"
-#include "MappedInstrument.h"
 #include "SoundDriver.h"
+#include "SoundDriverFactory.h"
+#include "MappedInstrument.h"
 #include "Profiler.h"
+#include "PluginFactory.h"
 
 // The default latency and read-ahead values are actually sent
 // down from the GUI every time playback or recording starts
 // so the local values are kind of meaningless.
 //
 //
-RosegardenSequencerApp::RosegardenSequencerApp(
-        const std::vector<std::string> &jackArgs):
+RosegardenSequencerApp::RosegardenSequencerApp() :
     DCOPObject("RosegardenSequencerIface"),
-    m_sequencer(0),
+    m_driver(0),
     m_transportStatus(STOPPED),
     m_songPosition(0, 0),
     m_lastFetchSongPosition(0, 0),
@@ -93,25 +93,25 @@ RosegardenSequencerApp::RosegardenSequencerApp(
     // ALSA/JACK interface for both playback and recording. MappedStudio
     // aduio faders are also created.
     //
-    m_sequencer = new Rosegarden::Sequencer(m_studio, jackArgs);
-    m_studio->setSequencer(m_sequencer);
+    m_driver = Rosegarden::SoundDriverFactory::createDriver(m_studio);
+    m_studio->setSoundDriver(m_driver);
 
-    if (!m_sequencer)
+    if (!m_driver)
     {
         SEQUENCER_DEBUG << "RosegardenSequencer object could not be allocated"
                         << endl;
         close();
     }
 
-    m_sequencer->setAudioBufferSizes(m_audioMix, m_audioRead, m_audioWrite,
+    m_driver->setAudioBufferSizes(m_audioMix, m_audioRead, m_audioWrite,
 				     m_smallFileSize);
 
-    m_sequencer->setSequencerDataBlock(m_sequencerMapper.getSequencerDataBlock());
-    m_sequencer->setExternalTransportControl(this);
+    m_driver->setSequencerDataBlock(m_sequencerMapper.getSequencerDataBlock());
+    m_driver->setExternalTransportControl(this);
 
     // set this here and now so we can accept async midi events
     //
-    m_sequencer->record(Rosegarden::ASYNCHRONOUS_MIDI);
+    m_driver->record(Rosegarden::ASYNCHRONOUS_MIDI);
 
     // Check for new clients every so often
     //
@@ -125,9 +125,9 @@ RosegardenSequencerApp::RosegardenSequencerApp(
 RosegardenSequencerApp::~RosegardenSequencerApp()
 {
     SEQUENCER_DEBUG << "RosegardenSequencer - shutting down" << endl;
-    m_sequencer->shutdown();
-    delete m_sequencer;
+    m_driver->shutdown();
     delete m_studio;
+    delete m_driver;
     delete m_controlBlockMmapper;
 }
 
@@ -153,7 +153,7 @@ RosegardenSequencerApp::stop()
     SEQUENCER_DEBUG << "RosegardenSequencerApp::stop() - stopping" << endl;
 
     // process pending NOTE OFFs and stop the Sequencer
-    m_sequencer->stopPlayback();
+    m_driver->stopPlayback();
 
     // the Sequencer doesn't need to know these once
     // we've stopped.
@@ -227,6 +227,7 @@ RosegardenSequencerApp::fetchEvents(Rosegarden::MappedComposition &composition,
     else
 */
     getSlice(composition, start, end, firstFetch);
+    applyLatencyCompensation(composition);
 }
 
 
@@ -253,6 +254,24 @@ RosegardenSequencerApp::getSlice(Rosegarden::MappedComposition &composition,
 }
 
 
+void
+RosegardenSequencerApp::applyLatencyCompensation(Rosegarden::MappedComposition &composition)
+{
+    Rosegarden::RealTime maxLatency = m_driver->getMaximumPlayLatency();
+    if (maxLatency == Rosegarden::RealTime::zeroTime) return;
+
+    for (Rosegarden::MappedComposition::iterator i = composition.begin();
+	 i != composition.end(); ++i) {
+
+	Rosegarden::RealTime instrumentLatency =
+	    m_driver->getInstrumentPlayLatency((*i)->getInstrument());
+
+	(*i)->setEventTime((*i)->getEventTime() +
+			   maxLatency - instrumentLatency);
+    }
+}
+	     
+
 // The first fetch of events from the core/ and initialisation for
 // this session of playback.  We fetch up to m_readAhead ahead at
 // first at then top up at each slice.
@@ -266,7 +285,7 @@ RosegardenSequencerApp::startPlaying()
 
     // This will reset the Sequencer's internal clock
     // ready for new playback
-    m_sequencer->initialisePlayback(m_songPosition);
+    m_driver->initialisePlayback(m_songPosition);
 
     m_mC.clear();
     fetchEvents(m_mC, m_songPosition, m_songPosition + m_readAhead, true);
@@ -274,14 +293,18 @@ RosegardenSequencerApp::startPlaying()
     // process whether we need to or not as this also processes
     // the audio queue for us
     //
-    m_sequencer->processEventsOut(m_mC, false);
+    m_driver->processEventsOut(m_mC, m_songPosition, m_songPosition + m_readAhead);
+
+    std::vector<Rosegarden::MappedEvent> audioEvents;
+    m_metaIterator->getAudioEvents(audioEvents);
+    m_driver->initialiseAudioQueue(audioEvents);
 
 //    SEQUENCER_DEBUG << "RosegardenSequencerApp::startPlaying: pausing to simulate high-load environment" << endl;
-//    ::sleep(1);
+//    ::sleep(2);
 
     // and only now do we signal to start the clock
     //
-    m_sequencer->startClocks();
+    m_driver->startClocks();
 
     incrementTransportToken();
 
@@ -301,13 +324,16 @@ RosegardenSequencerApp::keepPlaying()
     }
     if (fetchEnd > m_lastFetchSongPosition) {
 	fetchEvents(m_mC, m_lastFetchSongPosition, fetchEnd, false);
-	m_lastFetchSongPosition = fetchEnd;
     }
 
     // Again, process whether we need to or not to keep
     // the Sequencer up-to-date with audio events
     //
-    m_sequencer->processEventsOut(m_mC, false);
+    m_driver->processEventsOut(m_mC, m_lastFetchSongPosition, fetchEnd);
+
+    if (fetchEnd > m_lastFetchSongPosition) {
+	m_lastFetchSongPosition = fetchEnd;
+    }
     
     return true; // !isEndOfCompReached(); - until we sort this out, we don't stop at end of comp.
 }
@@ -319,14 +345,12 @@ RosegardenSequencerApp::updateClocks()
 {
     Rosegarden::Profiler profiler("RosegardenSequencerApp::updateClocks");
 
+    m_driver->runTasks();
+
     checkExternalTransport();
 
 //SEQUENCER_DEBUG << "RosegardenSequencerApp::updateClocks" << endl; 
 
-    // Attempt to send MIDI clock 
-    //
-    m_sequencer->sendMidiClock();
-
     // If we're not playing etc. then that's all we need to do
     //
     if (m_transportStatus != PLAYING &&
@@ -334,36 +358,38 @@ RosegardenSequencerApp::updateClocks()
         m_transportStatus != RECORDING_AUDIO)
         return;
 
-    Rosegarden::RealTime newPosition = m_sequencer->getSequencerTime();
+    Rosegarden::RealTime newPosition = m_driver->getSequencerTime();
 
     // Go around the loop if we've reached the end
     //
     if (isLooping() && newPosition >= m_loopEnd)
     {
+	Rosegarden::RealTime oldPosition = m_songPosition;
+
         // Remove the loop width from the song position and send
         // this position to the GUI
         //
         newPosition = m_songPosition = m_lastFetchSongPosition = m_loopStart;
 
-	m_sequencer->stopClocks();
+	m_driver->stopClocks();
 
         // Reset playback using this jump
         //
-        m_sequencer->resetPlayback(m_songPosition);
+        m_driver->resetPlayback(oldPosition, m_songPosition);
 
 	m_mC.clear();
 	fetchEvents(m_mC, m_songPosition, m_songPosition + m_readAhead, true);
 
-	m_sequencer->processEventsOut(m_mC, false);
+	m_driver->processEventsOut(m_mC, m_songPosition, m_songPosition + m_readAhead);
 
-	m_sequencer->startClocks();
+	m_driver->startClocks();
     }
     else
     {
         m_songPosition = newPosition;
 
-        if (m_songPosition <= m_sequencer->getStartPosition())
-            newPosition = m_sequencer->getStartPosition();
+        if (m_songPosition <= m_driver->getStartPosition())
+            newPosition = m_driver->getStartPosition();
     }
 
     // Remap the position pointer
@@ -398,7 +424,7 @@ RosegardenSequencerApp::notifySequencerStatus()
 void
 RosegardenSequencerApp::sleep(const Rosegarden::RealTime &rt)
 {
-    m_sequencer->sleep(rt);
+    m_driver->sleep(rt);
 }
 
 
@@ -413,7 +439,9 @@ RosegardenSequencerApp::jumpTo(long posSec, long posNsec)
     if (posSec < 0 && posNsec < 0)
         return;
 
-    m_sequencer->stopClocks();
+    m_driver->stopClocks();
+
+    Rosegarden::RealTime oldPosition = m_songPosition;
 
     m_songPosition = m_lastFetchSongPosition =
 	Rosegarden::RealTime(posSec, posNsec);
@@ -423,7 +451,7 @@ RosegardenSequencerApp::jumpTo(long posSec, long posNsec)
 	    (m_songPosition);
     }
 
-    m_sequencer->resetPlayback(m_songPosition);
+    m_driver->resetPlayback(oldPosition, m_songPosition);
 
     // Now prebuffer as in startPlaying:
 
@@ -433,12 +461,12 @@ RosegardenSequencerApp::jumpTo(long posSec, long posNsec)
     // process whether we need to or not as this also processes
     // the audio queue for us
     //
-    m_sequencer->processEventsOut(m_mC, false);
+    m_driver->processEventsOut(m_mC, m_songPosition, m_songPosition + m_readAhead);
 
 //!!!    SEQUENCER_DEBUG << "RosegardenSequencerApp::jumpTo: pausing to simulate high-load environment" << endl;
 //!!!    ::sleep(1);
 
-    m_sequencer->startClocks();
+    m_driver->startClocks();
     
     incrementTransportToken();
 
@@ -450,7 +478,7 @@ RosegardenSequencerApp::jumpTo(long posSec, long posNsec)
 void
 RosegardenSequencerApp::processRecordedMidi()
 {
-    Rosegarden::MappedComposition *mC = m_sequencer->getMappedComposition();
+    Rosegarden::MappedComposition *mC = m_driver->getMappedComposition();
 
     if (mC->empty() || !m_controlBlockMmapper) return;
 
@@ -465,7 +493,7 @@ RosegardenSequencerApp::processRecordedMidi()
     m_sequencerMapper.updateRecordingBuffer(mC);
 
     applyFiltering(mC, m_controlBlockMmapper->getThruFilter());
-    m_sequencer->processEventsOut(*mC, true);
+    m_driver->processEventsOut(*mC);
 }
 
 
@@ -474,11 +502,8 @@ RosegardenSequencerApp::processRecordedMidi()
 void
 RosegardenSequencerApp::processRecordedAudio()
 {
-    //!!! Nothing to do here: the recording time is sent back to the GUI
-    // in the sequencer mapper as a normal case.  What we probably
-    // should be doing is updating the sequencer mapper with levels,
-    // instead of sending them through mapped events.  Next thing to
-    // do I guess...
+    // Nothing to do here: the recording time is sent back to the GUI
+    // in the sequencer mapper as a normal case.
 }
 
 
@@ -490,10 +515,10 @@ RosegardenSequencerApp::processRecordedAudio()
 void
 RosegardenSequencerApp::processAsynchronousEvents()
 {
-    Rosegarden::MappedComposition *mC = m_sequencer->getMappedComposition();
+    Rosegarden::MappedComposition *mC = m_driver->getMappedComposition();
 
     if (mC->empty()) {
-	m_sequencer->processPending();
+	m_driver->processPending();
 	return;
     }
 
@@ -511,7 +536,7 @@ RosegardenSequencerApp::processAsynchronousEvents()
 	m_sequencerMapper.setControlBlock(m_controlBlockMmapper->getControlBlock());
     }
 
-    SEQUENCER_DEBUG << "processAsynchronousEvents: have " << mC->size() << " events" << endl;
+//    SEQUENCER_DEBUG << "processAsynchronousEvents: have " << mC->size() << " events" << endl;
 
     int instrumentId = m_controlBlockMmapper->getInstrumentForTrack
 	(m_controlBlockMmapper->getSelectedTrack());
@@ -525,9 +550,9 @@ RosegardenSequencerApp::processAsynchronousEvents()
     arg << mC;
 
     applyFiltering(mC, m_controlBlockMmapper->getThruFilter());
-    m_sequencer->processEventsOut(*mC, true);
+    m_driver->processEventsOut(*mC);
 
-    SEQUENCER_DEBUG << "processAsynchronousEvents: sent " << mC->size() << " events" << endl;
+//    SEQUENCER_DEBUG << "processAsynchronousEvents: sent " << mC->size() << " events" << endl;
 
     if (!kapp->dcopClient()->send(ROSEGARDEN_GUI_APP_NAME,
                                  ROSEGARDEN_GUI_IFACE_NAME,
@@ -544,7 +569,7 @@ RosegardenSequencerApp::processAsynchronousEvents()
     // Process any pending events (Note Offs or Audio) as part of
     // same procedure.
     //
-    m_sequencer->processPending();
+    m_driver->processPending();
 }
 
 
@@ -569,7 +594,7 @@ RosegardenSequencerApp::record(const Rosegarden::RealTime &time,
                                const Rosegarden::RealTime &audioRead,
                                const Rosegarden::RealTime &audioWrite,
 			       long smallFileSize,
-                               int recordMode)
+                               long recordMode)
 {
     TransportStatus localRecordMode = (TransportStatus) recordMode;
 
@@ -578,7 +603,7 @@ RosegardenSequencerApp::record(const Rosegarden::RealTime &time,
     {
         if (localRecordMode == STARTING_TO_RECORD_MIDI)
         {
-            if(m_sequencer->record(Rosegarden::RECORD_MIDI) == false)
+            if(m_driver->record(Rosegarden::RECORD_MIDI) == false)
             {
                 stop();
                 return 0;
@@ -604,7 +629,7 @@ RosegardenSequencerApp::record(const Rosegarden::RealTime &time,
         // Get the Sequencer to prepare itself for recording - if
         // this fails we stop.
         //
-        if (m_sequencer->record(Rosegarden::RECORD_MIDI) == false)
+        if (m_driver->record(Rosegarden::RECORD_MIDI) == false)
         {
             stop();
             return 0;
@@ -643,10 +668,10 @@ RosegardenSequencerApp::record(const Rosegarden::RealTime &time,
         }
 
         // set recording filename
-        m_sequencer->setRecordingFilename(std::string(audioFileName.data()));
+        m_driver->setRecordingFilename(std::string(audioFileName.data()));
 
         // set recording
-        if (m_sequencer->record(Rosegarden::RECORD_AUDIO) == false)
+        if (m_driver->record(Rosegarden::RECORD_AUDIO) == false)
         {
             SEQUENCER_DEBUG << "couldn't start recording - "
                             << "perhaps audio file path wrong?"
@@ -669,7 +694,7 @@ RosegardenSequencerApp::record(const Rosegarden::RealTime &time,
 
     // Ensure that playback is initialised
     //
-    m_sequencer->initialisePlayback(m_songPosition);
+    m_driver->initialisePlayback(m_songPosition);
 
     return play(time, readAhead, audioMix, audioRead, audioWrite, smallFileSize);
 }
@@ -720,9 +745,7 @@ RosegardenSequencerApp::play(const Rosegarden::RealTime &time,
         m_transportStatus = STARTING_TO_PLAY;
     }
 
-    //!!! Stop the clocks while we do all the dirty work.  We'll
-    //restart them at the end of startPlaying().
-    m_sequencer->stopClocks();
+    m_driver->stopClocks();
 
     // Set up buffer size
     //
@@ -735,7 +758,7 @@ RosegardenSequencerApp::play(const Rosegarden::RealTime &time,
     m_audioWrite = audioWrite;
     m_smallFileSize = smallFileSize;
 
-    m_sequencer->setAudioBufferSizes(m_audioMix, m_audioRead, m_audioWrite,
+    m_driver->setAudioBufferSizes(m_audioMix, m_audioRead, m_audioWrite,
 				     m_smallFileSize);
 
     cleanupMmapData();
@@ -948,7 +971,7 @@ RosegardenSequencerApp::record(long timeSec,
 			       long audioWriteSec,
 			       long audioWriteNsec,
 			       long smallFileSize,
-                               int recordMode)
+                               long recordMode)
 
 {
     return record(Rosegarden::RealTime(timeSec, timeNSec),
@@ -968,7 +991,7 @@ RosegardenSequencerApp::setLoop(const Rosegarden::RealTime &loopStart,
     m_loopStart = loopStart;
     m_loopEnd = loopEnd;
 
-    m_sequencer->setLoop(loopStart, loopEnd);
+    m_driver->setLoop(loopStart, loopEnd);
 }
 
 
@@ -988,7 +1011,7 @@ RosegardenSequencerApp::setLoop(long loopStartSec,
 unsigned int
 RosegardenSequencerApp::getSoundDriverStatus()
 {
-    return m_sequencer->getSoundDriverStatus();
+    return m_driver->getStatus();
 }
 
 
@@ -996,19 +1019,19 @@ RosegardenSequencerApp::getSoundDriverStatus()
 int
 RosegardenSequencerApp::addAudioFile(const QString &fileName, int id)
 {
-    return((int)m_sequencer->addAudioFile(fileName.utf8().data(), id));
+    return((int)m_driver->addAudioFile(fileName.utf8().data(), id));
 }
 
 int
 RosegardenSequencerApp::removeAudioFile(int id)
 {
-    return((int)m_sequencer->removeAudioFile(id));
+    return((int)m_driver->removeAudioFile(id));
 }
 
 void
 RosegardenSequencerApp::clearAllAudioFiles()
 {
-    m_sequencer->clearAudioFiles();
+    m_driver->clearAudioFiles();
 }
 
 void
@@ -1020,7 +1043,7 @@ RosegardenSequencerApp::setMappedInstrument(int type, unsigned char channel,
         (Rosegarden::Instrument::InstrumentType)type;
     Rosegarden::MidiByte mChannel = (Rosegarden::MidiByte)channel;
 
-    m_sequencer->setMappedInstrument(
+    m_driver->setMappedInstrument(
             new Rosegarden::MappedInstrument (mType, mChannel, mID));
 
 }
@@ -1033,7 +1056,7 @@ RosegardenSequencerApp::processSequencerSlice(Rosegarden::MappedComposition mC)
 {
     // Use the "now" API
     //
-    m_sequencer->processEventsOut(mC, true);
+    m_driver->processEventsOut(mC);
 }
 
 void
@@ -1060,8 +1083,7 @@ RosegardenSequencerApp::processMappedEvent(unsigned int id,
 
     Rosegarden::MappedComposition mC;
 
-    SEQUENCER_DEBUG << "processMappedEvent() - sending out single event"
-                    << endl;
+//    SEQUENCER_DEBUG << "processMappedEvent(data) - sending out single event at time " << mE->getEventTime() << endl;
 
     /*
     std::cout << "ID = " << mE->getInstrument() << std::endl;
@@ -1072,7 +1094,7 @@ RosegardenSequencerApp::processMappedEvent(unsigned int id,
 
     mC.insert(mE);
 
-    m_sequencer->processEventsOut(mC, true);
+    m_driver->processEventsOut(mC);
 }
 
 void
@@ -1080,7 +1102,9 @@ RosegardenSequencerApp::processMappedEvent(MappedEvent mE)
 {
     Rosegarden::MappedComposition mC;
     mC.insert(new MappedEvent(mE));
-    m_sequencer->processEventsOut(mC, true);
+    SEQUENCER_DEBUG << "processMappedEvent(ev) - sending out single event at time " << mE.getEventTime() << endl;
+
+    m_driver->processEventsOut(mC);
 }
 
 // Get the MappedDevice (DCOP wrapped vector of MappedInstruments)
@@ -1088,82 +1112,92 @@ RosegardenSequencerApp::processMappedEvent(MappedEvent mE)
 Rosegarden::MappedDevice
 RosegardenSequencerApp::getMappedDevice(unsigned int id)
 {
-    return m_sequencer->getMappedDevice(id);
+    return m_driver->getMappedDevice(id);
 }
 
 unsigned int
 RosegardenSequencerApp::getDevices()
 {
-    return m_sequencer->getDevices();
+    return m_driver->getDevices();
 }
 
 int
 RosegardenSequencerApp::canReconnect(int type)
 {
-    return m_sequencer->canReconnect(type);
+    return m_driver->canReconnect((Rosegarden::Device::DeviceType)type);
 }
 
 unsigned int
 RosegardenSequencerApp::addDevice(int type, unsigned int direction)
 {
-    return m_sequencer->addDevice(type, direction);
+    return m_driver->addDevice((Rosegarden::Device::DeviceType)type,
+			       (Rosegarden::MidiDevice::DeviceDirection)direction);
 }
 
 void
 RosegardenSequencerApp::removeDevice(unsigned int deviceId)
 {
-    m_sequencer->removeDevice(deviceId);
+    m_driver->removeDevice(deviceId);
 }
 
 unsigned int
 RosegardenSequencerApp::getConnections(int type, unsigned int direction)
 {
-    return m_sequencer->getConnections(type, direction);
+    return m_driver->getConnections((Rosegarden::Device::DeviceType)type,
+				    (Rosegarden::MidiDevice::DeviceDirection)direction);
 }
 
 QString
 RosegardenSequencerApp::getConnection(int type, unsigned int direction,
 				      unsigned int connectionNo)
 {
-    return m_sequencer->getConnection(type, direction, connectionNo);
+    return m_driver->getConnection((Rosegarden::Device::DeviceType)type,
+				   (Rosegarden::MidiDevice::DeviceDirection)direction,
+				   connectionNo);
 }
 
 void
 RosegardenSequencerApp::setConnection(unsigned int deviceId,
 				      QString connection)
 {
-    m_sequencer->setConnection(deviceId, connection);
+    m_driver->setConnection(deviceId, connection);
 }
 
 void
 RosegardenSequencerApp::setPlausibleConnection(unsigned int deviceId,
 					       QString connection)
 {
-    m_sequencer->setPlausibleConnection(deviceId, connection);
+    m_driver->setPlausibleConnection(deviceId, connection);
 }
 
 unsigned int
 RosegardenSequencerApp::getTimers()
 {
-    return m_sequencer->getTimers();
+    return m_driver->getTimers();
 }
 
 QString
 RosegardenSequencerApp::getTimer(unsigned int n)
 {
-    return m_sequencer->getTimer(n);
+    return m_driver->getTimer(n);
 }
 
 QString
 RosegardenSequencerApp::getCurrentTimer()
 {
-    return m_sequencer->getCurrentTimer();
+    return m_driver->getCurrentTimer();
 }
 
 void
 RosegardenSequencerApp::setCurrentTimer(QString timer)
 {
-    m_sequencer->setCurrentTimer(timer);
+    m_driver->setCurrentTimer(timer);
+}
+
+void
+RosegardenSequencerApp::setLowLatencyMode(bool ll)
+{
+    m_driver->setLowLatencyMode(ll);
 }
 
 void
@@ -1200,20 +1234,20 @@ RosegardenSequencerApp::setAudioMonitoring(long value)
     std::vector<unsigned int> inputPorts;
 
     if (bValue &&
-            m_sequencer->getRecordStatus() == Rosegarden::ASYNCHRONOUS_MIDI)
+            m_driver->getRecordStatus() == Rosegarden::ASYNCHRONOUS_MIDI)
     {
-        m_sequencer->record(Rosegarden::ASYNCHRONOUS_AUDIO);
+        m_driver->record(Rosegarden::ASYNCHRONOUS_AUDIO);
         SEQUENCER_DEBUG << "RosegardenSequencerApp::setAudioMonitoring - "
                         << "monitoring audio input" << endl;
         return;
     }
 
     if (bValue == false &&
-            m_sequencer->getRecordStatus() == Rosegarden::ASYNCHRONOUS_AUDIO)
+            m_driver->getRecordStatus() == Rosegarden::ASYNCHRONOUS_AUDIO)
     {
         SEQUENCER_DEBUG << "RosegardenSequencerApp::setAudioMonitoring - "
                         << "monitoring MIDI input" << endl;
-        m_sequencer->record(Rosegarden::ASYNCHRONOUS_MIDI);
+        m_driver->record(Rosegarden::ASYNCHRONOUS_MIDI);
     }
     
 }
@@ -1221,20 +1255,20 @@ RosegardenSequencerApp::setAudioMonitoring(long value)
 void
 RosegardenSequencerApp::setAudioMonitoringInstrument(unsigned int id)
 {
-    m_sequencer->setAudioMonitoringInstrument(id);
+    m_driver->setAudioMonitoringInstrument(id);
 }
 
 
 Rosegarden::MappedRealTime
 RosegardenSequencerApp::getAudioPlayLatency()
 {
-    return Rosegarden::MappedRealTime(m_sequencer->getAudioPlayLatency());
+    return Rosegarden::MappedRealTime(m_driver->getAudioPlayLatency());
 }
 
 Rosegarden::MappedRealTime
 RosegardenSequencerApp::getAudioRecordLatency()
 {
-    return Rosegarden::MappedRealTime(m_sequencer->getAudioRecordLatency());
+    return Rosegarden::MappedRealTime(m_driver->getAudioRecordLatency());
 }
 
 // Initialise the virtual studio with a few audio faders and
@@ -1248,20 +1282,6 @@ RosegardenSequencerApp::initialiseStudio()
     // clear down the studio before we start adding anything
     //
     m_studio->clear();
-
-    // Create a plugin manager
-    //
-    Rosegarden::MappedAudioPluginManager *pM =
-      dynamic_cast<Rosegarden::MappedAudioPluginManager*>(
-        m_studio->createObject(
-            Rosegarden::MappedObject::AudioPluginManager, true)); // read-only
-
-    if (pM)
-        SEQUENCER_DEBUG << "created plugin manager" << endl;
-
-    // This creates new MappedPlugin objects under the studio
-    //
-    pM->discoverPlugins(m_studio);
 }
 
 
@@ -1271,6 +1291,46 @@ RosegardenSequencerApp::setMappedProperty(int id,
                                           float value)
 {
 
+//    SEQUENCER_DEBUG << "setProperty: id = " << id
+//                    << " : property = \"" << property << "\""
+//		    << ", value = " << value << endl;
+
+
+    Rosegarden::MappedObject *object = m_studio->getObjectById(id);
+
+    if (object)
+        object->setProperty(property, value);
+}
+
+void
+RosegardenSequencerApp::setMappedProperties(const Rosegarden::MappedObjectIdList &ids,
+					    const Rosegarden::MappedObjectPropertyList &properties,
+					    const Rosegarden::MappedObjectValueList &values)
+{
+    Rosegarden::MappedObject *object = 0;
+    Rosegarden::MappedObjectId prevId = 0;
+
+    for (size_t i = 0;
+	 i < ids.size() && i < properties.size() && i < values.size();
+	 ++i) {
+
+	if (i == 0 || ids[i] != prevId) {
+	    object = m_studio->getObjectById(ids[i]);
+	    prevId = ids[i];
+	}
+
+	if (object) {
+	    object->setProperty(properties[i], values[i]);
+	}
+    }
+}
+
+void
+RosegardenSequencerApp::setMappedProperty(int id,
+                                          const QString &property,
+                                          const QString &value)
+{
+
     SEQUENCER_DEBUG << "setProperty: id = " << id
                     << " : property = \"" << property << "\""
 		    << ", value = " << value << endl;
@@ -1283,19 +1343,30 @@ RosegardenSequencerApp::setMappedProperty(int id,
 }
 
 void
-RosegardenSequencerApp::setMappedProperty(int id, const QString &property,
-        Rosegarden::MappedObjectValueList value)
+RosegardenSequencerApp::setMappedPropertyList(int id, const QString &property,
+					      const Rosegarden::MappedObjectPropertyList &values)
 {
-    SEQUENCER_DEBUG << "setProperty: id = " << id
-                    << " : property list size = \"" << value.size()
+    SEQUENCER_DEBUG << "setPropertyList: id = " << id
+                    << " : property list size = \"" << values.size()
                     << "\"" << endl;
 
     Rosegarden::MappedObject *object = m_studio->getObjectById(id);
 
-    if (object)
-        object->setProperty(property, value);
+    if (object) {
+	try {
+	    object->setPropertyList(property, values);
+	} catch (QString err) {
+	    QByteArray data;
+	    QDataStream arg(data, IO_WriteOnly);
+	    arg << err;
+	    kapp->dcopClient()->send(ROSEGARDEN_GUI_APP_NAME,
+				     ROSEGARDEN_GUI_IFACE_NAME,
+				     "showError(QString)",
+				     data);
+	}
+    }
 }
-
+	
 
 int
 RosegardenSequencerApp::getMappedObjectId(int type)
@@ -1335,11 +1406,53 @@ RosegardenSequencerApp::getPropertyList(int id,
     return list;
 }
 
+std::vector<QString>
+RosegardenSequencerApp::getPluginInformation()
+{
+    std::vector<QString> list;
+
+    Rosegarden::PluginFactory::enumerateAllPlugins(list);
+
+    return list;
+}
+
+QString
+RosegardenSequencerApp::getPluginProgram(int id, int bank, int program)
+{
+    Rosegarden::MappedObject *object = m_studio->getObjectById(id);
+
+    if (object) {
+	Rosegarden::MappedPluginSlot *slot =
+	    dynamic_cast<Rosegarden::MappedPluginSlot *>(object);
+	if (slot) {
+	    return slot->getProgram(bank, program);
+	}
+    }
+
+    return QString();
+}
+
+unsigned long
+RosegardenSequencerApp::getPluginProgram(int id, const QString &name)
+{
+    Rosegarden::MappedObject *object = m_studio->getObjectById(id);
+
+    if (object) {
+	Rosegarden::MappedPluginSlot *slot =
+	    dynamic_cast<Rosegarden::MappedPluginSlot *>(object);
+	if (slot) {
+	    return slot->getProgram(name);
+	}
+    }
+
+    return 0;
+}
+
 unsigned int
 RosegardenSequencerApp::getSampleRate() const
 {
-    if (m_sequencer)
-        return m_sequencer->getSampleRate();
+    if (m_driver)
+        return m_driver->getSampleRate();
 
     return 0;
 }
@@ -1351,7 +1464,7 @@ RosegardenSequencerApp::createMappedObject(int type)
 {
     Rosegarden::MappedObject *object =
               m_studio->createObject(
-                      Rosegarden::MappedObject::MappedObjectType(type), false);
+                      Rosegarden::MappedObject::MappedObjectType(type));
 
     if (object)
     {
@@ -1369,21 +1482,6 @@ RosegardenSequencerApp::createMappedObject(int type)
 int 
 RosegardenSequencerApp::destroyMappedObject(int id)
 {
-#ifdef HAVE_LADSPA
-    Rosegarden::MappedLADSPAPlugin *plugin =
-        dynamic_cast<Rosegarden::MappedLADSPAPlugin*>
-	(m_studio->getObjectByIdAndType
-	 (id, Rosegarden::MappedObject::LADSPAPlugin));
-
-    if (plugin)
-    {
-        std::cout << "RosegardenSequencerApp::destroyMappedObject - "
-                  << "removing plugin instance" << std::endl;
-        m_sequencer->removePluginInstance(plugin->getInstrument(),
-                                          plugin->getPosition());
-    }
-#endif
-
     return(int(m_studio->destroyObject(Rosegarden::MappedObjectId(id))));
 }
 
@@ -1400,7 +1498,7 @@ RosegardenSequencerApp::connectMappedObjects(int id1, int id2)
     if (m_transportStatus == PLAYING ||
 	m_transportStatus == RECORDING_MIDI ||
 	m_transportStatus == RECORDING_AUDIO) {
-	Rosegarden::RealTime seqTime = m_sequencer->getSequencerTime();
+	Rosegarden::RealTime seqTime = m_driver->getSequencerTime();
 	jumpTo(seqTime.sec, seqTime.nsec);
     }
 }
@@ -1427,8 +1525,7 @@ void
 RosegardenSequencerApp::clearStudio()
 {
     SEQUENCER_DEBUG << "clearStudio()" << endl;
-    m_sequencer->removePluginInstances();
-    m_studio->clearTemporaries();
+    m_studio->clear();
     m_sequencerMapper.getSequencerDataBlock()->clearTemporaries();
 
 } 
@@ -1441,19 +1538,33 @@ RosegardenSequencerApp::setMappedPort(int pluginId,
     Rosegarden::MappedObject *object =
         m_studio->getObjectById(pluginId);
 
-#ifdef HAVE_LADSPA
+    Rosegarden::MappedPluginSlot *slot = 
+        dynamic_cast<Rosegarden::MappedPluginSlot *>(object);
 
-    Rosegarden::MappedLADSPAPlugin *plugin = 
-        dynamic_cast<Rosegarden::MappedLADSPAPlugin*>(object);
-
-    if (plugin)
-    {
-        plugin->setPort(portId, value);
+    if (slot) {
+        slot->setPort(portId, value);
+    } else {
+	SEQUENCER_DEBUG << "no such slot" << endl;
     }
+}
 
+float
+RosegardenSequencerApp::getMappedPort(int pluginId,
+                                      unsigned long portId)
+{
+    Rosegarden::MappedObject *object =
+        m_studio->getObjectById(pluginId);
 
-#endif // HAVE_LADSPA
+    Rosegarden::MappedPluginSlot *slot = 
+        dynamic_cast<Rosegarden::MappedPluginSlot *>(object);
 
+    if (slot) {
+        return slot->getPort(portId);
+    } else {
+	SEQUENCER_DEBUG << "no such slot" << endl;
+    }
+    
+    return 0;
 }
 
 void
@@ -1466,7 +1577,7 @@ RosegardenSequencerApp::slotCheckForNewClients()
         m_transportStatus == RECORDING_AUDIO)
         return;
 
-    if (m_sequencer->checkForNewClients())
+    if (m_driver->checkForNewClients())
     {
         SEQUENCER_DEBUG << "client list changed" << endl;
     }
@@ -1481,14 +1592,14 @@ RosegardenSequencerApp::setQuarterNoteLength(long timeSec, long timeNSec)
     SEQUENCER_DEBUG << "RosegardenSequencerApp::setQuarterNoteLength"
                     << Rosegarden::RealTime(timeSec, timeNSec) << endl;
 
-    m_sequencer->setMIDIClockInterval(
+    m_driver->setMIDIClockInterval(
             Rosegarden::RealTime(timeSec, timeNSec) / 24);
 }
 
 QString
 RosegardenSequencerApp::getStatusLog()
 {
-    return m_sequencer->getStatusLog();
+    return m_driver->getStatusLog();
 }
 
 
@@ -1524,12 +1635,9 @@ void RosegardenSequencerApp::dumpFirstSegment()
 void
 RosegardenSequencerApp::rationalisePlayingAudio()
 {
-    if (!m_metaIterator) return;
-
-    std::vector<MappedEvent> &segmentAudio =
-	m_metaIterator->getPlayingAudioFiles(m_songPosition);
-
-    m_sequencer->rationalisePlayingAudio(segmentAudio, m_songPosition);
+    std::vector<Rosegarden::MappedEvent> audioEvents;
+    m_metaIterator->getAudioEvents(audioEvents);
+    m_driver->initialiseAudioQueue(audioEvents);
 }
 
 
@@ -1539,7 +1647,7 @@ RosegardenSequencerApp::transportChange(TransportRequest request)
     TransportPair pair(request, Rosegarden::RealTime::zeroTime);
     m_transportRequests.push_back(pair);
 
-    std::cout << "RosegardenSequencerApp::transportJump: " << request << std::endl;
+    std::cout << "RosegardenSequencerApp::transportChange: " << request << std::endl;
 
     if (request == TransportNoChange) return m_transportToken;
     else return m_transportToken + 1;
