@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -72,7 +72,7 @@ public:
 		       long audioWriteSec,
 		       long audioWriteNsec,
 		       long smallFileSize,
-                       int recordMode) = 0;
+                       long recordMode) = 0;
 
     // stop the sequencer
     //
@@ -214,6 +214,8 @@ public:
     //
     virtual void setCurrentTimer(QString timer) = 0;
 
+    virtual void setLowLatencyMode(bool lowLatMode) = 0;
+
     // Set audio monitoring Instrument - tells the sequencer that the
     // gui is currently monitoring audio and which Instrument to report
     // the input level against - this is so we can get a real time
@@ -234,12 +236,24 @@ public:
                                    const QString &property,
                                    float value) = 0;
 
+    // Set many properties on many MappedObjects
+    //
+    virtual void setMappedProperties(const Rosegarden::MappedObjectIdList &ids,
+				     const Rosegarden::MappedObjectPropertyList &properties,
+				     const Rosegarden::MappedObjectValueList &values) = 0;
+
+    // Set a string property on a MappedObject
+    //
+    virtual void setMappedProperty(int id,
+                                   const QString &property,
+                                   const QString &value) = 0;
+
     // Set a MappedObject to a property list
     //
-    virtual void setMappedProperty(
+    virtual void setMappedPropertyList(
                                int id,
                                const QString &property,
-                               std::vector<float> value) = 0;
+                               const Rosegarden::MappedObjectPropertyList &values) = 0;
 
     // Get a mapped object id for a object type
     //
@@ -250,6 +264,19 @@ public:
     virtual std::vector<QString> getPropertyList(int id,
                                                  const QString &property) = 0;
 
+    // Get a list of available plugins
+    //
+    virtual std::vector<QString> getPluginInformation() = 0;
+
+    // Nasty hack: program name/number mappings are one thing that
+    // mapped object properties can't cope with
+    //
+    virtual QString getPluginProgram(int mappedId, int bank, int program) = 0;
+
+    // Nastier hack: return value is bank << 16 + program
+    //
+    virtual unsigned long getPluginProgram(int mappedId, const QString &name) = 0;
+    
     // Cheat - we can't use a call (getPropertyList) during playback
     // so we use this method to set port N on plugin X.
     //
@@ -257,6 +284,9 @@ public:
                                unsigned long id,
                                float value) = 0;
 
+    virtual float getMappedPort(int pluginIn,
+				unsigned long id) = 0;
+
     // Create a (transient, writeable) object
     //
     virtual int createMappedObject(int type) = 0;
