@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -357,7 +357,7 @@ ArtsDriver::stopPlayback()
 
 
 void
-ArtsDriver::resetPlayback(const RealTime &position)
+ArtsDriver::resetPlayback(const RealTime &oldPosition, const RealTime &position)
 {
     //allNotesOff();
     RealTime artsPlayStartTime = RealTime(m_artsPlayStartTime.sec,
@@ -456,6 +456,10 @@ ArtsDriver::processNotesOff(const RealTime &time)
 void
 ArtsDriver::processAudioQueue(bool /*now*/)
 {
+#ifdef NOT_DEFINED
+/*** Removed following change to SoundDriver structure for audio files.
+     This would need to be heavily revised */
+
     // Now check queue for events that need playing
     std::list<PlayableAudioFile*>::iterator it;
     RealTime currentTime = getSequencerTime();
@@ -518,7 +522,7 @@ ArtsDriver::processAudioQueue(bool /*now*/)
                defunctEvents++;
     }
     while(defunctEvents);
-
+#endif
 }
 
 RealTime
@@ -575,8 +579,12 @@ ArtsDriver::getMappedComposition()
     
 void
 ArtsDriver::processMidiOut(const MappedComposition &mC,
-                           bool now)
+                           const RealTime &sliceStart,
+                           const RealTime &sliceEnd)
 {
+    // special case for unqueued events
+    bool now = (sliceStart == RealTime::zeroTime && sliceEnd == RealTime::zeroTime);
+
     Arts::MidiEvent event;
 
     // MidiCommandStatus
@@ -947,9 +955,19 @@ ArtsDriver::processMidiIn(const Arts::MidiCommand &midiCommand,
 
 
 void
-ArtsDriver::processEventsOut(const MappedComposition &mC,
-                             bool now)
+ArtsDriver::processEventsOut(const MappedComposition &mC)
 {
+    processEventsOut(mC, RealTime::zeroTime, RealTime::zeroTime);
+}
+
+void
+ArtsDriver::processEventsOut(const MappedComposition &mC,
+                             const RealTime &sliceStart,
+                             const RealTime &sliceEnd)
+{
+    // special case for unqueued events
+    bool now = (sliceStart == RealTime::zeroTime && sliceEnd == RealTime::zeroTime);
+
     // Start playback if it isn't already
     //
     if (m_startPlayback)
@@ -966,6 +984,10 @@ ArtsDriver::processEventsOut(const MappedComposition &mC,
     {
         if ((*i)->getType() == MappedEvent::Audio)
         {
+#ifdef NOT_DEFINED
+/*** Removed following change to SoundDriver structure for audio files.
+     This would need to be heavily revised */
+
             PlayableAudioFile *audioFile =
                     new PlayableAudioFile((*i)->getInstrument(),
                                           getAudioFile((*i)->getAudioID()),
@@ -974,16 +996,18 @@ ArtsDriver::processEventsOut(const MappedComposition &mC,
                                           (*i)->getDuration());
 
             queueAudio(audioFile);
+#endif
         }
     }
 
     // do MIDI events
-    processMidiOut(mC, now);
+    processMidiOut(mC, sliceStart, sliceEnd);
 
     // do any audio events
     processAudioQueue(now);
 }
 
+
 bool
 ArtsDriver::record(RecordStatus recordStatus)
 {
