@@ -3,7 +3,7 @@
   Rosegarden-4
   A sequencer and musical notation editor.
 
-  This program is Copyright 2000-2004
+  This program is Copyright 2000-2005
   Guillaume Laurent   <glaurent@telegraph-road.org>,
   Chris Cannam        <cannam@all-day-breakfast.com>,
   Richard Bown        <bownie@bownie.com>
@@ -22,6 +22,7 @@
 
 #ifndef HAVE_ALSA
 
+#include <map>
 #include <arts/artsmidi.h>
 #include <arts/soundserver.h>
 #include <arts/artsflow.h>     // aRts audio subsys
@@ -52,7 +53,7 @@ public:
     virtual void initialise();
     virtual void initialisePlayback(const RealTime &position);
     virtual void stopPlayback();
-    virtual void resetPlayback(const RealTime &position);
+    virtual void resetPlayback(const RealTime &oldPosition, const RealTime &position);
     virtual void allNotesOff();
     virtual void processNotesOff(const RealTime &time);
 
@@ -60,13 +61,14 @@ public:
 
     virtual MappedComposition *getMappedComposition();
     
-    virtual void processEventsOut(const MappedComposition &mC,
-                                  bool now);
+    virtual void processEventsOut(const MappedComposition &mC);
 
     virtual bool record(RecordStatus recordStatus);
 
     virtual void processPending();
 
+    virtual bool areClocksRunning() const { return true; }
+
     // Not supported
     //
     virtual unsigned int getSampleRate() const { return 0; }
@@ -78,6 +80,10 @@ public:
     void processMidiIn(const Arts::MidiCommand &midiCommand,
                        const Arts::TimeStamp &timeStamp);
 
+    virtual void processEventsOut(const MappedComposition &mC,
+				  const RealTime &sliceStart,
+				  const RealTime &sliceEnd);
+
     // Some Arts helper methods 'cos the basic Arts::TimeStamp
     // method is a bit unhelpful
     //
@@ -97,7 +103,7 @@ public:
     // Plugin instance management - do nothing for the moment
     //
     virtual void setPluginInstance(InstrumentId /*id*/,
-                                   unsigned long /*pluginId*/,
+                                   QString /*pluginIdent*/,
                                    int /*position*/) {;}
 
     virtual void removePluginInstance(InstrumentId /*id*/,
@@ -110,22 +116,66 @@ public:
                                             unsigned long /*portNumber*/,
                                             float /*value*/) {;}
 
+    virtual float getPluginInstancePortValue(InstrumentId id,
+					     int position,
+					     unsigned long portNumber) { return 0.0; }
+
     virtual void setPluginInstanceBypass(InstrumentId /*id*/,
                                          int /*position*/,
                                          bool /*value*/) {;}
 
+    virtual QStringList getPluginInstancePrograms(InstrumentId id,
+						  int position) { return QStringList(); }
+
+    virtual QString getPluginInstanceProgram(InstrumentId id,
+					     int position) { return QString::null; }
+
+    virtual QString getPluginInstanceProgram(InstrumentId id,
+					     int position,
+					     int bank,
+					     int program) { return QString::null; }
+
+    virtual unsigned long getPluginInstanceProgram(InstrumentId id,
+						   int position,
+						   QString name) { return 0; }
+    
+    virtual void setPluginInstanceProgram(InstrumentId id,
+					  int position,
+					  QString program) {;}
+
+    virtual QString configurePlugin(InstrumentId id,
+				    int position,
+				    QString key,
+				    QString value) { return QString::null; }
+
+    virtual void setAudioBussLevels(int bussId,
+				    float dB,
+				    float pan) {}
+
+    virtual void setAudioInstrumentLevels(InstrumentId id,
+					  float dB,
+					  float pan) {}
+
+
     virtual bool checkForNewClients() { return false; }
     
     virtual void setLoop(const RealTime &/*loopStart*/,
                          const RealTime &/*loopEnd*/) {;}
 
-    virtual void sendMidiClock() {;}
-
     virtual void getAudioInstrumentNumbers(InstrumentId &audioInstrumentBase,
 					   int &audioInstrumentCount) {
 	audioInstrumentBase = AudioInstrumentBase;
 	audioInstrumentCount = 16;
     }
+ 
+    virtual void getSoftSynthInstrumentNumbers(InstrumentId &ssInstrumentBase,
+					       int &ssInstrumentCount) {
+	ssInstrumentBase = SoftSynthInstrumentBase;
+	ssInstrumentCount = 0;
+    }
+
+    virtual void claimUnwantedPlugin(void *plugin) { }
+    virtual void scavengePlugins() { }
 
     virtual std::vector<PlayableAudioFile*> getPlayingAudioFiles() 
         { return std::vector<PlayableAudioFile*>(); }
@@ -133,11 +183,14 @@ public:
 protected:
     virtual void generateInstruments();
     virtual void processAudioQueue(bool now);
+
     virtual void processMidiOut(const MappedComposition &mC,
-                                bool now);
+                                const RealTime &sliceStart,
+				const RealTime &sliceEnd);
 
     void sendDeviceController(MidiByte controller, MidiByte value);
-
+    std::map<unsigned int, MappedEvent*>        m_noteOnMap;
+    
 private:
     // aRts sound server reference
     //
