@@ -5,7 +5,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2006
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -204,18 +204,6 @@ AudioPlayQueue::clear()
     m_maxBuffers = 0;
 }
 
-bool
-AudioPlayQueue::empty() const
-{
-    return m_unscheduled.empty() && m_files.empty();
-}
-
-size_t 
-AudioPlayQueue::size() const
-{
-    return m_unscheduled.size() + m_files.size();
-}
-
 void
 AudioPlayQueue::getPlayingFiles(const RealTime &sliceStart,
 				const RealTime &sliceDuration,
@@ -338,7 +326,7 @@ AudioPlayQueue::getPlayingFilesForInstrument(const RealTime &sliceStart,
 #endif
 
 	    size_t j = 0;
-	    for (j = 0; j < written; ++j) {
+	    for (size_t j = 0; j < written; ++j) {
 		if (playing[j] == f) break;
 	    }
 	    if (j < written) break; // already have it
