@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2006
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -85,16 +85,6 @@ public:
      */
     void clear();
 
-    /**
-     * Return true if the queue is empty.
-     */
-    bool empty() const;
-
-    /**
-     * Return the total number of files in the queue.  (May be slow.)
-     */
-    size_t size() const;
-
     /**
      * Look up the files playing during a given slice and return them
      * in the passed FileSet.  The pointers returned are still owned
