@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -29,6 +29,7 @@
 #include "RealTime.h"
 #include "RingBuffer.h"
 #include "RunnablePluginInstance.h"
+#include "AudioPlayQueue.h"
 
 namespace Rosegarden
 {
@@ -51,6 +52,8 @@ public:
     // exit and clean up, before destruction.
     void terminate();
 
+    bool running() const { return m_running; }
+
     int getLock();
     int tryLock();
     int releaseLock();
@@ -58,6 +61,7 @@ public:
 
 protected:
     virtual void threadRun() = 0;
+    virtual int getPriority() { return 0; }
 
     std::string       m_name;
 
@@ -67,6 +71,7 @@ protected:
     pthread_t         m_thread;
     pthread_mutex_t   m_lock;
     pthread_cond_t    m_condition;
+    bool              m_running;
     volatile bool     m_exiting;
 
 private:
@@ -106,7 +111,7 @@ public:
 
     /**
      * A buss is "dormant" if every readable sample on every one of
-     * its buffers is zero.  It can therefore be safely ignored during
+     * its buffers is zero.  It can therefore be safely skipped during
      * playback.
      */
     bool isBussDormant(int buss) {
@@ -124,6 +129,12 @@ public:
 	}
     }
 
+    /// For call from MappedStudio.  Pan is in range -100.0 -> 100.0
+    void setBussLevels(int buss, float dB, float pan);
+
+    /// For call regularly from anywhere in a non-RT thread
+    void updateInstrumentConnections();
+
 protected:
     virtual void threadRun();
 
@@ -138,12 +149,17 @@ protected:
 
     struct BufferRec
     {
-	BufferRec() : dormant(true), buffers() { }
+	BufferRec() : dormant(true), buffers(), instruments(),
+		      gainLeft(0.0), gainRight(0.0) { }
 	~BufferRec();
 
 	bool dormant;
 
 	std::vector<RingBuffer<sample_t> *> buffers;
+	std::vector<bool> instruments; // index is instrument id minus base
+
+	float gainLeft;
+	float gainRight;
     };
 
     typedef std::map<int, BufferRec> BufferMap;
@@ -168,20 +184,34 @@ public:
 
     void setBussMixer(AudioBussMixer *mixer) { m_bussMixer = mixer; }
 
-    void setPlugin(InstrumentId id, int position, unsigned int pluginId);
+    void setPlugin(InstrumentId id, int position, QString identifier);
     void removePlugin(InstrumentId id, int position);
     void removeAllPlugins();
     void setPluginPortValue(InstrumentId id, int position,
 			    unsigned int port, float value);
+    float getPluginPortValue(InstrumentId id, int position,
+			     unsigned int port);
     void setPluginBypass(InstrumentId, int position, bool bypass);
+    QStringList getPluginPrograms(InstrumentId, int);
+    QString getPluginProgram(InstrumentId, int);
+    QString getPluginProgram(InstrumentId, int, int, int);
+    unsigned long getPluginProgram(InstrumentId, int, QString);
+    void setPluginProgram(InstrumentId, int, QString);
+    QString configurePlugin(InstrumentId, int, QString, QString);
     void resetAllPlugins();
+    void destroyAllPlugins();
+
+    RunnablePluginInstance *getSynthPlugin(InstrumentId id) { return m_synths[id]; }
+
+    /**
+     * Return the total of the plugin latencies for a given instrument
+     * or buss id.
+     */
+    size_t getPluginLatency(unsigned int id);
 
     /**
      * Prebuffer.  This should be called only when the transport is
      * not running. 
-     *
-     * This function is called from AudioBussMixer::fillBuffers, so
-     * you do not normally need to call it yourself.
      */
     void fillBuffers(const RealTime &currentTime);
 
@@ -190,10 +220,20 @@ public:
      */
     void emptyBuffers(RealTime currentTime = RealTime::zeroTime);
 
+    /**
+     * An instrument is "empty" if it has no audio files, synths or
+     * plugins assigned to it, and so cannot generate sound.  Empty
+     * instruments can safely be ignored during playback.
+     */
+    bool isInstrumentEmpty(InstrumentId id) {
+	return m_bufferMap[id].empty;
+    }
+
     /**
      * An instrument is "dormant" if every readable sample on every
-     * one of its buffers is zero.  It can therefore be safely ignored
-     * during playback.
+     * one of its buffers is zero.  Dormant instruments can safely be
+     * skipped rather than mixed during playback, but they should not
+     * be ignored (unless also empty).
      */
     bool isInstrumentDormant(InstrumentId id) {
 	return m_bufferMap[id].dormant;
@@ -213,22 +253,32 @@ public:
 	}
     }
 
+    /// For call from MappedStudio.  Pan is in range -100.0 -> 100.0
+    void setInstrumentLevels(InstrumentId instrument, float dB, float pan);
+
 protected:
     virtual void threadRun();
 
-    void processBlocks(bool forceFill, bool &readSomething);
+    virtual int getPriority() { return 3; }
+
+    void processBlocks(bool &readSomething);
     void processEmptyBlocks(InstrumentId id);
-    bool processBlock(InstrumentId id, PlayableAudioFileList&, bool forceFill,
-		      bool &readSomething);
+    bool processBlock(InstrumentId id, PlayableAudioFile **, size_t, bool &readSomething);
     void generateBuffers();
 
     AudioFileReader  *m_fileReader;
     AudioBussMixer   *m_bussMixer;
     unsigned int      m_blockSize;
 
-    typedef std::map<int, RunnablePluginInstance *> PluginList;
+    // The plugin data structures will all be pre-sized and so of
+    // fixed size during normal run time; this will allow us to add
+    // and edit plugins without locking.
+    typedef std::vector<RunnablePluginInstance *> PluginList;
     typedef std::map<InstrumentId, PluginList> PluginMap;
+    typedef std::map<InstrumentId, RunnablePluginInstance *> SynthPluginMap;
+    RunnablePluginInstance *getPluginInstance(InstrumentId, int);
     PluginMap m_plugins;
+    SynthPluginMap m_synths;
 
     // maintain the same number of these as the maximum number of
     // channels on any audio instrument
@@ -236,7 +286,9 @@ protected:
 
     struct BufferRec
     {
-	BufferRec() : dormant(true), filledTo(RealTime::zeroTime), buffers() { }
+	BufferRec() : empty(true), dormant(true), zeroFrames(0),
+		      filledTo(RealTime::zeroTime), channels(2),
+		      buffers(), gainLeft(0.0), gainRight(0.0), volume(0.0) { }
 	~BufferRec();
 
 	bool empty;
@@ -267,8 +319,11 @@ public:
 
     bool kick(bool wantLock = true);
 
-    void updateReadyStatuses(PlayableAudioFileList &audioQueue);
-    void updateDefunctStatuses();
+    /**
+     * Prebuffer.  This should be called only when the transport is
+     * not running. 
+     */
+    void fillBuffers(const RealTime &currentTime);
 
 protected:
     virtual void threadRun();
