@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2004
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -74,6 +74,16 @@ public:
         { return (m_fileName + std::string(".pk")); }
 
 
+    //!!! NOT IMPLEMENTED YET
+    // 
+    virtual bool decode(const unsigned char *sourceData,
+			size_t sourceBytes,
+			size_t targetSampleRate,
+			size_t targetChannels,
+			size_t targetFrames,
+			std::vector<float *> &targetData,
+			bool addToResultBuffers = false) { return false; }
+
 protected:
 
 };
