@@ -4,7 +4,7 @@
     Rosegarden-4
     A sequencer and musical notation editor.
 
-    This program is Copyright 2000-2006
+    This program is Copyright 2000-2005
         Guillaume Laurent   <glaurent@telegraph-road.org>,
         Chris Cannam        <cannam@all-day-breakfast.com>,
         Richard Bown        <bownie@bownie.com>
@@ -61,7 +61,6 @@ DSSIPluginInstance::DSSIPluginInstance(PluginFactory *factory,
     m_sampleRate(sampleRate),
     m_latencyPort(0),
     m_run(false),
-    m_runSinceReset(false),
     m_bypassed(false),
     m_grouped(false)
 {
@@ -119,7 +118,6 @@ DSSIPluginInstance::DSSIPluginInstance(PluginFactory *factory,
     m_sampleRate(sampleRate),
     m_latencyPort(0),
     m_run(false),
-    m_runSinceReset(false),
     m_bypassed(false),
     m_grouped(false)
 {
@@ -176,7 +174,6 @@ DSSIPluginInstance::init()
 					   (i, data));
 
 		m_backupControlPortsIn.push_back(0.0);
-		m_portChangedSinceProgramChange.push_back(false);
 
 	    } else {
 		LADSPA_Data *data = new LADSPA_Data(0.0);
@@ -204,15 +201,8 @@ DSSIPluginInstance::init()
 size_t
 DSSIPluginInstance::getLatency()
 {
-#ifdef DEBUG_DSSI
-//    std::cerr << "DSSIPluginInstance::getLatency(): m_latencyPort " << m_latencyPort << ", m_run " << m_run << std::endl;
-#endif
-
     if (m_latencyPort) {
 	if (!m_run) run(RealTime::zeroTime);
-#ifdef DEBUG_DSSI 
-	std::cerr << "DSSIPluginInstance::getLatency(): latency is " << (size_t)(*m_latencyPort + 0.1) << std::endl;
-#endif
 	return (size_t)(*m_latencyPort + 0.1);
     }
     return 0;
@@ -221,28 +211,10 @@ DSSIPluginInstance::getLatency()
 void
 DSSIPluginInstance::silence()
 {
-#ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance::silence: m_run " << m_run << ", m_runSinceReset " << m_runSinceReset << std::endl;
-#endif
-
-    if (m_run && !m_runSinceReset) {
-	return;
-    }
     if (m_instanceHandle != 0) {
 	deactivate();
 	activate();
     }
-    m_runSinceReset = false;
-}
-
-void
-DSSIPluginInstance::discardEvents()
-{
-#ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance::discardEvents" << std::endl;
-#endif
-
-    m_eventBuffer.reset();
 }
 
 void
@@ -529,7 +501,7 @@ void
 DSSIPluginInstance::selectProgramAux(QString program, bool backupPortValues)
 {
 #ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance[" << this << "]::selectProgram(" << program << ", " << backupPortValues << ")" << std::endl;
+    std::cerr << "DSSIPluginInstance::selectProgram(" << program << ")" << std::endl;
 #endif
 
     if (!m_descriptor) return;
@@ -566,14 +538,9 @@ DSSIPluginInstance::selectProgramAux(QString program, bool backupPortValues)
     m_descriptor->select_program(m_instanceHandle, bankNo, programNo);
     pthread_mutex_unlock(&m_processLock);
 
-#ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance::selectProgram(" << program << "): made select_program(" << bankNo << "," << programNo << ") call" << std::endl;
-#endif
-
     if (backupPortValues) {
 	for (size_t i = 0; i < m_backupControlPortsIn.size(); ++i) {
 	    m_backupControlPortsIn[i] = *m_controlPortsIn[i].second;
-	    m_portChangedSinceProgramChange[i] = false;
 	}
     }
 }
@@ -582,35 +549,25 @@ void
 DSSIPluginInstance::activate()
 {
 #ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance[" << this << "]::activate" << std::endl;
+    std::cerr << "DSSIPluginInstance::activate" << std::endl;
 #endif
 
     if (!m_descriptor || !m_descriptor->LADSPA_Plugin->activate) return;
+    m_eventBuffer.reset();
     m_descriptor->LADSPA_Plugin->activate(m_instanceHandle);
 
-    for (size_t i = 0; i < m_backupControlPortsIn.size(); ++i) {
-	if (m_portChangedSinceProgramChange[i]) {
-#ifdef DEBUG_DSSI
-	    std::cerr << "DSSIPluginInstance::activate: setting port " << m_controlPortsIn[i].first << " to " << m_backupControlPortsIn[i] << std::endl;
-#endif
-	    *m_controlPortsIn[i].second = m_backupControlPortsIn[i];
-	}
-    }
-
     if (m_program) {
 #ifdef DEBUG_DSSI
 	std::cerr << "DSSIPluginInstance::activate: restoring program " << m_program << std::endl;
 #endif
 	selectProgramAux(m_program, false);
+    }
 
-	for (size_t i = 0; i < m_backupControlPortsIn.size(); ++i) {
-	    if (m_portChangedSinceProgramChange[i]) {
+    for (size_t i = 0; i < m_backupControlPortsIn.size(); ++i) {
 #ifdef DEBUG_DSSI
-		std::cerr << "DSSIPluginInstance::activate: setting port " << m_controlPortsIn[i].first << " to " << m_backupControlPortsIn[i] << std::endl;
+	std::cerr << "DSSIPluginInstance::activate: setting port " << m_controlPortsIn[i].first << " to " << m_backupControlPortsIn[i] << std::endl;
 #endif
-		*m_controlPortsIn[i].second = m_backupControlPortsIn[i];
-	    }
-	}
+	*m_controlPortsIn[i].second = m_backupControlPortsIn[i];
     }
 }
 
@@ -618,11 +575,9 @@ void
 DSSIPluginInstance::connectPorts()
 {
     if (!m_descriptor || !m_descriptor->LADSPA_Plugin->connect_port) return;
-#ifdef DEBUG_DSSI
     std::cerr << "DSSIPluginInstance::connectPorts: " << m_audioPortsIn.size() 
 	      << " audio ports in, " << m_audioPortsOut.size() << " out, "
 	      << m_outputBufferCount << " output buffers" << std::endl;
-#endif
 
     assert(sizeof(LADSPA_Data) == sizeof(float));
     assert(sizeof(sample_t) == sizeof(float));
@@ -664,7 +619,7 @@ void
 DSSIPluginInstance::setPortValue(unsigned int portNumber, float value)
 {
 #ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance[" << this << "]::setPortValue(" << portNumber << ") to " << value << std::endl;
+    std::cerr << "DSSIPluginInstance::setPortValue(" << portNumber << ") to " << value << std::endl;
 #endif
     for (unsigned int i = 0; i < m_controlPortsIn.size(); ++i) {
         if (m_controlPortsIn[i].first == portNumber) {
@@ -679,7 +634,6 @@ DSSIPluginInstance::setPortValue(unsigned int portNumber, float value)
 	    }
             (*m_controlPortsIn[i].second) = value;
 	    m_backupControlPortsIn[i] = value;
-	    m_portChangedSinceProgramChange[i] = true;
         }
     }
 }
@@ -804,10 +758,6 @@ DSSIPluginInstance::handleController(snd_seq_event_t *ev)
 {
     int controller = ev->data.control.param;
 
-#ifdef DEBUG_DSSI_PROCESS
-    std::cerr << "DSSIPluginInstance::handleController " << controller << std::endl;
-#endif
-
     if (controller == 0) { // bank select MSB
 	
 	m_pending.msb = ev->data.control.value;
@@ -835,17 +785,12 @@ DSSIPluginInstance::run(const RealTime &blockTime)
     static snd_seq_event_t localEventBuffer[EVENT_BUFFER_SIZE];
     int evCount = 0;
 
-    bool needLock = false;
-    if (m_descriptor->select_program) needLock = true;
-
-    if (needLock) {
-	if (pthread_mutex_trylock(&m_processLock) != 0) {
-	    for (size_t ch = 0; ch < m_audioPortsOut.size(); ++ch) {
-		memset(m_outputBuffers[ch], 0, m_blockSize * sizeof(sample_t));
-	    }
-	    return;
+    if (pthread_mutex_trylock(&m_processLock) != 0) {
+	for (size_t ch = 0; ch < m_audioPortsOut.size(); ++ch) {
+	    memset(m_outputBuffers[ch], 0, m_blockSize * sizeof(sample_t));
 	}
-    }
+	return;
+    }	
 
     if (m_grouped) {
 	runGrouped(blockTime);
@@ -862,8 +807,7 @@ DSSIPluginInstance::run(const RealTime &blockTime)
 	    }
 	}
 	m_run = true;
-	m_runSinceReset = true;
-	if (needLock) pthread_mutex_unlock(&m_processLock);
+	pthread_mutex_unlock(&m_processLock);
 	return;
     }
 
@@ -916,20 +860,10 @@ DSSIPluginInstance::run(const RealTime &blockTime)
     }
 
     if (m_pending.program >= 0 && m_descriptor->select_program) {
-
 	int program = m_pending.program;
 	int bank = m_pending.lsb + 128 * m_pending.msb;
-
-#ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance::run: making select_program(" << bank << "," << program << " call" << std::endl;
-#endif
-
 	m_pending.lsb = m_pending.msb = m_pending.program = -1;
 	m_descriptor->select_program(m_instanceHandle, bank, program);
-
-#ifdef DEBUG_DSSI
-    std::cerr << "DSSIPluginInstance::run: made select_program(" << bank << "," << program << " call" << std::endl;
-#endif
     }
 
 #ifdef DEBUG_DSSI_PROCESS
@@ -948,17 +882,9 @@ DSSIPluginInstance::run(const RealTime &blockTime)
 #endif
 
  done:
-    if (needLock) pthread_mutex_unlock(&m_processLock);
+    pthread_mutex_unlock(&m_processLock);
 
-    if (m_audioPortsOut.size() == 0) {
-	// copy inputs to outputs
-	for (size_t ch = 0; ch < m_idealChannelCount; ++ch) {
-	    size_t sch = ch % m_audioPortsIn.size();
-	    for (size_t i = 0; i < m_blockSize; ++i) {
-		m_outputBuffers[ch][i] = m_inputBuffers[sch][i];
-	    }
-	}
-    } else if (m_idealChannelCount < m_audioPortsOut.size()) {
+    if (m_idealChannelCount < m_audioPortsOut.size()) {
 	if (m_idealChannelCount == 1) {
 	    // mix down to mono
 	    for (size_t ch = 1; ch < m_audioPortsOut.size(); ++ch) {
@@ -979,7 +905,6 @@ DSSIPluginInstance::run(const RealTime &blockTime)
 
     m_lastRunTime = blockTime;
     m_run = true;
-    m_runSinceReset = true;
 }
 
 void
